
note
	
	description: "Convenience routines for condition handling"
	author: "David Hollenberg"
	date: "March 26, 2004"

class CONDITION_UTILITIES
inherit
	CONDITION_HANDLER_ACCESS

feature -- Operations

	handle_hard_error (s: STRING)
			-- Handle a hard non-retryable error with text `s'
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_hard_error
			cond.disable_retry
			cond.set_text (s)
			cond.handle
		end

	handle_soft_error (s: STRING)
			-- Handle a soft non-retryable error with text `s'
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_soft_error
			cond.disable_retry
			cond.set_text (s)
			cond.handle
		end

	handle_user_error (s: STRING)
			-- Handle a hard non-retryable error with text `s'
			-- that is due to a user error (e.g., entering
			-- invalid date)
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_hard_error
			cond.disable_retry
			cond.disable_notify
			cond.set_text (s)
			cond.handle
		end

	handle_internal_error (s: STRING)
			-- Handle a hard non-retryable error with text `s'
			-- that is an internal error
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_hard_error
			cond.disable_retry
			cond.set_text (s)
			cond.handle
		end

	handle_unsupported (s: STRING)
			-- Handle a hard non-retryable error that is
			-- an error reporting unsupported condition
			-- `s'
		require
			text_exists: s /= Void
		do
			handle_hard_error ("Unsupported condition: " + s)
		end

	warn (s: STRING)
			-- Handle a non-retryable warning with text `s'
			-- without offering a Notify option
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_warning
			cond.disable_retry
			cond.disable_notify
			cond.set_text (s)
			cond.handle
		end

	warn_with_notify (s: STRING)
			-- Handle a non-retryable warning with text `s',
			-- offering a Notify option
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_warning
			cond.disable_retry
			cond.set_text (s)
			cond.handle
		end

	warn_aligned (s: STRING)
			-- Handle a non-retryable warning with text `s',
			-- where columns should be aligned (usually means
			-- a fixed width font must be used)
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_warning
			cond.disable_retry
			cond.disable_notify
			cond.enable_column_alignment
			cond.set_text (s)
			cond.handle
		end

	question (s: STRING): BOOLEAN
			-- User's answer to question with text `s'
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_question
			cond.set_text (s)
			cond.handle
			Result := cond.answer
		end

	confirm (s: STRING)
			-- Get confirmation from user about whether to
			-- proceed, with message `s'
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_confirmation
			cond.set_text (s + "?")
			cond.handle
		end

	confirm_aligned (s: STRING)
			-- Get confirmation from user about whether to
			-- proceed, with message `s',
			-- where columns should be aligned (usually means
			-- a fixed width font must be used)
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_confirmation
			cond.enable_column_alignment
			cond.set_text (s + "?")
			cond.handle
		end

	inform (s: STRING)
			-- Inform user about something described by `s'
		require
			text_exists: s /= Void
		local
			cond: EXPLICIT_CONDITION
		do
			create cond
			cond.set_information
			cond.set_text (s)
			cond.handle
		end

end
