
note
	
	author: "David Hollenberg"
	date:	"January 31, 1996"
	description: "Constants for exceptions related to operations on a %
		%relational database"

class DB_EXCEPTIONS

feature -- Interface errors

	Db_call_interface_error: STRING = "DB call interface error"
			-- Name of exception raised when Database Call
			-- Interface error occurs

feature -- Programming errors

	Db_access_error: STRING = "DB connection access error"
			-- Name of exception raised when program attempts
			-- to do an operation on a connection which is not
			-- allowed by current access level

	Db_uncommitted_changes: STRING = "DB uncommitted changes"
			-- Name of exception raised when program attempts
			-- to do an operation with uncommitted changes which
			-- must only be done when there are no such changes.

	Db_unknown_statement_type: STRING = "DB unknown statement type"
			-- Name of exception raised when program cannot
			-- determine type of potential SQL statement but
			-- database server accepts it

	Db_illegal_statement_type: STRING = "DB illegal statement type"
			-- Name of exception raised when program can
			-- determine type of potential SQL statement and
			-- this type of statement is not allowed because
			-- of the model of database interaction we have
			-- constructed (e.g., "commit" is illegal in our model)

	Db_wrong_statement_type: STRING = "DB wrong statement type"
			-- Name of exception raised when wrong string which
			-- represents wrong type of statement is passed
			-- to a routine (e.g., passing a query to `sql_change',
			-- which expects a DML statement)

	Db_inconsistent_type: STRING = "DB inconsistent stmt type"
			-- Name of exception raised when program detects
			-- an inconsistency between the type of statement
			-- indicated by first word of statement and type
			-- indicated by database server

	Db_non_query_statement: STRING = "String not query statement"
			-- Name of exception raised when a SQL_STATEMENT
			-- is requested for a string that is
			-- supposed to be a query but the string does
			-- not represent a query

	Db_non_dml_statement: STRING = "String not DML statement"
			-- Name of exception raised when a SQL_STATEMENT
			-- is requested for a string that is
			-- supposed to be a DML statement but the
			-- string does not represent a DML statement

	Db_non_ddl_statement: STRING = "String not DDL statement"
			-- Name of exception raised when a SQL_STATEMENT
			-- is requested for a string that is
			-- supposed to be a DDL statement but the
			-- string does not represent a DDL statement

	Db_non_block_statement: STRING = "String not block statement"
			-- Name of exception raised when a SQL_STATEMENT
			-- is requested for a string that is
			-- supposed to be a block of SQL statements but the
			-- string does not represent such a block
			-- (e.g., PL/SQL block)

	Db_already_registered: STRING = "DB rowid dup registration"
			-- Name of exception raised when an item with a
			-- rowid is registered as part of the current
			-- transaction, but an item with the same rowid
			-- is already registered

	Db_not_registered: STRING = "DB row not registered"
			-- Name of exception raised on an attempt to
			-- re-register a row that is not registered

	Db_already_connection: STRING = "DB row has connection"
			-- Name of exception raised when a row is
			-- registered explicitly, but it already has an
			-- associated connection

	Db_wrong_connection: STRING = "DB wrong connection"
			-- Name of exception raised when an attempt is made
			-- to do an operation with the wrong connection
			-- (.e.g, trying to registered a rowid which is
			-- associated with one connection with a different
			-- connection)

	Db_row_not_locked: STRING = "DB row not locked"
			-- Name of exception raised when an attempt is made
			-- to do an operation with a row which requires
			-- that it be locked (such as register it), but
			-- the row is not locked

	Db_no_rows_found: STRING = "DB no rows found"
			-- Name of exception raised when a query that is
			-- expected to return exactly one row returns
			-- zero rows

	Db_multiple_rows_found: STRING = "DB multiple rows found"
			-- Name of exception raised when a query that is
			-- expected to return exactly one row returns
			-- more than one row

	Db_missing_row: STRING = "DB row key not found"
			-- Name of exception raised when an attempt is made
			-- to retrieve a row by key but there is no such
			-- row in the database

	Db_multiple_key_rows: STRING = "DB multiple key matches"
			-- Name of exception raised when an attempt is made
			-- to retrieve a row by key and there is more than
			-- one matching row in the database

	Db_invalid_new_row: STRING = "DB new row not allowed"
			-- Name of exception raised when an attempt is made
			-- to create a new row by key for a DB_IDENTIFIED_ROW

	Db_existing_row: STRING = "DB row key exists"
			-- Name of exception raised when an attempt is made
			-- to create a row by key but there is already a
			-- row with this key in the database

	Db_no_insert: STRING = "DB row insert not allowed"
			-- Name of exception raised when an attempt is made
			-- to change a field in a new row and the row
			-- is not allowed to be inserted into the database

	Db_no_update: STRING = "DB field update not allowed"
			-- Name of exception raised when an attempt is made
			-- to change a field in an old row and the field
			-- is not allowed to be updated in the database

	Db_no_copy: STRING = "DB row copy not allowed"
			-- Name of exception raised when an attempt is made
			-- to copy into a row from another row and
			-- row copy is not allowed

	Db_no_detached_twin: STRING = "DB row detached twin not allowed"
			-- Name of exception raised when an attempt is made
			-- to call `detached_twin' on a row
			-- when row copy is not allowed

	Db_insert_existing: STRING = "DB insert row with rowid"
			-- Name of exception raised when an attempt is made
			-- to insert immediate a row into database which
			-- already has a rowid

	Db_update_new: STRING = "DB update row without rowid"
			-- Name of exception raised when an attempt is made
			-- to update immediate a row in database which
			-- does not have a rowid

	Db_null_copy: STRING = "DB null copy into non-null attribute"
			-- Name of exception raised when an attempt is made
			-- to copy a null value into an attribute which
			-- cannot be null

	Db_deleted_row_access: STRING = "DB access deleted row"
			-- Name of exception raised when an attempt is made
			-- to do an operation with a row which is marked
			-- as deleted (the delete may or may not have
			-- been posted)

	Db_corrupted_row_access: STRING = "DB access corrupted row"
			-- Name of exception raised when an attempt is made
			-- to do an operation with a row which is marked
			-- as corrupted (invalidated by rollback)

	Db_empty_row_access: STRING = "DB access empty row"
			-- Name of exception raised when an attempt is made
			-- to do an operation with a row which is empty

	Db_reuse_registered: STRING = "DB reuse registered row"
			-- Name of exception raised when an attempt is made
			-- to reuse a row which is registered (fill it
			-- in via a query or creation)

	Db_change_key: STRING = "DB change key field"
			-- Name of exception raised when an attempt is made
			-- to change a key field in a row which has a rowid

	Db_no_connection: STRING = "DB no connection"
			-- Name of exception raised when an attempt is made
			-- to do an operation with a row which does not have
			-- an associated database connection

	Db_set_model_row: STRING = "DB bad set model row"
			-- Name of exception raised when an attempt is made
			-- to set `model_row' for a DB_ROWS_QUERY but
			-- this feature is not an attribute (or it is
			-- and descendant class forgot to redefine
			-- `set_model_row')

	Db_multiple_tables: STRING = "DB multiple tables"
			-- Name of exception raised when an attempt is made
			-- to do an operation allowed only for a row from
			-- one table, on a super-row from multiple joined
			-- tables

	Db_unbound_placeholder: STRING = "Placeholder not bound"
			-- Name of exception raised when a
			-- SQL_STATEMENT `execute' is done and an
			-- unbound placeholder is detected.  This can
			-- only happen with a reused statement.

	Db_invalid_placeholder_value: STRING = "Invalid placeholder value"
			-- Name of exception raised when a
			-- placeholder is bound to a value of static type ANY
			-- and the dynamic type is one which we don't know
			-- how to translate into a database datatype

	Db_placeholder_too_long: STRING = "Placeholder name too long"
			-- Name of exception raised when an attempt to
			-- bind a placeholder name which is too long is
			-- detected.  This error is normally detected as
			-- a precondition violation, but must also be
			-- caught when assertion monitoring is off in order
			-- to avoid overwriting fixed length data structures.

	Db_undefined_select_item: STRING = "Select item not defined"
			-- Name of exception raised when a a
			-- SQL_STATEMENT `execute' is done and an
			-- undefined placeholder is detected.  This
			-- can only happen with a reused statement.

	Db_conformance_error: STRING = "Database nonconforming arg"
			-- Name of exception raised when argument passed to
			-- database interface routine is not of the
			-- right type (e.g., passing a Sybase error handler
			-- to `set_error_handler' for an Oracle connection)

feature -- Database errors

	Db_invalid_boolean: STRING = "Invalid database boolean"
			-- Name of exception raised when a character value
			-- read from the database which is supposed to be
			-- a boolean has an invalid value

	Db_invalid_alert_wait_status: STRING = "Invalid alert wait status"
			-- Name of exception raised when an invalid status
			-- is returned from a database "wait for signal"
			-- operation

feature -- Transaction errors

	Db_transaction_failure: STRING = "DB transaction failed"
			-- Name of exception raised when a database
			-- transaction fails because an unlocked row
			-- was changed or deleted (detected when the
			-- row is locked)

end
