
note
	
	author: "David Hollenberg"
	date:	"March 26, 1996"
	description: "Named commonly used sets of options for a database %
		%query by key"

class DB_QUERY_OPTIONS

feature -- Options

	old_row: DB_QUERY_OPTION
			-- Option to retrieve an existing row by key
			-- without locking it.  Raises an exception if
			-- no row with specified key
		once
			create Result
			Result.set_old_row
		end
	
	old_no_exception_row: DB_QUERY_OPTION
			-- Option to retrieve an existing row by key
			-- without locking it.  Sets row `is_empty'
			-- if no row with specified key (does not raise
			-- exception)
		once
			create Result
			Result.set_old_row
			Result.set_no_exception (True)
		end
	
	old_locked_row: DB_QUERY_OPTION
			-- Option to retrieve an existing row by key
			-- and lock it, using wait option for connection
		once
			create Result
			Result.set_old_row
			Result.set_lock (True)
		end
	
	old_locked_row_wait: DB_QUERY_OPTION
			-- Option to retrieve an existing row by key
			-- and lock it, waiting if row is busy
		once
			create Result
			Result.set_old_row
			Result.set_wait (True)
			Result.set_lock (True)
		end
	
	old_no_exception_locked_row: DB_QUERY_OPTION
			-- Option to retrieve an existing row by key
			-- and lock it, using wait option for
			-- connection.  Sets row `is_empty' if no row
			-- with specified key (does not raise
			-- exception)
		once
			create Result
			Result.set_old_row
			Result.set_lock (True)
			Result.set_no_exception (True)
		end
	
	old_no_exception_locked_row_wait: DB_QUERY_OPTION
			-- Option to retrieve an existing row by key
			-- and lock it, waiting if row is
			-- busy.  Sets row `is_empty' if no row with
			-- specified key (does not raise exception)
		once
			create Result
			Result.set_old_row
			Result.set_lock (True)
			Result.set_wait (True)
			Result.set_no_exception (True)
		end
	
	old_no_exception_locked_row_nowait: DB_QUERY_OPTION
			-- Option to retrieve an existing row by key
			-- and lock it, without waiting if row is
			-- busy.  Sets row `is_empty' if no row with
			-- specified key (does not raise exception)
		once
			create Result
			Result.set_old_row
			Result.set_lock (True)
			Result.set_wait (False)
			Result.set_no_exception (True)
		end
	
	old_or_new_row: DB_QUERY_OPTION
			-- Option to retrieve an existing row by key
			-- or create a new registered row to be inserted
			-- if no existing row has specified key
		once
			create Result
			Result.set_old_or_new_row
		end
	
	old_or_new_locked_row: DB_QUERY_OPTION
			-- Option to retrieve an existing row by key
			-- or create a new registered row to be inserted
			-- if no existing row has specified key.  In
			-- addition, if there is an existing row with
			-- specified key it will be locked using
			-- wait option for connection
		once
			create Result
			Result.set_old_or_new_row
			Result.set_lock (True)
		end
	
	new_row: DB_QUERY_OPTION
			-- Option to create a new registered row to be
			-- inserted.  Does not check database table
			-- for already existing row with specified key
		once
			create Result
			Result.set_new_row
		end
	
	new_row_checked: DB_QUERY_OPTION
			-- Option to create a new registered row to be
			-- inserted.  Database is checked to see
			-- whether there is already an existing row
			-- with specified key and it is an error if
			-- there is such a row
		once
			create Result
			Result.set_new_row
			Result.set_new_rows_checked (True)
		end
	
end
