
note
	
	author: "David Hollenberg"
	date:	"February 11, 2000"
	description: "Miscellaneous database utilities"

class DB_UTILITIES

inherit
	ANY
	DB_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end

feature -- Dates

	db_date_from_unix_time (unix_time: INTEGER): DB_DATE
			-- Database date with same date and time as
			-- `unix_time', which is number of seconds since
			-- January 1, 1970
		local
			s: STRING
			parts: LIST [STRING]
		do
			parts := su.broken_into_words (s)
	    		-- Reformat from  "dayofweek Month day timeofday year"
	    		--            to  "day-Month-year timeofday"
			s.wipe_out
			s.append (parts.i_th (3))	-- Day of month
			s.extend ('-')
			s.append (parts.i_th (2))	-- Month
			s.extend ('-')
			s.append (parts.i_th (5))	-- Year
			s.extend (' ')
			s.append (parts.i_th (4))	-- Time of day
			Result := db.date_and_time (s)
		end

feature -- Date properties

	is_date (d: DB_DATE): BOOLEAN
			-- Is `d' a non-Void and non-null date?
		do
			Result := d /= Void and then (not d.is_null)
		end

	not_date (d: DB_DATE): BOOLEAN
			-- Is `d' not a valid date (i.e., is `d' Void
			-- or null)?
		do
			Result := d = Void or else d.is_null
		end

feature	-- List mass deletion

	delete_all_rows(lst: LIST[DB_DELETABLE_ROW])
		-- delete (from the db) all the rows in lst
		do
			if lst /= Void then
				lst.do_all(agent {DB_DELETABLE_ROW}.delete)
			end
		end

	delete_and_wipe_out(lst: LIST[DB_DELETABLE_ROW])
		-- delete (from the db) all the rows in lst and
		-- wipe_put the list too
		do
			if lst /= Void then
				delete_all_rows(lst)
				lst.wipe_out
			end
		end

feature	-- Conversion

	character_to_boolean (c: CHARACTER): BOOLEAN
			-- `c' converted to a boolean
		do
			if c = 'Y' then
				Result := True
			elseif c = 'N' then
				Result := False
			else
				handle_hard_error ("character '" + c.out + " does not represent a valid database boolean")
			end
		end

	string_to_boolean (s: STRING): BOOLEAN
			-- `s' converted to a boolean
		require
			s_not_void: s /= Void
		do
			if s.count = 1 then
				Result := character_to_boolean (s.item (1))
			else
				handle_hard_error ("string %"" + s + "%" does not represent a valid database boolean")
			end
		end

	boolean_to_db_character (b: BOOLEAN): CHARACTER
			-- `b' converted to database representation of
			-- boolean as a single characer
		do
			if b then
				Result := 'Y'
			else
				Result := 'N'
			end
		end

	boolean_to_db_string (b: BOOLEAN): STRING
			-- `b' converted to database representation of
			-- boolean as a string
		do
			create Result.make (1)
			Result.extend (boolean_to_db_character (b))
		end

end
