
note
	
	author: "Joel Goldberg (reworked by David Hollenberg)"
	date:	"March 15, 2002"
	description: "Used to turn a DB_IDENTIFIED_QUERY or %
		%DB_ANONYMOUS_QUERY into an effective class.  Not needed %
		%by clients."

deferred class QUERY_EFFECTOR [G -> DB_ROW]

feature -- Properties

	model_row: G
			-- Model row whose dynamic type determines the
			-- type of new Eiffel row objects created when
			-- fetching, if non-void.  Useful to create
			-- new Eiffel object rows whose type is a
			-- descendant of G

	hint: STRING
			-- Hint for cost-based optimizer
	
	binder: DB_BINDER
			-- Binder for query (can be Void if no placeholders
			-- in filter)

	filter_condition: STRING
			-- Filter condition for query (no condition if Void)

	order_by: STRING
			-- Order by clause for query (no clause if Void)

	executed: BOOLEAN
			-- Has `Current' been executed?
		deferred
		end
	
feature	-- binding and filtering

	set_model_row (m: G)
			-- Set `model_row' to `m'
		do
			model_row := m
		ensure
			value_set: model_row = m
		end
	
	set_hint (h: STRING)
			-- Set `hint' to `h'
		do
			hint := h
		ensure
			value_set: hint = h
		end
	
	set_hint_rule
			-- Set `hint' to "RULE"
		do
			set_hint ("rule")
		end
	
	set_binder (b: DB_BINDER)
			-- Set `binder' to `b'
		do
			binder := b
		ensure
			value_set: binder = b
		end

	bind (q: SQL_QUERY)
			-- Bind placeholders in `q' using `binder',
			-- if `binder' is not Void
		require
			query_exists: q /= Void
			statement_reserved: q.is_reserved
			valid_statement_type: q.is_bindable_statement
		do
			if binder /= Void then
				binder.bind (q)
	    		end
		end

	set_filter_condition (f: STRING)
			-- Set `filter_condition' to `f'
		require
			not_executed: not executed
		do
			filter_condition := f
		ensure
			value_set: filter_condition = f
		end

	set_f_and_b (f: STRING b: DB_BINDER)
			-- Set `filter_condition' to `f' and
			-- `binder' to `b'
		require
			not_executed: not executed
		do
			filter_condition := f
	    		binder := b
		ensure
			filter_set: filter_condition = f
			binder_set: binder = b
		end

	set_order_by (o: STRING)
			-- Set `order_by' to `o'
		require
			not_executed: not executed
		do
			order_by := o
		ensure
			value_set: order_by = o
		end

end
