
note
	
	author: "David Hollenberg"
	date:	"December 4, 1995"
	description: "Constants for internal and external datatypes codes %
		%used by Oracle"
		
class ORACLE_DATATYPE_CODES

feature -- Codes for Oracle internal datatypes
	
	varchar2_internal_type: INTEGER = 1
		-- Oracle internal code for VARCHAR2 datatype
	
	number_internal_type: INTEGER = 2
		-- Oracle internal code for NUMBER datatype
	
	long_internal_type: INTEGER = 8
		-- Oracle internal code for LONG datatype
	
	rowid_internal_type: INTEGER = 11
		-- Oracle internal code for ROWID datatype
	
	date_internal_type: INTEGER = 12
		-- Oracle internal code for DATE datatype
	
	raw_internal_type: INTEGER = 23
		-- Oracle internal code for RAW datatype
	
	long_raw_internal_type: INTEGER = 24
		-- Oracle internal code for LONG RAW datatype
	
	char_internal_type: INTEGER = 96
		-- Oracle internal code for CHAR datatype
	
	mlslabel_internal_type: INTEGER = 106
		-- Oracle internal code for MLSLABEL datatype
		-- (Note: manual says value is 105, but database returns
		-- 106, although describing table in SQL*Plus shows
		-- data type of RAW MLSLABEL instead of MLSLABEL)
	
feature -- Codes for Oracle external datatypes
	
	varchar2_external_type: INTEGER = 1
	
	number_external_type: INTEGER = 2
	
	integer_external_type: INTEGER = 3
	
	float_external_type: INTEGER = 4
	
	string_external_type: INTEGER = 5
	
	varnum_external_type: INTEGER = 6
	
	decimal_external_type: INTEGER = 7
	
	long_external_type: INTEGER = 8
	
	varchar_external_type: INTEGER = 9
	
	rowid_external_type: INTEGER = 11
	
	date_external_type: INTEGER = 12
	
	varraw_external_type: INTEGER = 15
	
	raw_external_type: INTEGER = 23
	
	long_raw_external_type: INTEGER = 24
	
	unsigned_external_type: INTEGER = 68
	
	display_external_type: INTEGER = 91
	
	long_varchar_external_type: INTEGER = 94
	
	long_varraw_external_type: INTEGER = 95
	
	char_external_type: INTEGER = 96
	
	charz_external_type: INTEGER = 97
	
	cursor_variable_external_type: INTEGER = 102
	
	mlslabel_external_type: INTEGER = 105
	
feature -- Lengths for Oracle external datatypes
	
	integer_external_length: INTEGER
			-- Length of integer
		external
			"C macro use %"oracle.h%""
		alias
			"integer_length"
		end
	
	real_external_length: INTEGER
			-- Length of real
		external
			"C macro use %"oracle.h%""
		alias
			"real_length"
		end
	
	double_external_length: INTEGER
			-- Length of double
		external
			"C macro use %"oracle.h%""
		alias
			"double_length"
		end
	
	natural_64_external_length: INTEGER
			-- Length of NATURAL_64
		external
			"C macro use %"oracle.h%""
		alias
			"natural_64_length"
		end
	
	character_external_length: INTEGER
			-- Length of character
		external
			"C macro use %"oracle.h%""
		alias
			"character_length"
		end
	
	date_external_length: INTEGER
			-- Length of date in Oracle internal format
		external
			"C macro use %"oracle.h%""
		alias
			"oracle_date_length"
		ensure
			nonnegative_result: Result >= 0
		end
	
	rowid_external_length: INTEGER
			-- Length of rowid in Oracle internal format
		external
			"C macro use %"oracle.h%""
		alias
			"oracle_rowid_length"
		ensure
			nonnegative_result: Result >= 0
		end
	
	string_count_length: INTEGER
			-- Length of the count field for a counted string
		external
			"C macro use %"oracle.h%""
		end

feature -- Translation to datatype names
	
	name_for_internal_code (code: INTEGER): STRING
			-- Name corresponding to internal datatype code `code'
		do
			inspect code
			when varchar2_internal_type then
				Result := "VARCHAR2"
			when number_internal_type then
				Result := "NUMBER"
			when long_internal_type then
				Result := "LONG"
			when rowid_internal_type then
				Result := "ROWID"
			when date_internal_type then
				Result := "DATE"
			when raw_internal_type then
				Result := "RAW"
			when long_raw_internal_type then
				Result := "LONG RAW"
			when char_internal_type then
				Result := "CHAR"
			when mlslabel_internal_type then
				Result := "MLSLABEL"
			else
				Result := "unknown"
			end
		ensure
			result_exists: Result /= Void
		end
	
	name_for_external_code (code: INTEGER): STRING
			-- Name corresponding to external datatype code `code'
		do
			inspect code
			when varchar2_external_type then
				Result := "VARCHAR2"
			when number_external_type then
				Result := "NUMBER"
			when integer_external_type then
				Result := "INTEGER"
			when float_external_type then
				Result := "FLOAT"
			when string_external_type then
				Result := "STRING"
			when varnum_external_type then
				Result := "VARNUM"
			when decimal_external_type then
				Result := "DECIMAL"
			when long_external_type then
				Result := "LONG"
			when varchar_external_type then
				Result := "VARCHAR"
			when rowid_external_type then
				Result := "ROWID"
			when date_external_type then
				Result := "DATE"
			when varraw_external_type then
				Result := "VARRAW"
			when raw_external_type then
				Result := "RAW"
			when long_raw_external_type then
				Result := "LONG RAW"
			when unsigned_external_type then
				Result := "UNSIGNED"
			when display_external_type then
				Result := "DISPLAY"
			when long_varchar_external_type then
				Result := "LONG VARCHAR"
			when long_varraw_external_type then
				Result := "LONG VARRAW"
			when char_external_type then
				Result := "CHAR"
			when charz_external_type then
				Result := "CHARZ"
			when cursor_variable_external_type then
				Result := "CURSOR VARIABLE"
			when mlslabel_external_type then
				Result := "MLSLABEL"
			else
				Result := "unknown"
			end
		ensure
			result_exists: Result /= Void
		end
	
feature -- Validity
	
	is_valid_internal_datatype (code: INTEGER): BOOLEAN
			-- Is `code' a valid internal datatype code?
		do
			Result := True
			inspect code
			when varchar2_internal_type then
			when number_internal_type then
			when long_internal_type then
			when rowid_internal_type then
			when date_internal_type then
			when raw_internal_type then
			when long_raw_internal_type then
			when char_internal_type then
			when mlslabel_internal_type then
			else
				Result := False
			end
		end
	
	is_valid_external_datatype (code: INTEGER): BOOLEAN
			-- Is `code' a valid external datatype code?
		do
			Result := True
			inspect code
			when varchar2_external_type then
			when number_external_type then
			when integer_external_type then
			when float_external_type then
			when string_external_type then
			when varnum_external_type then
			when decimal_external_type then
			when long_external_type then
			when varchar_external_type then
			when rowid_external_type then
			when date_external_type then
			when varraw_external_type then
			when raw_external_type then
			when long_raw_external_type then
			when unsigned_external_type then
			when display_external_type then
			when long_varchar_external_type then
			when long_varraw_external_type then
			when char_external_type then
			when charz_external_type then
			when cursor_variable_external_type then
			when mlslabel_external_type then
			else
				Result := False
			end
		end
	
end
