
note
	description:
		"Interface to MOSIS Packaging Utilities package in Oracle"
	author: "David Hollenberg"
	date:	"April 26, 2006"

class PACKAGING_UTILITIES_INTERFACE

feature -- Operations

	package_max_die_size (conn: DB_CONNECTION pkg_id, downbond_id: STRING): CONST_POINT
			-- Maximum die size for package `pkg_id' taking
			-- into account `downbond_id'
		require
			conn_not_void: conn /= Void
			has_block_access: conn.has_block_access
			pkg_id_not_void: pkg_id /= Void
			pkg_id_not_empty: not pkg_id.is_empty
		local
			b: SQL_BLOCK
			x, y: DOUBLE
		do
			conn.discard_package_state
				-- Avoid "existing state of packages has
				-- been discarded" Oracle error
			b := conn.sql_block (plsql_to_compute_max_die_size)
			b.set_named_bind_capacity (4)
			b.bind_name_to_string (ph_package_id, pkg_id, False)
			b.bind_name_to_string (ph_downbond_id, downbond_id, False)
			b.bind_name_to_double (ph_result_x, x, False)
			b.bind_name_to_double (ph_result_y, y, False)
			b.execute
			x := b.double_bound_to_name (ph_result_x)
			y := b.double_bound_to_name (ph_result_y)
			b.release
			create Result.make_xy (x, y)
		end

feature {NONE} -- Implementation

	plsql_to_compute_max_die_size: STRING =
		"begin %N%
		%   select packaging_utilities.package_max_die_size_x (:package_id, :downbond_id), %N%
		%          packaging_utilities.package_max_die_size_y (:package_id, :downbond_id) into :result_x, :result_y %N%
		%   from dual; %N%
		%end;"

	ph_result_x: STRING = "result_x"

	ph_result_y: STRING = "result_y"

	ph_package_id: STRING = "package_id"

	ph_downbond_id: STRING = "downbond_id"

end
