class CONTENT_TYPES_ROW

inherit
	CONTENT_TYPES_RAW

	FILE_UTILITIES_ACCESS
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
create

	make_template, make_null, make_by_key, 
	make_by_type_and_subtype

feature {NONE} -- Alternate creation

	make_by_type_and_subtype (conn: DB_CONNECTION opt: DB_QUERY_OPTION a_type, a_subtype: STRING)
			-- Row with content_type `a_type' and
			-- content subtype `a_subtype'
		require
			conn_not_void: conn /= Void
			conn_open: conn.is_open
			a_type_not_void: a_type /= Void
			a_subtype_not_void: a_subtype /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (a_type, a_subtype)
	    		make_by_alternate_key(conn, opt, type_and_subtype_filter, b)
		end

feature	-- Properties

	full_content_type: STRING
			-- Full content type (type and subtype
			-- separated by "/")
		do
			Result := mime_content_type + "/" + mime_content_subtype
		end

feature	-- pick a row based on a file name

	content_type_for_filename (conn: DB_CONNECTION; fname: STRING): like Current
		-- find (longest) match on tail of fname vs filename_suffix
		-- matching is case-independent
		-- Void if no match
		--  (if UNKNOWN is in the table, there will always be a match)
	require connection_exists: conn /= Void
		file_name_exists: fname /= Void
	local b: DB_BIND_S; lst: LIST[like Current]
	do  create b.make (fu.file_name_part (fname))
	    lst := all_rows_filtered_ordered
				(conn,
				 "lower(:1) like '%%'||lower(filename_suffix)", b,
				 "nvl(length(filename_suffix),0)")
	    if not lst.is_empty then
		Result := lst.last
	    end
	end;

feature -- Collections

	content_types_for_drawing (draw_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All content types for drawing ID `draw_id'
		require
			draw_id_not_void: draw_id /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (draw_id)
			Result := all_rows_filtered (conn, "content_type_id in (select content_type_id from drawing_formats where drawing_id = :1)", b)
			check_row_count (Result, range)
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	type_and_subtype_filter: STRING = "mime_content_type = :1 and mime_content_subtype = :2"

end -- CONTENT_TYPES_ROW
