class CUST_IC_DESIGN_FILES_MROW

inherit
	CUST_IC_DESIGN_FILES_MRAW
		redefine
			delete
		end;

	CUST_IC_DESIGN_FILES_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		redefine
			anchored_layer
		end;

	MOSIS_PROD_VALUES
		export {NONE} all
		end;

	MOSIS_DOCS_VALUES
		export {NONE} all
		end;

	FILE_UTILITIES_ACCESS
		export {NONE} all
		end;

	DB_UTILITIES
		export {NONE} all
		end;

create

	make_template, make_null, make_by_key,
	make_revision_layout, make_cust_layout, make_final_layout

feature -- default deliverable

	set_default_deliverable
		-- copy deliverable from my file_role
	do  set_deliverable(file_role_row.deliverable)
	end;

feature	-- file_no selection

	set_next_file_no
		-- set file_no to next available value.
		-- to prevent conflict with other processes, all users
		-- MUST first lock the cid row for this design.
		-- NOTE: can only be used once per transaction!!
	local fno: INTEGER; b: DB_BIND_I;
	do  create b.make(design_no);
	    fno := connection.selected_integer(
			"nvl(max(file_no),0)", table_and_alias_names,
			"design_no = :1", b);
	    set_file_no(fno+1)
	end;

feature	-- filename_prefix construction

	set_standard_filename_prefix(sfx: STRING)
		-- set filename_prefix to standard_filename_prefix(sfx)
	do  set_filename_prefix(standard_filename_prefix(sfx))
	end;

feature	-- delete associated file and mark it deleted

	delete_file
		-- delete this file (and make undeliverable)
	do  if not is_null_filename_prefix then
		fu.delete_if_exists(full_filename)
		set_date_file_deleted(connection.transaction_date)
	    end;
	    set_deliverable(false)
	end;

feature	-- cust_ic_design_file_layers management

	delete
		-- first delete the layers
	do  wipe_out_layers;
	    precursor;
	    reregister_last
	end;

        wipe_out_layers
                -- go to a state with no known layers
        do  if have_any_layers then
                -- (force fetch if cache was Void)
                delete_and_wipe_out(cached_layers)
                -- remove from db (at next post) and from the list
                -- (cached list is of MROWs, which are deletable)
            end
        end;

        add_layer(lyr_name: STRING; dens: DOUBLE; is_null_dens: BOOLEAN)
                -- add a new layer to layers
        require have_name: su.not_empty(lyr_name);
                new_name: not is_layer_present(lyr_name);
                valid_dens: is_null_dens or (0 <= dens and dens <= 1.0)
        local lyr: CUST_IC_DESIGN_FILE_LAYERS_MROW
        do  create lyr.make_null(connection);
            lyr.set_design_no(design_no);
	    lyr.set_file_no(file_no);
            lyr.set_layer_name(lyr_name);
            if is_null_dens then
                lyr.set_null_density
            else
                lyr.set_density(dens)
            end
            my_layers.extend(lyr)
        end;

feature {NONE}

	anchored_layer: CUST_IC_DESIGN_FILE_LAYERS_MROW
	do
	end;

end -- CUST_IC_DESIGN_FILES_MROW
