class CUST_IC_DESIGN_FILES_ROW

inherit
	CUST_IC_DESIGN_FILES_RAW

	STRING_UTILITIES_ACCESS

	FILE_UTILITIES_ACCESS

	DB_QUERY_OPTIONS
	export {NONE} all end;

	MOSIS_PROD_VALUES
	export {NONE} all end;

	FILE_ROLE_CACHE

	CONTENT_TYPE_CACHE
	
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key,
	make_revision_layout, make_cust_layout, make_final_layout

feature	-- status

	exists: BOOLEAN
		-- does the file exist (or, rather, _should_ the file exist)
	do  Result := (not is_null_date_file_posted) and is_null_date_file_deleted
	end;

feature	-- cust_ic_design

	cust_ic_design: CUST_IC_DESIGNS_ROW
	do  if cached_cust_ic_design = Void
	     then create cached_cust_ic_design.make_by_key(
				connection, old_row, design_no)
	    end;
	    Result := cached_cust_ic_design
	end;

feature	-- selection for the above (from master list)

	set_cust_ic_design(cid: CUST_IC_DESIGNS_ROW)
	require exists: cid /= Void
		matches: cid.design_no = design_no
	do  cached_cust_ic_design := cid
	end;

feature	-- design_techns_row

	design_techns_row: DESIGN_TECHNS_ROW
	require	have_techn: not is_null_design_techn
	do  if cached_design_techn = Void or else
	       (not equal (design_techn, cached_design_techn.design_techn)) then
		create cached_design_techn.make_by_key(connection, old_row, design_techn)
	    end;
	    Result := cached_design_techn
	end;

feature	-- run_wafer_lot (for WAFER_LOT_DATA or IBM_KERF_DATA files only)

	run_wafer_lot: RUN_WAFER_LOTS_ROW
	require have_lot: not is_null_wafer_lot_no;
		-- NOTE: table constraint says that is_null_wafer_lot_no = is_null_wafer_run_id
	do  if cached_wafer_lot = Void then
		create cached_wafer_lot.make_by_key
				(connection, old_row, wafer_run_id, wafer_lot_no)
	    end;
	    Result := cached_wafer_lot
	end;

feature	-- filename parts
	-- construction depends on whether we have a filename_prefix or
	--	a wafer_run_id and wafer_lot_no
	-- (must have one or the other or both)

	filename: STRING
		-- my simple filename
	require have_a_handle: not (is_null_filename_prefix and is_null_wafer_run_id)
	do  if is_null_wafer_run_id then	-- normal case
		Result := filename_prefix + content_type.filename_suffix
	    else				-- kerf data case
		Result := fu.file_name_part (full_filename)
	    end
	end;

	full_filename: STRING
		-- my fully qualified filename
	require have_a_handle: not (is_null_filename_prefix and is_null_wafer_run_id)
	do  if is_null_wafer_run_id then
		Result := cust_ic_design.full_source_directory_filename
				(filename)
	    else
		Result := run_wafer_lot.vendor_pcm_data_filename
	    end
	end;

feature	-- standard filename constructors (does not use filename_prefix)

	standard_filename_prefix(sfx: STRING): STRING 
		-- <design_no>[_<revision_no>][_]sfx
		-- where sfx is the provided suffix for the prefix
	do  Result := design_no.out;
	    if not is_null_revision_no then
		Result.extend('_');
		Result.append(revision_no.out)
	    end;
	    if su.not_empty(sfx) then
		Result.extend('_');
		Result.append(sfx)
	    end;
	end;

	standard_filename(sfx: STRING): STRING
		-- standard_filename
	do  Result := standard_filename_prefix(sfx) 
			+ content_type.filename_suffix
	end;

	standard_full_filename(sfx: STRING): STRING
		-- standard_full_filename
	do  Result := cust_ic_design.full_source_directory_filename
				(standard_filename(sfx))
	end;

feature {NONE} -- alternate creation for layout files

	make_revision_layout(conn: DB_CONNECTION; opt: DB_QUERY_OPTION;
				dno, rno: INTEGER)
	local b: DB_BIND_II;
	do  create b.make(dno, rno);
	    make_by_alternate_key(conn, opt, revision_filter, b)
	end;

	revision_filter: STRING =
	"design_no = :1 and revision_no = :2 and file_role in (select file_role from file_roles where is_layout = 'Y')"

	make_cust_layout(conn: DB_CONNECTION; opt: DB_QUERY_OPTION;
				dno: INTEGER)
	local b: DB_BIND_I;
	do  create b.make(dno);
	    make_by_alternate_key(conn, opt, cust_layout_filter, b)
	end;

	cust_layout_filter: STRING =
	"(design_no, file_no) = %
	%(select design_no, cust_file_no from cust_ic_design_attributes %
	%where design_no = :1)"

	make_final_layout(conn: DB_CONNECTION; opt: DB_QUERY_OPTION;
				dno: INTEGER)
	local b: DB_BIND_I;
	do  create b.make(dno);
	    make_by_alternate_key(conn, opt, final_layout_filter, b)
	end;

	final_layout_filter: STRING =
	"(design_no, file_no) = %
	%(select design_no, final_file_no from cust_ic_design_attributes %
	%where design_no = :1)"

feature -- my layers (layout only)

	have_any_layers: BOOLEAN
		-- do I have any layers at all?
	do  Result := my_layers.count > 0
	end

	is_layer_present(lname: STRING): BOOLEAN
		-- is lname one of my layers?
	do  Result :=
		my_layers.there_exists
			(agent {like anchored_layer}.is_my_layer_name(lname))
	end;
	
	layer(a_lnam: STRING): like anchored_layer
	local lyrs: like my_layers
	do  lyrs := my_layers
	    from  lyrs.start
	    until lyrs.after or Result /= Void
	    loop  if lyrs.item.is_my_layer_name(a_lnam) then
			Result := lyrs.item
		  end;
		  lyrs.forth
	    end;
	end;

	my_layers: LIST[like anchored_layer]
	local r: like anchored_layer; b: DB_BIND_II;
	do  if cached_layers = Void then
		create r.make_template
		create b.make(design_no, file_no)
		cached_layers := r.all_rows_filtered_ordered(connection,
					"design_no = :1 and file_no = :2",
					b, "layer_name")
	    end;
	    Result := cached_layers
	end;

feature -- Collections

	rows_for_design (dno: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All rows for design `dno' ordered by
			-- file_no
		require
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (dno)
			Result := all_rows_filtered_ordered (conn, "design_no = :1", b, "file_no")
			check_row_count (Result, range)
		ensure
			result_not_void: Result /= Void
		end

	rows_for_design_and_role (dno: INTEGER a_role: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All rows for design_no `dno' with
			-- file role `a_role'
		require
			a_role_not_void: a_role /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_IS
		do
			create b.make (dno, a_role)
			Result := all_rows_filtered (conn, "design_no = :1 and file_role = :2", b)
			check_row_count (Result, range)
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- caches and anchors

	cached_design_techn: DESIGN_TECHNS_ROW

	cached_cust_ic_design: CUST_IC_DESIGNS_ROW

	cached_wafer_lot: RUN_WAFER_LOTS_ROW

	anchored_layer: CUST_IC_DESIGN_FILE_LAYERS_ROW
	do
	ensure Result = Void
	end;

	cached_layers: LIST[like anchored_layer]

end -- CUST_IC_DESIGN_FILES_ROW
