class CUST_IC_DESIGN_QUESTIONNAIRES_ROW

inherit

	CUST_IC_DESIGN_QUESTIONNAIRES_RAW

	ITEM_COUNT_RANGES_ACCESS

	DB_QUERY_OPTIONS
		export
			{NONE} all
		end

	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

create

	make_template, make_null, make_by_key

feature -- lookup

	answer (q_id: STRING): like anchored_yn_answer
	    require
	        q_exists: q_id /= Void
	    do
		from
			submitted_answers.start
		until
			Result /= Void or submitted_answers.after
		loop
			if q_id.is_equal (submitted_answers.item.paragraph_id) then
				Result := submitted_answers.item
			end
			submitted_answers.forth
		end
	    end


	questionnaire_summary: HASH_TABLE[ECC_QUESTIONNAIRE_ANSWER_SUMMARY,STRING]

feature -- status

	no_count: INTEGER
		do
			from
				questionnaire_summary.start
			until
				questionnaire_summary.after
			loop
				Result := Result + questionnaire_summary.item_for_iteration.negative_response_count
				questionnaire_summary.forth
			end
		end

	yes_count: INTEGER
		do
			from
				questionnaire_summary.start
			until
				questionnaire_summary.after
			loop
				Result := Result + questionnaire_summary.item_for_iteration.positive_response_count
				questionnaire_summary.forth
			end
		end

	one_yes_answer: like anchored_yn_answer

	count_submitted_answers
		do
			create effective_answers.make (submitted_answers.count)
			add_to_effective_answers (submitted_answers)
			calculate_effective_answers
		end

	count_effective_answers
		local
			psa: like prior_submitted_answers
		do
			psa := prior_submitted_answers
			create effective_answers.make (submitted_answers.count + psa.count)
			add_to_effective_answers (submitted_answers)
			add_to_effective_answers (psa)
			calculate_effective_answers
		end

	complete_section_count: INTEGER
		do
			count_effective_answers
			from
				questionnaire_summary.start
			until
				questionnaire_summary.after
			loop
				if questionnaire_summary.item_for_iteration.missing_response_count = 0 then
					Result := Result + 1
				end
				questionnaire_summary.forth
			end
		end

	has_unanswered_questions: BOOLEAN
		local
			unanswered_questions_count: INTEGER
		do
			count_effective_answers
			from
				questionnaire_summary.start
			until
				questionnaire_summary.after
			loop
				unanswered_questions_count := unanswered_questions_count + questionnaire_summary.item_for_iteration.missing_response_count
				questionnaire_summary.forth
			end
			Result := unanswered_questions_count > 0
		end

feature -- Associated rows lookup

	submitted_answers: DYNAMIC_LIST[like anchored_yn_answer]
		-- all YN answers in paragraph_id order
		local
			yn_rec: like anchored_yn_answer
		do
			if cached_submitted_answers = Void then
				create yn_rec.make_template
				cached_submitted_answers :=
					yn_rec.design_submission_yn_answers (design_no, submission_no, connection, expected_count.At_least_zero)
			end
			Result := cached_submitted_answers
		ensure
			Result_exists: Result /= Void
		end

	prior_submitted_answers: DYNAMIC_LIST[like anchored_yn_answer]
		-- the YN answers for prior submissions
		local
			yn_rec: like anchored_yn_answer
		do
			create yn_rec.make_template
			Result :=  yn_rec.design_yn_answers (design_no, submission_no - 1, connection, expected_count.At_least_zero)
		end

feature {NONE} -- Implementation of associated rows lookup

	anchored_yn_answer: CUST_IC_DESIGN_YN_ANSWERS_ROW
		do
		end

	cached_submitted_answers: DYNAMIC_LIST[like anchored_yn_answer]

	effective_answers: HASH_TABLE[like anchored_yn_answer,STRING]


	add_to_effective_answers (aans: like cached_submitted_answers)
		require
			answer_array_exists: aans /= Void
			effective_answers_exists: effective_answers /= Void
		local
			ans: like anchored_yn_answer
		do
			if effective_answers = Void then
				create effective_answers.make (aans.count)
			end
			from
				aans.start
			until
				aans.after
			loop
				ans := aans.item
				if not effective_answers.has (ans.paragraph_id) then
					effective_answers.extend (ans, ans.paragraph_id)
				end
				aans.forth
			end
		end

	calculate_effective_answers
		local
			one_yes: like anchored_yn_answer
			y_count: INTEGER
		do
			from
				create questionnaire_summary.make (8)
				effective_answers.start
			until
				effective_answers.after
			loop
				if effective_answers.item_for_iteration.answer then
					one_yes := effective_answers.item_for_iteration
					y_count := y_count + 1
				end
				increment_summary (effective_answers.item_for_iteration)
				effective_answers.forth
			end
			if y_count = 1 then
				one_yes_answer := one_yes
			else
				one_yes_answer := Void
			end
		end

	increment_summary (row: like anchored_yn_answer)
		local
			section: ECC_QUESTIONNAIRE_ANSWER_SUMMARY
			row_questionnaire_id: STRING
		do
			row_questionnaire_id := row.ecc_questionnaire_id
			questionnaire_summary.search (row_questionnaire_id)
			if questionnaire_summary.found then
				section := questionnaire_summary.found_item
				if section.version_no = row.version_no then
					section.increment_count(row.answer)
				-- else answer is for prior version so ignore this answer
				--      (should never get here because older answers are not inserted
				--       into the effective answers set)
				end
			else
				from
					all_questionnaire_versions.start
				until
					all_questionnaire_versions.after or section /= Void
				loop
					if all_questionnaire_versions.item.has_key (row.ecc_questionnaire_id, row.version_no) then
						section := all_questionnaire_versions.item
						section.clear_negative_response
						section.clear_positive_response
						questionnaire_summary.extend (section, row_questionnaire_id)
						section.increment_count(row.answer)
					end
					all_questionnaire_versions.forth
				end
			end
		end

	all_questionnaire_versions: DYNAMIC_LIST [ECC_QUESTIONNAIRE_VERSIONS_EROW]
		local
			q: ECC_QUESTIONNAIRE_VERSIONS_EROW
		once
			create q.make_template
			Result := q.all_version_rows (connection, expected_count.At_least_zero)
		end

feature -- Collections

	questionnaires_for_design (design: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All cust IC design questionnaire for design_no `design', ordered by submission_no descending
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (design)
			Result := all_rows_filtered_ordered (conn, "design_no = :1", b, "submission_no desc")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

end
