
note

	author: "Generated"
	date: "Fri Feb 27 14:00:36 PST 2004"

class CUST_IC_DESIGNS_ROW
inherit
	CUST_IC_DESIGNS_RAW

	DB_QUERY_OPTIONS
	    export
		{NONE} all
	    end;

	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end

	CONDITION_UTILITIES
	    export
		{NONE} all
	    end;

	STRING_UTILITIES_ACCESS

	MOSIS_PROD_VALUES
	    export
		{NONE} all
	    end;

	GEOMETRY_RESOLUTION_ACCESS
	    export
		{NONE} all
	    end;

create
	make_template, make_null, make_by_key

feature	-- status

	is_passed: BOOLEAN
	local dasv: DESIGN_ACCEPT_STATE_VALUES
	do  Result := equal(design_accept_state, dasv.passed)
	end;

	is_not_passed: BOOLEAN
	local dasv: DESIGN_ACCEPT_STATE_VALUES
	do  Result := not equal(design_accept_state, dasv.passed)
	end;

feature {NONE} -- the two distinguished cust_ic_design_files

	cust_layout: like anchored_cidf_row
	do  if cached_cust_layout = Void or else
	       cached_cust_layout.file_no /= cust_file_no then
		create cached_cust_layout.make_by_key
			(connection, old_row, design_no, cust_file_no)
	    end;
	    Result := cached_cust_layout
	end;

	final_layout: like anchored_cidf_row
	do  if cached_final_layout = Void or else
	       cached_final_layout.file_no /= final_file_no then
		if final_file_no = cust_file_no then
			-- share the single cidf_row
			cached_final_layout := cust_layout
		else
			create cached_final_layout.make_by_key
			(connection, old_row, design_no, final_file_no)
		end
	    end;
	    Result := cached_final_layout
	end;

feature	-- selection of the "layout of interest" (cust or final)

	is_using_final_layout: BOOLEAN
		-- is the layout_of_interest the final_layout or the cust_layout?

	is_using_cust_layout: BOOLEAN
		-- is layout_of_interest = cust_layout?
		-- (even if is_using_final_layout)
	do  Result := layout_of_interest = cust_layout
	end;

	layout_of_interest: like anchored_cidf_row
		-- the one or the other
	do  if is_using_final_layout then
		Result := final_layout
	    else
		Result := cust_layout
	    end
	end;

	use_cust_layout
		-- make the cust_layout be the layout_of_interest
	do  is_using_final_layout := false
	end;

	use_final_layout
		-- make the final_layout be the layout_of_interest
	do  is_using_final_layout := true
	end;

feature	-- various attributes of the layout_of_interest 
	--   (things that used to be in the cid itself)

	design_techn: DESIGN_TECHNS_ROW
		-- this is a db row, not a string
	do  Result := layout_of_interest.design_techns_row
	end;

	design_techn_id: STRING
		-- this is a string, not a db row!!
	do  Result := design_techn.design_techn_id
	end;

	geometry_format: STRING
	do  Result := layout_of_interest.content_type_id
	end;

	is_gds: BOOLEAN
	local ctv: CONTENT_TYPE_VALUES
	do  Result := equal(geometry_format, ctv.gds)
	end;

	is_cif: BOOLEAN
	local ctv: CONTENT_TYPE_VALUES
	do  Result := equal(geometry_format, ctv.cif)
	end;

	source_filename: STRING
	do  Result := layout_of_interest.filename_prefix
	end;
	
	is_null_source_filename: BOOLEAN
	do  Result := layout_of_interest.is_null_filename_prefix
	end;

	layout_file_exists: BOOLEAN
	do  Result := layout_of_interest.exists
	end;

	is_preliminary: BOOLEAN
	do  Result := layout_of_interest.is_preliminary
	end;

feature	-- prior cust_layout

	prior_cust_layout: like anchored_cidf_row
		-- the last cidf for layout before cust_layout 
		-- (may be Void)
	local cidf: like anchored_cidf_row; cidfs: LIST[like anchored_cidf_row];
		b: DB_BIND_SII; frv: FILE_ROLE_VALUES
	do  create cidf.make_template;
	    create b.make(frv.submitted_layout, design_no, 
				cust_layout.revision_no)
	    cidfs := cidf.all_rows_filtered_ordered
				(connection, prior_cust_layout_filter, b, "revision_no")
	    if not cidfs.is_empty then
		Result := cidfs.last
	    end
	end;

	prior_cust_layout_filter: STRING =
	"file_role = :1 and design_no = :2 and revision_no < :3"

feature	-- full source directory filename
	
	full_source_directory_filename(fnam: STRING): STRING
		-- fully qualified filename for fnam
		--  in our source_directory as a NEW string
	do 
	end


feature	-- my ic_fab_instances

	ifi_of_interest: like anchored_ic_fab_instance
		-- one particular fab instance, as selected 
		--  by find_ifi_of_interest
		-- if find_ifi_of_interest hasn't been called,
		--  we call it here
	do  if cached_ifi_of_interest = Void then
		find_ifi_of_interest(0, Void)
	    end;
	    Result := cached_ifi_of_interest
	end;

	ic_fab_instances: LIST[like anchored_ic_fab_instance]
		-- all valid fab instances in date_fab_requested order
	local r: like anchored_ic_fab_instance; b: DB_BIND_I
	do  if cached_ic_fab_instances = Void then
		create r.make_template
		create b.make(design_no)
		cached_ic_fab_instances := r.all_rows_filtered_ordered
			(connection, ic_fab_instance_filter, b, "date_fab_requested")
	    end;
	    Result := cached_ic_fab_instances
	end;

	find_ifi_of_interest(fno: INTEGER; rid: STRING)
		-- if fno > 0, we want THIS m_fab_no
		-- if rid is specified, we want THIS run_id
		-- if neither, we want unfabbed ifi, or, if none,
		--  use latest ifi
	require just_one: fno = 0 or su.is_empty(rid)
	local ifis: like ic_fab_instances; p: like anchored_ic_fab_instance;
		pick: BOOLEAN;
		c: CURSOR
	do  ifis := ic_fab_instances
	    c := ifis.cursor
	    from  ifis.start;
		  cached_ifi_of_interest := Void;
	    until ifis.after
	    loop  p := ifis.item;
		  -- is this ifi the one (pick)?
		  if su.not_empty(rid) then
			pick := equal(rid, p.run_id)
		  elseif fno > 0 then
			pick := fno = p.m_fab_no
		  else
			pick := not p.has_fab_id
		  end;
		  if pick then
			if cached_ifi_of_interest /= Void
			 then handle_hard_error("FIND_IFI_OF_INTEREST: multiple pending for " + design_no.out)
			end;
			cached_ifi_of_interest := p;	-- the one just picked
		  end;
		  ifis.forth;
	    end;
	    ifis.go_to (c)
	    -- no unfabbed; use the last (= latest) one
	    if fno = 0 and su.is_empty(rid) and cached_ifi_of_interest = Void
	     then cached_ifi_of_interest := p end;
	end;

	tiered_fab_instances: LIST [like anchored_ic_fab_instance]
			-- List of fab instances for tiers, in tier_no
			-- order (1 to N)
		require 
			is_3dic_parent: is_3dic_parent
		local
			cids: LIST [like Current]
			cid: like Current
		do
			cids := tiered_cids
			create {ARRAYED_LIST [like anchored_ic_fab_instance]} Result.make (cids.count)
			from
				cids.start
			until
				cids.after
			loop
				cid := cids.item
				cid.find_ifi_of_interest (0, Void)
				Result.extend (cid.ifi_of_interest)
				cids.forth
			end
		end

feature -- my questionnaires

	design_questionnaires: LIST[like anchored_design_questionnaire]
		-- all valid cust_ic_design_questionnaires in submission_no desc order
		local
			r: like anchored_design_questionnaire
			b: DB_BIND_I
		do
			if cached_design_questionnaires = Void then
				create r.make_template
				create b.make(design_no)
				cached_design_questionnaires := 
					   r.all_rows_filtered_ordered (connection,
									design_questionnaire_filter,
									b,
									"submission_no desc")
		end
		Result := cached_design_questionnaires
	end

feature -- status

	design_center: POINT
			-- center as a point
	do  create Result.make_xy(center_x, center_y)
	end;

	design_size: POINT
			-- size as a point
	do  create Result.make_xy(design_size_x, design_size_y)
	end;

	layout_area: DOUBLE
			-- Area of layout
		do
			Result := design_size_x * design_size_y
		end

	design_bounding_box: RECTANGLE 
			-- bounding box for the layout
	do  create Result.make_by_size_and_center
				(design_size_x, design_size_y,
				 center_x, center_y)
	end;

	bonded_pad_count: INTEGER
			-- number of bonded cust_ic_design_pads
	do  Result := count_cid_pads("design_no = :1 and bonded = %'Y%' and pad_type is null")
	end;

	unbonded_pad_count: INTEGER
			-- number of unbonded cust_ic_design_pads
	do  Result := count_cid_pads("design_no = :1 and bonded = %'N%' and pad_type is null")
	end;

	total_pad_count: INTEGER
			-- number of cust_ic_design_pads
	do  Result := count_cid_pads("design_no = :1 and pad_type is null")
	end;

	has_multiple_fab_instances: BOOLEAN
			-- Are there multiple fab instances for `design_no'?
	local	b: DB_BIND_I
	do
		create b.make (design_no);
		Result := any_connection.selected_count
				("ic_fab_instances", by_design_no, b) > 1
	end;

	is_final_layout_derived: BOOLEAN
		-- Is the final_layout different from the cust_layout?
	do  Result := final_file_no /= cust_file_no
	end;

	is_frozen: BOOLEAN
		-- is the layout frozen (cannot be changed by customer)?
	do  Result := design_frozen or design_cancelled or
			(not is_null_date_layout_locked) or
			is_final_layout_derived
	end;

	is_itar: BOOLEAN
			-- Is `Current' an ITAR design
		local
			type: EXPORT_CONTROL_TYPE_VALUES
		do
			Result := export_control_type ~ type.Itar
		end

feature -- Associated rows lookup

feature	{NONE}

	count_cid_pads(where: STRING): INTEGER
		-- where string must have "design_no = :1" as only bound variable
	local b: DB_BIND_I
	do  create b.make(design_no);
	    Result := connection.selected_count
					("cust_ic_design_pads", where, b)
	end;

feature -- Queries

	rows_with_design_no (n: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All IC designs with design number `n'
		require
			design_no_nonegative: n >= 0
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (n)
			Result := all_rows_filtered (conn, by_design_no, b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- stuff related to chamfer reconstuction in the overhead
	--  that we generate

	is_overhead_viable(a_style: STRING): BOOLEAN
		-- can we generate overhead a la a_style?
	require	valid_style: a_style /= Void;
	local styl: OVERHEAD_STYLE_VALUES
	do  Result := (is_chamfer_reconstruction_required(a_style)
			implies (not is_null_chamfer_edge))
			-- if we need to perform chamfer reconstruction,
			-- then we better have a discovered chamfer_edge
		and   (equal(a_style, styl.internal)
			implies (not is_null_label_box_size_x))
	end;

	is_chamfer_reconstruction_required(a_style: STRING): BOOLEAN
		-- given that our overhead is in a_style, will
		-- chamfer reconstruction be necessary?
	local ostyl: OVERHEAD_STYLE_VALUES
	do  Result := equal(a_style, ostyl.external_) and
		      chamfer_layer /= Void
	end;

	chamfer_layer: STRING
		-- layer for the 4 corner chamfer triangles
		-- (Void if chamfer reconstruction does not apply)
		-- this value is strictly a function of the design techn
	local b: DB_BIND_S;
	do  create b.make(design_techn_id);
	    Result := connection.safe_selected_string
			("layer_name", chamfer_layer_tables,
			 chamfer_layer_filter, b);
	    if Result.is_empty then Result := Void end
	end;

	is_null_chamfer_edge: BOOLEAN 
		-- do we lack a measured chamfer edge length?
	local inv: ITEM_NAME_VALUES
	do  Result := not have_measurement(inv.chamfer_edge)
	end;

	chamfer_edge: DOUBLE
		-- the measured chamfer edge length
		-- (or 0 if we don't have one and you didn't check)
	local inv: ITEM_NAME_VALUES
	do  Result := get_measurement_value(inv.chamfer_edge)
	end;

feature {NONE} -- details for overhead generation

	chamfer_layer_tables: STRING =
	"design_techn_id_layers dtil, layer_roles lr"

	chamfer_layer_filter: STRING =
	"dtil.layer_role = lr.layer_role and dtil.design_techn_id = :1%
	% and lr.is_corner_chamfer = %'Y%'"

feature	-- cust_ic_design_measurements
	-- (the routines to set or remove a measurement are in
	--   CUST_IC_DESIGNS_MROW)

	have_measurement(iname: STRING): BOOLEAN 
		-- do we have a measurement for item_name iname?
	do  Result := a_measurement(iname) /= Void
	end;

	get_measurement_value(iname: STRING): DOUBLE 
		-- get the value for a measurement
		--  (0 if not present and you didn't check first)
	local foo: like a_measurement
	do  foo := a_measurement(iname)
	    if foo /= Void then Result := foo.numeric_value end
	end;

feature	-- basic support for 3DIC bundles

	is_3dic_tier: BOOLEAN
		-- is this a tiered element of a 3DIC?
	do  Result := not is_null_design_no_3dic
	end;

	is_3dic_parent: BOOLEAN
		-- is this a 3DIC parent (the 3DIC itself)?
	do  Result := design_techn.is_3dic
	end;

	is_3dic_element: BOOLEAN
		-- is this either part of a 3DIC?
	do  Result := is_3dic_tier or is_3dic_parent
	end;

	parent_cid: like Current
		-- for a 3dic tier, the parent
	require	is_3dic_tier: is_3dic_tier
	do  if cached_3dic_parent = Void then
		create cached_3dic_parent.make_by_key(connection, old_row, design_no_3dic)
	       end;
	    Result := cached_3dic_parent
	end;

	tiered_cids: LIST[like Current]
		-- in tier_no order (1 to <n>)
		-- (empty if not yet created)
	require is_3dic_parent: is_3dic_parent
	local b: DB_BIND_I; a_cid: like Current
	do  if cached_tiered_cids = Void then
		create b.make(design_no)
		create a_cid.make_template
		cached_tiered_cids := a_cid.all_rows_filtered_ordered
					(connection, "design_no_3dic = :1", b, "tier_no_3dic")
		cached_tiered_cids.do_all(agent {like Current}.use_for_parent(Current))
	    end;
	    Result := cached_tiered_cids
	end;

	nth_tier_cust_ic_design (n: INTEGER): like Current
			-- Cust IC design for tier `n'
		require
			is_3dic_parent: is_3dic_parent
		local
			list: like tiered_cids
			cid: like Current
		do
			list := tiered_cids
			from
				list.start
			until
				list.after or Result /= Void
			loop
				cid := list.item
				if cid.tier_no_3dic = n then
					Result := cid
				end
				list.forth
			end
			if Result = Void then
				handle_hard_error ("cannot find cust IC design for tier " + n.out + " for parent design_no " + design_no.out)
			end
		end

	use_for_parent(a_cid: like Current)
		-- use this cid as the parent_cid rather than fetching it
	require	have_cid: a_cid /= Void
		my_parent: a_cid.design_no = design_no_3dic
	local
		ifi: like anchored_ic_fab_instance
		l_run_id: STRING
	do  cached_3dic_parent := a_cid
	    -- find ifi to match his run
	    ifi := a_cid.ifi_of_interest
	    if ifi /= Void then
	    	l_run_id := ifi.run_id
	    else
	    	l_run_id := ""
	    end
	    find_ifi_of_interest(0, l_run_id)
	end;

	use_for_tier(a_cid: like Current)
		-- use this cid, by tier_no_3dic, in tiered_cids
		-- (important when the cids are mrows to avoid multiple instances)
		-- (this should happen only when one of the tiers is created first)
	require	have_cid: a_cid /= Void
		my_cid: a_cid.design_no_3dic = design_no
		valid_tier_no: tiered_cids.valid_index(a_cid.tier_no_3dic)
	local tcids: like tiered_cids; tno: INTEGER; o_cid: like Current
	do  -- find ifi to match his run
	    find_ifi_of_interest(0, a_cid.ifi_of_interest.run_id)
	    tcids := tiered_cids;
	    tno := a_cid.tier_no_3dic
	    o_cid := tcids.i_th(tno)
	    if o_cid.is_registered then
		handle_hard_error("use_for_tier: replacing a modified cid")
	    end;
	    tcids.put_i_th(a_cid, tno)
	end;

	all_sizes_match: BOOLEAN
		-- are I and my tiers all the same size?
	require	is_3dic_parent: is_3dic_parent
	do  Result := tiered_cids.for_all(agent one_size_matches(Current, ?))
	end;

feature	{NONE} -- cache for 3DIC cross-linkage

	cached_3dic_parent: like Current

	cached_tiered_cids: LIST[like Current]

	one_size_matches(a_cid, b_cid: like Current): BOOLEAN
		-- are these 2 cids the same design_size?
	do  Result := gres.eq(a_cid.design_size_x, b_cid.design_size_x)
			and
		      gres.eq(a_cid.design_size_y, b_cid.design_size_y)
	end;

feature {NONE} -- standard symbolic link maintenance

	repair_links_descr: STRING

	repair_layout_symbolic_links
		-- maintain the links <dno>_[cust/final].<format>
		-- this is my action_item do_it routine; it is queued
		-- for commit action when a new layout cidf is created
	local cust_fnam, final_fnam, dnos: STRING;
		cust_cif_lnam, cust_gds_lnam, final_cif_lnam, final_gds_lnam: STRING
	do  repair_links_descr := "begin symbolic link fixup"
	    -- the two target names
	    cust_fnam := cust_layout.standard_full_filename(Void);
	    final_fnam := final_layout.standard_full_filename(Void);
	    -- the 4 canonical symbolic link names
	    dnos := design_no.out
	    cust_cif_lnam := full_source_directory_filename(dnos + "_cust.cif")
	    cust_gds_lnam := full_source_directory_filename(dnos + "_cust.gds")
	    final_cif_lnam := full_source_directory_filename(dnos + "_final.cif")
	    final_gds_lnam := full_source_directory_filename(dnos + "_final.gds")
	    -- now repair each of the 4 names
	    repair_one_symbolic_link(cust_fnam, cust_cif_lnam)
	    repair_one_symbolic_link(cust_fnam, cust_gds_lnam)
	    repair_one_symbolic_link(final_fnam, final_cif_lnam)
	    repair_one_symbolic_link(final_fnam, final_gds_lnam)
	    -- and sign off
	end;

	repair_one_symbolic_link(target, symlink: STRING)
		-- arguments are full paths
		-- if target's extension matches symlink's, link is created,
		--  otherwise, old link is just removed
	require have_target: target /= Void
		have_symlink: symlink /= Void
	do
	end;

feature {NONE} -- implementation stuff

	by_design_no: STRING = "design_no = :1";

	anchored_cidf_row: CUST_IC_DESIGN_FILES_ROW
	do
	end;

	cached_cust_layout,
	cached_final_layout: like anchored_cidf_row;

	anchored_ic_fab_instance: IC_FAB_INSTANCES_ROW
	do
	end;

	cached_ifi_of_interest: like anchored_ic_fab_instance

	cached_ic_fab_instances: LIST[like anchored_ic_fab_instance]

	ic_fab_instance_filter: STRING = "design_no = :1 and date_invalidated is null"

	anchored_design_questionnaire: CUST_IC_DESIGN_QUESTIONNAIRES_ROW
		do
		end

	cached_design_questionnaires: LIST[like anchored_design_questionnaire]

	design_questionnaire_filter: STRING = "design_no = :1"

	a_measurement(iname: STRING): CUST_IC_DESIGN_MEASUREMENTS_MROW
		-- measurement with item_name iname or Void
		-- NOTE: clients never see or handle any
		--   CUST_IC_DESIGN_MEASUREMENTS_MROWs
	local r: CUST_IC_DESIGN_MEASUREMENTS_MROW; b: DB_BIND_I
	do  if cached_measurements = Void
	     then create r.make_template;
		  create b.make(design_no)
		  cached_measurements := 
			r.all_rows_filtered(connection, by_design_no, b)
	    end;
	    from  cached_measurements.start
	    until cached_measurements.after or Result /= Void
	    loop  if equal(cached_measurements.item.item_name, iname)
		   then Result := cached_measurements.item
		   else cached_measurements.forth
		  end;
	    end;
	ensure Result_is_item: 
		(Result = Void) or else (Result = cached_measurements.item)
	end;

	cached_measurements: DYNAMIC_LIST[CUST_IC_DESIGN_MEASUREMENTS_MROW]

end -- CUST_IC_DESIGNS_ROW
