class CUST_QUOTES_ROW

inherit
	CUST_QUOTES_RAW

	EFFECTIVE_ON_DATE

	MOSIS_MPRO_VALUES
	MOSIS_DOCS_VALUES

	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

	HAS_M_CUST_QUOTE_NO

create
	make_template, make_null, make_by_key, make_by_full_cust_quote_id

feature {NONE} -- alternate Creation

	make_by_full_cust_quote_id (c: DB_CONNECTION; o: DB_QUERY_OPTION;
			the_full_cust_quote_id: STRING)
	require	connection_exists: c /= Void;
		option_exists: o /= Void;
		for_old_row: o.require_old_row
	local	b: DB_BIND_S;
	do
		create b.make(the_full_cust_quote_id)
		make_by_alternate_key(c, o, full_id_where_clause, b)
	end;

	full_id_where_clause: STRING =
	"cust_quote_id || '-' || revision = :1";

feature	-- some aspects of this quote

	has_medium_quantity_quote: BOOLEAN
		-- are there any line items for medium quantity parts?
	local lis: like the_line_items;
	do  lis := the_line_items
	    from  lis.start 
	    until lis.after or Result
	    loop  Result := lis.item.my_part.is_quantity_quotable_part
		  lis.forth
	    end;
	end;

	has_dedicated_run_quote: BOOLEAN
		-- are there any line items for dedicated runs?
	local lis: like the_line_items;
	do  lis := the_line_items
	    from  lis.start 
	    until lis.after or Result
	    loop  Result := lis.item.my_part.is_fab_run
		  lis.forth
	    end;
	end;

feature {NONE} -- quote revision

	full_id_for_revision(rev: STRING): STRING
		-- the "cust_quote_id" for revision rev
	do  create Result.make(cust_quote_id.count+rev.count+1)
	    Result.append(cust_quote_id);
	    Result.extend('-');
	    Result.append(rev)
	end;

feature	-- quote revision issues

	full_cust_quote_id: STRING
		-- the "cust_quote_id" that the user sees
	do  Result := full_id_for_revision(revision)
	end;

	is_current_revision: BOOLEAN
		-- is this quote the latest revision?
	do  Result := equal(revision, latest_revision)
	end;

	latest_revision: STRING
	local b: DB_BIND_S;
	do  if cached_latest_revision = Void
	     then create b.make(cust_quote_id);
		  cached_latest_revision :=
			connection.selected_string
				("max(revision)", "cust_quotes", 
				 "cust_quote_id = :1", b);
	    end
	    Result := cached_latest_revision
	end;

	latest_quote: CUST_QUOTES_ROW
		-- the latest revision (may be me)
	do  if is_current_revision
	     then Result := Current
	     else create Result.make_by_full_cust_quote_id
				(connection, option.old_row,
				 full_id_for_revision(latest_revision))
	    end
	ensure equal(cust_quote_id, Result.cust_quote_id)
	end

feature	-- the quote line items

	the_line_items: LIST[like anchored_line_item]
		--
	local r: like anchored_line_item; b: DB_BIND_I;
	do  if cached_line_items /= Void
	     then -- already cached
	    elseif connection /= Void
	     then -- fetch from connection
		  create r.make_template;
		  create b.make(m_cust_quote_no)
		  cached_line_items := r.all_rows_filtered_ordered
					(connection,
					 "m_cust_quote_no = :1", b,
					 "cust_quote_line_no")
	     else -- no connection: create empty list
		  create {ARRAYED_LIST[like anchored_line_item]}
			cached_line_items.make(4)
	    end
	    Result := cached_line_items
	ensure Result /= Void
	end;

feature	-- Account

	is_cached_account: BOOLEAN
			-- Is customer account cached?
		do
		end

	uncache_account: BOOLEAN
			-- Remove customer account from cached
		do
		end

feature -- Collections

	valid_fab_quotes_for_production_and_account (prod: STRING acct: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All quotes for accounts that have an IC
			-- order on production `prod', are valid for
			-- an order on `prod' and have a line item for
			-- a FAB RUN part, ordered by full quote ID
		require
			prod_not_void: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_ISSS
			fab_type: IC_FAB_INSTANCE_TYPE_VALUES
			part_type: PART_TYPE_VALUES
		do
			create b.make (acct, prod, fab_type.Customer_fab, part_type.Fab_run)
			Result := all_rows_filtered_ordered (conn, 
			  "cust_acct_no in %
			  % (select cwo.cust_acct_no %
			  %  from cust_wos cwo, ic_fab_instances ifi %
			  %  where cwo.m_fab_no = ifi.m_fab_no %
			  %   and cwo.cust_acct_no = :1 %
			  %   and cwo.production_id = :2 %
			  %   and ifi.ic_fab_instance_type = :3 %
			  %   and trunc(nvl(cq.date_expires, sysdate)) >= nvl(cwo.date_for_funding, cwo.date_for_pricing)) %
			  % and exists (select 1 from cust_quote_line_items cqli, m_parts mp %
			  %  where cqli.m_part_no = mp.m_part_no %
			  %    and cqli.m_cust_quote_no = cq.m_cust_quote_no %
			  %    and mp.part_type = :4)", 
				b, "cust_quote_id, revision desc")
			check_row_count (Result, range)
		end

	valid_fab_quotes_for_production (prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All quotes for accounts that have an IC order
			-- on production `prod', are valid for some order and
			-- have a line item for a FAB RUN part,
			-- ordered by full quote ID
		require
			prod_not_void: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SSS
			fab_type: IC_FAB_INSTANCE_TYPE_VALUES
			part_type: PART_TYPE_VALUES
		do
			create b.make (prod, fab_type.Customer_fab, part_type.Fab_run)
			Result := all_rows_filtered_ordered (conn, 
			  "cust_acct_no in %
			  % (select cwo.cust_acct_no %
			  %  from cust_wos cwo, ic_fab_instances ifi %
			  %  where cwo.m_fab_no = ifi.m_fab_no %
			  %   and cwo.production_id = :1 %
			  %   and ifi.ic_fab_instance_type = :2 %
			  %   and trunc(nvl(cq.date_expires, sysdate)) >= nvl(cwo.date_for_funding, cwo.date_for_pricing) %
			  % union %
			  % select dedicated_cust_acct_no %
			  % from   scheduled_runs sr, runs r, productions p %
			  % where  sr.sched_run_no = r.sched_run_no %
			  %   and  r.run_id = p.run_id %
			  %   and trunc(nvl(cq.date_expires, sysdate)) >= sysdate %
			  %   and  p.production_id = :1 %
			  % ) %
			  % and exists (select 1 from cust_quote_line_items cqli, m_parts mp %
			  %  where cqli.m_part_no = mp.m_part_no %
			  %    and cqli.m_cust_quote_no = cq.m_cust_quote_no %
			  %    and mp.part_type = :3)", 
				b, "cust_quote_id, revision desc")
			check_row_count (Result, range)
		end

	unexpired_fab_quotes_for_account (acct_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All quotes for account `acct_no' that are
			-- not expired and have a line item for a FAB
			-- RUN part, ordered by full quote ID
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_IS
			part_type: PART_TYPE_VALUES
		do
			create b.make (acct_no, part_type.Fab_run)
			Result := all_rows_filtered_ordered (conn, 
			   "cust_acct_no = :1 %
			  % and trunc(nvl(date_expires, sysdate)) >= trunc(sysdate) %
			  % and exists (select 1 from cust_quote_line_items cqli, m_parts mp %
			  %  where cqli.m_part_no = mp.m_part_no %
			  %    and cqli.m_cust_quote_no = cq.m_cust_quote_no %
			  %    and mp.part_type = :2)", 
				b, "cust_quote_id, revision desc")
			check_row_count (Result, range)
		end

	unexpired_fab_quotes_for_part_number (part_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All quotes that are for accounts that have
			-- an unexpired quote with a line item for
			-- `part_no' and that have a line item for a FAB
			-- RUN part, ordered by full quote ID
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_IS
			part_type: PART_TYPE_VALUES
		do
			create b.make (part_no, part_type.Fab_run)
			Result := all_rows_filtered_ordered (conn, 
			   "cust_acct_no in (select distinct cust_acct_no from cust_quotes cq, cust_quote_line_items cqli where cq.m_cust_quote_no = cqli.m_cust_quote_no and nvl(cq.date_expires, sysdate) >= trunc(sysdate) and cqli.m_part_no = :1) %
			  % and trunc(nvl(date_expires, sysdate)) >= trunc(sysdate) %
			  % and exists (select 1 from cust_quote_line_items cqli, m_parts mp %
			  %  where cqli.m_part_no = mp.m_part_no %
			  %    and cqli.m_cust_quote_no = cq.m_cust_quote_no %
			  %    and mp.part_type = :2)", 
				b, "cust_quote_id, revision desc")
			check_row_count (Result, range)
		end

	unexpired_fab_quotes_for_acct_and_part_number (acct_no, part_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All quotes that are for `acct_no' that have
			-- a line item for `part_no' and that have a
			-- line item for a FAB RUN part, ordered by
			-- full quote ID
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_IIS
			part_type: PART_TYPE_VALUES
		do
			create b.make (acct_no, part_no, part_type.Fab_run)
			Result := all_rows_filtered_ordered (conn, "[
				cust_acct_no = :1
			    and trunc(nvl(date_expires, sysdate)) >= trunc(sysdate)
			    and exists (select 1 from cust_quote_line_items cqli, m_parts mp
			   	 where cqli.m_part_no = mp.m_part_no
			           and cqli.m_cust_quote_no = cq.m_cust_quote_no
			           and cqli.m_part_no = :2
			           and mp.part_type = :3)
			    ]", 
				b, "cust_quote_id, revision desc")
			check_row_count (Result, range)
		end

feature	-- the location of the pdf copy of this quote

	content_type_id: STRING
	do  Result := ctype_is_pdf
	end;

	quote_pdf_filename: STRING
		-- the known location of the generated pdf copy
	do  Result := su.string_of_4
			(quote_directory_name,
			 full_cust_quote_id,
			 ".",
			 ctype_is_pdf)
	    Result.to_lower
	end;

	quote_directory_name: STRING
		-- the fixed directory in which all quote PDF are saved
	once  Result := connection.selected_string
				("char_value",
				 "admin_application_parameters",
				 "application = %'REPORTS%' and%
				 % parameter_name = %'$OQUOTE%'", Void)
	end;

feature {NONE} -- details

	anchored_line_item: CUST_QUOTE_LINE_ITEMS_ROW
	do
	ensure Result = Void
	end;

	cached_line_items: LIST[like anchored_line_item]

	cached_latest_revision: STRING;

end -- CUST_QUOTES_ROW
