
note

	author: "Generated"
	date: "Sat Feb 21 09:30:28 PST 2004"

class CUST_WOS_ROW
inherit
	CUST_WOS_RAW
	ROW_COUNT_CHECK_ACCESS
	DB_QUERY_OPTIONS
		export {NONE} all end;

	PRICING_ENVIRONMENT
	rename	quote_no		as m_cust_quote_no,
		is_null_quote_no	as is_null_m_cust_quote_no
	end;
	HAS_DESIGN_NUMBER

create
	make_template, make_null, make_by_key

feature	-- simple stuff

	is_export_order: BOOLEAN
		-- is this order an export order?
	local b: DB_BIND_I;
	do create b.make(m_order_no)
	   Result := connection.selected_boolean
				("cust_wo_status.is_export_order(:1)",
				 "dual", Void, b)
	end;

	is_subject_to_export_charge: BOOLEAN
		-- is this order subject to the export order handling charge?
	local b: DB_BIND_I;
	do create b.make(m_order_no)
	   Result := connection.selected_boolean
				("cust_wo_status.is_subject_to_export_charge(:1)",
				 "dual", Void, b)
	end;

	hard_date_for_funding: DB_DATE
		-- if date_for_funding is null, use date_for_pricing
	do  if is_null_date_for_funding
	     then Result := date_for_pricing
	     else Result := date_for_funding
	    end
	ensure Result /= Void
	end;

	pricing_group: STRING
		-- my (price_schedule's) pricing group
	local b: DB_BIND_S;
	do  create b.make(pricing_schedule)
	    Result := connection.selected_string("pricing_group",
				"pricing_schedules",
				"pricing_schedule = :1", b)
	end;

	pricing_descr: STRING
		-- my (price_schedule's) descr
	local b: DB_BIND_S;
	do  create b.make(pricing_schedule)
	    Result := connection.selected_string("descr",
				"pricing_schedules",
				"pricing_schedule = :1", b)
	end;

	needs_final_bonding_diagrams: BOOLEAN
			-- Does project need any final bonding diagrams?
			-- (Note: If there are no packaging line
			-- items, then we use the IC packaging for the
			-- fab instance)
		do
			Result := (not is_null_m_fab_no) and then fab_instance.is_customer_project
		end

	final_diagram_bonding_names: LIST [STRING]
			-- Bonding names for all packaging line items
			-- for order that require preparation of a final
			-- bonding diagram
		do
			create {ARRAYED_LIST [STRING]} Result.make (line_items.count)
			line_items.do_if (agent add_line_item_bonding_name (?, Result), agent {like anchored_line_item}.needs_final_bonding_diagram)
		end

	is_mosis_still_paying: BOOLEAN
			-- Is MOSIS still paying for order (i.e., is
			-- the funding alloc < 0)?
		do
			Result := m_funding_alloc_no < 0
		end

	is_on_account (n: INTEGER): BOOLEAN
			-- Is work order on account `n'?
		do
		end

	is_on_account_and_quote (acct_no, quote_no: INTEGER): BOOLEAN
			-- Is work order on account `acct_no' and
			-- compatible with quote `quote_no' (either has
			-- that quote number or is null)?
		do
		end

	is_non_ic_on_account_and_quote (acct_no, quote_no: INTEGER): BOOLEAN
			-- Is work order a non-IC order on account
			-- `acct_no' and compatible with quote
			-- `quote_no' (either has that quote number or
			-- is null)?
		do
			Result := is_null_m_fab_no and is_on_account_and_quote (acct_no, quote_no)
		end

	ic_quantity_ordered: INTEGER
			-- Number of IC parts ordered
		do
			private_quantity_ordered := 0
			line_items.do_all (agent add_line_item_ic_quantity)
			Result := private_quantity_ordered
		end

	needs_eccn (wtf_row: WAFER_TECHN_FAMILIES_ROW): BOOLEAN
			-- Does this work order need an ECCN if
			-- project goes on a run for wafer techn
			-- family `wtf_row'?
		do
			if is_null_m_fab_no then
			else
				
			end
		end

	has_eccn: BOOLEAN
		do
			Result := not (is_null_assigned_ecc_paragraph_id and is_null_assigned_eccn_value)
		end

	has_fab_run_part: BOOLEAN
		-- do any of my line items have a fab_run part?
	do
		Result := line_items.there_exists
				(agent {like anchored_line_item}.is_fab_run_part)
	end;

	has_shipments: BOOLEAN
			-- Does any of the line items for `Current' 
			-- have a shipment?
		do
			Result := line_items.there_exists
				(agent {like anchored_line_item}.is_on_shipment)
		end

	has_shippable_shipped_line_item: BOOLEAN
			-- Are any of the shippable line items for
			-- `Current' on a shipment (i.e., are any
			-- shippable line items shipped)?
		do
			Result := line_items.there_exists
				(agent {like anchored_line_item}.is_shippable_and_on_shipment)
		end

	all_line_items_are_shipped: BOOLEAN
			-- Do all of the line items for `Current' 
			-- are on a shipment?
		do
			Result := line_items.for_all
				(agent {like anchored_line_item}.is_on_shipment)
		end

	is_itar: BOOLEAN
			-- Is design for `Current' an ITAR design?
		do
			Result := fab_instance.cust_ic_design.is_itar
		end

	is_certificate_eligible: BOOLEAN
			-- Can we generate a fab certificate for
			-- `Current'?
		do
		end

	is_certificate_eligible_ic: BOOLEAN
			-- Is `Current' an order for an IC that is
			-- eligible for a fab certificate?
		do
			Result := not is_null_m_fab_no and then not fab_instance.cust_ic_design.is_3dic_parent and is_certificate_eligible
		end

	is_3dic_tier_order: BOOLEAN
			-- Is `Current' an order for a tier of
			-- a 3DIC design?
		do
			Result := not is_null_m_fab_no and then fab_instance.cust_ic_design.is_3dic_tier
		end

	not_3dic_tier_order: BOOLEAN
			-- Is `Current' not an order for a tier of
			-- a 3DIC design?
		do
			Result := not is_3dic_tier_order
		end

feature -- report

	eccn_assignment: STRING
		require
			eccn_set: has_eccn
		do
			if not is_null_assigned_eccn_value then
				Result := assigned_eccn_value
			else
				-- this should be formatted better
				Result := assigned_ecc_paragraph_id
			end
		end

feature	-- cust_wo_line_items access

	line_items: LIST[like anchored_line_item]
	local r: like anchored_line_item; b: DB_BIND_I; 
	do  if cached_line_items = Void
	     then create r.make_template;
		  r.set_the_wo(Current);
		  create b.make(m_order_no);
		  cached_line_items :=
			r.all_rows_filtered_ordered
				(connection, "m_order_no = :1", b,
				 "cust_wo_line_no");
	    end
	    Result := cached_line_items
	end;

	select_line_items(master: LIST[like anchored_line_item])
		-- select my line items out of master
	local it: like anchored_line_item;
	do  create {LINKED_LIST[like anchored_line_item]}
			cached_line_items.make;
	    from  master.start
	    until master.after
	    loop  it := master.item
		  if it.m_order_no = m_order_no
		   then	-- one of mine: adopt it
			it.set_the_wo(Current);
			cached_line_items.extend(it)
		  end;
		  master.forth
	    end
	end;

	is_cached_line_items: BOOLEAN
	do  Result := cached_line_items /= Void
	end;

	uncache_line_items
	do  cached_line_items := Void
	end;

	next_wo_line_no: INTEGER
			-- Next work order line item to use,
			-- based on the ones already used in `line_items'
		local
			list: like line_items
			n: INTEGER
		do  
			list := line_items
			from
				Result := 0
				list.start
			until
				list.after
			loop
				n := list.item.cust_wo_line_no
				if n > Result then
					Result := n
				end
				list.forth
			end
			Result := Result + 1
		end

	resequence_line_items
			-- Resequence line items for `Current' so that they
			-- are in the standard order.  Uncaches line items
		require
			has_block_access: connection.has_block_access
		local 
			b: SQL_BLOCK
		do
			uncache_line_items
			b := resequence_line_items_block
			b.execute
			b.release
		end

feature {NONE} -- Implementation

	anchored_line_item: CUST_WO_LINE_ITEMS_ROW
		-- for anchoring; always Void
	do
	ensure Result = Void
	end;

	cached_line_items: LIST[like anchored_line_item]

	resequence_line_items_block: SQL_BLOCK
			-- Create and return a SQL_BLOCK that when executed
			-- will resequence line items for `Current' so that they
			-- are in the standard order.
		require
			has_block_access: connection.has_block_access
		do
			Result := connection.sql_block (resequence_line_items_stmt)
			Result.set_named_bind_capacity (1)
			Result.bind_name_to_integer (ph_m_order_no, m_order_no, False)
		end

	ph_m_order_no: STRING = "ord_no"

	resequence_line_items_stmt: STRING =
		"begin cust_wo_utilities.resequence_line_items(:ord_no); end;"

feature	-- Fab instance access

	fab_instance: like anchored_fab_instance
			-- Fab instance
		require
			has_fab_instance: not is_null_m_fab_no
		do  
			if cached_fab_instance = Void then
				create cached_fab_instance.make_by_key (connection, Old_row, m_fab_no)
			end
			Result := cached_fab_instance
		ensure
			result_not_void: Result /= Void
		end

	select_fab_instance (list: LIST [like anchored_fab_instance])
		-- Select my fab instance out of `list'
		local 
			f: like anchored_fab_instance
		do
	    		cached_fab_instance := Void
	    		from
				list.start
	    		until
				list.after or cached_fab_instance /= Void
	    		loop
				f := list.item
				if f.m_fab_no = m_fab_no then
					cached_fab_instance := f
				end
				list.forth
			end
		end

	is_cached_fab_instance: BOOLEAN
		do  
			Result := cached_fab_instance /= Void
		end

	uncache_fab_instance
		do  
			cached_fab_instance := Void
		end

	has_design_number: BOOLEAN
		do
			Result := not is_null_m_fab_no
		end

	design_number: INTEGER
		do
			Result := fab_instance.design_no
		end

feature {NONE} -- private fab instance stuff

	anchored_fab_instance: IC_FAB_INSTANCES_ROW
		-- For anchoring; always Void
		do
		ensure 
			result_void: Result = Void
		end

	cached_fab_instance: like anchored_fab_instance

feature	-- cust_acct access

	is_cached_cust_acct: BOOLEAN
	do 
	end;

	uncache_cust_acct
	do
	end;

feature -- cust_quote access

	cust_quote: CUST_QUOTES_ROW
	require not is_null_m_cust_quote_no
	do  if	cached_cust_quote = Void or else
		cached_cust_quote.m_cust_quote_no /= m_cust_quote_no
	     then create cached_cust_quote.make_by_key(connection, 
					old_row, m_cust_quote_no)
	    end;
	    Result := cached_cust_quote
	end;

	full_cust_quote_id: STRING
		-- assembled "full" quote id (includes rev letter)
	require not is_null_m_cust_quote_no
	do  Result := cust_quote.full_cust_quote_id
	end;

	select_cust_quote(master: LIST[CUST_QUOTES_ROW])
	do  from  master.start;
		  cached_cust_quote := Void
	    until master.after or cached_cust_quote /= Void
	    loop  if master.item.m_cust_quote_no = m_cust_quote_no
		   then cached_cust_quote := master.item
		  end;
		  master.forth
	    end
	end;

	is_cached_cust_quote: BOOLEAN
	do  Result := cached_cust_quote /= Void
	end;

	uncache_cust_quote
	do  cached_cust_quote := Void
	end;

feature {NONE} -- cust_quote holder

	cached_cust_quote: CUST_QUOTES_ROW

feature	-- shipping modalities

	FedEX_shipping_mode: STRING
		-- default FedEX shipping method
		-- per Mosis policy as of 2-May-2007
		-- (DEV runs treated same as SHR)
	require on_run: not fab_instance.is_null_run_id
	local run: RUNS_ROW; rtyp: RUN_TYPE_VALUES;
	do  if is_export_order then
		Result := International_Priority
	    else
		create run.make_by_key(connection, old_row, fab_instance.run_id);
	    end
	end;

	is_shipping_mode_upgradable: BOOLEAN
		-- should we offer a shipping upgrade (for a fee)
	do
	end;

feature {NONE} -- more shipping details

	gets_fast_shipping(an_org: INTEGER): BOOLEAN 
		-- is org_no = an_org a FAST SHIPPING type?
	local b: DB_BIND_IS; otyps: ORG_TYPE_VALUES
	do  create b.make(an_org, otyps.fast_shipping);
	    Result := 0 < connection.selected_count
				("org#org_types",
				 "org_no = :1 and org_type = :2", b)
	end;

	Overnight_Priority: STRING = "Overnight Priority"

	International_Priority: STRING = "International Priority"

	Second_Business_Day: STRING = "2nd Business Day"

feature -- Collections
	
	commercial_work_orders_for_run (prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All work orders for customer projects on production
			-- `prod', that are for a commercial account
		require
			prod_exists: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SSS
			ic_fab_instance_type: IC_FAB_INSTANCE_TYPE_VALUES
		do
		ensure
			result_exists: Result /= Void
		end

	test_vehicle_orders_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All work orders for test vehicles on run
			-- `run'
		require
			run_not_void: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			ic_fab_instance_type: IC_FAB_INSTANCE_TYPE_VALUES
		do
			create b.make (run, ic_fab_instance_type.Test_vehicle)
			Result := all_rows_filtered (conn, "m_fab_no in (select m_fab_no from ic_fab_instances where run_id = :1 and ic_fab_instance_type = :2)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	cust_non_tier_work_orders_for_production (prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All work orders for customer projects on production
			-- `prod' that are not for tier designs on
			-- a 3DIC run
		require
			prod_exists: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			ic_fab_instance_type: IC_FAB_INSTANCE_TYPE_VALUES
		do
			create b.make (prod, ic_fab_instance_type.Customer_fab)
			Result := all_rows_filtered (conn, "[
				production_id = :1 and m_fab_no in 
				(select m_fab_no from ic_fab_instances ifi, cust_ic_designs cid 
				  where ifi.design_no = cid.design_no 
				    and cid.tier_no_3dic is null 
				    and ifi.ic_fab_instance_type = :2)
				]", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	work_orders_for_production (prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All work orders for production `prod',
			-- sorted by die ID
		require
			prod_exists: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (prod)
			Result := all_rows_filtered_ordered (conn, "production_id = :1", b, 
				"(select die_id from ic_fab_instances ifi where m_fab_no = cwo.m_fab_no)")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	certificate_work_orders_for_production (prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All work orders for production `prod'
			-- whose customer account is eligible for
			-- a fab certificate
		require
			prod_exists: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			list: like certificate_work_orders_for_production
		do
			list := work_orders_for_production (prod, conn, range)
			Result := list.twin
			Result.wipe_out
			list.do_if (agent Result.extend, agent {like Current}.is_certificate_eligible_ic)
		ensure
			result_exists: Result /= Void
		end

	fab_run_orders_for_production (prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All work orders for production `prod' that
			-- have a line item whose part is of type FAB RUN
		require
			prod_exists: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			part_type: PART_TYPE_VALUES
		do
			create b.make (prod, part_type.Fab_run)
			Result := all_rows_filtered (conn, "production_id = :1 and exists (select 1 from cust_wo_line_items cwli, m_parts mp where cwli.m_part_no = mp.m_part_no and cwli.m_order_no = cwo.m_order_no and mp.part_type = :2)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	orders_for_fab_no (fno: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All work orders for fab_no `fno'
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (fno)
			Result := all_rows_filtered_ordered 
					(conn, "m_fab_no = :1", b, 
					"(select date_production_closed from productions p where p.production_id = cwo.production_id)")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	orders_for_fab_no_and_prod (fno: INTEGER prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All work orders for fab_no `fno' that are on
			-- production `prod' (or any production if `prod' is
			-- empty)
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			prod_not_void: prod /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_IS
			where: STRING
		do
			create b.make (fno, prod)
			where := "m_fab_no = :1"
			if prod.is_empty then
				where.append (" and :2 is null")
			else
				where.append (" and production_id = :2")
			end
			Result := all_rows_filtered_ordered (conn, where, b, "production_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	orders_for_fab_no_prod_and_pkging_no (fno: INTEGER prod: STRING pkging_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All work orders for fab_no `fno' that are on
			-- production `prod' (or any production if `prod' is
			-- empty) and that have a line item linked to IC
			-- packaging `pkging_no'
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			prod_not_void: prod /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SII
			where: STRING
		do
			create b.make (prod, fno, pkging_no)
			create where.make (100)
			if prod.is_empty then
				where.append (":1 is null")
			else
				where.append ("production_id = :1")
			end
			where.append  (" and m_fab_no = :2")
			where.append  (" and exists (select 1 from cust_wo_line_items cwli, ic_packagings ip %N%
				%   where cwo.m_order_no = cwli.m_order_no and cwli.ic_packaging_no = :3)")
			Result := all_rows_filtered_ordered (conn, where, b, "production_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	orders_for_wafer_lot(rid: STRING; lno: INTEGER; conn: DB_CONNECTION): DYNAMIC_LIST [like Current]
		-- all orders for parts on this run_wafer_lot (by production and process)
	local b: DB_BIND_SI; where: STRING;
	do
		create b.make(rid, lno);
		where := "(m_fab_no, production_id) in %
			 %(select ifi.m_fab_no, rwl.production_id%
			 % from ic_fab_instance_processes ifip,%
			 %  ic_fab_instances ifi, run_wafer_lots rwl%
			 % where ifip.m_fab_no = ifi.m_fab_no%
			 %  and ifip.wafer_process = rwl.wafer_process%
			 %  and ifi.run_id = rwl.run_id%
			 %  and rwl.run_id = :1 and rwl.wafer_lot_no = :2)"
		Result := all_rows_filtered (conn, where, b)
	end;

feature {NONE} -- Implementation

	add_line_item_bonding_name (cwli: like anchored_line_item list: LIST [STRING])
			-- Add bonding name for `cwli' (which must be a
			-- packaging line item) to `list'
		do
			list.extend (cwli.ic_packaging.bonding_name)
		end

	add_line_item_ic_quantity (cwli: like anchored_line_item)
		do
			if cwli.is_ic_part then
				private_quantity_ordered := private_quantity_ordered + cwli.part_quantity_ordered * cwli.part.quantity_in_part
			end
		end

	private_quantity_ordered: INTEGER
			-- For calculating IC quantity ordered from line items

end
