
note

	author: "Generated"
	date: "Thu Mar 11 15:52:47 PST 2004"

class CUT_DIE_SITES_ROW
inherit
	CUT_DIE_SITES_RAW
	ROW_COUNT_CHECK_ACCESS
	DB_QUERY_OPTIONS
		export {NONE} all
		end

create
	make_template, make_null, make_by_key

feature -- Properties

	rows_for_die (a_run_id, a_die_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All rows for `a_run_id' and `a_die_id'
		require
			a_run_id_exists: a_run_id /= Void
			a_die_id_exists: a_die_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (a_run_id, a_die_id)
			Result := all_rows_filtered (conn, die_filter, b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_run (a_run_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All rows for `a_run_id', ordered
			-- by cut pattern number
		require
			a_run_id_exists: a_run_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (a_run_id)
			Result := all_rows_filtered_ordered (conn, "run_id = :1", b, "cut_pattern_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_run_and_production (a_run_id, a_prod_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All rows for for cut patterns assigned
			-- to run `a_run_id' and production `a_prod_id'
		require
			a_run_id_exists: a_run_id /= Void
			a_prod_id_exists: a_prod_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (a_run_id, a_prod_id)
			Result := all_rows_filtered (conn, "(run_id, cut_pattern_no) in (select run_id, cut_pattern_no from run_cut_patterns where run_id = :1 and production_id = :2)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_run_and_pattern (a_run_id: STRING pat_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All rows for `a_run_id' and `pat_no'
		require
			a_run_id_exists: a_run_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SI
		do
			create b.make (a_run_id, pat_no)
			Result := all_rows_filtered (conn, "run_id = :1 and cut_pattern_no = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_ic_packaging (packaging_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All cut die sites for an IC Packaging
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (packaging_no)
			Result := all_rows_filtered (conn, ic_packaging_filter, b)
			check_row_count (Result, range)
		end

feature {NONE} -- Implementation

	die_filter: STRING = "(run_id, mask_site_no) in (%
		%select distinct dl.run_id, dl.mask_site_no %
		%from die_locations dl %
		%where dl.run_id = :1 and dl.die_id = :2)"

	ic_packaging_filter: STRING = "(run_id, cut_pattern_no, mask_site_no) in (%
		%select distinct rcp.run_id, rcp.cut_pattern_no, dl.mask_site_no %
		%from ic_packagings icp, ic_fab_instances ifi, die_locations dl, cust_wo_line_items cwli, run_cut_patterns rcp, run_packaging_lots rpl %
		%where rcp.run_id=rpl.run_id and rcp.packaging_lot_no=rpl.packaging_lot_no %
		%and rpl.run_id=cwli.packaging_run_id and rpl.packaging_lot_no=cwli.packaging_lot_no %
		%and cwli.ic_packaging_no = icp.ic_packaging_no %
		%and dl.run_id = ifi.run_id and dl.die_id = ifi.die_id %
		%and ifi.m_fab_no = icp.m_fab_no %
		%and icp.ic_packaging_no = :1)"

feature	-- Cut sizes in reticle coordinates

	reticle_size_x: DOUBLE
		-- size of cut die in reticle coordinates
	do  if die.is_rotated_sideways then
		Result := die_size_cut_y
	    else
		Result := die_size_cut_x
	    end
	end;

	reticle_size_y: DOUBLE
		-- size of cut die in reticle coordinates
	do  if die.is_rotated_sideways then
		Result := die_size_cut_x
	    else
		Result := die_size_cut_y
	    end
	end;

feature	-- access to dies_row and litho_sizes_row

	die_location: DIE_LOCATIONS_ROW
		-- my die_location
	do  if cached_die_location = Void then
		create cached_die_location.make_by_key
				(connection, old_row, run_id, mask_site_no)
	    end
	    Result := cached_die_location
	end;

	die: DIES_ROW
		-- my die
	local dloc: DIE_LOCATIONS_ROW
	do  if cached_die = Void then
		dloc := die_location;
		create cached_die.make_by_key
				(connection, old_row, dloc.run_id, dloc.die_id)
	    end
	    Result := cached_die
	end;

feature {NONE} -- caches for above

	cached_die_location: DIE_LOCATIONS_ROW

	cached_die: DIES_ROW

end
