
note

	author: "Generated"
	date: "Thu Jun 12 14:26:02 PDT 2003"

class DIES_ROW
inherit
	ABSTRACT_DIE
		redefine
			selects_designated_dt
		end
	DIES_RAW
		rename
			cats_status as job_status,
			maxlen_cats_status as maxlen_job_status,
			is_null_cats_status as is_null_job_status
		end
	ROW_COUNT_CHECK_ACCESS
	NULL_VALUE_CHECK_ACCESS
	ITEM_COUNT_RANGES_ACCESS
	CONST_BATCH_JOB_REQUESTOR
	DB_QUERY_OPTIONS
	DIE_PART_ID_QUERIES
		export
			{NONE} all
		end
create
	make_template, make_null, make_by_key

feature -- Properties

	usage: STRING
			-- How part id is used
		do
			Result := "run " + run_id + ", die " + die_id
		end

	die_part_id_usages (seq: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All uses of parts in sequence `seq' by runs
			-- in the DIES table
		require
			sequence_exists: seq /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			labeled_entity: LABELED_ENTITY_VALUES
		do
			create b.make (seq, labeled_entity.Die)
			Result := all_rows_filtered (conn, die_part_id_query (seq), b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	dies_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All dies for run `run', ordered by die id
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "run_id = :1", b, "die_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	test_vehicle: M_TEST_VEHICLES_ROW
			-- Test vehicle for `Current'
		require
			has_data_for_probing: has_data_for_probing
		local
			row: M_TEST_VEHICLES_ROW
		do
			if cached_test_vehicle = Void then
				create row.make_template
				cached_test_vehicle := row.test_vehicles_for_fab_id (run_id, die_id, connection, expected_count.exactly_one).first
			end
			Result := cached_test_vehicle
		ensure
			result_exists: Result /= Void
		end

	test_objects: LIST [M_TEST_OBJECTS_EROW]
			-- All test objects for `Current'
		require
			is_probeable: is_probeable
		local
			row: M_TEST_OBJECTS_EROW
		do
			if private_test_objects = Void then
				create row.make_template
				private_test_objects := row.test_objects_for_design_no (design_no, connection, expected_count.At_least_one)
			end
			Result := private_test_objects
		ensure
			result_exists: Result /= Void
		end

	is_probeable: BOOLEAN
			-- Can `Current' be probed?
		local
			status: TEST_VEHICLE_STATUS_VALUES
		do
			Result := (is_test_vehicle or else (is_cust_die and has_data_for_probing)) and then not test_vehicle.test_vehicle_status.is_equal (status.retired)
		end

	has_data_for_probing: BOOLEAN
			-- Does `Current' have enough data to
			-- allow it to be probed?
		do
			if not is_cached_has_data_for_probing then
				fetch_has_data_for_probing
			end
			Result := cached_has_data_for_probing
		end

	fab_id: STRING
		-- my fabrication_id (upper case) as a new string
	do  Result := run_id + die_id
	end;

	fab_id_lc: STRING
		-- my fabrication_id in lower case
	do  Result := fab_id
	    Result.to_lower
	end;

	design_no: INTEGER
		do
			Result := ic_fab_instance.design_no
		end

	is_on_3dic_tier (a_tier: WAFER_TECHN_3D_TIERS_ROW): BOOLEAN
			-- Is `Current' on `a_tier'?
		do
			Result := (a_tier = Void and is_null_tier_no_3dic) or (a_tier /= Void and then a_tier.tier_no = tier_no_3dic)
		end

	is_3dic_tier: BOOLEAN
			-- Is `Current' on some 3DIC tier?
		do
			Result := not is_null_tier_no_3dic
		end

	is_non_3dic_tier: BOOLEAN
			-- Is `Current' either a normal die or
			-- a 3DIC die, but not a 3DIC tier?
		do
			Result := not is_3dic_tier
		end

	is_3dic_tier_scribe: BOOLEAN
			-- Is `Current' a scribe on some 3DIC tier?
		do
			Result := is_scribe and is_3dic_tier
		end

	is_3dic_tier_filler: BOOLEAN
			-- Is `Current' a filler on some 3DIC tier?
		do
			Result := is_filler and is_3dic_tier
		end

	is_3dic_tier_non_vendor_ghost: BOOLEAN
			-- Is `Current' a non-vendor ghost die 
			-- on some 3DIC tier?
		do
			Result := is_not_vendor_ghost and is_3dic_tier
		end

	is_3dic_tier_and_needs_overhead_file: BOOLEAN
			-- Is `Current' a die on some 3DIC tier
			-- that needs an overhead file?
		do
			Result := needs_overhead_file and is_3dic_tier
		end

	selects_designated_dt: BOOLEAN
			-- `<precursor>'
		do
			Result := precursor and is_null_tier_no_3dic
		end

	is_normal_filler: BOOLEAN
			-- Is `Current' a normal filler?
		local
			type: FILLER_TYPE_VALUES
		do
			Result := is_filler and filler_type.is_equal (type.Normal)
		end

	is_temporary_filler: BOOLEAN
			-- Is `Current' a temporary filler?
		local
			type: FILLER_TYPE_VALUES
		do
			Result := is_filler and filler_type.is_equal (type.Temporary)
		end

	is_placeholder_filler: BOOLEAN
			-- Is `Current' a placeholder filler?
		local
			type: FILLER_TYPE_VALUES
		do
			Result := is_filler and filler_type.is_equal (type.Placeholder)
		end

feature	-- orientation (for scribes)

	is_vertical: BOOLEAN
		-- Is scribe_orientation vertical?
	local	orient: ORIENTATION_VALUES
	do	Result := equal(scribe_orientation, orient.vertical)
	end;

	is_horizontal: BOOLEAN
		-- Is scribe_orientation horizontal?
	local	orient: ORIENTATION_VALUES
	do	Result := equal(scribe_orientation, orient.horizontal)
	end;

feature	-- my die locations

	locations: LIST[DIE_LOCATIONS_ROW]
	local r: DIE_LOCATIONS_ROW; b: DB_BIND_SS;
	do  if cached_locations = Void
	     then create r.make_template;
		  create b.make(run_id, die_id);
		  cached_locations := r.all_rows_filtered_ordered
				(connection, 
				"run_id = :1 and die_id = :2", b,
				"mask_site_no")
	    end;
	    Result := cached_locations
	ensure Result /= Void
	end;

	printed_box(l: DIE_LOCATIONS_ROW): RECTANGLE
		-- the box for location l
	require same_die: equal(l.run_id, run_id) and 
			  equal(l.die_id, die_id)
	do  create Result.make_by_size_and_center
			(die_size_printed_x, die_size_printed_y,
			 l.location_x, l.location_y);
	end;

	printed_boxes: ARRAYED_LIST[RECTANGLE]
		-- one box for each die location
	do  create Result.make(locations.count)
	    from  locations.start
	    until locations.after
	    loop  Result.extend(printed_box(locations.item));
		  locations.forth
	    end;
	ensure Result.count = locations.count
	end;

feature	-- my cust_ic_design and ic_fab_instance (if I have one)

	ic_fab_instance: IC_FAB_INSTANCES_ROW
		-- returns Void if not relevant to this die
	local b: DB_BIND_SS;
	do  if cached_ifis = Void
	     then create Result.make_template;
		  create b.make(run_id, die_id);
		  cached_ifis := Result.all_rows_filtered_ordered
				 (connection, 
				 "run_id = :1 and die_id = :2", b,
				 "project_on_die")
		  Result := Void
	    end;
	    if cached_ifis.count > 0
	     then Result := cached_ifis.first
	    end
	end;

	cust_ic_design: CUST_IC_DESIGNS_ROW
		-- returns Void if not relevant to this die
	local ifi: IC_FAB_INSTANCES_ROW
	do  ifi := ic_fab_instance;
	    if ifi /= Void
	     then Result := ifi.cust_ic_design
		  Result.use_final_layout
	    elseif not is_null_test_vehicle_design_no 
	     then	-- die is test vehicle, but ifi not yet created
		  if cached_tv_cid = Void
		   then create cached_tv_cid.make_by_key
					(connection, old_row, test_vehicle_design_no)
			cached_tv_cid.use_final_layout
		  end;
		  Result := cached_tv_cid
	    end
	end;

	uncache_fab_instance
			-- Remove fab instance from cache
		do
			cached_ifis := Void
		end

	uncache_cust_ic_design
			-- Remove cust ID design from cache
		do
			cached_tv_cid := Void
		end

feature -- Properties

	needs_fracture_script: BOOLEAN
			-- Does die need a fracture script?
		do
			Result := not is_macrocell and not is_vendor_ghost
		end

	still_needs_fracture_script: BOOLEAN
			-- Does die need a fracture script which hasn't
			-- ben generated yet?
		local
			status: JOB_STATUS_VALUES
		do
			Result := needs_fracture_script and job_status.is_equal (status.Not_created)
		end

	description: STRING
			-- Description of die suitable for
			-- error message
		local
			ifi: like ic_fab_instance
		do
			ifi := ic_fab_instance
			if ifi /= Void then
				Result := ifi.design_no.out
			else
				Result := die_type + " " + die_id
			end
		end

feature	-- geometries row (for vendor_geometry and ghost)

	has_geometry: BOOLEAN
		-- is there a GEOMETRIES_ROW associated with this die?
	do  Result := not is_null_geometry_id
	end;

	geometries_row: GEOMETRIES_ROW
	require has_geometry: has_geometry
	do  if cached_geometry = Void then
		create cached_geometry.make_by_key(connection, old_row, geometry_id)
	    end;
	    Result := cached_geometry
	end;

	is_internal_to_payload: BOOLEAN
			-- Is `Current' internal to the payload area?
		do
			Result := not is_external_to_payload
		end

	is_external_to_payload: BOOLEAN
			-- Is `Current' external to the payload area?
		do
			Result := is_vendor_geometry_or_ghost and then geometries_row.external_to_payload
		end

feature	-- die_attributes and pieces thereof

	attributes: DIE_ATTRIBUTES_ROW
	local r: DIE_ATTRIBUTES_ROW; b: DB_BIND_SS; 
	do  if cached_attributes = Void
	     then create r.make_template;
		  create b.make(run_id, die_id);
		  cached_attributes := r.all_rows_filtered
				(connection, 
				 "run_id = :1 and die_id = :2", b).first;
	    end;
	    Result := cached_attributes
	end;

	cache_attributes (r: like attributes)
			-- Set `attributes' to `r'
			-- (optimization to avoid slow fetch
			-- if caller already has it in memory)
		require
			r_not_void: r /= Void
		do
			cached_attributes := r
		ensure
			cached_attributes_set: cached_attributes = r
		end

	uncache_attributes
			-- Remove die attributes from cache
		do
			cached_attributes := Void
		end

	design_techn: STRING
	do  Result := attributes.design_techn
	end;

	design_techn_id: STRING
	do  Result := attributes.design_techn_id
	end;

	wafer_design_techn: STRING
	do  Result := attributes.wafer_design_techn
	end;

	wafer_design_techn_id: STRING
	do  Result := attributes.wafer_design_techn_id
	end;

	geometry_format: STRING
	do  Result := attributes.geometry_format
	end;

	overhead_style: STRING 
	do  Result := attributes.overhead_style
	end;

	design_die_offset: POINT
	do  create Result.make_xy(attributes.design_die_offset_x,
				  attributes.design_die_offset_y)
	end;

feature {NONE} -- cached data

	cached_locations: LIST[DIE_LOCATIONS_ROW]

	cached_ifis: LIST[IC_FAB_INSTANCES_ROW]

	cached_tv_cid: CUST_IC_DESIGNS_ROW

	cached_attributes: DIE_ATTRIBUTES_ROW

	cached_geometry: GEOMETRIES_ROW

feature {NONE} -- Implementation

	fetch_has_data_for_probing
			-- Fetch indication of whether
			-- die has enough data to be probed
		local
			row: M_TEST_VEHICLES_ROW
			list: LIST [M_TEST_VEHICLES_ROW]
		do
			create row.make_template
			list := row.test_vehicles_for_fab_id (run_id, die_id, connection, expected_count.At_least_zero)
			cached_has_data_for_probing := not list.is_empty
			is_cached_has_data_for_probing := True
		end

	private_test_objects: LIST [M_TEST_OBJECTS_EROW]
			-- Test objects for `Current' if previously fetched

	cached_test_vehicle: M_TEST_VEHICLES_ROW
			-- Test vehicle for die that is a test vehicle,
			-- cached in memory

	cached_has_data_for_probing: BOOLEAN
			-- Indicator of whether `Current' has enough
			-- data to allow it to be probed, cached
			-- in memory

	is_cached_has_data_for_probing: BOOLEAN
			-- Indicator of whether `cached_has_data_for_probing'
			-- is valid

end
