class ECC_QUESTIONNAIRE_TEXT_ROW

inherit
	ECC_QUESTIONNAIRE_TEXT_RAW

	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

create

	make_template, make_by_key

feature -- Status

	key_matches (p_id: like paragraph_id v_no: like version_no): BOOLEAN
		require
			p_id_exists: p_id /= Void
		do
			Result := not is_null_paragraph_id and paragraph_id.is_equal (p_id) and version_no = v_no
		end

feature -- Collections

	question_rows (q_id: STRING ver: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All questionniare text rows that are questions (i.e. is_question='Y') for the
			-- specified questionnaire ID (`q_id') and version_no (`ver')
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SI
		do
			create b.make (q_id, ver)
			Result := all_rows_filtered_ordered (conn, questions_filter, b, "paragraph_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	current_version_rows (questions_only: BOOLEAN conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All questionniare text rows that do not have another row with a higher version and the same
			-- paragraph ID.  If `questions_only' is True then return only the rows that are questions (i.e. is_question='Y')
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			filter: STRING
		do
			filter := version_filter
			if questions_only then
				filter := filter + question_filter
			end
			Result := all_rows_filtered_ordered (conn, filter, Void, "paragraph_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end


feature {NONE} -- Collection implementation

	version_filter: STRING = "version_no = (select max(version_no) from ecc_questionnaire_text x where x.paragraph_id = eccqt.paragraph_id)"

	question_filter: STRING = " and is_question='Y'"

	questions_filter: STRING = "ecc_questionnaire_id = :1 and version_no = :2 and is_question='Y'"

end -- ECC_QUESTIONNAIRE_TEXT_ROW
