class ECC_QUESTIONNAIRE_VERSIONS_EROW

inherit
	ECC_QUESTIONNAIRE_VERSIONS_ERAW

	ECC_QUESTIONNAIRE_ANSWER_SUMMARY
		rename
			section_id as ecc_questionnaire_id
		end

	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end

create

	make_template, make_null, make_detached


feature -- Status

	has_key (p_id: like ecc_questionnaire_id v_no: like version_no): BOOLEAN
		require
			p_id_exists: p_id /= Void
		do
			Result := not is_null_ecc_questionnaire_id and ecc_questionnaire_id.is_equal (p_id) and version_no = v_no
		end

	question_count: INTEGER
		do
			Result := questions.count
		end

	questions: DYNAMIC_LIST[ECC_QUESTIONNAIRE_TEXT_ROW]
		local
			r: ECC_QUESTIONNAIRE_TEXT_ROW
		do
			if cached_questions = Void then
				create r.make_template
				cached_questions := r.question_rows (ecc_questionnaire_id, version_no, connection, expected_count.at_least_zero)
			end
			Result := cached_questions
		end

feature -- {NONE} Implementation

	cached_questions: DYNAMIC_LIST[ECC_QUESTIONNAIRE_TEXT_ROW]

feature -- collections

	current_rows (conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST[like Current]
			-- All questionnaire versions rows that are active
		require
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_not_void: range /= Void
		do
			Result := all_rows_filtered_ordered (conn, current_rows_filter, Void, "ecc_questionnaire_id, version_no desc")
			check_row_count (Result, range)
		ensure
			result_not_void: Result /= Void
		end

	current_row_on_date (id: STRING d: DB_DATE conn: DB_CONNECTION): like Current
		local
			l: DYNAMIC_LIST[like Current]
			b: DB_BIND_SD
		do
			create b.make (id, d)
			l := all_rows_filtered_ordered (conn, current_rows_on_date_filter, b, "version_no desc")
			if l.count > 0 then
				Result := l.first
			end
		end

	all_version_rows (conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST[like Current]
			-- All questionnaire versions rows
		require
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_not_void: range /= Void
		do
			Result := all_rows_ordered (conn, "ecc_questionnaire_id, version_no desc")
			check_row_count (Result, range)
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- implementation

	current_rows_filter: STRING = "is_active = 'Y' %
			%and date_effective <= trunc (sysdate) %
			%and not exists (select 'x' from ecc_questionnaire_versions %
			%where ecc_questionnaire_id = eqv.ecc_questionnaire_id%
			%  and version_no > eqv.version_no)"

	current_rows_on_date_filter: STRING = "ecc_questionnaire_id = :1 %
			%and date_effective <= :2"


end -- ECC_QUESTIONNAIRE_VERSIONS_ROW
