
note

	author: "Generated"
	date: "Thu Jun 12 13:50:27 PDT 2003"

class FIELD_SETS_ROW
inherit
	FIELD_SETS_RAW
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		end
	DB_QUERY_OPTIONS
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key

feature	-- my litho_size

	litho_size: like anchored_litho_size
	do  if not(cached_litho_size /= Void and then
		   equal(litho_size_id, cached_litho_size.litho_size_id))
	     then create cached_litho_size.make_template
		  cached_litho_size := 
			cached_litho_size.litho_sizes_with_name
				(litho_size_id, connection, 
				 expected_count.exactly_one).first
	    end;
	    Result := cached_litho_size
	end;

	is_cached_litho_size: BOOLEAN
			-- Is `litho_size' currently cached?
		do
			Result := cached_litho_size /= Void
		end

	set_litho_size(master: like cached_litho_size)
		-- can be used to provide the correct litho_size from a cache
		-- or to clear local cache and force subsequent refetch
		-- this is NOT a way to change litho_size_id!!
	require	Void_or_mine: master = Void or else
				equal(litho_size_id, master.litho_size_id)
	do  cached_litho_size := master
	end;

feature -- Queries

	available_field_sets_ordered (run, wafer_techn: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All available field sets for `run' which has `wafer_techn'
		require
			run_not_void: run /= Void
			wafer_techn_exists: wafer_techn /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SSB
		do
			create b.make (run, wafer_techn, True)
			Result := all_rows_filtered_ordered (conn, "[
				field_set_name in
				   (select field_set_name from wafer_techn#field_sets
				    where wafer_techn = :2 and available = :3)
				or field_set_name in    
				   (select field_set_name from field_sets fs, litho_sizes ls
				    where fs.litho_size_id = ls.litho_size_id and ls.run_id = :1)
				]", b, "field_set_name")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	field_sets_ordered (run, wafer_techn: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All field sets ever associated with `run'
			-- or `wafer_techn'
		require
			run_not_void: run /= Void
			wafer_techn_exists: wafer_techn /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (run, wafer_techn)
			Result := all_rows_filtered_ordered (conn, "[
				field_set_name in
				   (select field_set_name from wafer_techn#field_sets
				    where wafer_techn = :2)
				or field_set_name in    
				   (select field_set_name from field_sets fs, litho_sizes ls
				    where fs.litho_size_id = ls.litho_size_id and ls.run_id = :1)
				or field_set_name in    
				   (select field_set_name from runs
				    where wafer_techn = :2 and field_set_name is not null)
				]", b, "field_set_name")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	custom_field_sets_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All field sets whose litho size is a custom
			-- one for `run'
		require
			run_not_void: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "[
				field_set_name in    
				   (select field_set_name from field_sets fs, litho_sizes ls
				    where fs.litho_size_id = ls.litho_size_id and ls.run_id = :1)
				]", b, "field_set_name")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	field_sets_with_name (name: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All field sets named `name'
		require
			name_exists: name /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (name)
			Result := all_rows_filtered (conn, "field_set_name = :1", b)
			check_row_count (Result, range)
		end

feature -- Properties

	field_names: LIST [STRING]
			-- Names of fields for `Current'
		do
			if cached_field_names = Void then
				fetch_field_names
			end
			Result := cached_field_names
		ensure
			result_not_void: Result /= Void
		end

	is_cached_field_names: BOOLEAN
			-- Is `field_names' currently cached?
		do
			Result := cached_field_names /= Void
		end

	description: STRING
			-- Description of field set suitable for display
			-- when selecting field set for a run
		local
			ls: like litho_size
		do
			ls := litho_size
			Result := descr + " (" + nu.formatted_number_or_integer (ls.payload_size_x, 6) + " x " + nu.formatted_number_or_integer (ls.payload_size_y, 6) + ")"
		end

	packing_target: STRING
			-- Packing target for litho size for `Current'
		do
			Result := litho_size.packing_target
		end

	has_fab_location: BOOLEAN
			-- Does `Current' have a fab location?
		do
			Result := not is_null_fab_location
		end

	has_wafer_type: BOOLEAN
			-- Does `Current' have an associated wafer type?
		do
			Result := not litho_size.is_null_wafer_type_id
		end

	has_mask_size: BOOLEAN
			-- Does `Current' have an associated mask size?
		do
			Result := not litho_size.is_null_mask_size_id
		end

	has_tweezer_margin: BOOLEAN
			-- Does `Current' have an associated tweezer margin?
		do
			Result := not litho_size.is_null_tweezer_margin
		end

	has_primary_flat_location: BOOLEAN
			-- Does `Current' have an associated primary flat location?
		do
			Result := not litho_size.is_null_primary_flat_location
		end

	is_null_pattern_file_renaming_style: BOOLEAN
		do
			Result := litho_size.is_null_pattern_file_renaming_style
		end

	pattern_file_renaming_style: STRING
		do
			Result := litho_size.pattern_file_renaming_style
		end

	wafer_type_id: STRING
		do
			Result := litho_size.wafer_type_id
		end

	mask_size_id: STRING
		do
			Result := litho_size.mask_size_id
		end

	primary_flat_location: STRING
		do
			Result := litho_size.primary_flat_location
		end

	tweezer_margin: DOUBLE
		do
			Result := litho_size.tweezer_margin
		end

	payload_size_x: DOUBLE
		do
			Result := litho_size.payload_size_x
		end

	payload_size_y: DOUBLE
		do
			Result := litho_size.payload_size_y
		end

	field_size_x: DOUBLE
		do
			Result := litho_size.field_size_x
		end

	field_size_y: DOUBLE
		do
			Result := litho_size.field_size_y
		end

	external_kerf_x: DOUBLE
		do
			Result := litho_size.external_kerf_x
		end

	external_kerf_y: DOUBLE
		do
			Result := litho_size.external_kerf_y
		end

feature -- Operations

	uncache_field_names
			-- Remove field names from cache, if present
		do
			cached_field_names := Void
		ensure
			field_names_not_cached: not is_cached_field_names
		end

	uncache_litho_size
			-- Remove litho size from cache, if present
		do
			cached_litho_size := Void
		ensure
			litho_size_not_cached: not is_cached_litho_size
		end

feature {NONE} -- Implementation

	fetch_field_names
		local
			b: DB_BIND_S
		do
			create b.make (field_set_name)
			cached_field_names := any_connection.selected_strings ("field_name", "field_names", "field_set_name = :1", "field_name", False, b)
		end

	cached_field_names: LIST [STRING]
			-- Names of fields for `Current', cached in memory

	cached_litho_size: like anchored_litho_size

	anchored_litho_size: LITHO_SIZES_ROW
		do
		ensure Result = Void
		end;

end
