class FIELD_STEP_PATTERNS_ROW

inherit
	FIELD_STEP_PATTERNS_RAW
	ITEM_COUNT_RANGES_ACCESS
	ROW_COUNT_CHECK_ACCESS

create
	make_template, make_null, make_by_key

feature -- Queries

	available_step_patterns_ordered (fs_name: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Step patterns for field set `fs_name',
			-- order by step pattern name
		require
			fs_name_not_void: fs_name /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SB
		do
			create b.make (fs_name, True)
			Result := all_rows_filtered_ordered (conn, "field_set_name = :1 and available = :2", b, "field_step_pattern")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	custom_step_patterns_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All field step patterns whose litho size is a custom
			-- one for `run'
		require
			run_not_void: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "[
				field_set_name in    
				   (select field_set_name from field_sets fs, litho_sizes ls
				    where fs.litho_size_id = ls.litho_size_id and ls.run_id = :1)
				]", b, "field_step_pattern")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature	-- my field_set

	field_set: like anchored_field_set
	do  if not(cached_field_set /= Void and then
		   equal(field_set_name, cached_field_set.field_set_name))
	     then create cached_field_set.make_template
		  cached_field_set := 
			cached_field_set.field_sets_with_name
				(field_set_name, connection, 
				 expected_count.exactly_one).first
	    end;
	    Result := cached_field_set
	end;

	set_field_set(master: like cached_field_set)
		-- can be used to provide the correct field_set from a cache
		-- or to clear local cache and force subsequent refetch
		-- this is NOT a way to change field_set_name!!
	require	Void_or_mine: master = Void or else
				equal(field_set_name, master.field_set_name)
	do  cached_field_set := master
	end;

feature	-- my field_step_pattern#field_locs

	field_locations: LIST[like anchored_field_location]
	local r: like anchored_field_location; b: DB_BIND_S
	do  if cached_field_locations = Void
	     then create r.make_template;
		  create b.make(field_step_pattern)
		  cached_field_locations :=
			r.all_rows_filtered_ordered
				(connection,
				 "field_step_pattern = :1", b,
				 "copy_no")
	    end;
	    Result := cached_field_locations
	end;

feature	-- bounding box of all my field_locations (in wafer coordinates)

	field_locs_bounding_box: RECTANGLE
		-- new RECTANGLE showing the stepped extrema
	local x, y, lx, hx, ly, hy: DOUBLE; cfl: like cached_field_locations;
		fl: FIELD_STEP_PATTERNXFIELD_LOCS_ROW;
		siz: CONST_POINT
	do  cfl := field_locations;
	    from  cfl.start
	    until cfl.after
	    loop  fl := cfl.item
		  x := fl.location_x; y := fl.location_y;  -- center of fl
		  lx := lx.min(x);  ly := ly.min(y);
		  hx := hx.max(x);  hy := hy.max(y);
		  cfl.forth
	    end;
	    siz := field_set.litho_size.field_size;
	    create Result.make_by_corners(lx - siz.x/2, ly - siz.y/2,
					  hx + siz.x/2, hy + siz.y/2)
	end;

feature	{NONE} -- caches and stuff

	anchored_field_set: FIELD_SETS_ROW
	do
	ensure Result = Void
	end;

	cached_field_set: like anchored_field_set

	anchored_field_location: FIELD_STEP_PATTERNXFIELD_LOCS_ROW
	do
	ensure Result = Void
	end;

	cached_field_locations: LIST[like anchored_field_location]

end -- FIELD_STEP_PATTERNS_ROW
