
note

	author: "David Hollenberg"

class GEOMETRIES_ROW
inherit
	GEOMETRIES_RAW
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	NULL_VALUE_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	MOSIS_PROD_VALUES
		export	{NONE} all	end

	GEOMETRY_RESOLUTION_ACCESS
		export	{NONE} all	end

create
	make_template, make_null, make_by_key, make_detached

feature	-- filename substitution

	final_geometry_filename(sps: SUBSTITUTION_PARAMETER_SET): STRING
		-- return the substituted geometry_filename
		-- plus extension (unless mebes)
	require have_sps: sps /= Void
	do  Result := sps.substituted_text (geometry_filename)
	    Result.to_lower	
		-- Original or substituted text may be upper case, so
		-- lower case
	    if equal(geometry_format, fmt_is_cif) then
		Result := Result + ".cif"
	    elseif equal(geometry_format, fmt_is_gds) then
		Result := Result + ".gds"
	    end
	end;

feature -- Queries

	payload_rows (wt, fs: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Rows for wafer techn `wt' and field set
			-- `fs' that are internal to payload, ordered
			-- by geometry_id
		require
			wt_not_void: wt /= Void
			fs_not_void: fs /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SSSB
			usage: GEOMETRY_USAGE_VALUES
		do
			create b.make (wt, fs, usage.Die, False)
			Result := all_rows_filtered_ordered (conn, 
				wt_fs_ext_filter, b, "geometry_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	nonpayload_rows (wt, fs: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Rows for wafer techn `wt' and field set
			-- `fs' that are external to payload, ordered
			-- by geometry_id
		require
			wt_not_void: wt /= Void
			fs_not_void: fs /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SSSB
			usage: GEOMETRY_USAGE_VALUES
		do
			create b.make (wt, fs, usage.Die, True)
			Result := all_rows_filtered_ordered (conn, 
				wt_fs_ext_filter, b, "geometry_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	fixed_locations (fs: STRING): LIST [GEOMETRYXFIXED_LOCATIONS_ROW]
			-- Fixed locations for `Current' and
			-- field set `fs'
		require
			fs_not_void: fs /= Void
		do
			if cached_fixed_locations = Void then
				create cached_fixed_locations.make (10)
			end
			cached_fixed_locations.search (fs)
			if cached_fixed_locations.found then
				Result := cached_fixed_locations.found_item
			else
				fetch_fixed_locations (fs)
				cached_fixed_locations.search (fs)
				if cached_fixed_locations.found then
					Result := cached_fixed_locations.found_item
				else
					handle_hard_error ("internal error in fixed_locations")
				end
			end
		end

	has_fixed_locations (fs: STRING): BOOLEAN
			-- Does `Current' have any fixed locations for
			-- field set `fs'?
		require
			fs_not_void: fs /= Void	
		do
			Result := not fixed_locations (fs).is_empty
		end

	is_cached_fixed_locations (fs: STRING): BOOLEAN
			-- Are fixed locations for field set
			-- `fs' currently cached?
		require
			fs_not_void: fs /= Void	
		do
			if cached_fixed_locations /= Void then
				cached_fixed_locations.search (fs)
				Result := cached_fixed_locations.found
			end
		end

	quantities (fs: STRING): LIST [GEOMETRY_QUANTITIES_ROW]
			-- Quantities for `Current' and field set `fs'
		require
			fs_not_void: fs /= Void
		do
			if cached_quantities = Void then
				create cached_quantities.make (10)
			end
			cached_quantities.search (fs)
			if cached_quantities.found then
				Result := cached_quantities.found_item
			else
				fetch_quantities (fs)
				cached_quantities.search (fs)
				if cached_quantities.found then
					Result := cached_quantities.found_item
				else
					handle_hard_error ("internal error in quantities")
				end
			end
		end

	has_quantities (fs: STRING): BOOLEAN
			-- Does `Current' have any quantities for
			-- field set `fs'?
		require
			fs_not_void: fs /= Void	
		do
			Result := not quantities (fs).is_empty
		end

	is_cached_quantities (fs: STRING): BOOLEAN
			-- Are quantities for field set `fs' currently cached?
		require
			fs_not_void: fs /= Void	
		do
			if cached_quantities /= Void then
				cached_quantities.search (fs)
				Result := cached_quantities.found
			end
		end

	is_rotatable (fs: STRING): BOOLEAN
			-- Is vendor geometry represented by `Current'
			-- rotatable for field set `fs'?
		require
			fs_not_void: fs /= Void	
		do
			Result := has_quantities (fs) and then quantities (fs).first.is_rotatable
		end
	
	die_type: STRING
			-- Die type for `Current'
		local
			type: DIE_TYPE_VALUES
		do
			if external_files then
				Result := type.Vendor_ghost
			else
				Result := type.Vendor_geometry
			end
		end

	non_null_size_x: like size_x
			-- Value of `size_x', with error check 
			-- for nullness
		do
			check_not_null ("size_x", is_null_size_x, Current)
			Result := size_x
		end

	non_null_size_y: like size_y
			-- Value of `size_y', with error check 
			-- for nullness
		do
			check_not_null ("size_y", is_null_size_y, Current)
			Result := size_y
		end

	non_null_design_techn: like design_techn
			-- Value of `design_techn', with error check 
			-- for nullness
		do
			check_not_null ("design_techn", is_null_design_techn, Current)
			Result := design_techn
		end

	filename_has_part_id: BOOLEAN
			-- Does `geometry_filename' include
			-- a part ID substitution variable?
		local
			val: SUBSTITUTION_PARAMETER_VALUES
		do
			Result := geometry_filename.substring_index (val.Subst_var_partno, 1) > 0
		end

feature	-- geometry_structures access

	structure_by_role(a_role: STRING): GEOMETRY_STRUCTURES_ROW
		-- find geometry_structure with geometry_role = a_role
	local strus: like structures
	do  from  strus := structures;
		  strus.start
	    until strus.after or Result /= Void
	    loop  if equal(strus.item.geometry_role, a_role) then
			Result := strus.item
		  end;
		  strus.forth
	    end
	end;

	structure_by_name(a_name: STRING): GEOMETRY_STRUCTURES_ROW
		-- find geometry_structure with structure_name = a_name
	local strus: like structures
	do  from  strus := structures;
		  strus.start
	    until strus.after or Result /= Void
	    loop  if equal(strus.item.structure_name, a_name) then
			Result := strus.item
		  end;
		  strus.forth
	    end
	end;

	structures: LIST[GEOMETRY_STRUCTURES_ROW]
		-- all the geometry_sructures for my geometry_id
	local r: GEOMETRY_STRUCTURES_ROW; b: DB_BIND_S;
	do  if cached_structures = Void then
		create r.make_template;
		create b.make(geometry_id);
		cached_structures := r.all_rows_filtered(connection, "geometry_id = :1", b)
	    end;
	    Result := cached_structures
	end;

feature	-- CSR/SealRing synthesis

	SealRingWidth: DOUBLE
		-- width (in microns) of the seal ring that will be built
	local grolv: GEOMETRY_ROLE_VALUES; stru: GEOMETRY_STRUCTURES_ROW
		orient: INTEGER
	do
	    stru :=structure_by_role(grolv.sr_segment)
	    orient := stru.drawn_orientation
	    if (orient \\ 180) = 0 then
		Result := stru.bounding_box.size_x
	    else
		Result := stru.bounding_box.size_y
	    end
	end;

feature {NONE} -- elements of build_CSR_structure

	CSR_corner_size: DOUBLE
		-- size of CSR_CORNER (which must be square)

feature -- Modification

	uncache_fixed_locations
			-- Clear all fixed locations from cache
		do
			cached_fixed_locations := Void
		end

	uncache_quantities
			-- Clear all quantities from cache
		do
			cached_quantities := Void
		end

	uncache_structures
			-- Clear all structures from cache
		do
			cached_structures := Void
		end

feature {NONE} -- Implementation

	fetch_fixed_locations (fs: STRING)
			-- Fetch fixed_locations for `Current'
			-- and field set `fs'
		require
			fs_not_void: fs /= Void
		local
			row: GEOMETRYXFIXED_LOCATIONS_ROW
			list: LIST [GEOMETRYXFIXED_LOCATIONS_ROW]
		do
			create row.make_template
			list := row.rows_for_id_and_field_set (geometry_id, fs, connection, expected_count.At_least_zero)
			cached_fixed_locations.put (list, fs)
		end
		
	fetch_quantities (fs: STRING)
			-- Fetch quantities for `Current'
			-- and field set `fs'
		require
			fs_not_void: fs /= Void
		local
			row: GEOMETRY_QUANTITIES_ROW
			list: LIST [GEOMETRY_QUANTITIES_ROW]
		do
			create row.make_template
			list := row.rows_for_id_and_field_set (geometry_id, fs, connection, expected_count.At_most_one)
			cached_quantities.put (list, fs)
		end
		
	cached_fixed_locations: HASH_TABLE [LIST [GEOMETRYXFIXED_LOCATIONS_ROW], STRING]
			-- Table of cached fixed locations, indexed
			-- by field set name
		
	cached_quantities: HASH_TABLE [LIST [GEOMETRY_QUANTITIES_ROW], STRING]
			-- Table of cached quantities, indexed
			-- by field set name
		
	cached_structures: like structures
			-- list of cached geometry_structures

	wt_fs_ext_filter: STRING
			-- Filter on wafer techn, field set and "external
			-- to payload"
		do
			Result := 
				"wafer_techn = :1 and %N%
				% geometry_usage = :3 and %N%
				% external_to_payload = :4 and %N%
				% (geometry_id in (select geometry_id from geometry#fixed_locations where field_set_name = :2) or %N%
				%  geometry_id in (select geometry_id from geometry_quantities where field_set_name = :2))"
		ensure
			result_not_void: Result /= Void
		end

end
