
note

	author: "David Hollenberg"
	date: "Fri Feb 27 10:32:55 PST 2004"

class IC_FAB_INSTANCE_PROCESSES_ROW
inherit
	IC_FAB_INSTANCE_PROCESSES_RAW
	ROW_COUNT_CHECK_ACCESS
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key

feature -- Collections

	rows_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All fab instances and processes ordered for
			-- for run `run', ordered by m_fab_no and
			-- process name
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "m_fab_no in (select m_fab_no from ic_fab_instances where run_id = :1)", b, "m_fab_no, wafer_process")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	multiple_process_rows_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All rows which have multiple processes
			-- ordered on run `run'
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "(m_fab_no, 0) in (select ifip.m_fab_no, decode(count(*), 1, 1, 0) from ic_fab_instances ifi, ic_fab_instance_processes ifip where ifip.m_fab_no = ifi.m_fab_no and ifi.run_id = :1 group by ifip.m_fab_no)", b, "m_fab_no, wafer_process")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	fab_instance: IC_FAB_INSTANCES_ROW
			-- Fab instance for `Current'
		do
			if not is_cached_fab_instance then
				fetch_fab_instance
			end
			Result := cached_fab_instance
		end

	is_cached_fab_instance: BOOLEAN
			-- Is fab instance cached?
		do
			Result := cached_fab_instance /= Void
		end

feature -- Modification

	uncache_fab_instance
			-- Remove fab instance from cache
		do
			cached_fab_instance := Void
		ensure
			fab_instance_not_cached: not is_cached_fab_instance
		end

feature {NONE} -- Implementation

	fetch_fab_instance
			-- Fetch fab instance for `Current' and put it
			-- in `cached_fab_instance'
		do
			create cached_fab_instance.make_by_key (connection, option.Old_row, m_fab_no)
		end

	cached_fab_instance: like fab_instance
			-- Fab instance for `Current', cached in memory

end
