
note

	author: "Generated"
	date: "Fri Feb 27 10:32:55 PST 2004"

class IC_FAB_INSTANCES_ROW
inherit
	IC_FAB_INSTANCES_RAW

	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end

	MOSIS_PROD_VALUES
		export {NONE} all end;

	ROW_COUNT_CHECK_ACCESS

	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
create
	make_template, make_null, make_by_key

feature

	is_on_run: BOOLEAN
		-- am I actually on a run yet (even if not visible)?
		-- if not locked onto run, then, no, I'm not yet on a run
	do  Result := not (is_null_run_id or is_null_date_locked)
	end;

	has_fab_id: BOOLEAN
		-- is_on_run and die_id assigned
	do  Result := not (is_null_run_id or is_null_die_id or is_null_date_locked)
	end;

	is_assigned: BOOLEAN
			-- Is `Current' assigned to a run
			-- (even if it is not locked)?
		do  
			Result := not is_null_run_id
		end

	is_assigned_to_run (a_run_id: STRING): BOOLEAN
			-- Is `Current' assigned to run `a_run_id'?
			-- (even if it is not locked)?
		do  
			Result := is_assigned and then run_id.is_equal (a_run_id)
		end

	can_be_assigned_to_run (s: STRING): BOOLEAN
			-- Can `Current' be assigned to run `s'?
			-- True if unassigned or if already assigned to `s'
		do
			Result := is_null_run_id or run_id.is_equal (s)
		end

	is_frozen: BOOLEAN
		-- am I frozen (no changes allowed)
	do  Result := not is_null_date_locked
	end;

	was_locked_after_date (a_date: DB_DATE): BOOLEAN
			-- Was `Current' locked after `a_date'?
		do  
			Result := not is_null_date_locked and then a_date < date_locked
		end

	fab_id: STRING
	require is_on_run	-- (weaker than just has_fab_id)
	do  Result := su.string_of_2(run_id, die_id)
	end;

	funding_po_id: STRING
		-- the auth_id to use in find_funding_alloc
	do  if is_null_approved_funding_po_id
	     then Result := submitted_funding_po_id
	     else Result := approved_funding_po_id
	    end
	end;

	ordered_wafer_process: STRING
			-- The ordered wafer process (empty string if none,
			-- first process name in alphabetical order if
			-- multiple ordered processes)
		require 
			is_assigned: is_assigned
		local
			procs: LIST [STRING]
		do 
			procs := ordered_wafer_processes
			if not procs.is_empty then
				Result := procs.first
			else
				Result := ""
			end
		end

	ordered_wafer_processes: LIST [STRING]
			-- List of wafer processes for `Current'
		require 
			is_assigned: is_assigned
		local 
			b: DB_BIND_I
		do  
			create b.make (m_fab_no);
	    		Result := connection.selected_strings (
				"wafer_process",
				"ic_fab_instance_processes",
				"m_fab_no = :1",
				"wafer_process", False, b)
		end

	is_customer_project: BOOLEAN
			-- Is `Current' a customer project?
		local
			type: IC_FAB_INSTANCE_TYPE_VALUES
		do  
			Result := ic_fab_instance_type.is_equal (type.Customer_fab)
		end

	is_test_vehicle: BOOLEAN
			-- Is `Current' a customer project?
		local
			type: IC_FAB_INSTANCE_TYPE_VALUES
		do  
			Result := ic_fab_instance_type.is_equal (type.Test_vehicle)
		end

	tiered_fab_instances: LIST [like Current]
			-- List of fab instances for tiers, in tier_no
			-- order (1 to N)
		require 
			is_3dic_parent: cust_ic_design.is_3dic_parent
		do
			Result := cust_ic_design.tiered_fab_instances
		end

	has_shipments: BOOLEAN
			-- Does work order for `Current' have any
			-- shipments?
		do
			Result := work_order.has_shipments
		end

	is_overhead_viable (a_style: STRING): BOOLEAN
			-- Can we generate overhead in `a_style'?
		require
			a_style_not_void: a_style /= Void
		do
			Result := cust_ic_design.is_overhead_viable (a_style)
		end

	is_3dic_tier: BOOLEAN
			-- Is this a tiered element of a 3DIC?
		do  
			Result := cust_ic_design.is_3dic_tier
		end

	is_3dic_parent: BOOLEAN
			-- Is this a 3DIC parent (the 3DIC itself)?
		do  
			Result := cust_ic_design.is_3dic_parent
		end

	is_3dic_element: BOOLEAN
			-- Is this either part of a 3DIC?
		do  
			Result := cust_ic_design.is_3dic_element
		end

	tier_no_3dic: INTEGER
			-- Tier number for fab instance
		require
			is_3dic_tier: is_3dic_tier
		do  
			Result := cust_ic_design.tier_no_3dic
		end

	nth_tier_fab_instance (a_tier_no: INTEGER): like Current
			-- Fab instance for `a_tier_no'
		require 
			is_3dic_parent: cust_ic_design.is_3dic_parent
		local
			list: like tiered_fab_instances
			ifi: like Current
		do  
			list := tiered_fab_instances
			from
				list.start
			until
				list.after or Result /= Void
			loop
				ifi := list.item
				if ifi.tier_no_3dic = a_tier_no then
					Result := ifi
				end
				list.forth
			end
			if Result = Void then
				handle_hard_error ("cannot find fab instance for tier " + a_tier_no.out + " for parent m_fab_no " + m_fab_no.out)
			end
		end

feature -- run_type_requested stuff

	rtr_word: STRING
		-- the run_type_word for the run_type_requested
	local b: DB_BIND_S
	do  create b.make(run_type_requested)
	    Result := connection.safe_selected_string
				("run_type_word", "run_types",
				 "run_type = :1", b)
	end;

	rtr_is_shared: BOOLEAN
	do  Result := equal(run_type_requested, rtype_is_shared)
	end;

	rtr_is_dedicated: BOOLEAN
	do  Result := equal(run_type_requested, rtype_is_dedicated)
	end;

	rtr_is_trusted_foundry: BOOLEAN
	do  Result := equal(run_type_requested, rtype_is_trusted_foundry)
	end;

	rtr_is_development: BOOLEAN
	do  Result := equal(run_type_requested, rtype_is_development)
	end;

	awaiting_target_sched_run_no: BOOLEAN
		-- are we ready to find a sched_run_no, and lack one
	do  Result := is_null_target_sched_run_no
	end;

feature -- Queries

	fab_instances_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All fab instances for run `run', 
			-- ordered by design number
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "run_id = :1", b, "design_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	cust_fab_instances_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All fab instances for run `run'
			-- that are for customer projects
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			type: IC_FAB_INSTANCE_TYPE_VALUES
		do
			create b.make (run, type.Customer_fab)
			Result := all_rows_filtered (conn, "run_id = :1 and ic_fab_instance_type = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	cancelled_fab_instances_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All fab instances for run `run'
			-- that are for customer projects that
			-- have been cancelled or that are
			-- for refabs that have been cancelled,
			-- order by design number
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			type: IC_FAB_INSTANCE_TYPE_VALUES
		do
			create b.make (run, type.Customer_fab)
			Result := all_rows_filtered_ordered (conn, "run_id = :1 and ic_fab_instance_type = :2 and (date_invalidated is not null or 'Y' = (select design_cancelled from cust_ic_designs where design_no = ifi.design_no))", b, "design_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	itar_fab_instances_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All fab instances ITAR projects
			-- on run `run'
		require
			run_not_void: run /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_not_void: range /= Void
		local
			b: DB_BIND_SS
			type: EXPORT_CONTROL_TYPE_VALUES
		do
			create b.make (run, type.Itar)
			Result := all_rows_filtered (conn, "run_id = :1 and exists (select design_no from cust_ic_designs where design_no = ifi.design_no and export_control_type = :2)", b)
			check_row_count (Result, range)
		ensure
			result_not_void: Result /= Void
		end

	row_for_run_and_die (run, die: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): like Current
			-- Fab instance for run `run' and die `die' or
			-- Void if none
		require
			run_exists: run /= Void
			die_exists: die /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			list: LIST [like Current]
		do
			create b.make (run, die)
			list := all_rows_filtered (conn, "run_id = :1 and die_id = :2", b)
			check_row_count (list, range)
			if not list.is_empty then
				Result := list.first
			end
		end

	rows_for_design_no (dno: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Fab instances for design number `dno'
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (dno)
			Result := all_rows_filtered (conn, "design_no = :1", b)
			check_row_count (Result, range)
		end

	rows_on_production (prod_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Fab instances for work orders for
			-- production `prod_id'
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (prod_id)
			Result := all_rows_filtered (conn, "m_fab_no in (select m_fab_no from cust_wos where production_id = :1)", b)
			check_row_count (Result, range)
		end

	multiple_process_rows_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Fab instances for customer projects for run
			-- `run' that have more than one (or that have
			-- zero) processes ordered, ordered by design
			-- number
		require
			run_not_void: run /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			type: IC_FAB_INSTANCE_TYPE_VALUES
		do
			create b.make (run, type.Customer_fab)
			Result := all_rows_filtered_ordered (conn, "run_id = :1 and ic_fab_instance_type = :2 and 1 <> (select count(*) from ic_fab_instance_processes where m_fab_no = ifi.m_fab_no)", b, "design_no")
			check_row_count (Result, range)
		end

	rows_targeted_to_sched_run (a_sched_run_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All fab instances targeted to sched_run_no
			-- `a_sched_run_no'
		require
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (a_sched_run_no)
			Result := all_rows_filtered_ordered (conn, "target_sched_run_no = :1", b, "design_no")
			check_row_count (Result, range)
		ensure
			result_not_void: Result /= Void
		end

feature -- Associated rows lookup

	cust_ic_design: like anchored_cust_ic_design
			-- Customer IC Design for this Fabrication Instance
		do
			if cached_cust_ic_design = Void then
				create cached_cust_ic_design.make_by_key (connection, option.Old_row, design_no)
			end
			Result := cached_cust_ic_design
		ensure
			result_not_void: Result /= Void
		end

	uncache_cust_ic_design
			-- Remove customer IC design from cache, if present
		do
			cached_cust_ic_design := Void
		ensure
			cust_ic_design_not_cached: not is_cached_cust_ic_design
		end

	is_cached_cust_ic_design: BOOLEAN
			-- Is `cust_ic_design' currently cached?
		do
			Result := cached_cust_ic_design /= Void
		end

	explicit_options: LIST [IC_FAB_INSTANCE_OPTIONS_ROW]
			-- Explicitly requested options for fab instance
			-- (does not include options determined by layers
			-- present)
		do
			if cached_explicit_options = Void then
				fetch_explicit_options
			end
			Result := cached_explicit_options
		ensure
			result_not_void: Result /= Void
		end

	uncache_explicit_options
			-- Remove explicit options from cache, if present
		do
			cached_explicit_options := Void
		ensure
			explicit_options_not_cached: not is_cached_explicit_options
		end

	is_cached_explicit_options: BOOLEAN
			-- Is `explicit_options' currently cached?
		do
			Result := cached_explicit_options /= Void
		end

	production_work_order (prod_id: STRING): like anchored_work_order
			-- Work order for this fab instance and
			-- production ID `prod_id' (or Void if none)
			-- or unique work order for fab instance if
			-- `prod_id' is Void.  Error if more than one
		local
			list: LIST [like anchored_work_order]
		do
			if prod_id /= Void then
				list := work_orders_for_production (prod_id, expected_count.At_most_one)
				if not list.is_empty then
					Result := list.first
				end
			else
				Result := work_order
			end
		ensure
			result_not_void: Result /= Void
		end

	work_order: like anchored_work_order
			-- Work order for this fab instance
			-- (error if more than one)
		do
			if cached_work_order = Void then
				cached_work_order := fetch_work_orders (expected_count.exactly_one).first
			end
			Result := cached_work_order
		ensure
			result_not_void: Result /= Void
		end

	uncache_work_order
			-- Remove work order from cache, if present
		do
			cached_work_order := Void
		ensure
			work_order_not_cached: not is_cached_work_order
		end

	is_cached_work_order: BOOLEAN
			-- Is `work_order' currently cached?
		do
			Result := cached_work_order /= Void
		end

	work_orders: DYNAMIC_LIST[like anchored_work_order]
			-- All work orders for this fab instance (not cached)
		do
			Result := fetch_work_orders (expected_count.at_least_one)
		ensure
			result_not_empty: not Result.is_empty
		end

	cust_acct_nos: LIST[INTEGER]
			-- All distinct cust_acct_no values in our cust_wos
		local b: DB_BIND_I
		do
			create b.make(m_fab_no)
			Result := connection.selected_integers
					("cust_acct_no", "cust_wos", "m_fab_no = :1",
					 "cust_acct_no", true, b)
		end

	replies: LIST [M_REPLIES_ROW]
			-- Replies to requests for this fab instance,
			-- in reverse order of date processed
		local
			row: M_REPLIES_ROW
		do
			if cached_replies = Void then
				create row.make_template
				cached_replies := row.replies_for_fab_no (m_fab_no, connection, expected_count.At_least_zero)
			end
			Result := cached_replies
		ensure
			result_not_void: Result /= Void
		end

	uncache_replies
			-- Remove replies from cache, if present
		do
			cached_replies := Void
		ensure
			replies_not_cached: not is_cached_replies
		end

	is_cached_replies: BOOLEAN
			-- Is `replies' currently cached?
		do
			Result := cached_replies /= Void
		end

	the_die: DIES_ROW
			-- die for this fab instance
			-- (sorry about the name, but I couldn't call this routine 'die')
		do
			if is_null_die_id then
				cached_die := Void
			elseif cached_die = Void then
				create cached_die.make_by_key(connection, option.old_row, run_id, die_id)
			end;
			Result := cached_die
		end;

	uncache_die
		do
			cached_die := Void
		end

	is_cached_die: BOOLEAN
		do
			Result := cached_die /= Void
		end;

feature	-- layer and option testing

	is_layer_present(a_layer: STRING): BOOLEAN
		-- is this layer present (in my cust_ic_design)?
	do  Result := cust_ic_design.layout_of_interest.is_layer_present(a_layer)
	end;

	are_2_layers_present(a_layer, b_layer: STRING): BOOLEAN
		-- are these layers both present?
	do  Result := is_layer_present(a_layer) and
		      is_layer_present(b_layer)
	end;

	is_option_present(an_option: STRING): BOOLEAN
		-- is an_option present in explicit_options?
	local xopt_list: like explicit_options
	do  xopt_list := explicit_options
	    from  xopt_list.start
	    until xopt_list.after or Result
	    loop  Result := equal(an_option, xopt_list.item.option_id)
		  xopt_list.forth
	    end
	end;

feature {NONE} -- Implementation of associated rows lookup

	cached_cust_ic_design: like cust_ic_design

	cached_replies: like replies

	anchored_work_order: CUST_WOS_ROW
	do
	ensure Result = Void
	end;

	anchored_cust_ic_design: CUST_IC_DESIGNS_ROW
			-- For anchoring only
		do
		ensure 
			result_void: Result = Void
		end

	cached_work_order: like anchored_work_order

	cached_explicit_options: like explicit_options
			-- Explicitly requested options for fab instance,
			-- cached in memory

	cached_die: like the_die

	fetch_work_orders(a_range: ITEM_COUNT_RANGE): DYNAMIC_LIST[like anchored_work_order]
		local
			row: like anchored_work_order
		do
			create row.make_template
			Result := row.orders_for_fab_no (m_fab_no, connection, a_range)
		end

	work_orders_for_production (a_prod_id: STRING a_range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like anchored_work_order]
		local
			row: like anchored_work_order
		do
			create row.make_template
			Result := row.orders_for_fab_no_and_prod (m_fab_no, a_prod_id, connection, a_range)
		end

	fetch_explicit_options
		local
			row: IC_FAB_INSTANCE_OPTIONS_ROW
		do
			create row.make_template
			cached_explicit_options := row.options_for_fab_no (m_fab_no, connection, expected_count.At_least_zero)
		end

end
