
note

	author: "Generated"
	date: "Fri Feb 27 10:32:55 PST 2004"

class IC_PACKAGINGS_ROW
inherit
	IC_PACKAGINGS_RAW
	ROW_COUNT_CHECK_ACCESS
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end

	MOSIS_PROD_VALUES
	export {NONE} all
	end;

	STRING_UTILITIES_ACCESS
	export {NONE} all
	end;

create
	make_template, make_null, make_by_key

feature	-- handle NONE and DEFAULT package_requested

	req_is_none: BOOLEAN
	do  Result := equal(package_requested, package_is_none)
	end;

	req_is_default: BOOLEAN
	do  Result := equal(package_requested, package_is_default)
	end;

	have_a_wiring: BOOLEAN
		-- do we have a computed wiring in place?
	do  Result := not my_wiring.is_empty
	end;

feature	-- quantity_packaged_requested

	qty_is_none: BOOLEAN
		-- quantity_packaged_requested = 0 (none)
	do  Result := (not is_null_quantity_packaged_requested) and
			(quantity_packaged_requested = 0)
	end;

feature	-- my package (if any) and my bonding (wires)

	my_package: PACKAGES_ROW
		-- Void iff package_id is Void
	local r: PACKAGES_ROW; b: DB_BIND_S;
	do  if cached_package = Void or else
	       not equal(cached_package.package_id, package_id)
	     then if is_null_package_id
		   then cached_package := Void
		   else create r.make_template
			create b.make(package_id);
			cached_package := r.all_rows_filtered
					(connection,
					 "package_id = :1", b).first
		  end
	    end;
	    Result := cached_package
	end;

	set_my_package(a_pkg: PACKAGES_ROW)
		-- save db fetch (MUST be set if Void connection)
	require exists: a_pkg /= Void
		mine: equal(a_pkg.package_id, package_id)
	do  cached_package := a_pkg
	end;

	my_wiring: LIST[IC_PACKAGING_WIRES_ROW]
	do  Result := my_mwiring
	end;

	my_mwiring: LIST[IC_PACKAGING_WIRES_MROW]
		-- straight out of the db
	local r: IC_PACKAGING_WIRES_MROW; b: DB_BIND_I;
	do  if cached_wiring /= Void
	     then	-- have it already
	    elseif connection /= Void
	     then create r.make_template;
		  create b.make(ic_packaging_no);
		  cached_wiring := r.all_rows_filtered_ordered
					(connection, 
					 "ic_packaging_no = :1", b,
					 "wire_no")
	     else -- guess there just aren't any wires!
		  create {LINKED_LIST[IC_PACKAGING_WIRES_MROW]}
			cached_wiring.make
	    end;
	    Result := cached_wiring
	ensure Result /= Void
	end;

	needs_manual_bonding: BOOLEAN
			-- Does the the final bonding diagram have to
			-- be produced manually, by incorporating a
			-- picture of the chip into another file?
	do  if not is_null_package_id then
		Result := my_package.leadframe
	    end
	end;

	needs_individual_diagram: BOOLEAN
			-- Does packaging need an individual PostScript
			-- and PDF file generated?
		do  
			Result := customer_providing_diagram and (not my_package.leadframe or have_a_wiring)
			-- Do it even if package doesn't have connections
			-- (requested by Robert)
			-- and my_package.have_known_connections
		end

	needs_final_diagram: BOOLEAN
			-- Does packaging need a final (vendor) diagram
			-- generated for the customer?
		do  
			Result := True
		end

feature	-- ifi and cid stuff

	die_id: STRING
			-- Die id for this packaging
		do  if not is_null_m_fab_no then
			Result := fab_instance.die_id
		    else
			Result := "??"
		    end
		ensure 
			result_not_void: Result /= Void
		end

	run_id: STRING
			-- Run id for this packaging
		do  if not is_null_m_fab_no then
			Result := fab_instance.run_id
		    else
			Result := "????"
		    end
		ensure 
			result_not_void: Result /= Void
		end

	design_no: INTEGER
			-- Design No for this packaging
	do  if not is_null_m_fab_no then
		Result := fab_instance.design_no
	    end;
	end;

	fab_instance: IC_FAB_INSTANCES_ROW
			-- Fab instance for this packaging
			-- (can be Void if Void connection AND not set)
		do  
			if cached_fab_instance = Void and (not is_null_m_fab_no) then 
				create cached_fab_instance.make_by_key (connection, option.Old_row, m_fab_no)
	    		end
	    		Result := cached_fab_instance
		ensure 
			result_not_void: is_null_m_fab_no = (Result = Void)
			result_not_empty:  Result /= Void implies (not Result.is_empty)
		end

	possible_fab_instance: IC_FAB_INSTANCES_ROW
			-- Fab instance for this packaging
			-- `Result' is empty if the fab instance disappeared
		do  
			if cached_fab_instance = Void and (not is_null_m_fab_no) then 
				create cached_fab_instance.make_by_key (connection, option.Old_no_exception_row, m_fab_no)
		    		Result := cached_fab_instance
				if Result.is_empty then
					cached_fab_instance := Void
				end
			else
		    		Result := cached_fab_instance
	    		end
		ensure 
			result_not_void: is_null_m_fab_no = (Result = Void)
			good_result_cached: Result /= Void and then (not Result.is_empty) implies is_cached_fab_instance
		end

	fab_instance_bonding_description: STRING
			-- Description of bonding suitable for error
			-- messages (includes info from fab instance)
		do  if not is_null_m_fab_no then
			Result := fab_instance.die_id + "   " + bonding_name + "   " + fab_instance.design_no.out
		    else
			Result := "??   " + bonding_name + "   0"
		    end
		ensure 
			result_not_void: Result /= Void
		end

	uncache_fab_instance
			-- Remove fab instance from cache, if present
		do  
			cached_fab_instance := Void
		ensure 
			fab_instance_not_cached: not is_cached_fab_instance
		end

	is_cached_fab_instance: BOOLEAN
			-- Is `fab_instance' currently cached?
		do  
			Result := cached_fab_instance /= Void
		end

	select_fab_instance(master: LIST[like cached_fab_instance])
	require master /= Void
	do  from  master.start; uncache_fab_instance
	    until master.after or is_cached_fab_instance
	    loop  if master.item.m_fab_no = m_fab_no
		   then cached_fab_instance := master.item
		  end;
		  master.forth
	    end
	end;

	set_fab_instance(ifi: like cached_fab_instance)
	require	exists: ifi /= Void
		matches: ifi.m_fab_no = m_fab_no
	do  cached_fab_instance := ifi
	end;

	cust_ic_design: CUST_IC_DESIGNS_ROW
	do  if cached_cust_ic_design = Void and connection /= Void
	     then create cached_cust_ic_design.make_by_key
				(connection, option.old_row, design_no)
	    end;
	    Result := cached_cust_ic_design
	end;

	set_cust_ic_design(cid: like cached_cust_ic_design)
	require	exists: cid /= Void
		matches: cid.design_no = design_no
	do  cached_cust_ic_design := cid
	end;

feature	-- check bonding and friends

	bonding_name: STRING
		-- the package_id plus the bonding_id
	do  if is_null_package_id
	     then Result := package_is_none
	     else Result := package_id
	    end;
	    if not is_null_bonding_id
	     then Result := su.string_of_3(Result, "-", bonding_id)
	    end
	end;

	bonding_diagram_status(post_dc: BOOLEAN): STRING
		-- string describing state of the bonding diagram
		-- post_dc is true iff we are post design check
		-- (pre-dc changes message re generated diagrams)
	local dbnds: STRING
	do  if qty_is_none or is_null_package_id
	     then Result := Void
	    elseif customer_providing_diagram
	     then if (not is_null_date_diagram_received)
		   then Result := su.string_of_2(
				"bonding diagram received on ",
				date_diagram_received.to_date_string)
		  elseif have_generated_final_pdf
		   then Result := "waiting for your bonding diagram"
		   else Result := "blank bonding diagram not yet released"
		  end;
		  dbnds := downbond_location_phrase;
		  if su.not_empty(dbnds)
		   then Result :=su.string_of_3(Result, "; downbonds: ", dbnds)
		  end;
	    elseif have_a_wiring
	     then if is_custom_wiring
		   then Result := "custom bonding diagram"
		   else Result := "MOSIS generated bonding diagram"
		  end
	    elseif not post_dc
	     then Result := "MOSIS to generate bonding diagram"
	    elseif can_check_bonding
	     then Result := "bonding diagram generation failed"
	     else Result := "bonding diagram cannot be generated by MOSIS"
	    end
	end;

	can_check_bonding: BOOLEAN
		-- is this a package for which we CAN check the bonding
	local pkg: PACKAGES_ROW
	do  pkg := my_package;
	    Result := pkg /= Void and then
			pkg.have_known_connections
	end;

	check_bonding: BOOLEAN
		-- should the design check server try to check_bonding?
	do  Result := (not customer_providing_diagram) and
		      (not qty_is_none) and
		      (can_check_bonding)
	end;

	can_generate_customer_pdf: BOOLEAN
		-- can we generate a diagram for the customer?
	do  Result := (my_package /= Void) and then
		      (my_package.have_known_connections)
	end;

	should_generate_customer_pdf: BOOLEAN
		-- we should if we can AND we haven't
	do  Result := can_generate_customer_pdf and
			cid_files.is_empty
	end;

	have_generated_final_pdf: BOOLEAN
		-- have we already generated the FINAL bonding diagram?
	local cidfs: like cid_files
	do  cidfs := cid_files
	    from  cidfs.start
	    until cidfs.after or Result
	    loop  Result := equal(cidfs.item.file_role, fir_is_f_bnd_diagram)
		  cidfs.forth
	    end
	end;

	ignore_the_wires: BOOLEAN
		-- should we ignore the wires in generating a diagram
		-- (because they are probably an obsolete artifact)
	do  Result := customer_providing_diagram and
			(not is_custom_wiring)
	end;

	is_approved: BOOLEAN
		-- have we approved the bonding?
	do  Result := not is_null_date_diagram_approved
	end;

	can_sell_packaged: BOOLEAN
		-- can we write up this packaging item
	do  Result := (not is_null_package_id) and
		      (customer_providing_diagram or is_approved)
	end;

	diagram_type: STRING
		-- "blank" or "custom" or "MOSIS-generated"
	do  if not have_a_wiring
	     then Result := "blank"
	    elseif is_custom_wiring
	     then Result := "custom"
	     else Result := "MOSIS-generated"
	    end
	end;

	rotation_text: STRING
		-- text describing design_package_rotation
		-- (null design_package_rotation is treated as a 0)
	do  if is_null_design_package_rotation or design_package_rotation = 0 then
		Result := "None"
	    elseif design_package_rotation = 90 then
		Result := "90 degrees CW"
	    elseif design_package_rotation = 180 then
		Result := "180 degrees"
	    elseif design_package_rotation = 270 then
		Result := "270 degrees CW"
	    end
	end;

feature	-- file names associated with my diagram(s)

	final_diagram_name(suffx, ext: STRING): STRING
		-- suffx is "final" or "blank"; ext may be empty
	do  
		Result := final_diagram_name_for_production (run_id, suffx, ext)
	end;

	final_diagram_rundir_name(prod_id, ext: STRING): STRING
			-- Individual diagram name in run directory
			-- for production `prod_id'
		do  
			if have_a_wiring then
				Result := final_diagram_name_for_production (prod_id, "final", ext)
	    		else
				Result := final_diagram_name_for_production (prod_id, "blank", ext)
	    		end
		end

	final_diagram_presentation_name: STRING
		-- doc server presentation name for the final diagram
	do  Result := final_diagram_name("final", Void)
	end;

feature	-- associated cust_ic_design_file(s) (shouldn't be more than 1)

	cid_files: LIST[like anchored_cid_file]
	local r: like anchored_cid_file; b: DB_BIND_I;
	do  if cached_cid_files /= Void
	     then
	    elseif connection /= Void
	     then create r.make_template;
		  create b.make(ic_packaging_no)
		  cached_cid_files := 
			r.all_rows_filtered
				(connection, "ic_packaging_no = :1", b)
	     else create {LINKED_LIST[like anchored_cid_file]}
			cached_cid_files.make
	    end;
	    Result := cached_cid_files
	end;

	select_cid_files(master: LIST[like anchored_cid_file])
	do  create {LINKED_LIST[like anchored_cid_file]}
		  cached_cid_files.make;
	    from  master.start
	    until master.after
	    loop  if master.item.ic_packaging_no = ic_packaging_no
		   then cached_cid_files.extend(master.item)
		  end;
		  master.forth
	    end
	end;

feature {NONE} -- associated cust_ic_design_files stuff

	anchored_cid_file: CUST_IC_DESIGN_FILES_ROW
	do
	ensure Result = Void
	end;

	cached_cid_files: LIST[like anchored_cid_file]

feature	--- max die size stuff

	max_die_size: MAX_DIE_SIZES_ROW
		-- MAX_DIE_SIZES is the view that gives the maximum die
		-- size for this ic_packaging, considering both the
		-- package cavity limits and the downbonded sides
	require unchanged: not is_changed;	-- downbond_location_reqd
						-- must be current in the db
	local r: MAX_DIE_SIZES_ROW; b: DB_BIND_I
	do  if is_null_package_id or connection = Void
	     then cached_max_die_size := Void
	    elseif cached_max_die_size = Void or else
		   not equal(package_id, cached_max_die_size.package_id)
	     then create r.make_template;
		  create b.make(ic_packaging_no)
		  cached_max_die_size := 
			r.all_rows_filtered
				(connection, 
				 "ic_packaging_no = :1", b).first
			-- max_die_sizes has 1 row for every ic_packaging
			--  with a non-null package_id
	    end;
	    Result := cached_max_die_size
	end;

	max_die_in_um: CONST_RECTANGLE
	require have_pkg: not is_null_package_id
	local mdsiz: like max_die_size
	do  mdsiz := max_die_size;
	    create Result.make_by_size_and_center
				(mdsiz.max_die_size_x,
				 mdsiz.max_die_size_y,
				 0, 0)
	end;

feature	-- downbond_location stuff

	downbond_location_id: STRING
		-- mine or my package's if mine is null
	do  if not is_null_downbond_location_reqd
	     then Result := downbond_location_reqd
	    elseif my_package /= Void
	     then Result := my_package.downbond_location_default
	     else Result := Void
	    end
	end;

	downbond_location_phrase: STRING
		-- just mine (don't go to the package)
	local b: DB_BIND_S;
	do  if (not is_null_downbond_location_reqd) and connection /= Void
	     then create b.make(downbond_location_reqd);
		  Result := connection.selected_string
			("downbond_location_phrase",
			 "downbond_locations",
			 "downbond_location_id = :1", b)
	    end
	end;

feature	{NONE} -- cached package and bonding (wires)

	cached_package: PACKAGES_ROW

	cached_wiring: LIST[IC_PACKAGING_WIRES_MROW]

	cached_fab_instance: IC_FAB_INSTANCES_ROW

	cached_cust_ic_design: CUST_IC_DESIGNS_ROW

	cached_max_die_size: MAX_DIE_SIZES_ROW

feature -- Properties

	rows_for_complex_criteria (dno, run, prod, die, pkg, bnd: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Packagings constrained by design number `dno', 
			-- `run', `prod', `die', `pkg' and bonding id `bnd'.
			-- For values that are empty, no constraint is
			-- applied
		require
			dno_exists: dno /= Void
			run_exists: run /= Void
			prod_exists: prod /= Void
			die_exists: die /= Void
			pkg_exists: pkg /= Void
			bnd_exists: bnd /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SSSSSS
			where: STRING
		do
			create b.make (dno, run, prod, die, pkg, bnd)
			create where.make (100)
			where.append ("m_fab_no in %N")
			where.append ("(select m_fab_no from ic_fab_instances ifi where 0 = 0%N")
			if dno.is_empty then
				where.append (" and :1 is null")
			else
				where.append (" and design_no = :1")
			end
			if run.is_empty then
				where.append (" and :2 is null")
			else
				where.append (" and run_id = :2")
			end
			if prod.is_empty then
				where.append (" and :3 is null")
			else
				where.append (" and exists (select 1 from cust_wos where m_fab_no = ifi.m_fab_no and production_id = :3)")
			end
			if die.is_empty then
				where.append (" and :4 is null")
			else
				where.append (" and die_id = :4")
			end
			where.append (")%N")
			if pkg.is_empty then
				where.append (" and :5 is null")
			else
				where.append (" and package_id = :5")
			end
			if bnd.is_empty then
				where.append (" and :6 is null")
			else
				where.append (" and bonding_id = :6")
			end
			Result := all_rows_filtered_ordered (conn, where, b, "m_lookup.fab_instance_fab_id(m_fab_no), package_id, bonding_id")
			check_row_count (Result, range)
		end

	rows_needing_diagram_for_pkg_lot (prod_id: STRING lot_no: INTEGER null_lot: BOOLEAN conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Packagings that need a bonding diagram for
			-- production `prod_id' on packaging lot `lot_no'
			-- (if `null_lot' is False); otherwise packagings
			-- that need a bonding diagram for `prod_id'
			-- and that are not assigned to a packaging lot
		require
			prod_id_not_void: prod_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BINDER
			where: STRING
			table: HASH_TABLE [like Current, INTEGER]
			list: LIST [like Current]
			r: like Current
		do
			if null_lot then
				create {DB_BIND_S} b.make (prod_id)
			else
				create {DB_BIND_SI} b.make (prod_id, lot_no)
			end
			
			where := "ic_packaging_no in %N%
				% (select ip.ic_packaging_no %N%
				%  from ic_packagings ip, %N%
				%       ic_fab_instances ifi, %N%
				%       cust_wos cw, %N%
				%       cust_wo_line_items cwli, %N"
			if not null_lot then
				where.append ("       run_packaging_lots rpl,%N")
			end
			where.append (
				"       m_parts mp %N%
				% where ip.m_fab_no = ifi.m_fab_no %N%
				%   and ifi.m_fab_no = cw.m_fab_no %N%
				%   and cw.m_order_no = cwli.m_order_no %N%
				%   and cwli.m_part_no = mp.m_part_no %N")
			if null_lot then
				where.append (
				"   and cwli.packaging_run_id is null %N%
				%   and cwli.packaging_lot_no is null %N")
			else
				where.append (
				"   and cwli.packaging_run_id = rpl.run_id %N%
				%   and cwli.packaging_lot_no = rpl.packaging_lot_no %N")
			end
			where.append (
				"   and cwli.ic_packaging_no = ip.ic_packaging_no %N%
				%   and cw.production_id = :1 %N%
				%   and mp.packaged = 'Y' %N")
			if not null_lot then
				where.append (
				"   and rpl.packaging_lot_no = :2 %N")
			end
			where.append (")")
			
			list := all_rows_filtered_ordered (conn, where, b, "m_lookup.fab_instance_fab_id(m_fab_no), package_id, bonding_id")
			
			-- Remove duplicates 
			
			Result := list.twin
			Result.wipe_out
			create table.make (list.count)
			from
				list.start
			until
				list.after
			loop
				r := list.item
				table.search (r.ic_packaging_no)
				if not table.found then
					Result.extend (r)
					table.put (r, r.ic_packaging_no)
				end
				list.forth
			end
			check_row_count (Result, range)
		end

feature {NONE} -- Implementation

	final_diagram_name_for_production (prod_id, suffx, ext: STRING): STRING
			-- 'Name of file for bonding diagram for
			-- production `prod_id'.  `suffx' is "final"
			-- or "blank"; ext may be empty
		do  
			Result := su.string_of_8 (prod_id, su.underscore, die_id, su.underscore, design_no.out, su.underscore, bonding_name, "_diagram_");
	    		Result := su.string_of_3(Result, suffx, ext)
	    		Result.to_lower
		end;

end
