
note

	author: "Generated"
	date: "Wed Feb 11 14:31:07 PST 2004"

class LITHO_SIZES_ROW
inherit
	LITHO_SIZES_RAW
	ROW_COUNT_CHECK_ACCESS
	NULL_VALUE_CHECK_ACCESS
	ITEM_COUNT_RANGES_ACCESS
	DB_QUERY_OPTIONS
		export {NONE} all end
	DB_OPTIONS_ACCESS
		export 
			{NONE} all 
		end


create
	make_template, make_null, make_by_key, make_detached

feature -- Properties

	external_kerf: CONST_POINT
			-- Size of the external kerf
		do  
			create Result.make_xy (external_kerf_x, external_kerf_y)
		end

	external_kerf_x: DOUBLE
			-- Size of the external kerf in x
		do  
			if not is_null_field_size_x then
				Result := field_size_x - payload_size_x
			else
				Result := 0.0
			end
		end

	external_kerf_y: DOUBLE
			-- Size of the external kerf in y
		do  
			if not is_null_field_size_y then
				Result := field_size_y - payload_size_y
			else
				Result := 0.0
			end
		end

	litho_sizes_with_name (name: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All litho sizes named `name'
		require
			name_exists: name /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (name)
			Result := all_rows_filtered (conn, "litho_size_id = :1", b)
			check_row_count (Result, range)
		end

	custom_litho_sizes_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): DYNAMIC_LIST [like Current]
			-- All litho sizes which are custom
			-- ones for `run'
		require
			run_not_void: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "run_id = :1", b, "litho_size_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	field_size: CONST_POINT
			-- Size of field in microns, in wafer coordinates
		do
			check_not_null ("field_size_x", is_null_field_size_x, Current)
			check_not_null ("field_size_y", is_null_field_size_y, Current)
			create Result.make_xy (field_size_x, field_size_y)
		ensure
			result_exists: Result /= Void
		end

	payload_size: CONST_POINT
			-- Size of payload in microns, in wafer coordinates
		do
			check_not_null ("payload_size_x", is_null_payload_size_x, Current)
			check_not_null ("payload_size_y", is_null_payload_size_y, Current)
			create Result.make_xy (payload_size_x, payload_size_y)
		ensure
			result_exists: Result /= Void
		end

	payload_center: CONST_POINT
			-- Location of center of payload in microns relative to
			-- center of field, in wafer coordinates
		do
			check_not_null ("payload_location_x", is_null_payload_location_x, Current)
			check_not_null ("payload_location_y", is_null_payload_location_y, Current)
			create Result.make_xy (payload_location_x, payload_location_y)
		ensure
			result_exists: Result /= Void
		end

	payload_box: CONST_RECTANGLE
			-- Rectangle that defines extent of payload in 
			-- microns relative to center of field, in 
			-- wafer coordinates
		do
			create Result.make_size_and_center (payload_size, payload_center)
		ensure
			result_exists: Result /= Void
		end

	merged_size: CONST_POINT
			-- Size of merged pattern file in microns, in mask
			-- coordinates
		do
			check_not_null ("merged_pattern_file_size_x", is_null_merged_pattern_file_size_x, Current)
			check_not_null ("merged_pattern_file_size_y", is_null_merged_pattern_file_size_y, Current)
			create Result.make_xy (merged_pattern_file_size_x, merged_pattern_file_size_y)
		ensure
			result_exists: Result /= Void
		end

	merged_center: CONST_POINT
			-- Location of center of merged pattern file
			-- in microns, relative to center of field in mask
			-- coordinates
		do
			check_not_null ("merged_pattern_file_location_x", is_null_merged_pattern_file_location_x, Current)
			check_not_null ("merged_pattern_file_location_y", is_null_merged_pattern_file_location_y, Current)
			create Result.make_xy (merged_pattern_file_location_x, merged_pattern_file_location_y)
		ensure
			result_exists: Result /= Void
		end

	merged_box: CONST_RECTANGLE
			-- Rectangle that defines extent of merged
			-- pattern files in microns, relative to
			-- center of field in mask coordinates
		do
			create Result.make_size_and_center (merged_size, merged_center)
		ensure
			result_exists: Result /= Void
		end


	has_left_flat: BOOLEAN
			-- Does `Current' have a flat or notch on left
			-- side of wafer?
		local
			flat: WAFER_FLAT_LOCATION_VALUES
			main_flat: STRING
		do
			main_flat := non_null_primary_flat_location
			Result := main_flat.is_equal (flat.Left) or main_flat.is_equal (flat.Unknown)
		end

	has_right_flat: BOOLEAN
			-- Does `Current' have a flat or notch on
			-- right side of wafer?
		local
			flat: WAFER_FLAT_LOCATION_VALUES
			main_flat: STRING
		do
			main_flat := non_null_primary_flat_location
			Result := main_flat.is_equal (flat.Right) or main_flat.is_equal (flat.Unknown)
		end

	has_top_flat: BOOLEAN
			-- Does `Current' have a flat or notch on top
			-- side of wafer?
		local
			flat: WAFER_FLAT_LOCATION_VALUES
			main_flat: STRING
		do
			main_flat := non_null_primary_flat_location
			Result := main_flat.is_equal (flat.Top) or main_flat.is_equal (flat.Unknown)
		end

	has_bottom_flat: BOOLEAN
			-- Does `Current' have a flat or notch on
			-- bottom side of wafer?
		local
			flat: WAFER_FLAT_LOCATION_VALUES
			main_flat: STRING
		do
			main_flat := non_null_primary_flat_location
			Result := main_flat.is_equal (flat.Bottom) or main_flat.is_equal (flat.Unknown)
		end

	non_null_wafer_type_id: like wafer_type_id
			-- Value of `wafer_type_id', with error 
			-- check for nullness
		do
			check_not_null ("wafer_type_id", is_null_wafer_type_id, Current)
			Result := wafer_type_id
		end

	non_null_primary_flat_location: like primary_flat_location
			-- Value of `primary_flat_location', with error 
			-- check for nullness
		do
			check_not_null ("primary_flat_location", is_null_primary_flat_location, Current)
			Result := primary_flat_location
		end

	wafer_diameter: DOUBLE
			-- Diameter of wafers
		do
			if wafer_type /= Void then
				Result := wafer_type.wafer_diameter
			else
				Result := 0
			end
		end

feature	-- decrypting the packing target

	target: PACKING_TARGETS_ROW
			-- Packing target info for `Current'
		do
			if cached_packing_target = Void then
				fetch_packing_target
			end
			Result := cached_packing_target
		ensure
			result_not_void: Result /= Void
		end

	uncache_packing_target
			-- Remove `target' from cache, if present
		do
			cached_packing_target := Void
		ensure
			target_not_cached: not is_cached_packing_target
		end

	is_cached_packing_target: BOOLEAN
			-- Is `target' currently cached?
		do
			Result := cached_packing_target /= Void
		end

	is_standard_reticle: BOOLEAN
			-- Is `target' a standard (non-custom) reticle?
		do
			Result := target.is_standard_reticle
		end

	is_custom_reticle: BOOLEAN
			-- Is `target' a custom reticle?
		do
			Result := target.is_custom_reticle
		end

	is_reticle: BOOLEAN 
			-- Is `target' a reticle?
		do  
			Result := target.is_reticle
		end

	is_non_reticle: BOOLEAN 
			-- Is `target' not a reticle?
		do  
			Result := not is_reticle
		end

	is_custom_target: BOOLEAN
			-- Is `target' a custom target?
		do  
			Result := target.is_custom
		end

	is_unpacked_die: BOOLEAN
			-- Does `target' indicate unpacked die?
		do 
			Result := target.is_unpacked_die
		end

feature	-- mask_sizes_row and wafer_types_row

	mask_size: MASK_SIZES_ROW
	do  if is_null_mask_size_id
	     then cached_mask_size := Void
	    elseif cached_mask_size = Void
	     then create cached_mask_size.make_by_key
				(connection, old_row, mask_size_id)
	    end;
	    Result := cached_mask_size
	end;

	wafer_type: WAFER_TYPES_ROW
	do  if is_null_wafer_type_id
	     then cached_wafer_type := Void
	    elseif cached_wafer_type = Void
	      then create cached_wafer_type.make_template;
			cached_wafer_type := 
				cached_wafer_type.wafer_types_with_name
					(wafer_type_id, connection,
					 expected_count.exactly_one).first
	    end;
	    Result := cached_wafer_type
	end;

feature	{NONE} -- Implementation

	fetch_packing_target
		do
			create cached_packing_target.make_by_key (connection, option.Old_row, packing_target)
		end

	cached_mask_size: MASK_SIZES_ROW

	cached_wafer_type: WAFER_TYPES_ROW

	cached_packing_target: like target
			-- Packing target row, cached in memory

end
