
note

	author: "Generated"
	date: "Fri Feb 27 10:31:28 PST 2004"

class M_TEST_VEHICLES_ROW
inherit
	M_TEST_VEHICLES_RAW
	ROW_COUNT_CHECK_ACCESS
	WAFER_OPTION_UTILITIES_INTERFACE
		export
			{NONE} all
		end
create
	make_template, make_null

feature -- Properties

	test_vehicle_with_design_no (n: INTEGER a_wafer_techn: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All test vehicles with design number `n'
		require
			design_no_nonegative: n >= 0
			a_wafer_techn_exists: a_wafer_techn /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_IS
		do
			create b.make (n, a_wafer_techn)
			Result := all_rows_filtered (conn, "design_no = :1 and wafer_techn = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	test_vehicles_for_fab_id (a_run_id, a_die_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All test vehicles for die on `a_run_id' 
			-- with `a_die_id'
		require
			a_run_id_not_void: a_run_id /= Void
			a_die_id_not_void: a_die_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (a_run_id, a_die_id)
			Result := all_rows_filtered (conn, "design_no in (select design_no from ic_fab_instances ifi where run_id = :1 and die_id = :2) and wafer_techn = (select wafer_techn from runs where run_id = :1)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	test_vehicles_for_wafer_techn (wt: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All test vehicles compatible with wafer
			-- techn `wt' whose status indicates that they
			-- can be included on a run, ordered by design
			-- number
		require
			wt_not_void: wt /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			status: TEST_VEHICLE_STATUS_VALUES
		do
			create b.make (wt, status.Fab_and_test)
			Result := all_rows_filtered_ordered (conn, "wafer_techn = :1 and test_vehicle_status = :2", b, "design_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	test_vehicles_for_wafer_techn_and_priority (wt: STRING high_pri: BOOLEAN conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All test vehicles compatible with wafer
			-- techn `wt' whose status indicates that they
			-- can be included on a run and whose high priority
			-- status matches `high_pri', ordered by design
			-- number
		require
			wt_not_void: wt /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SSB
			status: TEST_VEHICLE_STATUS_VALUES
		do
			create b.make (wt, status.Fab_and_test, high_pri)
			Result := all_rows_filtered_ordered (conn, "wafer_techn = :1 and test_vehicle_status = :2 and high_priority = :3", b, "design_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	wafer_techn_layer_errors: LIST [STRING]
			-- Wafer techn layer errors for `Current'
		do  
			if cached_wafer_techn_layer_errors = Void then
				fetch_wafer_techn_layer_errors
			end
			Result := cached_wafer_techn_layer_errors
		ensure
			result_not_void: Result /= Void
		end

	wafer_techn_layer_error_text: STRING
			-- Description of wafer techn layer errors as a 
			-- single string
		do  
			Result := "Wafer techn layer incompatibilities: " + su.joined_by (wafer_techn_layer_errors, ", ")
		ensure
			result_not_void: Result /= Void
		end

	is_cached_wafer_techn_layer_errors: BOOLEAN
			-- Are wafer techn layer errors currently cached?
		do  
			Result := cached_wafer_techn_layer_errors /= Void
		end

	is_selectable: BOOLEAN
			-- Is `Current' selectable?
		do  
			Result := may_be_fabricated and wafer_techn_layer_errors.is_empty
		end

	may_be_fabricated: BOOLEAN
			-- May `Current' be fabricated, according to its status
		local
			status: TEST_VEHICLE_STATUS_VALUES
		do  
			Result := test_vehicle_status.is_equal (status.Fab_and_test)
		end

feature -- Modification

	uncache_wafer_techn_layer_errors
			-- Remove wafer techn layer errors from cache
		do  
			cached_wafer_techn_layer_errors := Void
		end

feature {NONE} -- Implementation

	fetch_wafer_techn_layer_errors
			-- Fetch wafer techn layer error
		local	b: DB_BIND_S; dtid: STRING
		do
			create b.make(design_techn);
			dtid := connection.selected_string
					("design_techn_id", "design_techns", "design_techn = :1", b)
			compute_design_wt_layer_errors (connection, design_no, dtid, wafer_techn)
			cached_wafer_techn_layer_errors := design_wt_layer_errors
		end
		
	cached_wafer_techn_layer_errors: LIST [STRING]
			-- Wafer techn layer errors for `Current', cached
			-- in memory

end
