note
	description:
		"Package bonding connections."
	author: "Barden E. Smith"
	date:	"August 30, 2004"

class PACKAGE_CONNECTIONS_ROW

inherit
	PACKAGE_CONNECTIONS_RAW
		select
			is_equal,
			out
		end

	PACKAGE_CONNECTION
		rename
			out as bonding_point_out,
			is_equal as bonding_point_is_equal
		end
	ROW_COUNT_CHECK_ACCESS

	DB_OPTIONS_ACCESS

create
	make_template, make_null, make_by_key

feature -- Properties

	rows_for_package_id (pkg: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All rows for package id `pkg'
		require
			pkg_exists: pkg /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (pkg)
			Result := all_rows_filtered (conn, "package_id = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end


	bonding_finger_layout: BONDING_FINGER_LAYOUTS_ROW
			-- Layout for bonding finger for connection
	do	if cached_bf_layout = Void then
			create cached_bf_layout.make_by_key
				(connection, option.old_row, 
				 bonding_finger_layout_no)
		end;
		Result := cached_bf_layout
	end;

feature -- Modification

	set_bonding_finger_layout (r: like bonding_finger_layout)
			-- Set `bonding_finger_layout' to `r'
		do
			cached_bf_layout := r
		ensure
			value_set: bonding_finger_layout = r
		end

	select_bonding_finger_layout (lst: LIST[like bonding_finger_layout])
			-- pick my layout from the list
		do
			from	lst.start;
				cached_bf_layout := Void
			until	lst.after or cached_bf_layout /= Void
			loop	if lst.item.bonding_finger_layout_no = bonding_finger_layout_no then
					cached_bf_layout := lst.item
				end;
				lst.forth
			end
		end

feature {NONE} -- cached data

	cached_bf_layout: BONDING_FINGER_LAYOUTS_ROW

end -- class PACKAGE_CONNECTIONS_ROW
