
note

	author: "Generated"
	date: "Sat Feb 28 10:50:43 PST 2004"

class PACKAGES_ROW
inherit
	PACKAGES_RAW
	ROW_COUNT_CHECK_ACCESS
	STRING_UTILITIES_ACCESS
		export {NONE} all end;
	ITEM_COUNT_RANGES
		export {NONE} all end;

create
	make_template, make_null

feature -- Properties

	packages_with_id (id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All packages with package id `id'
		require
			id_exists: id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (id)
			Result := all_rows_filtered (conn, "package_id = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	packages_by_mosis_default (default_package: BOOLEAN conn: DB_CONNECTION
					range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All packages where M_DEFAULT = `default_package'
		require
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_B
		do
			create b.make (default_package)
			Result := all_rows_filtered (conn, "m_default = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	packages_with_reduced_id (id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All packages with package id `id' or else
			-- all packages with id reduced_to_alphanumerics(id)
		require
			id_exists: id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local rid: STRING
		do  Result := packages_with_id(id, conn, at_least_zero);
		    if Result.is_empty then
			rid := su.reduced_to_alphanumerics(id);
			Result := packages_with_id(rid, conn, range)
		    end
		end;

	cavity_in_um: CONST_RECTANGLE
		-- a box depicting the package cavity, in um
		-- (origin is at center of cavity)
	do  create Result.make_by_size_and_center
			(package_cavity_size_x_mil * 25.4,
			 package_cavity_size_y_mil * 25.4,
			 0, 0)
	end;

	have_known_connections: BOOLEAN
		-- do we have a definition of the packaging_connections
		-- available in the db?
	do  Result := not my_connections.is_empty
	end;

	my_connections: LIST[PACKAGE_CONNECTIONS_ROW]
		-- all of my connections (pin/bonding finger pairs)
		-- origin is at center of cavity; dimensions in um
	local r: PACKAGE_CONNECTIONS_ROW; b: DB_BIND_S; 
	do  if cached_connections = Void
	     then create r.make_template;
		  create b.make(package_id);
		  cached_connections := r.all_rows_filtered_ordered
					(connection,
					 "package_id = :1", b,
					 "connection_no");
		  check
			cached_connections.is_empty or
			cached_connections.count = bonding_finger_count
		  end;
	    end;
	    Result := cached_connections
	ensure Result /= Void
	end;

	clear_cached_connections
		-- force a re-fetch
	do  cached_connections := Void
	end;

feature -- set_bonding_finger_layouts

	set_bonding_finger_layouts
		-- set bonding_finger_layout for all of my connections
		-- (there is usually a lot of finger sharing)
	local r: BONDING_FINGER_LAYOUTS_ROW;
		b: DB_BIND_S
		allf: LIST[BONDING_FINGER_LAYOUTS_ROW]
	do  -- get every finger layout that we use
	    create r.make_template
	    create b.make(package_id)
	    allf := r.all_rows_filtered
			(connection, bonding_finger_filter, b)
	    -- and let each connection pick out his own
	    my_connections.do_all(agent {PACKAGE_CONNECTIONS_ROW}.select_bonding_finger_layout(allf))
	end;

feature {NONE} -- details

	bonding_finger_filter: STRING =
	"bonding_finger_layout_no in%
	% (select distinct bonding_finger_layout_no from package_connections where package_id = :1)"
	cached_connections: like my_connections

end
