
note

	author: "David Hollenberg"

class PACKING_CUT_DIE_SITES_ROW
inherit
	PACKING_CUT_DIE_SITES_RAW
	PACKING_QUERIES
	DB_HASHABLE_ROW
		undefine
			initialize_new, initialize_old, fill_in
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key, make_detached

feature -- Queries

	rows_for_packing_version_and_pat (a_packing_id: STRING a_version_no, a_pattern_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Cut die sites for `a_packing_id',
			-- `a_version_no' and `a_pattern_no', ordered by die
			-- number and site number
		require
			a_packing_id_not_void: a_packing_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SII
		do
			create b.make (a_packing_id, a_version_no, a_pattern_no)
			Result := all_rows_filtered_ordered (conn, "packing_id = :1 and version_no = :2 and cut_pattern_no = :3", b, "die_no, site_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	hash_code: INTEGER
		local
			s: STRING
		do
			create s.make (packing_id.count + 10)
			s.append (packing_id)
			s.extend ('%U')
			s.append_integer (version_no)
			s.extend ('%U')
			s.append_integer (cut_pattern_no)
			Result := s.hash_code
		end
	
	die_location_foreign_key_columns: STRING
		do
			create Result.make (packing_id.count + 15)
			Result.append (packing_id)
			Result.extend ('%U')
			Result.append_integer (version_no)
			Result.extend ('%U')
			Result.append_integer (die_no)
			Result.extend ('%U')
			Result.append_integer (site_no)
		end
	
	die_location: like anchored_die_location
			-- Die location corresponding to `Current'

	cut_die: RECTANGLE
			-- Cut die
		do
			create Result.make_by_size_and_center (die_size_cut_x, die_size_cut_y, location_x, location_y)
		end

feature -- Modification

	set_die_location (dl: like die_location)
			-- Set `die_location' to `dl'
		do
			die_location := dl
		end

feature {NONE} -- Implementation

	anchored_die_location: PACKING_DIE_LOCATIONS_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

end
