
note

	author: "David Hollenberg"

class PACKING_CUT_LOCATIONS_ROW
inherit
	PACKING_CUT_LOCATIONS_RAW
	PACKING_QUERIES
	DB_HASHABLE_ROW
		undefine
			initialize_new, initialize_old, fill_in
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	HAS_POSITION
		rename
			position as location
		end

create
	make_template, make_null, make_by_key, make_detached

feature -- Queries

	rows_for_packing_version_and_pat (a_packing_id: STRING a_version_no, a_pattern_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Cut patterns for `a_packing_id',
			-- `a_version_no' and `a_pattern_no', ordered by cut 
			-- number
		require
			a_packing_id_not_void: a_packing_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SII
		do
			create b.make (a_packing_id, a_version_no, a_pattern_no)
			Result := all_rows_filtered_ordered (conn, "packing_id = :1 and version_no = :2 and cut_pattern_no = :3", b, "cut_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	hash_code: INTEGER
		local
			s: STRING
		do
			create s.make (packing_id.count + 12)
			s.append (packing_id)
			s.extend ('%U')
			s.append_integer (version_no)
			s.extend ('%U')
			s.append_integer (cut_pattern_no)
			s.extend ('%U')
			s.append_integer (cut_no)
			Result := s.hash_code
		end
	
	is_horizontal: BOOLEAN
			-- Is cut horizontal?
		local
			orient: ORIENTATION_VALUES
		do
			Result := orientation.is_equal (orient.Horizontal)
		end

	is_vertical: BOOLEAN
			-- Is cut vertical?
		local
			orient: ORIENTATION_VALUES
		do
			Result := orientation.is_equal (orient.Vertical)
		end

	has_position: BOOLEAN = True
			-- Is there a position?

end
