
note

	author: "David Hollenberg"

class PACKING_CUT_PATTERNS_MROW
inherit
	PACKING_CUT_PATTERNS_MRAW
		redefine
			detached_twin, copy_from_row,
			set_wafer_etest_name, set_null_wafer_etest_name
		end
	PACKING_CUT_PATTERNS_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		redefine
			anchored_cut_location, anchored_cut_die_site,
			anchored_die_location, anchored_die,
			cut_locations, cut_die_sites, set_cut_locations,
			set_cut_die_sites
		end
	DB_DETACHABLE_ROW
		export
			{PACKING_CUT_PATTERNS_MROW} detach_actions_internal
		undefine
			set_default_values
		redefine
			detached_twin, copy_from_row
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	ABSTRACT_EXT_CUT_DIST_SPEC

create
	make_template, make_null, make_by_key, make_detached

feature -- Properties

	cut_locations: LIST [like anchored_cut_location]
			-- Cut locations for `Current'
		do
			if is_detached then
				Result := detached_cut_locations
			else
				if cached_cut_locations = Void then
					fetch_cut_locations
				end
				Result := cached_cut_locations
			end
		end

	cut_die_sites: LIST [like anchored_cut_die_site]
			-- Cut die sites for `Current'
		do
			if is_detached then
				Result := detached_cut_die_sites
			else
				if cached_cut_die_sites = Void then
					fetch_cut_die_sites
				end
				Result := cached_cut_die_sites
			end
		end

	packaging_lot_no: INTEGER
			-- Number of associated packaging lot, if any

	is_null_packaging_lot_no: BOOLEAN
			-- Is there no associated packaging lot?

	detached_twin: like Current
			-- New object that is the same as `Current' but
			-- is detached from any database connection
		do
			Result := precursor {PACKING_CUT_PATTERNS_MRAW}
			call_detach_actions (Result)
		end

	deep_detached_twin: like Current
			-- Detached copy of `Current' with all
			-- dependent rows detached as well
		do
			Result := detached_twin
			Result.set_cut_locations (cut_locations.twin)
			Result.set_cut_die_sites (cut_die_sites.twin)
			Result.uncache_dependents
				-- Just in case: shouldn't be referenced
			Result.detach_dependents_deep
		end

	is_deep_unchanged: BOOLEAN
			-- Are `Current' and all its dependent rows
			-- unchanged from their original versions?
			-- Returns False if no original version
		do
		end

	new_horizontal_cut (a_location: DOUBLE): like anchored_cut_location
			-- New horizontal cut at `a_location'
		local
			orient: ORIENTATION_VALUES
		do
			max_cut_no := max_cut_no + 1
			Result := new_cut (orient.Horizontal, a_location)
		end

	new_vertical_cut (a_location: DOUBLE): like anchored_cut_location
			-- New vertical cut at `a_location'
		local
			orient: ORIENTATION_VALUES
		do
			max_cut_no := max_cut_no + 1
			Result := new_cut (orient.Vertical, a_location)
		end

feature -- Row copy

	initialize_from_pattern (pat: RUN_CUT_PATTERNS_ROW)
			-- Initialize from attributes of `pat'
		require
			pat_not_void: pat /= Void
		do
			set_external_cut_specs (pat)
		end

	copy_from_row (r: like original_row)
			-- Initialize `Current' from `r'
		do
			precursor {PACKING_CUT_PATTERNS_MRAW} (r)
			detach_actions_internal := r.detach_actions_internal
			cached_cut_locations := r.cached_cut_locations
			cached_cut_die_sites := r.cached_cut_die_sites
			max_cut_no := r.max_cut_no
			grid := r.grid
			is_read_only := r.is_read_only
			production_id := r.production_id
			is_sent_to_packager := r.is_sent_to_packager
			date_sent_to_packager := r.date_sent_to_packager
			is_actual_date_sent := r.is_actual_date_sent
			packaging_lot_no := r.packaging_lot_no
			is_null_packaging_lot_no := r.is_null_packaging_lot_no
		end

feature -- Modification

	set_is_read_only (b: BOOLEAN)
			-- Set `is_read_only' to `b'
		do
			is_read_only := b
		ensure
			read_only_set: is_read_only = b
		end

	set_production_id (p: like production_id)
			-- Set `production_id' to `p'
		do
			production_id := p
		ensure
			production_id_set: production_id = p
		end

	set_is_sent_to_packager (b: like is_sent_to_packager)
			-- Set `is_sent_to_packager' to `b'
		do
			is_sent_to_packager := b
		ensure
			is_sent_to_packager_set: is_sent_to_packager = b
		end

	set_date_sent_to_packager (d: like date_sent_to_packager)
			-- Set `date_sent_to_packager' to `d'
		do
			date_sent_to_packager := d
		ensure
			date_sent_to_packager_set: date_sent_to_packager = d
		end

	set_is_actual_date_sent (b: like is_actual_date_sent)
			-- Set `is_actual_date_sent' to `b'
		do
			is_actual_date_sent := b
		ensure
			date_sent_actual_set: is_actual_date_sent = b
		end

	set_cut_locations (list: like cut_locations)
			-- Set `cut_locations' to `list'
		do
			if is_detached then
				detached_cut_locations := list
			else
				precursor (list)
			end
		end

	set_cut_die_sites (list: like cut_die_sites)
			-- Set `cut_die_sites' to `list'
		do
			if is_detached then
				detached_cut_die_sites := list
			else
				precursor (list)
			end
		end

	set_wafer_etest_name (a_name: like wafer_etest_name)
			-- Set `wafer_etest_name' to `a_name'
		do
			precursor (a_name)
			uncache_etest_name
		end

	set_null_wafer_etest_name
			-- Set `wafer_etest_name' to null
		do
			precursor
			uncache_etest_name
		end

	set_packaging_lot_no (a_value: like packaging_lot_no)
			-- Set `packaging_lot_no' to `a_value'
		do
			packaging_lot_no := a_value
			is_null_packaging_lot_no := False
		end

	set_null_packaging_lot_no
			-- Set `packaging_lot_no' to null
		do
			packaging_lot_no := 0
			is_null_packaging_lot_no := True
		end

feature -- Operations

	update_for_new_version (n: INTEGER)
			-- Update `Current' with information for new
			-- version `n'
		do
			set_version_no (n)
			update_dependents_for_version (n)
		end

	renumber_cut_locations
			-- Renumber the cut locations for `Current'
			-- prior to saving changes made in cut pattern
			-- editor.  Not strictly necessary, but cuts
			-- are then numbered 1 - N.
		do
			max_cut_no := 0
			cut_locations.do_all (agent renumber_cut_location)
		end
	
	register_deep
			-- Register `Current' and all dependent rows
			-- on implicit connection
		do
			register_on_implicit_connection
			register_dependents_deep
		end

	add_cut (a_cut: like anchored_cut_location)
			-- Add `a_cut' to `Current'
		require
			a_cut_not_void: a_cut /= Void
		local
			cuts: like horizontal_cuts_table
		do
			cuts := table_for_cut (a_cut)
				-- Force calculation of hash table if not done
			cut_locations.extend (a_cut)
			add_cut_to_table (a_cut, cuts)
		end

	delete_cuts (a_list: LIST [like anchored_cut_location])
			-- Delete cuts in `a_list'
		do
			from
				a_list.start
			until
				a_list.after
			loop
				delete_cut (a_list.item)
				a_list.forth
			end
		end

	delete_cut (a_cut: like anchored_cut_location)
			-- Delete `a_cut'
		local
			cuts: like horizontal_cuts_table
		do
			cuts := table_for_cut (a_cut)
				-- Force calculation of hash table if not done
			cut_locations.prune_all (a_cut)
			remove_cut_from_table (a_cut, cuts)
		end

	remove_isolated_sites
			-- Remove all isolated sites for cut pattern
		do
			cut_die_sites.wipe_out
			uncache_isolated_die_locations
		end

	isolate (a_location: like anchored_die_location a_cut_die: CONST_RECTANGLE)
			-- Isolate `a_location' in cut pattern
		require
			a_location_not_void: a_location /= Void
			a_cut_die_not_void: a_cut_die /= Void
		local
			table: like isolated_die_locations
			l_cut_die_site: like anchored_cut_die_site
		do
			table := isolated_die_locations
			table.search (a_location)
			if not table.found then
				l_cut_die_site := new_cut_die_site (a_location, a_cut_die)
				table.put (l_cut_die_site, a_location)
				cut_die_sites.extend (l_cut_die_site)
			else
				handle_hard_error ("there is already an entry for die location " + a_location.identification + " in internal table of isolated die locations")
			end
		end

	unisolate (a_location: like anchored_die_location)
			-- Unisolate `a_location' in cut pattern
		local
			table: like isolated_die_locations
			l_cut_die_site: like anchored_cut_die_site
		do
			table := isolated_die_locations
			table.search (a_location)
			if table.found then
				l_cut_die_site := table.found_item
				cut_die_sites.prune_all (l_cut_die_site)
				table.remove (a_location)
			else
				handle_hard_error ("cannot find entry for die location " + a_location.identification + " in internal table of isolated die locations")
			end
		end

	set_isolated_site_cut_die (a_location: like anchored_die_location a_cut_die: CONST_RECTANGLE)
			-- Set cut die size and center from `a_cut_die' for 
			-- already isolated site `a_location' 
		require
			a_location_not_void: a_location /= Void
			a_cut_die_not_void: a_cut_die /= Void
		local
			table: like isolated_die_locations
			l_cut_die_site: like anchored_cut_die_site
		do
			table := isolated_die_locations
			table.search (a_location)
			if table.found then
				l_cut_die_site := table.found_item
				l_cut_die_site.set_location_x (a_cut_die.center_x)
				l_cut_die_site.set_location_y (a_cut_die.center_y)
				l_cut_die_site.set_die_size_cut_x (a_cut_die.size_x)
				l_cut_die_site.set_die_size_cut_y (a_cut_die.size_y)
			else
				handle_hard_error ("cannot find entry for die location " + a_location.identification + " in internal table of isolated die locations")
			end
		end

feature {PACKING_VERSIONS_MROW, PACKING_PACKAGING_LOTS_MROW} -- Implementation

	set_highest_used_cut_no
			-- Compute `max_cut_no' over all locations for
			-- cut pattern
		do
			max_cut_no := 0
			cut_locations.do_all (agent update_max_cut_no)
		end

feature {PACKING_CUT_PATTERNS_MROW} -- Implementation

	detach_dependents_deep
			-- Detach all dependent rows
		do
			detach_cut_locations_deep
			detach_cut_die_sites_deep
		end

feature {NONE} -- Implementation

	detach_cut_locations_deep
			-- Detach cut location rows
		do
			original_cut_locations := cut_locations.twin
			original_cut_locations.wipe_out
			original_cut_locations.append (cut_locations)
			cut_locations.wipe_out
			original_cut_locations.do_all (agent append_detached_cut_location (?, cut_locations))
		end

	detach_cut_die_sites_deep
			-- Detach all cut die site row
		do
			original_cut_die_sites := cut_die_sites.twin
			original_cut_die_sites.wipe_out
			original_cut_die_sites.append (cut_die_sites)
			cut_die_sites.wipe_out
			original_cut_die_sites.do_all (agent append_detached_cut_die_site (?, cut_die_sites))
		end

	append_detached_cut_location (cl: like anchored_cut_location list: like cut_locations)
			-- Append detached copy of `cl' to `list'
		do
			list.extend (cl.deep_detached_twin)
		end
		
	append_detached_cut_die_site (cds: like anchored_cut_die_site list: like cut_die_sites)
			-- Append detached copy of `cds' to `list'
		do
			list.extend (cds.deep_detached_twin)
		end
		
	update_dependents_for_version (n: INTEGER)
			-- Update all dependent rows prior to save of
			-- new version `n'
		do
			max_cut_no := 0
			cut_locations.do_all (agent renumber_cut_location)
			cut_locations.do_all (agent {like anchored_cut_location}.update_for_new_version (n))
			cut_die_sites.do_all (agent {like anchored_cut_die_site}.set_cut_pattern_no (cut_pattern_no))
			cut_die_sites.do_all (agent {like anchored_cut_die_site}.update_for_new_version (n))
		end

	renumber_cut_location (cl: PACKING_CUT_LOCATIONS_MROW)
		do
			max_cut_no := max_cut_no + 1
			cl.set_cut_pattern_no (cut_pattern_no)
			cl.set_cut_no (max_cut_no)
		end
	
	register_dependents_deep
			-- Register all dependent rows on implicit
			-- connection
		do
			-- Move "detached" versions back to "cached" versions
			-- since they will no longer be detached at end
			-- of routine
			
			cached_cut_locations := detached_cut_locations
			detached_cut_locations := Void
			cut_locations.do_all (agent {like anchored_cut_location}.register_deep)
			
			cached_cut_die_sites := detached_cut_die_sites
			detached_cut_die_sites := Void
			cut_die_sites.do_all (agent {like anchored_cut_die_site}.register_deep)
		end

	update_max_cut_no (cl: PACKING_CUT_LOCATIONS_MROW)
		do
			max_cut_no := max_cut_no.max (cl.cut_no)
		end
	
	new_cut (an_orientation: STRING a_location: DOUBLE): like anchored_cut_location
			-- New cut with `an_orientation' at `a_location'
			-- and `max_cut_no' as its cut number
		do
			create Result.make_detached (any_connection)
			Result.set_packing_id (packing_id)
			Result.set_version_no (version_no)
			Result.set_cut_pattern_no (cut_pattern_no)
			Result.set_cut_no (max_cut_no)
			Result.set_orientation (an_orientation)
			Result.set_cut_type ("NORMAL")
			Result.set_location (grid.rounded_center (a_location))
		end

	new_cut_die_site (a_location: like anchored_die_location a_cut_die: CONST_RECTANGLE): like anchored_cut_die_site
			-- New cut die site for `a_location'
		require
			a_location_not_void: a_location /= Void
			a_cut_die_not_void: a_cut_die /= Void
		do
			create Result.make_detached (any_connection)
			Result.set_packing_id (packing_id)
			Result.set_version_no (version_no)
			Result.set_cut_pattern_no (cut_pattern_no)
			Result.set_die_no (a_location.die_no)
			Result.set_site_no (a_location.site_no)
			Result.set_location_x (a_cut_die.center_x)
			Result.set_location_y (a_cut_die.center_y)
			Result.set_die_size_cut_x (a_cut_die.size_x)
			Result.set_die_size_cut_y (a_cut_die.size_y)
			Result.set_isolated (True)
			Result.set_die_location (a_location)
		end

	anchored_die: PACKING_DIES_MROW
			-- For anchoring only
		do
		end

	anchored_die_location: PACKING_DIE_LOCATIONS_MROW
			-- For anchoring only
		do
		end

	anchored_cut_location: PACKING_CUT_LOCATIONS_MROW
			-- For anchoring only
		do
		end

	anchored_cut_die_site: PACKING_CUT_DIE_SITES_MROW
			-- For anchoring only
		do
		end

	detached_cut_locations: like cut_locations
			-- List of detached cut locations (no explict 
			-- DB connection)

	detached_cut_die_sites: like cut_die_sites
			-- List of detached cut die sites (no explict 
			-- DB connection)

	original_cut_locations: like cut_locations
			-- Original list of cut locations before detach

	original_cut_die_sites: like cut_die_sites
			-- Original list of cut locations before detach

feature {PACKING_CUT_PATTERNS_MROW} -- Implementation

	max_cut_no: INTEGER
			-- Last cut number assigned during renumbering

end
