
note

	author: "David Hollenberg"

class PACKING_CUT_PATTERNS_ROW
inherit
	PACKING_CUT_PATTERNS_RAW
	ABSTRACT_CUT_PATTERN
	PACKING_QUERIES
	DB_HASHABLE_ROW
		undefine
			initialize_new, initialize_old, fill_in
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	GRIDABLE
	CONST_ABSTRACT_EXT_CUT_DIST_SPEC

create
	make_template, make_null, make_by_key, make_detached

feature -- Queries

	rows_for_packing_version (a_packing_id: STRING a_version_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Cut patterns for `a_packing_id' and
			-- `a_version_no', ordered by cut pattern number
		require
			a_packing_id_not_void: a_packing_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SI
		do
			create b.make (a_packing_id, a_version_no)
			Result := all_rows_filtered_ordered (conn, "packing_id = :1 and version_no = :2", b, "cut_pattern_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	hash_code: INTEGER
		local
			s: STRING
		do
			create s.make (packing_id.count + 5)
			s.append (packing_id)
			s.extend ('%U')
			s.append_integer (version_no)
			s.extend ('%U')
			s.append_integer (cut_pattern_no)
			Result := s.hash_code
		end
	
	is_read_only:  BOOLEAN
			-- Is `Current' read-only (cannot be modified
			-- or deleted)?

	production_id: STRING
			-- Production ID of production `Current' is
			-- for, if not Void

	is_null_production_id: BOOLEAN
			-- <Precursor>
		do
			Result := production_id = Void
		end

	is_sent_to_packager: BOOLEAN
			-- Is there an associated packaging lot which
			-- has already been sent to the packager?

	date_sent_to_packager: STRING
			-- Date associated packaging lot was sent to packager,
			-- if has an associated packaging lot and if not Void

	is_actual_date_sent: BOOLEAN
			-- Is the date associated packaging lot was
			-- sent to packager an actual date (not an
			-- estimate)?

	cut_locations: LIST [like anchored_cut_location]
			-- Cut locations for `Current'
		do
			if cached_cut_locations = Void then
				fetch_cut_locations
			end
			Result := cached_cut_locations
		end

	is_cached_cut_locations: BOOLEAN
			-- Are cut locations currently cached?
		do
			Result := cached_cut_locations /= Void
		end

	cut_die_sites: LIST [like anchored_cut_die_site]
			-- Cut die sites for `Current'
		do
			if cached_cut_die_sites = Void then
				fetch_cut_die_sites
			end
			Result := cached_cut_die_sites
		end

	is_cached_cut_die_sites: BOOLEAN
			-- Are cut die sites currently cached?
		do
			Result := cached_cut_die_sites /= Void
		end

	is_cached_isolated_die_locations: BOOLEAN
			-- Are isolated die locations currently cached?
		do
			Result := cached_isolated_die_locations /= Void
		end

	etest_name: WAFER_ETEST_NAMES_ROW
			-- Wafer etest name row for `Current'
		require
			has_etest_name: not is_null_wafer_etest_name
		do  
			if not is_cached_etest_name then 
	     			create cached_etest_name.make_by_key (any_connection, option.Old_row, wafer_etest_name)
	    		end
	    		Result := cached_etest_name
		ensure
			result_not_void: Result /= Void
		end;

	is_cached_etest_name: BOOLEAN
			-- Is etest name cached?
		do  
			Result := cached_etest_name /= Void
		end;

	wafer_etest_id: STRING
			-- Wafer etest ID for `Current'
		require
			has_etest_name: not is_null_wafer_etest_name
		do  
	    		Result := etest_name.wafer_etest_id
		end;

	has_cuts: BOOLEAN
			-- Are there any cuts for pattern?
		do
			Result := not cut_locations.is_empty
		end

	has_horizontal_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a horizontal cut at
			-- `a_location'?
		do
			Result := horizontal_cuts_table.has (grid.rounded_center (a_location))
		end

	has_vertical_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a vertical cut at
			-- `a_location'?
		do
			Result := vertical_cuts_table.has (grid.rounded_center (a_location))
		end

	horizontal_cuts: LIST [like anchored_cut_location]
			-- Horizontal cuts for cut pattern
		local
			cut: like anchored_cut_location
		do
			create {ARRAYED_LIST [like anchored_cut_location]} Result.make (cut_locations.count)
			from
				cut_locations.start
			until
				cut_locations.after
			loop
				cut := cut_locations.item
				if cut.is_horizontal then
					Result.extend (cut)
				end
				cut_locations.forth
			end
		end

	vertical_cuts: LIST [like anchored_cut_location]
			-- Vertical cuts for cut pattern
		local
			cut: like anchored_cut_location
		do
			create {ARRAYED_LIST [like anchored_cut_location]} Result.make (cut_locations.count)
			from
				cut_locations.start
			until
				cut_locations.after
			loop
				cut := cut_locations.item
				if cut.is_vertical then
					Result.extend (cut)
				end
				cut_locations.forth
			end
		end

	has_isolated_sites: BOOLEAN
			-- Are there any isolated sites for pattern?
		do
			Result := not cut_die_sites.is_empty
		end

	isolates_die (a_die: like anchored_die): BOOLEAN
			-- Does pattern isolate `a_die'?
		local
			locs: LIST [like anchored_die_location]
		do
			locs := a_die.die_locations
			from
				locs.start
			until
				locs.after or Result
			loop
				Result := Result or isolates_location (locs.item)
				locs.forth
			end
		end

	isolates_location (a_location: like anchored_die_location): BOOLEAN
			-- Does pattern isolate `a_location'?
		do
			Result := isolated_die_locations.has (a_location)
		end

	isolated_cut_die (a_location: like anchored_die_location): RECTANGLE
			-- Cut die for isolated location `a_location'
		require
			a_location_not_void: a_location /= Void
		local
			table: like isolated_die_locations
		do
			table := isolated_die_locations
			table.search (a_location)
			if table.found then
				Result := table.found_item.cut_die
			else
				handle_hard_error ("cannot find entry for die location " + a_location.identification + " in internal table of isolated die locations")
			end
		end

	isolated_location_count: INTEGER
			-- Number of die locations isolated by cut pattern
		do
			Result := cut_die_sites.count
		end

	label_description: STRING
			-- Description of cut pattern suitable for label
		local
			has_lot, has_quant: BOOLEAN
			xf_type: like current_xform_type
		do
			create Result.make (100)
			xf_type := current_xform_type
			if production_id /= Void then
				Result.append (production_id)
				Result.extend (' ')
			end
			Result.extend (' ')
			Result.append ("Cut Pattern ")
			Result.append_integer (cut_pattern_no)
			Result.append ("   (")
			Result.append (status)
			Result.append (")%N")
			has_lot := not is_null_wafer_etest_name
			has_quant := not is_null_quantity_wafers_needed
			if has_lot or has_quant then
				if has_lot then
					Result.append (etest_name.wafer_etest_id)
				else
					Result.append ("No etest ID")
				end
				Result.append (", ")
				if has_quant then
					Result.append_integer (quantity_wafers_needed)
					Result.extend (' ')
					Result.append (su.number (quantity_wafers_needed, "wafer", "wafers"))
				else
					Result.append ("No quantity")
				end
				Result.append (", ")
			end
			Result.append (xf_type.presentation_text)
			if xf_type.applies_to_strips then
				if not is_null_cut_pattern_no_parent then
					Result.append (", parent ")
					Result.append_integer (cut_pattern_no_parent)
				else
					Result.append (", no parent")
				end
			end
		end

	description: STRING
			-- Description of cut pattern
		local
			has_lot, has_quant: BOOLEAN
			xf_type: like current_xform_type
		do
			create Result.make (100)
			xf_type := current_xform_type
			Result.append ("Cut Pattern ")
			Result.append_integer (cut_pattern_no)
			Result.append ("   (")
			has_lot := not is_null_wafer_etest_name
			has_quant := not is_null_quantity_wafers_needed
			if has_lot or has_quant then
				if has_lot then
					Result.append (etest_name.wafer_etest_id)
				end
				if has_quant then
					if has_lot then
						Result.append (", ")
					end
					Result.append_integer (quantity_wafers_needed)
					Result.extend (' ')
					Result.append (su.number (quantity_wafers_needed, "wafer", "wafers"))
				end
				Result.append (", ")
			end
			Result.append (xf_type.presentation_text)
			if xf_type.applies_to_strips then
				if not is_null_cut_pattern_no_parent then
					Result.append (", parent ")
					Result.append_integer (cut_pattern_no_parent)
				else
					Result.append (", no parent")
				end
			end
			Result.extend (')')
		end

	description_for_selection: STRING
			-- Description of cut pattern when it
			-- is included in a list from which user
			-- selects one item
		local
			has_lot: BOOLEAN
		do
			create Result.make (30)
			Result.append_integer (cut_pattern_no)
			Result.append ("   ")
			if production_id /= Void then
				Result.append (production_id)
				Result.append ("   ")
			end
			has_lot := not is_null_wafer_etest_name
			if has_lot then
				Result.append (etest_name.wafer_etest_id)
			else
				Result.append ("No etest ID yet")
			end
		end

	status: STRING
			-- Status of cut pattern
		do
			if is_sent_to_packager then
				Result := "Sent " + date_sent_to_packager
				if not is_actual_date_sent then
					Result.append (" (Est)")
				end
			elseif is_completed then
				Result := "Completed"
			else
				Result := "Under Construction"
			end
		end

	has_parent: BOOLEAN
			-- Does `Current' have a parent cut pattern?
		do
			Result := not is_null_cut_pattern_no_parent
		end

	has_no_parent: BOOLEAN
			-- Does `Current' have no parent cut pattern?
		do
			Result := not has_parent
		end

feature -- Modification

	uncache_cut_locations
			-- Clear cut locations cache
		do
			cached_cut_locations := Void
		ensure
			cut_locations_not_cached: not is_cached_cut_locations
		end

	set_cut_locations (list: like cut_locations)
			-- Set `cut_locations' to `list'
		require
			list_not_void: list /= Void
		do
			cached_cut_locations := list
		ensure
			cut_locations_set: cut_locations = list
		end

	uncache_cut_die_sites
			-- Clear cut die sites cache
		do
			cached_cut_die_sites := Void
		ensure
			cut_die_sites_not_cached: not is_cached_cut_die_sites
		end

	uncache_isolated_die_locations
			-- Clear isolated die locations cache
		do
			cached_isolated_die_locations := Void
		ensure
			isolated_die_locations_not_cached: not is_cached_isolated_die_locations
		end

	uncache_etest_name
			-- Uncache `etest_name' for `Current'
		do  
			cached_etest_name := Void
		ensure
			etest_name_not_cached: not is_cached_etest_name
		end;

	set_cut_die_sites (list: like cut_die_sites)
			-- Set `cut_die_sites' to `list'
		require
			list_not_void: list /= Void
		do
			cached_cut_die_sites := list
		ensure
			cut_die_sites_set: cut_die_sites = list
		end

	uncache_dependents
			-- Clear all dependent rows from cache
		do
			uncache_cut_locations
			uncache_cut_die_sites
		end

	remove_dies_for_projects (table: HASH_TABLE [INTEGER, INTEGER])
			-- Remove all references to dies whose
			-- die_no is a key in `table'
		require
			table_not_void: table /= Void
		local
			old_list, new_list: like cut_die_sites
			site: like anchored_cut_die_site
		do
			from
				old_list := cut_die_sites
				new_list := old_list.twin
				new_list.wipe_out
				old_list.start
			until
				old_list.after
			loop
				site := old_list.item
				table.search (site.die_no)
				if not table.found then
					new_list.extend (site)
				end
				old_list.forth
			end
			set_cut_die_sites (new_list)
		end

feature {PACKING_CUT_PATTERNS_MROW} -- Implementaion

	cached_cut_locations: like cut_locations
			-- Cached list of cut locations
		
	cached_cut_die_sites: like cut_die_sites
			-- Cached list of cut die sites
		
feature {NONE} -- Implementation

	horizontal_cuts_table: HASH_TABLE [like anchored_cut_location, DOUBLE]
			-- Horizontal cuts for pattern, indexed by location
		local
			list: like cut_locations
			cut: like anchored_cut_location
		do
			if cached_horizontal_cuts_table = Void then
				create cached_horizontal_cuts_table.make (25)
				from
					list := cut_locations
					list.start
				until
					list.after
				loop
					cut := list.item
					if cut.is_horizontal then
						add_cut_to_table (cut, cached_horizontal_cuts_table)
					end
					list.forth
				end
			end
			Result := cached_horizontal_cuts_table
		end
	
	vertical_cuts_table: HASH_TABLE [like anchored_cut_location, DOUBLE]
			-- Vertical cuts for pattern, indexed by location
		local
			list: like cut_locations
			cut: like anchored_cut_location
		do
			if cached_vertical_cuts_table = Void then
				create cached_vertical_cuts_table.make (25)
				from
					list := cut_locations
					list.start
				until
					list.after
				loop
					cut := list.item
					if cut.is_vertical then
						add_cut_to_table (cut, cached_vertical_cuts_table)
					end
					list.forth
				end
			end
			Result := cached_vertical_cuts_table
		end
	
	table_for_cut (a_cut: like anchored_cut_location): like horizontal_cuts_table
			-- Appropriate hash table for `a_cut', depending
			-- on whether it is horizontal or vertical
		do
			if a_cut.is_horizontal then
				Result := horizontal_cuts_table
			else
				Result := vertical_cuts_table
			end
		end

	add_cut_to_table (a_cut: like anchored_cut_location a_table: like horizontal_cuts_table)
			-- Add `a_cut' to `a_table'
		require
			a_cut_not_void: a_cut /= Void
			a_table_not_void: a_table /= Void
		do
			a_table.put (a_cut, grid.rounded_center (a_cut.location))
		end

	remove_cut_from_table (a_cut: like anchored_cut_location a_table: like horizontal_cuts_table)
			-- Remove `a_cut' from `a_table'
		require
			a_cut_not_void: a_cut /= Void
			a_table_not_void: a_table /= Void
		do
			a_table.remove (grid.rounded_center (a_cut.location))
		end

	isolated_die_locations: HASH_TABLE [like anchored_cut_die_site, like anchored_die_location]
			-- Die locations isolated by pattern
		local
			list: like cut_die_sites
			cut_site: like anchored_cut_die_site
		do
			if cached_isolated_die_locations = Void then
				create cached_isolated_die_locations.make (25)
				from
					list := cut_die_sites
					list.start
				until
					list.after
				loop
					cut_site := list.item
					cached_isolated_die_locations.put (cut_site, cut_site.die_location)
					list.forth
				end
			end
			Result := cached_isolated_die_locations
		end
	
	cached_horizontal_cuts_table: like horizontal_cuts_table
			-- Horizontal cuts for pattern, indexed by location
			-- and cached
			
	cached_vertical_cuts_table: like vertical_cuts_table
			-- Vertical cuts for pattern, indexed by location
			-- and cached
			
	cached_isolated_die_locations: HASH_TABLE [like anchored_cut_die_site, like anchored_die_location]
			-- Cut die sites isolated by pattern, indexed by 
			-- die location and cached
			
	fetch_cut_locations
			-- Fetch cut locations for `Current'
		local
			row: like anchored_cut_location
		do
			create row.make_template
			cached_cut_locations := row.rows_for_packing_version_and_pat (packing_id, version_no, cut_pattern_no, any_connection, expected_count.At_least_zero)
		end
		
	fetch_cut_die_sites
			-- Fetch cut die sites for `Current'
		local
			row: like anchored_cut_die_site
		do
			create row.make_template
			cached_cut_die_sites := row.rows_for_packing_version_and_pat (packing_id, version_no, cut_pattern_no, any_connection, expected_count.At_least_zero)
		end
		
	anchored_die: PACKING_DIES_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

	anchored_die_location: PACKING_DIE_LOCATIONS_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

	anchored_cut_location: PACKING_CUT_LOCATIONS_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

	anchored_cut_die_site: PACKING_CUT_DIE_SITES_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

	cached_etest_name: like etest_name
			-- Row for `wafer_etest_name', cached in memory

end
