
note

	author: "David Hollenberg"

class PACKING_DIE_LOCATIONS_MROW
inherit
	PACKING_DIE_LOCATIONS_MRAW
		redefine
			detached_twin, copy_from_row
		end
	PACKING_DIE_LOCATIONS_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		end
	DB_DETACHABLE_ROW
		export
			{PACKING_DIE_LOCATIONS_MROW} detach_actions_internal
		undefine
			set_default_values
		redefine
			detached_twin, copy_from_row
		end

create
	make_template, make_null, make_by_key, make_detached

feature -- Row copy

	copy_from_row (r: like original_row)
			-- Initialize `Current' from `r'
		do
			precursor {PACKING_DIE_LOCATIONS_MRAW} (r)
			tier_3dic := r.tier_3dic
			detach_actions_internal := r.detach_actions_internal
		end

feature -- Properties

	detached_twin: like Current
			-- New object that is the same as `Current' but
			-- is detached from any database connection
		do
			Result := precursor {PACKING_DIE_LOCATIONS_MRAW}
			call_detach_actions (Result)
		end

	deep_detached_twin: like Current
			-- Detached copy of `Current' with all
			-- dependent rows detached as well
		do
			Result := detached_twin
		end

	is_deep_unchanged: BOOLEAN
			-- Are `Current' and all its dependent rows
			-- unchanged from their original versions?
			-- Returns False if no original version
		do
			Result := all_columns_equal_original
		end

feature -- Operations

	move_by (a_distance: CONST_POINT)
			-- Move `Current' by `a_distance'
		require
			a_distance_not_void: a_distance /= Void
		do
			if a_distance.x /= 0.0 then
				set_location_x (location_x + a_distance.x)
			end
			if a_distance.y /= 0.0 then
				set_location_y (location_y + a_distance.y)
			end
		end

	update_for_new_version (n: INTEGER)
			-- Update `Current' with information for new
			-- version `n'
		do
			set_version_no (n)
		end

	register_deep
			-- Register `Current' and all dependent rows
			-- on implicit connection
		do
			register_on_implicit_connection
		end

	set_default_packer_attributes (is_rot, can_move: BOOLEAN)
			-- Set attributes specific to the packer,
			-- where `rot' indicates whether location
			-- is rotatable and `can_move' indicates
			-- whether location can move or be duplicated
			-- or deleted
		do
			set_is_movable_in_x (can_move)
			set_is_movable_in_y (can_move)
			set_is_fixed_in_x (not can_move)
			set_is_fixed_in_y (not can_move)
			set_is_duplicatable (can_move)
			set_is_deletable (can_move)
			set_is_rotatable (is_rot)
		end

	set_fixed_vendor_geometry_packer_attributes
			-- Set attributes specific to the packer for
			-- a vendor geometry/ghost that has fixed locations
		do
			set_default_packer_attributes (False, False)
		end

	set_movable_vendor_geometry_packer_attributes (is_rot: BOOLEAN)
			-- Set attributes specific to the packer for
			-- a vendor geometry/ghost that has quantities
			-- rather than fixed locations, where `is_rot'
			-- indicates whether it is rotatable
		do
			set_default_packer_attributes (is_rot, True)
		end

end
