
note

	author: "David Hollenberg"

class PACKING_DIES_MROW
inherit
	PACKING_DIES_MRAW
		redefine
			detached_twin, copy_from_row
		end
	PACKING_DIES_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		redefine
			anchored_die_location, anchored_project,
			die_locations, project, set_die_locations, set_project
		end
	DB_DETACHABLE_ROW
		export
			{PACKING_DIES_MROW} detach_actions_internal
		redefine
			detached_twin, copy_from_row
		end

create
	make_template, make_null, make_by_key, make_detached

feature -- Properties

	die_locations: LIST [like anchored_die_location]
			-- Die locations for `Current'
		do
			if is_detached then
				Result := detached_die_locations
			else
				if cached_die_locations = Void then
					fetch_die_locations
				end
				Result := cached_die_locations
			end
		end

	uniform_white_space_border: DOUBLE
			-- Minimum amount of white space on all sides
			-- of project
		require
			is_project: is_cust_die or is_test_vehicle
		do
			Result := packing.uniform_white_space_border (project.overhead_style)
		end

	project: like anchored_project
			-- Project for `Current'
		do
			if is_detached then
				Result := detached_project
			else
				if cached_project = Void then
					fetch_project
				end
				Result := cached_project
			end
		end

	detached_twin: like Current
			-- New object that is the same as `Current' but
			-- is detached from any database connection
		do
			Result := precursor {PACKING_DIES_MRAW}
			call_detach_actions (Result)
		end

	deep_detached_twin: like Current
			-- Detached copy of `Current' with all
			-- dependent rows detached as well
		do
			Result := detached_twin
			Result.set_die_locations (die_locations.twin)
			if has_project then
				Result.set_project (project)
			end
			Result.uncache_dependents
				-- Just in case: shouldn't be referenced
			Result.detach_dependents_deep
		end

	is_deep_unchanged: BOOLEAN
			-- Are `Current' and all its dependent rows
			-- unchanged from their original versions?
			-- Returns False if no original version
		do
		end

	last_die_location: like anchored_die_location
			-- Last die location added by `add_die_location'

feature -- Row copy

	copy_from_row (r: like original_row)
			-- Initialize `Current' from `r'
		do
			precursor {PACKING_DIES_MRAW} (r)
			detach_actions_internal := r.detach_actions_internal
			cached_die_locations := r.die_locations
			cached_project := r.cached_project
			cached_vendor_geometry := r.cached_vendor_geometry
			wafer_process := r.wafer_process
			tier_3dic := r.tier_3dic
			max_site_no := r.max_site_no
			packing := r.packing
		end

feature -- Modification

	set_attributes_from_project (proj: like anchored_project a_packing: PACKINGS_ROW)
			-- Copy attributes of `proj', which is part of 
			-- `a_packing', into `Current'
		require
			proj_not_void: proj /= Void
		local
			wt: WAFER_TECHNS_ROW
			size: CONST_POINT
		do
			set_project (proj)
			set_project_no (proj.project_no)
			set_project_version_no (proj.project_version_no)
			set_null_wafer_etest_name
			wt := a_packing.wafer_techns_row
			size := wt.die_size_with_overhead (a_packing.effective_overhead_style (proj.overhead_style), a_packing.other_border, proj.design_size)
			set_die_size_printed_x (size.x)
			set_die_size_printed_y (size.y)
			if (die_wafer_rotation \\ 180) = 90 then
				swap_die_size_x_and_y
			end
			packing := a_packing
		end

	set_die_locations (list: like die_locations)
			-- Set `die_locations' to `list'
		do
			if is_detached then
				detached_die_locations := list
			else
				precursor (list)
			end
		end

	add_die_location (fld_name: STRING loc_x, loc_y: DOUBLE is_rot, can_move: BOOLEAN)
			-- Add new die location in field `fld_name',
			-- whose center is at (`loc_x', `loc_y')
			-- relative to center of field.  `is_rot'
			-- indicates whether location is rotatable and
			-- `can_move' indicates whether location can
			-- be moved, duplicated or deleted.  Leave new
			-- location in `last_die_location'
		require
			is_detached: is_detached
		local
			dl: like anchored_die_location
		do
			create dl.make_detached (any_connection)
			dl.set_packing_id (packing_id)
			dl.set_version_no (version_no)
			dl.set_die_no (die_no)
			dl.set_site_no (highest_used_site_number + 1)
			dl.set_field_name (fld_name)
			dl.set_location_x (loc_x)
			dl.set_location_y (loc_y)
			dl.set_default_packer_attributes (is_rot, can_move)
			detached_die_locations.extend (dl)
			last_die_location := dl
		end

	add_existing_die_location (a_die_location: like anchored_die_location)
			-- Add existing die location `dl' (for undo/redo)
		require
			a_die_location_not_void: a_die_location /= Void
		do
			detached_die_locations.extend (a_die_location)
		end

	remove_die_location (loc: PACKING_DIE_LOCATIONS_MROW)
			-- Remove die location `loc'
		require
			is_detached: is_detached
		do
			detached_die_locations.prune_all (loc)
		end

	set_project (p: like project)
			-- Set `project' to `p'
		do
			if is_detached then
				detached_project := p
			else
				precursor (p)
			end
		end

	assign_wafer_process
		require
			is_cust_die: is_cust_die
		do
			set_wafer_process (project.assigned_process)
		end

	assign_wafer_etest_name
			-- Assign wafer etest name to `Current' if
			-- there is only one possibility
		require
			is_cust_die: is_cust_die
		do
			if project.compatible_wafer_etest_names.count = 1 then
				set_wafer_etest_name (project.compatible_wafer_etest_names.first.wafer_etest_name)
			end
		end

	propagate_assigned_etest_name (table: HASH_TABLE [WAFER_PROCESSES_ROW, STRING])
			-- Propagate assigned wafer etest name to the
			-- project associated with `Current', using `table'
			-- to look up wafer processes
		require
			is_cust_die: is_cust_die
		do
			project.set_assigned_wafer_etest_name (assigned_wafer_etest_name, table)
		end

	set_official_processes
			-- Set assigned process from official data
			-- (obtained from project, where it has
			-- already been set)
		require
			is_cust_die: is_cust_die
		local
			l_name: STRING
		do
			assign_wafer_process
			l_name := project.compatible_wafer_etest_names.first.wafer_etest_name
			set_wafer_etest_name (l_name)
			if original_row /= Void then
				-- Avoid spurious mismatch with original
				-- row, which makes packer think row has changed
				original_row.set_wafer_etest_name (l_name)
			end
			set_alternate_wafer_etest_names (project.compatible_wafer_etest_names)
		end

	save_wafer_etest_name
			-- Save current `wafer_etest_name', if any, in
			-- `saved_wafer_etest_name'
		do
			if not is_null_wafer_etest_name then
				saved_wafer_etest_name := wafer_etest_name
			else
				saved_wafer_etest_name := Void
			end
		end

	restore_wafer_etest_name
			-- Restore `wafer_etest_name' from
			-- `saved_wafer_etest_name, if project is
			-- still compatible with the latter
		do
			if saved_wafer_etest_name /= Void and then compatible_wafer_etest_names.there_exists (agent {WAFER_ETEST_NAMES_ROW}.has_name (saved_wafer_etest_name)) then
				set_wafer_etest_name (saved_wafer_etest_name)
			end
			saved_wafer_etest_name := Void
		end

feature -- Operations

	set_rotation (n: INTEGER)
			-- Set rotation to `n' degrees clockwise
		local
			old_rot: INTEGER
		do
			old_rot := die_wafer_rotation
			set_die_wafer_rotation (n)
			if ((n - old_rot).abs \\ 180) = 90 then
				swap_die_size_x_and_y
			end
		end

	toggle_rotation
			-- Toggle rotation between 0 and 90 degrees clockwise
		local
			switch: BOOLEAN
		do
			if die_wafer_rotation = 0 then
				switch := True
				set_die_wafer_rotation (90)
			else
				switch := is_rotated_sideways
				set_die_wafer_rotation (0)
			end
			if switch then
				swap_die_size_x_and_y
			end
		end

	rotate_90_clockwise
			-- Rotate die 90 degrees clockwise
		local
			rot: INTEGER
		do
			rot := (die_wafer_rotation + 90) \\ 360
			set_die_wafer_rotation (rot)
			swap_die_size_x_and_y
		end

	rotate_90_counterclockwise
			-- Rotate die 90 degrees counterclockwise
		local
			rot: INTEGER
		do
			rot := (die_wafer_rotation + 270) \\ 360
			set_die_wafer_rotation (rot)
			swap_die_size_x_and_y
		end

	update_for_new_version (n: INTEGER)
			-- Update `Current' with information for new
			-- version `n'
		do
			set_version_no (n)
			update_dependents_for_version (n)
			if has_project then
				set_project_version_no (project.project_version_no)
			end
		end

	register_deep
			-- Register `Current' and all dependent rows
			-- on implicit connection
		do
			register_project_deep
			register_on_implicit_connection
			register_dependents_deep
		end

feature {PACKING_DIES_MROW} -- Implementation

	detach_dependents_deep
			-- Detach all dependent rows
		do
			detach_die_locations_deep
			if has_project then
				set_project (project.deep_detached_twin)
			end
		end

feature {NONE} -- Implementation

	set_alternate_wafer_etest_names (list: LIST [WAFER_ETEST_NAMES_ROW])
			-- Set `alternate_wafer_etest_names' and
			-- `alternate_assigned_wafer_etest_names' from
			-- the names in rows in `list'
		do
			create {ARRAYED_LIST [STRING]} alternate_wafer_etest_names.make (list.count)
			create {ARRAYED_LIST [WAFER_ETEST_NAMES_ROW]} alternate_assigned_wafer_etest_names.make (list.count)
			alternate_wafer_etest_names.compare_objects
			alternate_assigned_wafer_etest_names.compare_objects
			list.do_all (agent add_alternate_wafer_etest_name)
		end

	add_alternate_wafer_etest_name (row: WAFER_ETEST_NAMES_ROW)
			-- Add wafer etest name in `row' to
			-- `alternate_wafer_etest_names' and
			-- to `alternate_assigned_wafer_etest_names'
		do
			alternate_wafer_etest_names.extend (row.wafer_etest_name)
			alternate_assigned_wafer_etest_names.extend (row)
		end

	detach_die_locations_deep
			-- Detach die locations deeply
		do
			original_die_locations := die_locations.twin
			original_die_locations.wipe_out
			original_die_locations.append (die_locations)
			die_locations.wipe_out
			original_die_locations.do_all (agent append_detached_die_location (?, die_locations))
		end

	append_detached_die_location (dl: like anchored_die_location list: like die_locations)
			-- Append detached copy of `dl' to `list'
		do
			list.extend (dl.deep_detached_twin)
		end
		
	update_dependents_for_version (n: INTEGER)
			-- Update all dependent rows prior to save of
			-- new version `n'
		do
			max_site_no := 0
			die_locations.do_all (agent renumber_die_location)
			die_locations.do_all (agent {like anchored_die_location}.update_for_new_version (n))
		end

	highest_used_site_number: INTEGER
			-- Highest used site number among all die locations
		local
			locs: like die_locations
		do
			locs := die_locations
			from
				locs.start
			until
				locs.after
			loop
				Result := Result.max (locs.item.site_no)
				locs.forth
			end
		end
	
	renumber_die_location (dl: PACKING_DIE_LOCATIONS_MROW)
		do
			max_site_no := max_site_no + 1
			dl.set_die_no (die_no)
			dl.set_site_no (max_site_no)
		end
	
	register_project_deep
			-- Register project row (if any) on implicit
			-- connection.  Must be done before `Current'
			-- is registered to avoid foreign key constraint
			-- violation
		do
			-- Move "detached" versions back to "cached" versions
			-- since they will no longer be detached at end
			-- of routine
			
			if has_project then
				if not project.is_deep_unchanged then
					project.register_deep
				end
				cached_project := detached_project
				detached_project := Void
			end
		end

	register_dependents_deep
			-- Register all dependent rows on implicit
			-- connection
		do
			-- Move "detached" versions back to "cached" versions
			-- since they will no longer be detached at end
			-- of routine
			
			cached_die_locations := detached_die_locations
			detached_die_locations := Void
			die_locations.do_all (agent {like anchored_die_location}.register_deep)
		end

	swap_die_size_x_and_y
			-- Swap `die_size_x' and `die_size_y'
		local
			temp: like die_size_printed_x
		do
			temp := die_size_printed_x
			set_die_size_printed_x (die_size_printed_y)
			set_die_size_printed_y (temp)
		end

	saved_wafer_etest_name: like wafer_etest_name
			-- Value of `wafer_etest_name' when last
			-- saved.  Used to avoid losing lot split
			-- assignments, if still compatible, when
			-- projects are updated or locked

	anchored_die_location: PACKING_DIE_LOCATIONS_MROW
			-- For anchoring only
		do
		end

	anchored_project: PACKING_PROJECTS_MROW
			-- For anchoring only
		do
		end

	detached_die_locations: like die_locations
			-- List of detached die locations (no explict 
			-- DB connection)

	detached_project: like project
			-- Detached project (no explict DB connection)

	original_die_locations: like die_locations
			-- Original list of die locations before detach

feature {PACKING_DIES_MROW} -- Implementation

	max_site_no: INTEGER
			-- Last site number assigned during renumbering

end
