
note

	author: "David Hollenberg"

class PACKING_PARTITIONS_ROW
inherit
	PACKING_PARTITIONS_RAW
	DB_HASHABLE_ROW
		undefine
			initialize_new, initialize_old, fill_in
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	HAS_LOWER_LEFT_CORNER

create
	make_template, make_null, make_by_key, make_detached

feature -- Queries

	rows_for_packing_version (a_packing_id: STRING a_version: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Partitions for `a_packing_id' and `a_version'
		require
			a_packing_id_not_void: a_packing_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SI
		do
			create b.make (a_packing_id, a_version)
			Result := all_rows_filtered (conn, "packing_id = :1 and version_no = :2", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	rows_for_run (a_run_id: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Rows for packing whose intended run is `a_run_id'
		require
			a_run_id_not_void: a_run_id /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (a_run_id)
			Result := all_rows_filtered (conn, "packing_id in (select packing_id from packings where intended_run = :1)", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	hash_code: INTEGER
		do
			Result := concatenated_key_columns.hash_code
		end
	
	concatenated_key_columns: STRING
			-- Key columns concatenated together and
			-- separated by null characters
		do
			create Result.make (packing_id.count + 10)
			Result.append (packing_id)
			Result.extend ('%U')
			Result.append_integer (version_no)
			Result.extend ('%U')
			Result.append_integer (partition_no)
		end
	
	packing: PACKINGS_ROW
			-- Packing of which this partition is a part

	max_db_value: DOUBLE
			-- Maximum value that can fit in the corresponding
			-- database table
		require
			same_x_and_y_precision: precision_location_x = precision_location_y
			same_x_and_y_scale: scale_location_x = scale_location_y
		once
			Result := 10 ^ (precision_location_x - scale_location_x) - 10 ^ -scale_location_x
		end

	etest_name: WAFER_ETEST_NAMES_ROW
			-- Wafer etest names row for etest name
			-- for `Current'
		do
			if cached_etest_name = Void then
				create cached_etest_name.make_by_key (any_connection, option.Old_row, wafer_etest_name)
			end
			Result := cached_etest_name
		ensure
			result_not_void: Result /= Void
		end

	wafer_etest_id: STRING
			-- Wafer etest ID for `Current'
		do
			Result := etest_name.wafer_etest_id
		ensure
			result_not_void: Result /= Void
		end

	is_cached_etest_name: BOOLEAN
			-- Is `etest_name' currently cached?
		do
			Result := cached_etest_name /= Void
		end

	partition_size: CONST_POINT
			-- Size of `Current'
		do
			create Result.make_xy (partition_size_x, partition_size_y)
		end

	lower_left_corner: CONST_POINT
			-- <Precursor>
		do
			create Result.make_xy (location_x - (partition_size_x / 2.0), location_y - (partition_size_y / 2.0))
		end
	
feature -- Modification

	set_packing (a_packing: like packing)
			-- Set `packing' to `a_packing'
		do
			packing := a_packing
		end

	uncache_etest_name
			-- Remove etest name from cache, if present
		do
			cached_etest_name := Void
		ensure
			etest_name_not_cached: not is_cached_etest_name
		end

feature {NONE} -- Implementation

	cached_etest_name: WAFER_ETEST_NAMES_ROW
			-- Wafer etest names row for Current,
			-- cached in memory

end
