
note

	author: "David Hollenberg"

class PACKING_PROJECT_OPTIONS_ROW
inherit
	PACKING_PROJECT_OPTIONS_RAW
	PACKING_QUERIES
	ABSTRACT_PROCESS_OPTION
	HAS_FAB_OPTION
	DB_HASHABLE_ROW
		undefine
			initialize_new, initialize_old, fill_in
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	DB_QUERY_OPTIONS
		export {NONE}	all
		end

create
	make_template, make_null, make_by_key

feature -- Queries

	rows_for_project_version (a_packing_id: STRING a_project_no, a_proj_version: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Options for `a_packing_id', `a_project_no' and
			-- `a_proj_version'
		require
			a_packing_id_not_void: a_packing_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SII
		do
			create b.make (a_packing_id, a_project_no, a_proj_version)
			Result := all_rows_filtered (conn, "packing_id = :1 and project_no = :2 and project_version_no = :3", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	hash_code: INTEGER
		local
			s: STRING
		do
			create s.make (packing_id.count + option_id.count + 10)
			s.append (packing_id)
			s.extend ('%U')
			s.append_integer (project_no)
			s.extend ('%U')
			s.append_integer (project_version_no)
			s.extend ('%U')
			s.append (option_id)
			Result := s.hash_code
		end
	
	is_layer_option: BOOLEAN
			-- Is `Current' an option determined by layers
			-- present in design?
		do
			Result := not is_explicit
		end

	is_explicit_option: BOOLEAN
			-- Is `Current' an option that was explicitly
			-- requested by customer?
		do
			Result := is_explicit
		end

	display_name: STRING
			-- Option name for display purposes.
			-- Option ID is preceded by minus sign
			-- if not `option_in_process' and an
			-- asterisk is appended if option is
			-- explicit
		do
			create Result.make (option_id.count + 2)
			if not option_in_process then
				Result.extend ('-')
			end
			Result.append (option_id)
			if is_explicit then
				Result.extend ('*')
			end
		end

feature -- Operations

	add_to_list (list: LIST [like Current])
		require
			list_not_void: list /= Void
		do
			list.extend (Current)
		end

feature	-- the rest of essence_of_option

	option_name: STRING
	do  Result := option_id_row.option_name
	end;

	option_category: STRING
	do  Result := option_id_row.option_category
	end;

feature	-- my OPTION_IDS_ROW so I can know about myself

	option_id_row: OPTION_IDS_ROW
	do  if cached_option = Void
	     then create cached_option.make_by_key
				(connection, old_row, option_id)
	    end;
	    Result := cached_option
	end;

feature {NONE} -- cache

	cached_option: OPTION_IDS_ROW

end
