
note

	author: "David Hollenberg"

class PACKING_PROJECT_PACKAGINGS_ROW
inherit
	PACKING_PROJECT_PACKAGINGS_RAW
	PACKING_QUERIES
	DB_HASHABLE_ROW
		undefine
			initialize_new, initialize_old, fill_in
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	PACKAGING_UTILITIES_INTERFACE
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key, make_detached

feature -- Queries

	rows_for_project_version (a_packing_id: STRING a_project_no, a_proj_version: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Packagings for `a_packing_id', `a_project_no' and
			-- `a_proj_version'
		require
			a_packing_id_not_void: a_packing_id /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SII
		do
			create b.make (a_packing_id, a_project_no, a_proj_version)
			Result := all_rows_filtered (conn, "packing_id = :1 and project_no = :2 and project_version_no = :3", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

feature -- Properties

	hash_code: INTEGER
		local
			s: STRING
		do
			create s.make (packing_id.count + 15)
			s.append (packing_id)
			s.extend ('%U')
			s.append_integer (project_no)
			s.extend ('%U')
			s.append_integer (project_version_no)
			s.extend ('%U')
			s.append_integer (line_no)
			Result := s.hash_code
		end
	
	max_die_size: CONST_POINT
			-- Maximum die size dictated by `Current'
			-- or Void if no maximum limit.
			-- If required rotation is null, then
			-- the size in X will be less than or
			-- equal to the size in Y
		do
			if not is_cached_max_die_size then
				fetch_max_die_size
			end
			Result := cached_max_die_size
		end

	is_cached_max_die_size: BOOLEAN
			-- Is `max_die_size' currently cached?

	package: PACKAGES_ROW
			-- Package for `Current'
		require
			package_id_not_null: not is_null_package_id
		do
			if not is_cached_package then
				fetch_package
			end
			Result := cached_package
		end

	is_cached_package: BOOLEAN
			-- Is `package' currently cached?
		do
			Result := cached_package /= Void
		end

	display_name: STRING
			-- Text summarizing significant row for
			-- display purposes
		do
			create Result.make (100)
			Result.append ("Pkg: ")
			if not is_null_package_id then
				Result.append (package_id)
			else
				Result.append ("null")
			end
			if is_null_packaged then
				Result.extend ('*')
			elseif packaged then
				Result.extend ('+')
			else
				Result.extend ('-')
			end
			Result.extend (' ')
			Result.append ("Downbond: ")
			if is_null_downbond_location_reqd then
				Result.append ("null")
			else
				Result.append (downbond_location_reqd)
			end
			Result.extend (' ')
			Result.append ("Rot: ")
			if is_null_design_package_rotation_reqd then
				Result.append ("null")
			else
				Result.append_integer (design_package_rotation_reqd)
			end
			Result.extend (' ')
			Result.append ("Thickness: ")
			if is_null_die_thickness_mils then
				Result.append ("null")
			else
				Result.append_double (die_thickness_mils)
			end
			Result.extend (' ')
			Result.append ("Ord: ")
			if is_null_quantity_ordered then
				Result.append ("null")
			else
				Result.append_integer (quantity_ordered)
			end
			Result.extend (' ')
			Result.append ("Pkgd: ")
			if is_null_quantity_packaged_requested then
				Result.append ("null")
			else
				Result.append_integer (quantity_packaged_requested)
			end
			Result.extend (' ')
			Result.append ("Req Pkg: ")
			if is_null_package_requested then
				Result.append ("null")
			else
				Result.append (package_requested)
			end
		end

feature -- Operations

	uncache_max_die_size
			-- Remove `max_die_size' from cache
		do
			cached_max_die_size := Void
			is_cached_max_die_size := False
		end

	uncache_package
			-- Remove `package' from cache
		do
			cached_package := Void
		end

feature {NONE} -- Implementation

	fetch_max_die_size
			-- Fetch maximum die size dictated by `Current'
		local
			p: like max_die_size
		do
			if not is_null_package_id then
				p := package_max_die_size (any_connection, package_id, downbond_location_reqd)
				if is_null_design_package_rotation_reqd then
					if p.y < p.x then
						-- Put smaller dimension first
						create p.make_xy (p.y, p.x)
					end
				elseif design_package_rotation_reqd = 90 or design_package_rotation_reqd = 270 then
					create p.make_xy (p.y, p.x)
				end
			else
				p := Void
			end
			cached_max_die_size := p
			is_cached_max_die_size := True
		end

	fetch_package
			-- Fetch package for `Current'
		require
			package_id_not_null: not is_null_package_id
		local
			row: PACKAGES_ROW
		do
			create row.make_template
			cached_package := row.packages_with_id (package_id, any_connection, expected_count.Exactly_one).first
		end

feature {PACKING_PROJECT_PACKAGINGS_ROW} -- Implementation

	cached_max_die_size: CONST_POINT
			-- Maximum die size dictated by `Current',
			-- cached in memory

	cached_package: PACKAGES_ROW
			-- Package for `Current', cached in memory

end
