
note

	author: "David Hollenberg"

class PACKING_PROJECTS_MROW
inherit
	PACKING_PROJECTS_MRAW
		redefine
			copy_from_row
		end
	PACKING_PROJECTS_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		redefine
			anchored_packaging, anchored_option,
			anchored_fab_instance, anchored_cust_ic_design,
			packagings, options, set_packagings, set_options
		end

create
	make_template, make_null, make_by_key, make_detached,
	make_cust_project, make_test_vehicle,
	make_test_vehicle_from_fab_instance

feature {NONE} -- Creation

	make_cust_project (fno: INTEGER date: DB_DATE old_proj: PACKING_PROJECTS_ROW prod_id: STRING a_packing: like packing conn: DB_CONNECTION)
			-- Create detached row for customer project
			-- with fab_no `fno', selection and
			-- synchronization dates `date'. `old_proj'
			-- is the highest version of project if not Void.
			-- `prod_id' is the production ID
		require
			date_not_void: date /= Void
			conn_not_void: conn /= Void
		local
			type: IC_FAB_INSTANCE_TYPE_VALUES
			ifi: IC_FAB_INSTANCES_ROW
		do
			make_detached (conn)
			set_packing (a_packing)
			set_ic_fab_instance_type (type.Customer_fab)
			set_date_selected (date)
			set_m_fab_no (fno)
			ifi := fab_instance
			set_design_no (ifi.design_no)
			initialize_cust_project (date, old_proj, prod_id)
			is_initialized := True
		end

	make_test_vehicle (dno: INTEGER date: DB_DATE old_proj: PACKING_PROJECTS_ROW a_packing: like packing conn: DB_CONNECTION)
			-- Create detached row for test vehicle with
			-- design_no `dno' with selection and
			-- synchronization dates set to `date'.
			-- `old_proj' is the highest version of test
			-- vehicle if not Void
		require
			date_not_void: date /= Void
			conn_not_void: conn /= Void
		do
			make_test_vehicle_common (dno, date, old_proj, a_packing, conn)
			initialize_test_vehicle (date, old_proj)
			is_initialized := True
		end

	make_test_vehicle_from_fab_instance (dno: INTEGER ifi: IC_FAB_INSTANCES_ROW date: DB_DATE old_proj: PACKING_PROJECTS_ROW prod_id: STRING a_packing: like packing conn: DB_CONNECTION)
			-- Create detached row for test vehicle with
			-- design_no `dno' with selection and
			-- synchronization dates set to `date'.
			-- `old_proj' is the highest version of test
			-- vehicle if not Void
		require
			ifi_not_void: ifi /= Void
			date_not_void: date /= Void
			prod_id_not_void: prod_id /= Void
			conn_not_void: conn /= Void
		do
			make_test_vehicle_common (dno, date, old_proj, a_packing, conn)
			initialize_test_vehicle_from_fab_instance (ifi, date, old_proj, prod_id)
			is_initialized := True
		end

feature -- Properties

	is_initialized: BOOLEAN
			-- Has `Current' been initialized yet?

	packagings: LIST [like anchored_packaging]
			-- Packagings for `Current'
		do
			if is_detached then
				Result := detached_packagings
			else
				if cached_packagings = Void then
					fetch_packagings
				end
				Result := cached_packagings
			end
		end

	options: LIST [like anchored_option]
			-- Options for `Current'
		do
			if is_detached then
				Result := detached_options
			else
				if cached_options = Void then
					fetch_options
				end
				Result := cached_options
			end
		end

	deep_detached_twin: like Current
			-- Detached copy of `Current' with all
			-- dependent rows detached as well
		do
			Result := detached_twin
			Result.set_packagings (packagings.twin)
			Result.set_options (options.twin)
			Result.uncache_dependents
				-- Just in case: shouldn't be referenced
			Result.detach_dependents_deep
		end

	is_deep_unchanged: BOOLEAN
			-- Are `Current' and all its dependent rows
			-- unchanged from their original versions?
			-- Returns False if no original version
		do
		end

	new_packagings (wo: CUST_WOS_ROW prod_id: STRING): LIST [like anchored_packaging]
			-- Create new list of packagings from database
			-- (assumes project is locked)
		local
			wolis: LIST [CUST_WO_LINE_ITEMS_ROW]
			woli: CUST_WO_LINE_ITEMS_ROW
			pkging: like anchored_packaging
			n: INTEGER
		do
			if wo /= Void then
				-- We have a work order
				wolis := wo.line_items
			else
				create {ARRAYED_LIST [CUST_WO_LINE_ITEMS_ROW]} wolis.make (0)
			end
			create {ARRAYED_LIST [like anchored_packaging]} Result.make (wolis.count)
			from
				n := 0
				wolis.start
			until
				wolis.after
			loop
				woli := wolis.item
				if not woli.is_null_ic_packaging_no then
					create pkging.make_from_line_item (Current, woli)
					n := n + 1
					pkging.set_line_no (n)
					Result.extend (pkging)
				end
				wolis.forth
			end
		end

feature -- Modification

	initialize_cust_project (date: DB_DATE old_proj: PACKING_PROJECTS_ROW prod_id: STRING)
			-- Initialize cust project `Current' from data
			-- in database, using `old_proj' as the
			-- existing row if not Void and `date'
			-- as the synchronization date and `prod_id'
			-- as the production ID
		local
			ifi: IC_FAB_INSTANCES_ROW
			opts: LIST [like anchored_option]
			opt: like anchored_option
			explicit_opts: LIST [IC_FAB_INSTANCE_OPTIONS_ROW]
			list: LIST [WAFER_PROCESS_OPTION]
		do
			uncache_dependents
			uncache_foreign_keys
			set_new_project_key (old_proj)
			set_date_synchronized (date)
			ifi := fab_instance
			set_run_type_requested (ifi.run_type_requested)
			set_run_date_requested (ifi.run_date_requested)
			set_run_id_copy (ifi.run_id)
			set_net_address (ifi.net_address)
			set_phone_number (ifi.phone_number)
			initialize_from_fab_instance (ifi, prod_id)
			
			create {ARRAYED_LIST [like anchored_option]} opts.make (5)
			explicit_opts := fab_instance.explicit_options
			from
				explicit_opts.start
			until
				explicit_opts.after
			loop
				create opt.make_from_explicit_option (Current, explicit_opts.item)
				opts.extend (opt)
				explicit_opts.forth
			end
			
			fetch_layer_options (packing.wafer_techn)
			from
				list := computed_layer_options
				list.start
			until
				list.after
			loop
				create opt.make_from_layer_option (Current, list.item)
				opts.extend (opt)
				list.forth
			end
			set_options (opts)
		end

	initialize_test_vehicle (date: DB_DATE old_proj: PACKING_PROJECTS_ROW)
			-- Initialize cust project `Current' from data
			-- in database, using `old_proj' as the
			-- existing row if not Void and `date' as
			-- the synchronization date
		local
			style: OVERHEAD_STYLE_VALUES
			l_style: STRING
			tv: M_TEST_VEHICLES_ROW
			pkging_list: ARRAYED_LIST [like anchored_packaging]
			pkging: like anchored_packaging
			pkg_id: STRING
		do
			uncache_dependents
			uncache_foreign_keys
			set_date_synchronized (date)
			set_new_project_key (old_proj)
			if cust_ic_design.is_null_label_box_size_x then
				l_style := style.External_
			else
				l_style := style.Internal
			end
			set_overhead_style (l_style)
			copy_ic_design_fields
			
			-- Don't have max_allowed_die_size_x and y values
			-- since no fab instance for test vehicle yet.
			-- These are user preference values, not limits
			-- dictated by packaging constraints
		
			-- Don't set project_state or run_id - not meaningful 
			-- for test vehicle
			
			-- Don't set quantity_requested or
			-- quantity_unpackaged_requested - don't have
			-- this info for test vehicle (and may not apply)
			
			tv := test_vehicle
			
			if not tv.is_null_design_wafer_rotation_reqd then
				set_design_wafer_rotation_reqd (tv.design_wafer_rotation_reqd)
			else
				set_null_design_wafer_rotation_reqd
			end

			-- Set details

			create pkging_list.make (1)
			create pkging.make_detached (any_connection)
			pkging.set_key_from_project (Current)
			pkging.set_line_no (1)
			pkg_id := tv.package_id
			pkging.set_package_id (pkg_id)
			pkging.set_package_requested (pkg_id)
			if not pkging.is_null_package_id then
				pkging.set_packaged (True)
			else
				pkging.set_null_packaged
			end
			pkging_list.extend (pkging)
			set_packagings (pkging_list)
			set_options (create {ARRAYED_LIST [PACKING_PROJECT_OPTIONS_MROW]}.make (0))
		end

	initialize_test_vehicle_from_fab_instance (ifi: IC_FAB_INSTANCES_ROW date: DB_DATE old_proj: PACKING_PROJECTS_ROW prod_id: STRING)
			-- Initialize cust project `Current' from data
			-- in database, using `old_proj' as the
			-- existing row if not Void and `date' as
			-- the synchronization date
		require
			ifi_not_void: ifi /= Void
			date_not_void: date /= Void
			prod_id_not_void: prod_id /= Void
		do
			uncache_dependents
			uncache_foreign_keys
			set_date_synchronized (date)
			set_new_project_key (old_proj)
			initialize_from_fab_instance (ifi, prod_id)
			set_options (create {ARRAYED_LIST [PACKING_PROJECT_OPTIONS_MROW]}.make (0))
		end

	set_assigned_wafer_etest_name (a_value: like assigned_wafer_etest_name table: HASH_TABLE [WAFER_PROCESSES_ROW, STRING])
			-- Set `assigned_wafer_etest_name' to `a_value'
		local
			list: like compatible_processes
		do
			assigned_wafer_etest_name := a_value
			list := compatible_processes
			compatible_processes_with_assigned_etest_name := list.twin
			compatible_processes_with_assigned_etest_name.wipe_out
			list.do_all (agent add_process_if_etest_compatible (?, table))
		end

	set_official_processes (ifip_table: HASH_TABLE [LIST [STRING], INTEGER] wp_table: HASH_TABLE [WAFER_PROCESSES_ROW, STRING] wen_table: HASH_TABLE [WAFER_ETEST_NAMES_ROW, STRING])
			-- Set process information
			-- (`compatible_processes',
			-- `compatible_wafer_etest_ids',
			-- `compatible_wafer_etest_names'
			-- `assigned_wafer_etest_name') from data in
			-- official database tables, instead of
			-- computing which processes are compatible
			-- based on options
		require
			is_customer_project: is_customer_project
		local
			procs: LIST [STRING]
		do
			ifip_table.search (m_fab_no)
			if ifip_table.found then
				procs := ifip_table.found_item
				set_compatible_processes (procs)
				create {ARRAYED_LIST [STRING]} compatible_wafer_etest_ids.make (2)
				compatible_wafer_etest_ids.compare_objects
				create {ARRAYED_LIST [WAFER_ETEST_NAMES_ROW]} compatible_wafer_etest_names.make (2)
				procs.do_all (agent add_etest_id (?, wp_table))
				set_assigned_process (procs.first)
				set_assigned_wafer_etest_name (compatible_wafer_etest_names.first, wp_table)
			else
				handle_hard_error ("could not find customer project with m_fab_no " + m_fab_no.out + " (design_no " + design_no.out + " ) in lookup table to get its list of ic fab instance processes")
			end
		end

	set_packagings (list: like packagings)
			-- Set `packagings' to `list'
		do
			if is_detached then
				detached_packagings := list
			else
				precursor (list)
			end
		end

	set_options (list: like options)
			-- Set `options' to `list'
		do
			if is_detached then
				detached_options := list
			else
				precursor (list)
			end
		end

	set_packing (p: like packing)
		do
			cached_packing := p
		end

	set_new_project_key (old_proj: PACKING_PROJECTS_ROW)
			-- Set key columns in `Current', using `old_proj'
			-- as the existing row if not Void
		local
			proj_no, proj_vers_no: INTEGER
		do
			if not packing_id.is_equal (packing.packing_id) then
				set_packing_id (packing.packing_id)
			end
			if old_proj /= Void then
				proj_no := old_proj.project_no
				proj_vers_no := old_proj.project_version_no + 1
			else
				if not is_initialized then
					-- Not in any saved version of packing and
					-- this is an addition of a new project
					-- rather than an update of an existing one.
					-- Allocate new project_no
					packing.increment_highest_used_project_no
					proj_no := packing.highest_used_project_no
				else
					proj_no := project_no
				end
				proj_vers_no := 1
			end
			if project_no /= proj_no then
				set_project_no (proj_no)
			end
			if project_version_no /= proj_vers_no then
				-- Update key field, skipping "rowid /= Void"
				-- check.  This is OK since we never try
				-- to lock or delete these rows
				set_project_version_no_without_key_check (proj_vers_no)
			end
		end

feature -- Row copy

	copy_from_row (r: like original_row)
			-- Initialize `Current' from `r'
		do
			precursor (r)
			assigned_process := r.assigned_process
			compatible_processes := r.compatible_processes
			computed_layer_options := r.computed_layer_options
			design_wt_layer_errors := r.design_wt_layer_errors
			cached_packing := r.cached_packing
			cached_packagings := r.cached_packagings
			cached_options := r.cached_options
			cached_cust_ic_design := r.cached_cust_ic_design
			cached_fab_instance := r.cached_fab_instance
			cached_test_vehicle := r.cached_test_vehicle
			is_initialized := r.is_initialized
			is_selected := r.is_selected
			reason_unselectable := r.reason_unselectable
			compatible_wafer_etest_ids := r.compatible_wafer_etest_ids
			compatible_wafer_etest_names := r.compatible_wafer_etest_names
			assigned_wafer_etest_name := r.assigned_wafer_etest_name
		end

feature -- Operations

	register_deep
			-- Register `Current' and all dependent rows
			-- on implicit connection
		do
			register_on_implicit_connection
			register_dependents_deep
		end

feature {PACKING_PROJECTS_MROW} -- Implementation

	detach_dependents_deep
			-- Detach dependent rows from database connection
			-- deeply
		do
			detach_packagings_deep
			detach_options_deep
		end

feature {NONE} -- Implementation

	set_project_version_no_without_key_check (k: INTEGER)
			-- Set `project_version_no' to `k', skipping check
			-- for change of key field
		do
			change_field (True, False)
			project_version_no := k
			is_null_project_version_no := False
		end

	detach_packagings_deep
			-- Detach `packagings' from database connection, along
			-- with all dependents
		do
			original_packagings := packagings.twin
			original_packagings.wipe_out
			original_packagings.append (packagings)
			packagings.wipe_out
			original_packagings.do_all (agent append_detached_packaging (?, packagings))
		end

	detach_options_deep
			-- Detach `options' from database connection, along
			-- with all dependents
		do
			original_options := options.twin
			original_options.wipe_out
			original_options.append (options)
			options.wipe_out
			original_options.do_all (agent append_detached_option (?, options))
		end

	append_detached_packaging (p: like anchored_packaging list: like packagings)
			-- Append detached copy of `p' to `list'
		do
			list.extend (p.detached_twin)
		end
		
	append_detached_option (o: like anchored_option list: like options)
			-- Append detached copy of `o' to `list'
		do
			list.extend (o.detached_twin)
		end
		
	update_dependents_with_version (n: INTEGER)
			-- Update all dependent rows with project_no `n'
			-- prior to save of new version `n'
		do
			last_line_no := 0
			packagings.do_all (agent renumber_packaging)
			packagings.do_all (agent {like anchored_packaging}.set_project_version_no (n))
			options.do_all (agent {like anchored_option}.set_project_version_no (n))
		end

	renumber_packaging (p: PACKING_PROJECT_PACKAGINGS_MROW)
		do
			last_line_no := last_line_no + 1
			p.set_line_no (last_line_no)
		end
	
	register_dependents_deep
			-- Register all dependent rows on implicit
			-- connection
		do
			-- Move "detached" versions back to "cached" versions
			-- since they will no longer be detached at end
			-- of routine
			
			cached_packagings := detached_packagings
			detached_packagings := Void
			packagings.do_all (agent {like anchored_packaging}.register_deep)
			
			cached_options := detached_options
			detached_options := Void
			options.do_all (agent {like anchored_option}.register_deep)
		end

	add_process_if_etest_compatible (a_process: STRING table: HASH_TABLE [WAFER_PROCESSES_ROW, STRING])
		do
			table.search (a_process)
			if table.found then
				if assigned_wafer_etest_id.is_empty or table.found_item.wafer_etest_id.is_equal (assigned_wafer_etest_id) then
					compatible_processes_with_assigned_etest_name.extend (a_process)
				end
			else
				handle_hard_error ("cannot find process " + a_process + " in lookup table to get its wafer etest ID")
			end
		end

	make_test_vehicle_common (dno: INTEGER date: DB_DATE old_proj: PACKING_PROJECTS_ROW a_packing: like packing conn: DB_CONNECTION)
			-- Create detached row for test vehicle with
			-- design_no `dno' with selection and
			-- synchronization dates set to `date'.
			-- `old_proj' is the highest version of test
			-- vehicle if not Void
		require
			date_not_void: date /= Void
			conn_not_void: conn /= Void
		local
			type: IC_FAB_INSTANCE_TYPE_VALUES
		do
			make_detached (conn)
			set_packing (a_packing)
			set_ic_fab_instance_type (type.Test_vehicle)
			set_date_selected (date)
			set_design_no (dno)
			set_test_vehicle_design_no (dno)
		end

	initialize_from_fab_instance (ifi: IC_FAB_INSTANCES_ROW prod_id: STRING)
			-- Initialize `Current' from fab instance
			-- `ifi' and production ID `prod_id'
		local
			wo: CUST_WOS_ROW
			quant, acct_no: INTEGER
		do
			set_date_fab_requested (ifi.date_fab_requested)
			set_overhead_style (ifi.overhead_style)
			if not ifi.is_null_max_allowed_die_size_x then
				set_max_allowed_die_size_x (ifi.max_allowed_die_size_x)
			else
				set_null_max_allowed_die_size_x
			end
			if not ifi.is_null_max_allowed_die_size_y then
				set_max_allowed_die_size_y (ifi.max_allowed_die_size_y)
			else
				set_null_max_allowed_die_size_y
			end
			if not ifi.is_null_allow_oversize_die_on_run then
				set_allow_oversize_die_on_run (ifi.allow_oversize_die_on_run)
			else
				set_null_allow_oversize_die_on_run
			end
			if not ifi.is_null_oversize_die_incl_conditions then
				set_oversize_die_incl_conditions (ifi.oversize_die_incl_conditions)
			else
				set_null_oversize_die_incl_conditions
			end
			wo := ifi.production_work_order (prod_id)
			if wo /= Void then
				quant := wo.ic_quantity_ordered
			else
				quant := 0
			end
			set_quantity_ordered (quant)
			if not ifi.is_null_quantity_requested then
				set_quantity_requested (ifi.quantity_requested)
			else
				set_null_quantity_requested
			end
			if not ifi.is_null_quantity_unpackaged_requested then
				set_quantity_unpackaged_requested (ifi.quantity_unpackaged_requested)
			else
				set_null_quantity_unpackaged_requested
			end
			if not ifi.is_null_design_wafer_rotation_reqd then
				set_design_wafer_rotation_reqd (ifi.design_wafer_rotation_reqd)
			else
				set_null_design_wafer_rotation_reqd
			end
			copy_ic_design_fields
			if wo /= Void then
				acct_no := wo.cust_acct_no
			else
				acct_no := cust_ic_design.cust_acct_no
			end
			set_cust_acct_no (acct_no)
			if not ifi.is_null_design_die_rotation then
				set_design_die_rotation (ifi.design_die_rotation)
			else
				set_null_design_die_rotation
			end

			set_packagings (new_packagings (wo, prod_id))
		end

	copy_ic_design_fields
			-- Initialize from info in `cust_ic_design'
		local
			cid: CUST_IC_DESIGNS_ROW
			state: DESIGN_ACCEPT_STATE_VALUES
		do
			cid := cust_ic_design
			cid.use_final_layout
			set_is_preliminary (cid.is_preliminary)
			set_design_techn_id (cid.design_techn_id)
			set_export_control_type (cid.export_control_type)
			set_design_name (cid.design_name)
			set_design_accept_state (cid.design_accept_state)
			set_cust_acct_no (cid.cust_acct_no)
			if cid.design_accept_state.is_equal (state.Passed) then
				set_is_design_size_estimate (False)
				if not cid.is_null_design_size_x then
					set_design_size_x (cid.design_size_x)
				else
					set_null_design_size_x
				end
				if not cid.is_null_design_size_y then
					set_design_size_y (cid.design_size_y)
				else
					set_null_design_size_y
				end
			else
				set_is_design_size_estimate (True)
				if not cid.is_null_est_design_size_x then
					set_design_size_x (cid.est_design_size_x)
				else
					set_null_design_size_x
				end
				if not cid.is_null_est_design_size_y then
					set_design_size_y (cid.est_design_size_y)
				else
					set_null_design_size_y
				end
			end
			set_design_die_rotation (0)
		end

	anchored_packaging: PACKING_PROJECT_PACKAGINGS_MROW
			-- For anchoring only
		do
		end

	anchored_option: PACKING_PROJECT_OPTIONS_MROW
			-- For anchoring only
		do
		end

	anchored_fab_instance: IC_FAB_INSTANCES_MROW
			-- For anchoring only
		do
		end

	anchored_cust_ic_design: CUST_IC_DESIGNS_MROW
			-- For anchoring only
		do
		end

	last_line_no: INTEGER
			-- Last line number assigned during renumbering

	detached_packagings: like packagings
			-- List of detached packagings (no explict DB 
			-- connection)
		
	detached_options: like options
			-- List of detached options (no explict DB connection)
		
	original_packagings: like packagings
			-- Original list of packagings before detach

	original_options: like options
			-- Original list of options before detach


end
