
note

	author: "David Hollenberg"

class PACKING_VERSIONS_MROW
inherit
	PACKING_VERSIONS_MRAW
		redefine
			detached_twin, copy_from_row
		end
	PACKING_VERSIONS_ROW
		undefine
			initialize_new,
			initialize_new_by_key
		redefine
			anchored_partition,
			anchored_die, anchored_wafer_process, 
			anchored_die_location,
			anchored_cut_pattern, anchored_cut_die_site,
			anchored_project, 
			partitions, set_partitions,
			dies, set_dies, cut_patterns, set_cut_patterns,
			wafer_processes, set_wafer_processes
		end
	DB_DETACHABLE_ROW
		export
			{PACKING_VERSIONS_MROW} detach_actions_internal
		redefine
			detached_twin, copy_from_row
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	ABSTRACT_EXT_CUT_DIST_SPEC

create
	make_template, make_null, make_by_key, make_detached,
	make_initial_version

feature {NONE} -- Creation

	make_initial_version (p: like anchored_packing ls: LITHO_SIZES_ROW conn: DB_CONNECTION)
			-- Create initial version for packing `p' on database
			-- connection `conn'
		require
			p_not_void: p /= Void
			conn_not_void: conn /= Void
			ls_not_void: ls /= Void
		local
			pt: PACKING_TARGETS_ROW
			ps, fs: CONST_POINT
		do
			make_detached (conn)
			set_packing_id (p.packing_id)
			set_version_no (0)
			set_null_parent_version_no
			set_date_created (conn.system_date)
			set_creator (conn.user_name)
			set_version_id ("Initial version")
			set_packing (p)
			set_partitions (create {ARRAYED_LIST [like anchored_partition]}.make (0))
			set_cut_patterns (create {ARRAYED_LIST [like anchored_cut_pattern]}.make (0))
			set_wafer_processes (create {ARRAYED_LIST [like anchored_wafer_process]}.make (0))

			max_die_no := 0
			select_internal_vendor_geometry
			set_selection_date
			parent.setup_for_fast_project_access
			select_high_priority_test_vehicles
			pt := ls.target
			if pt.is_custom then
				-- Initialize current payload size for this
				-- version from litho_sizes row
				ps := ls.payload_size
				set_payload_size_x (ps.x)
				set_payload_size_y (ps.y)
				if pt.is_reticle then
					-- Iniialize current field size too
					fs := ls.field_size
					set_field_size_x (fs.x)
					set_field_size_y (fs.y)
				end
			end
		end

feature -- Properties

	partitions: LIST [like anchored_partition]
			-- Partitions for `Current'
		do
			if is_detached then
				Result := detached_partitions
			else
				if cached_partitions = Void then
					fetch_partitions
				end
				Result := cached_partitions
			end
		end

	dies: LIST [like anchored_die]
			-- Dies for `Current'
		do
			if is_detached then
				Result := detached_dies
			else
				if cached_dies = Void then
					fetch_dies
				end
				Result := cached_dies
			end
		end

	cut_patterns: LIST [like anchored_cut_pattern]
			-- Cut patterns for `Current'
		do
			if is_detached then
				Result := detached_cut_patterns
			else
				if cached_cut_patterns = Void then
					fetch_cut_patterns
				end
				Result := cached_cut_patterns
			end
		end

	wafer_processes: LIST [like anchored_wafer_process]
			-- Wafer processes for `Current'
		do
			if is_detached then
				Result := detached_wafer_processes
			else
				if cached_wafer_processes = Void then
					fetch_wafer_processes
				end
				Result := cached_wafer_processes
			end
		end

	detached_twin: like Current
			-- New object that is the same as `Current' but
			-- is detached from any database connection
		do
			Result := precursor {PACKING_VERSIONS_MRAW}
			call_detach_actions (Result)
		end

	deep_detached_twin: like Current
			-- Detached copy of `Current' with all
			-- dependent rows detached as well
		do
			Result := detached_twin
			Result.set_partitions (partitions.twin)
			Result.set_dies (dies.twin)
			Result.set_wafer_processes (wafer_processes.twin)
			Result.set_cut_patterns (cut_patterns.twin)
			Result.uncache_dependents
				-- Just in case: shouldn't be referenced
			Result.detach_dependents_deep
			Result.connect_die_sites_to_locations
		end

	is_deep_unchanged: BOOLEAN
			-- Are `Current' and all its dependent rows
			-- unchanged from their original versions?
			-- Returns False if no original version
		do
		end

	is_cut_pattern_set_deep_unchanged: BOOLEAN
			-- Is set of cut patterns unchanged from the
			-- original set?  Returns False if no
			-- original version for a cut pattern or
			-- for the cut pattern set
		do
		end

	new_cust_die (m_fab_no: INTEGER): like anchored_die
			-- New cust die for fab instance identified by
			-- by `m_fab_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called)
		local
			type: DIE_TYPE_VALUES
			proj: like anchored_project
		do
			proj := new_cust_die_project (m_fab_no)
			Result := new_initialized_die (type.Cust_die)
			Result.set_attributes_from_project (proj, packing)
		end

	new_test_vehicle_die (design_no: INTEGER): like anchored_die
			-- New test vehicle die for test vehicle identified
			-- by `design_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called)
		local
			type: DIE_TYPE_VALUES
			proj: like anchored_project
		do
			proj := new_test_vehicle_project (design_no)
			Result := new_initialized_die (type.Test_vehicle)
			Result.set_attributes_from_project (proj, packing)
		end

	new_test_vehicle_die_from_fab_instance (design_no: INTEGER ifi: IC_FAB_INSTANCES_ROW): like anchored_die
			-- New test vehicle die for test vehicle identified
			-- by `design_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called).  Alternate version that
			-- gets some data from the fab instance (for use
			-- when viewing packing or editing cut patterns for
			-- the official packing)
		require
			ifi_not_void: ifi /= Void
		local
			type: DIE_TYPE_VALUES
			proj: like anchored_project
		do
			proj := new_test_vehicle_project_from_fab_instance (design_no, ifi)
			Result := new_initialized_die (type.Test_vehicle)
			Result.set_attributes_from_project (proj, packing)
		end

	new_horizontal_scribe: like anchored_die
			-- New scribe die which is horizontal, without
			-- size or locations
		local
			orient: ORIENTATION_VALUES
		do
			Result := new_scribe_with_orientation (orient.Horizontal)
		end
		
	new_vertical_scribe: like anchored_die
			-- New scribe die which is vertical, without
			-- size or locations
		local
			orient: ORIENTATION_VALUES
		do
			Result := new_scribe_with_orientation (orient.Vertical)
		end
		
	new_scribe_with_orientation (o: STRING): like anchored_die
			-- New scribe with orientation `o', without
			-- size or locations
		require
			orientation_not_void: o /= Void
			orientation_valid: is_valid_orientation (o)
		local
			type: DIE_TYPE_VALUES
		do
			Result := new_initialized_die (type.Scribe)
			Result.set_scribe_orientation (o)
		end
		
	is_valid_orientation (s: STRING): BOOLEAN
			-- Is `s' a valid orientation?
		require
			s_not_void: s /= Void
		local
			orient: ORIENTATION_VALUES
		do
			Result := s.is_equal (orient.Horizontal) or s.is_equal (orient.Vertical)
		end

	new_filler_with_name_and_type (s, a_type: STRING): like anchored_die
			-- New filler die with name `s'
			-- and type `a_type'
		require
			s_not_void: s /= Void
			a_type_not_void: a_type /= Void
		local
			type: DIE_TYPE_VALUES
		do
			Result := new_initialized_die (type.Filler)
			Result.set_filler_name (s)
			Result.set_filler_type (a_type)
		end
		
	new_vendor_geometry (g: GEOMETRIES_ROW): like anchored_die
			-- New vendor geometry/ghost die for `g'
		local
			locs: LIST [GEOMETRYXFIXED_LOCATIONS_ROW]
			quants: LIST [GEOMETRY_QUANTITIES_ROW]
			rot: INTEGER
		do
			Result := new_initialized_die (g.die_type)
			Result.set_geometry_id (g.geometry_id)
			rot := g.rotation
			Result.set_die_wafer_rotation (rot)
			if rot = 0 or rot = 180 then
				Result.set_die_size_printed_x (g.non_null_size_x)
				Result.set_die_size_printed_y (g.non_null_size_y)
			else
				Result.set_die_size_printed_x (g.non_null_size_y)
				Result.set_die_size_printed_y (g.non_null_size_x)
			end
			Result.set_vendor_geometry (g)
			
			locs := g.fixed_locations (packing.field_set_name)
			quants := g.quantities (packing.field_set_name)
			if not locs.is_empty and not quants.is_empty then
				handle_hard_error ("vendor geometry " + g.geometry_id + " has both fixed locations and a quantity specification")
			elseif not locs.is_empty then
				add_vendor_geometry_locations (locs, Result)
			else
				-- Don't create any locations if just has 
				-- a quantity spec
			end
		end

	new_vendor_geometry_official (g: GEOMETRIES_ROW): like anchored_die
			-- New vendor geometry/ghost die for `g'.
			-- Alternate version used when synthesizing
			-- a packing from official database tables
		do
			Result := new_initialized_die (g.die_type)
			Result.set_geometry_id (g.geometry_id)
			Result.set_vendor_geometry (g)
		end

	new_partition_with_etest_name (a_name: STRING): like anchored_partition
			-- New partition for etest name `a_name'
		require
			a_name_not_void: a_name /= Void
		do
			Result := new_initialized_partition (a_name)
		end
		
	new_cust_die_project (m_fab_no: INTEGER): like anchored_project
			-- New cust die project for project identified
			-- by `m_fab_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called).  Die points to 
			-- project
		do
			create Result.make_cust_project (m_fab_no, selection_date, parent.cust_project_highest_version (m_fab_no), parent.production_id, packing, any_connection)
		end

	new_test_vehicle_project (design_no: INTEGER): like anchored_project
			-- New test vehicle project for test vehicle identified
			-- by `design_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called).  Die points to 
			-- project
		do
			create Result.make_test_vehicle (design_no, selection_date, parent.test_vehicle_highest_version (design_no), packing, any_connection)
		end

	new_test_vehicle_project_from_fab_instance (design_no: INTEGER ifi: IC_FAB_INSTANCES_ROW): like anchored_project
			-- New test vehicle project for test vehicle identified
			-- by `design_no', with selection and synchronization
			-- dates set to `selection_date' (must set it before
			-- this routine is called).  Die points to 
			-- project.  Alternate version that gets some data
			-- from fab instance
		require
			ifi_not_void: ifi /= Void
		do
			create Result.make_test_vehicle_from_fab_instance (design_no, ifi, selection_date, parent.test_vehicle_highest_version (design_no), parent.production_id, packing, any_connection)
		end

	ordered_processes: LIST [STRING]
			-- Names of processes to be ordered computed by
			-- last call to `assign_process_to_projects'

	added_projects: LIST [like anchored_project]
			-- Projects added by last call to 
			-- `add_dies_for_projects'

	added_dies: LIST [like anchored_die]
			-- Dies added by last call to 
			-- `add_dies_for_projects'

	updated_dies: LIST [like anchored_die]
			-- Dies updated by last call to 
			-- `update_dies_for_projects'

	removed_dies: LIST [like anchored_die]
			-- Dies removed by last call to 
			-- `remove_dies_for_projects'

	last_cut_pattern_position: INTEGER
			-- Position of last added cut pattern in cut
			-- pattern list

feature -- Row copy

	copy_from_row (r: like original_row)
			-- Initialize `Current' from `r'
		do
			precursor {PACKING_VERSIONS_MRAW} (r)
			detach_actions_internal := r.detach_actions_internal
			cached_partitions := r.cached_partitions
			cached_dies := r.cached_dies
			cached_cut_patterns := r.cached_cut_patterns
			cached_packing := r.cached_packing
			cached_wafer_processes := r.cached_wafer_processes
			die_locations_table := r.die_locations_table
			cust_die_table := r.cust_die_table
			test_vehicle_table := r.test_vehicle_table
			vendor_geometry_table := r.vendor_geometry_table
			selection_date := r.selection_date
			max_partition_no := r.max_partition_no
			max_die_no := r.max_die_no
			max_cut_pattern_no := r.max_cut_pattern_no
			ordered_processes := r.ordered_processes
			warnings := r.warnings
			cached_family_processes_table := r.cached_family_processes_table
		end

feature -- Modification

	set_partitions (list: like partitions)
			-- Set `partitions' to `list'
		do
			if is_detached then
				detached_partitions := list
			else
				precursor (list)
			end
		end

	set_dies (list: like dies)
			-- Set `dies' to `list'
		do
			if is_detached then
				detached_dies := list
			else
				precursor (list)
			end
		end

	set_cut_patterns (list: like cut_patterns)
			-- Set `cut_patterns' to `list'
		do
			if is_detached then
				detached_cut_patterns := list
			else
				precursor (list)
			end
		end

	set_wafer_processes (list: like wafer_processes)
			-- Set `wafer_processes' to `list'
		do
			if is_detached then
				detached_wafer_processes := list
			else
				precursor (list)
			end
		end

feature -- Operations

	select_internal_vendor_geometry
			-- Select vendor geometry which is internal to
			-- payload and add appropriate dependent rows
		local
			row: GEOMETRIES_ROW
			list: LIST [GEOMETRIES_ROW]
		do
			create row.make_template
			list := row.payload_rows (packing.wafer_techn, packing.field_set_name, any_connection, expected_count.At_least_zero)
			if dies = Void then
				set_dies (create {ARRAYED_LIST [like anchored_die]}.make (list.count))
			end
			check_for_3dic_vendor_geometry (list)
			list.do_all (agent append_vendor_geometry (?, dies))
		end

	select_external_vendor_geometry
			-- Select vendor geometry which is external to
			-- payload and add appropriate dependent rows
		local
			row: GEOMETRIES_ROW
			list: LIST [GEOMETRIES_ROW]
		do
			create row.make_template
			list := row.nonpayload_rows (packing.wafer_techn, packing.field_set_name, any_connection, expected_count.At_least_zero)
			if dies = Void then
				set_dies (create {ARRAYED_LIST [like anchored_die]}.make (list.count))
			end
			check_for_3dic_vendor_geometry (list)
			list.do_all (agent append_vendor_geometry (?, dies))
		end

	check_for_3dic_vendor_geometry (list: LIST [GEOMETRIES_ROW])
			-- Check for vendor geometry (but not vendor
			-- ghosts) if run is 3DIC, since this is not
			-- supported yet
		local
			g: GEOMETRIES_ROW
			vg_ids: ARRAYED_LIST [STRING]
		do
			if is_3dic then
				create vg_ids.make (list.count)
				from
					list.start
				until
					list.after
				loop
					g := list.item
					if not g.external_files then
						vg_ids.extend (g.geometry_id)
					end
					list.forth
				end
				if not vg_ids.is_empty then
					handle_unsupported ("vendor geometry on 3DIC run.%NGeometry IDs are: " + su.joined_by (vg_ids, ", "))
				end
			end
		end

	select_high_priority_test_vehicles
			-- Select test vehicles which are marked high
			-- priority and add appropriate dependent rows
		local
			row: M_TEST_VEHICLES_ROW
			list: LIST [M_TEST_VEHICLES_ROW]
		do
			create row.make_template
			list := row.test_vehicles_for_wafer_techn_and_priority (packing.wafer_techn, True, any_connection, expected_count.At_least_zero)
			if dies = Void then
				set_dies (create {ARRAYED_LIST [like anchored_die]}.make (list.count))
			end
			list.do_all (agent append_test_vehicle (?, dies))
		end

	register_deep
			-- Register `Current' and all dependent rows
			-- on implicit connection
		do
			register_on_implicit_connection
			register_dependents_deep
		end

	set_highest_used_numbers
			-- Set highest used numbers for die, die
			-- locations etc so that allocating a new
			-- entity will pick a number higher than any
			-- previously used number
		do
			set_highest_used_partition_no
			set_highest_used_die_no
			set_highest_used_cut_pattern_no
			cut_patterns.do_all (agent {like anchored_cut_pattern}.set_highest_used_cut_no)
		end

	update_for_new_version (a_name: STRING)
			-- Update `Current' with information for new
			-- version with optional version ID `a_name'
		local
			n: INTEGER
		do
			n := highest_used_version + 1
			if version_no /= 0 then
				set_parent_version_no (version_no)
			else
				set_null_parent_version_no
			end
			set_version_no (n)
			set_date_created (any_connection.transaction_date)
			set_creator (any_connection.user_name)
			set_version_id (a_name)
			if not is_null_version_id then
				set_packing_name (packing_id)
			else
				set_null_packing_name
			end
			update_dependents_for_version (n)
		end

	set_selection_date
			-- Set selection/synchronization date so that
			-- a whole batch of newly created or updated
			-- projects can get the exact same date
		do
			selection_date := any_connection.system_date
		end

	update_dies_for_projects (list: LIST [PACKING_PROJECTS_ROW])
			-- Update every project in `list' (and its
			-- corresponding die) with current information
			-- from the database.  Leave list of dies that
			-- were (possibly) updated in `updated_dies'.
		require
			list_not_void: list /= Void
		local
			table: HASH_TABLE [like anchored_die, INTEGER]
		do
			create {ARRAYED_LIST [like anchored_die]} updated_dies.make (list.count)
			set_selection_date
			parent.setup_for_fast_project_access
			create table.make (list.count)
			dies.do_if (agent add_die_to_table_under_project_no (?, table), agent {PACKING_DIES_MROW}.has_project)
			list.do_all (agent update_die_for_project (?, table))
			selection_date := Void
		end

	add_die_to_table_under_project_no (die: like anchored_die table: HASH_TABLE [like anchored_die, INTEGER])
			-- Add `die' to `table', using the project_no
			-- of its associated project as the key
		do
			table.put (die, die.project.project_no)
		end

	update_die_for_project (a_proj: like anchored_project table: HASH_TABLE [like anchored_die, INTEGER])
			-- Update project corresponding to `a_proj'
			-- and update its associated die with current
			-- information from the database, using
			-- `table' to look up the die
		require
			a_proj_not_void: a_proj /= Void
		local
			die: like anchored_die
			proj: like a_proj
		do
			table.search (a_proj.project_no)
			if table.found then
				die := table.found_item
				proj := die.project
			else
				handle_hard_error ("cannot find die in internal table for project_no " + a_proj.project_no.out)
			end
			
			-- Save current wafer etest name, so that
			-- we can restore it and not lose lot split
			-- assignments if the updated project is still
			-- compatible with the current wafer etest name
			die.save_wafer_etest_name
			
			if proj.is_customer_project then
				proj.initialize_cust_project (selection_date, parent.cust_project_highest_version (proj.m_fab_no), parent.production_id)
			else
				proj.initialize_test_vehicle (selection_date, parent.test_vehicle_highest_version (proj.design_no))
			end
			die.set_attributes_from_project (proj, packing)
			updated_dies.extend (die)
		end

	add_dies_for_projects (sel_list: LIST [PACKING_PROJECTS_ROW])
			-- Add a die to `dies' for every project in `sel_list'.
			-- Leave list of new projects in `added_projects'
			-- and list of new dies in `added_dies'
		require
			sel_list_not_void: sel_list /= Void
		do
			setup_for_build_die (sel_list.count)
			sel_list.do_all (agent add_die_for_project)
			selection_date := Void
		end

	add_die_for_project (p: PACKING_PROJECTS_ROW)
			-- Add a die to `dies' for `p'
		require
			p_not_void: p /= Void
		do
			build_die_for_project (p)
			dies.extend (last_die)
		end

	build_dies_for_projects (cmp_list: LIST [PACKING_PROJECTS_ROW])
			-- Build a packing die for every project in `cmp_list',
			-- for comparison purposes, but don't add to `dies'.
			-- Leave list of newly built projects in `added_projects'
			-- and list of newly built dies in `added_dies'
		require
			cmp_list_not_void: cmp_list /= Void
		do
			setup_for_build_die (cmp_list.count)
			cmp_list.do_all (agent build_die_for_project)
			selection_date := Void
		end

	setup_for_build_die (n: INTEGER)
			-- Do needed setup before building a packing
			-- die for a project
		require
			n_nonnegative: n >= 0
		do
			create {ARRAYED_LIST [like anchored_project]} added_projects.make (n)
			create {ARRAYED_LIST [like anchored_die]} added_dies.make (n)
			set_selection_date
			parent.setup_for_fast_project_access
		end

	build_die_for_project (p: PACKING_PROJECTS_ROW)
			-- Build a packing die for `p' and
			-- add it to `added_dies' (but not to `dies')
		require
			p_not_void: p /= Void
		do
			if p.is_customer_project then
				last_die := new_cust_die (p.m_fab_no)
			else
				last_die := new_test_vehicle_die (p.design_no)
			end
			added_dies.extend (last_die)
			added_projects.extend (last_die.project)
		end

	remove_dies_for_projects (del_list: LIST [PACKING_PROJECTS_ROW])
			-- Remove from `dies' all projects in `del_list'.
			-- Leave list of corresponding dies in `removed_dies'
		require
			del_list_not_void: del_list /= Void
		local
			old_list, new_list: like dies
			die: like anchored_die
			proj_table: HASH_TABLE [INTEGER, INTEGER]
			die_table: HASH_TABLE [INTEGER, INTEGER]
			pno: INTEGER
		do
			create {ARRAYED_LIST [like anchored_die]} removed_dies.make (del_list.count)
			create proj_table.make (del_list.count)
			del_list.do_all (agent add_project_to_table (?, proj_table))
			create die_table.make (del_list.count)

			from
				old_list := dies
				new_list := old_list.twin
				new_list.wipe_out
				old_list.start
			until
				old_list.after
			loop
				die := old_list.item
				if die.has_project then
					pno := die.project_no
					proj_table.search (pno)
					if not proj_table.found then
						new_list.extend (die)
					else
						removed_dies.extend (die)
						die_table.put (1, die.die_no)
					end
				else
					new_list.extend (die)
				end
				old_list.forth
			end
			set_dies (new_list)
			cut_patterns.do_all (agent {like anchored_cut_pattern}.remove_dies_for_projects (die_table))
		end

	remove_dies (del_list: LIST [PACKING_DIES_ROW])
			-- Remove from `dies' all dies in `del_list'
		require
			del_list_not_void: del_list /= Void
		local
			old_list, new_list: like dies
			die: like anchored_die
			die_table: HASH_TABLE [INTEGER, INTEGER]
			die_no: INTEGER
		do
			create die_table.make (del_list.count)
			del_list.do_all (agent add_die_to_table (?, die_table))

			from
				old_list := dies
				new_list := old_list.twin
				new_list.wipe_out
				old_list.start
			until
				old_list.after
			loop
				die := old_list.item
				die_no := die.die_no
				die_table.search (die_no)
				if not die_table.found then
					new_list.extend (die)
				end
				old_list.forth
			end
			set_dies (new_list)
			cut_patterns.do_all (agent {like anchored_cut_pattern}.remove_dies_for_projects (die_table))
		end

	remove_partitions (del_list: LIST [PACKING_PARTITIONS_ROW])
			-- Remove from `partitions' all partitions in `del_list'
		require
			del_list_not_void: del_list /= Void
		local
			old_list, new_list: like partitions
			partition: like anchored_partition
			partition_table: HASH_TABLE [INTEGER, INTEGER]
			partition_no: INTEGER
		do
			create partition_table.make (del_list.count)
			del_list.do_all (agent add_partition_to_table (?, partition_table))

			from
				old_list := partitions
				new_list := old_list.twin
				new_list.wipe_out
				old_list.start
			until
				old_list.after
			loop
				partition := old_list.item
				partition_no := partition.partition_no
				partition_table.search (partition_no)
				if not partition_table.found then
					new_list.extend (partition)
				end
				old_list.forth
			end
			set_partitions (new_list)
		end

	add_new_die (a_die: like anchored_die)
			-- Add `a_die' to `dies'
		require
			a_die_not_void: a_die /= Void
			same_version_or_zero: a_die.version_no = version_no or a_die.version_no = 0
		do
			dies.extend (a_die)
		end

	add_new_partition (a_partition: like anchored_partition)
			-- Add `a_partition' to `partitions'
		require
			a_partition_not_void: a_partition /= Void
			same_version_or_zero: a_partition.version_no = version_no or a_partition.version_no = 0
		do
			partitions.extend (a_partition)
		end

	compute_compatible_processes
			-- Compute compatible processes for each selected
			-- customer project in `Current'
		do
			compute_compatible_processes_for_list (selected_customer_projects)
		end
	
	compute_compatible_processes_for_list (list: like selected_customer_projects)
			-- Compute compatible processes for each 
			-- project in `list'
		do
			uncache_family_processes_table
				-- Make sure any added processes will be found
			compute_compatible_processes_for_projects (list, family_processes_table)
		end
	
	assign_process_to_projects
			-- Assign a process to each customer project,
			-- based on the list of selected projects and
			-- the compatible processes for each of these
			-- projects.  Then propagate the assigned process
			-- to the die for each project.  Leave any
			-- warnings in `warnings'
		local
			die_list: like selected_customer_dies
			proj_list: like selected_customer_projects
		do
			uncache_family_processes_table
				-- Make sure any added processes will be found
			die_list := selected_customer_dies
			proj_list := selected_customer_projects
			die_list.do_all (agent {like anchored_die}.restore_wafer_etest_name)
			die_list.do_all (agent {like anchored_die}.propagate_assigned_etest_name (family_processes_table))
			compute_ordered_processes (proj_list)
			
			proj_list.do_all (agent {like anchored_project}.assign_process (ordered_processes))
			dies.do_if (agent {like anchored_die}.assign_wafer_process, agent {like anchored_die}.is_cust_die)
			dies.do_if (agent {like anchored_die}.assign_wafer_etest_name, agent {like anchored_die}.is_cust_die)
			
			check_ordered_processes (family_processes_table)
		end
	
	report_process_warnings
			-- Report any warnings from last call to
			-- `assign_process_to_projects'
		local
			op: ANONYMOUS_OPERATION
		do
			if not warnings.is_empty then
				create op.make (agent warn_with_notify ("multiple ordered processes with same wafer etest ID - missing process or conceptual error.%N%N" + su.joined_by (warnings, "%N") + "%N"))
				op.enable_abort_caught
				op.execute
			end
		end

	set_processes_from_official_data
			-- Set process information for every
			-- customer project from data in official
			-- database tables
		local
			ifip_table:  HASH_TABLE [LIST [STRING], INTEGER]
			wp_table: HASH_TABLE [WAFER_PROCESSES_ROW, STRING]
			wen_table: HASH_TABLE [WAFER_ETEST_NAMES_ROW, STRING]
		do
			ifip_table := packing.fab_instance_processes_table
			wp_table := packing.wafer_processes_table
			wen_table := packing.wafer_etest_names_table
			selected_customer_projects.do_all (agent {PACKING_PROJECTS_MROW}.set_official_processes (ifip_table, wp_table, wen_table))
			selected_customer_dies.do_all (agent {PACKING_DIES_MROW}.set_official_processes)
		end
	
	add_cut_pattern (a_pattern_no, a_parent_pattern_no: INTEGER prod_id, a_pattern_type, a_field_name, a_wafer_etest_name: STRING quant: INTEGER has_quant: BOOLEAN ext_cut_specs: CONST_ABSTRACT_EXT_CUT_DIST_SPEC a_blade_width: DOUBLE)
			-- Add a new cut pattern `a_pattern_no' for
			-- production `prod_id' with `a_pattern_type',
			-- for field `a_field_name' and lot
			-- `a_wafer_etest_name'.  Leave position of
			-- new cut pattern in
			-- `last_cut_pattern_position'
		require
			prod_id_not_void: prod_id /= Void
			prod_id_not_empty: not prod_id.is_empty
			a_pattern_type_not_void: a_pattern_type /= Void
			a_pattern_type_not_empty: not a_pattern_type.is_empty
			a_field_name_not_void: a_field_name /= Void
			a_field_name_not_empty: not a_field_name.is_empty
			a_wafer_etest_name_not_void: a_wafer_etest_name /= Void
			ext_cut_specs_not_void: ext_cut_specs /= Void
			a_blade_width_positive: a_blade_width > 0.0
		local
			pat: PACKING_CUT_PATTERNS_MROW
		do
			create pat.make_detached (any_connection)
			pat.set_grid (grid)
			pat.set_packing_id (packing_id)
			pat.set_version_no (version_no)
			pat.set_cut_pattern_no (a_pattern_no)
			pat.set_field_name (a_field_name)
			pat.set_is_completed (False)
			pat.set_wafer_etest_name (a_wafer_etest_name)
			pat.set_cut_pattern_xform_type (a_pattern_type)
			if a_parent_pattern_no >= 0 then
				pat.set_cut_pattern_no_parent (a_parent_pattern_no)
			else
				pat.set_null_cut_pattern_no_parent
			end
			pat.set_saw_blade_width (a_blade_width)
			pat.set_external_cut_specs (ext_cut_specs)
			if has_quant then
				pat.set_quantity_wafers_needed (quant)
			else
				pat.set_null_quantity_wafers_needed
			end
			pat.set_production_id (prod_id)
			pat.set_cut_locations (create {ARRAYED_LIST [PACKING_CUT_LOCATIONS_MROW]}.make (0))
			pat.set_cut_die_sites (create {ARRAYED_LIST [PACKING_CUT_DIE_SITES_MROW]}.make (0))
			cut_patterns.extend (pat)
				-- FIXME: should probably put cut pattern in
				-- correct position by cut_pattern_no
			last_cut_pattern_position := cut_patterns.count
		end

	remove_cut_pattern (a_pattern_no: INTEGER)
			-- Remove new cut pattern whose number is 
			-- `a_pattern_no'
		local
			list: like cut_patterns
			pat: PACKING_CUT_PATTERNS_MROW
			found: BOOLEAN
		do
			from
				list := cut_patterns
				list.start
			until
				list.after or found
			loop
				pat := list.item
				if pat.cut_pattern_no = a_pattern_no then
					list.remove
					found := True
				else
					list.forth
				end
			end
		end

	remove_cut_patterns
			-- Remove all cut patterns
		do
			cut_patterns.wipe_out
		end

feature {PACKING_VERSIONS_MROW} -- Implementation

	detach_dependents_deep
			-- Detach dependent rows from database connection
			-- deeply
		do
			detach_partitions_deep
			detach_dies_deep
			detach_wafer_processes_deep
			detach_cut_patterns_deep
			cut_patterns.do_all (agent {like anchored_cut_pattern}.uncache_isolated_die_locations)
		end

feature {NONE} -- Implementation

	last_die: like anchored_die
			-- Last die built by `build_die_for_project'

	set_highest_used_partition_no
			-- Compute highest used partition_no and put
			-- in `max_partition_no'
		do
			max_partition_no := 0
			partitions.do_all (agent update_max_partition_no)
		end

	set_highest_used_die_no
			-- Compute highest used die_no and put in `max_die_no'
		do
			max_die_no := 0
			dies.do_all (agent update_max_die_no)
		end

	set_highest_used_cut_pattern_no
			-- Compute highest used cut_pattern_no and put in 
			-- `max_cut_pattern_no'
		do
			max_cut_pattern_no := 0
			cut_patterns.do_all (agent update_max_cut_pattern_no)
		end

	compute_compatible_processes_for_projects (list: like selected_customer_projects table: like family_processes_table)
			-- Compute list of compatible processes for each
			-- project in `list', using `table' to lookup
			-- information about processes
		do
			list.do_all (agent {like anchored_project}.compute_compatible_processes (table))
		end
	
	compute_ordered_processes (proj_list: like selected_customer_projects)
			-- Compute processes to be ordered, where `proj_list'
			-- is list of all customer projects, and put
			-- it in `ordered_processes'
		do
		end
	
	check_ordered_processes (proc_table: like family_processes_table)
			-- Check `ordered_processes' to verify that no two
			-- processes have the same wafer etest ID.
			-- Add a warning to `warnings' for each wafer etest 
			-- ID that corresponds to more than one ordered process
		local
			proc, etest_id: STRING
			table: HASH_TABLE [LIST [STRING], STRING]
			proc_list: LIST [STRING]
			bad_etest_ids: ARRAYED_LIST [STRING]
		do
			create {ARRAYED_LIST [STRING]} warnings.make (0)
			create table.make (10)
			create bad_etest_ids.make (0)
			from
				ordered_processes.start
			until
				ordered_processes.after
			loop
				proc := ordered_processes.item
				proc_table.search (proc)
				if proc_table.found then
					etest_id := proc_table.found_item.wafer_etest_id
					table.search (etest_id)
					if table.found then
						proc_list := table.found_item
						if proc_list.count = 1 then
							bad_etest_ids.extend (etest_id)
						end
					else
						create {ARRAYED_LIST [STRING]} proc_list.make (1)
						table.put (proc_list, etest_id)
					end
					proc_list.extend (proc)
				else
					handle_hard_error ("cannot find ordered wafer process " + proc + " in table of all processes for wafer techn family")
				end
				ordered_processes.forth
			end
			
			if not bad_etest_ids.is_empty then
				from
					bad_etest_ids.start
				until
					bad_etest_ids.after
				loop
					etest_id := bad_etest_ids.item
					proc_list := table.item (etest_id)
					warnings.extend ("Wafer etest ID " + etest_id + " shared by processes " + su.joined_by (proc_list, ", "))
					bad_etest_ids.forth
				end
			end
		end
	
	add_warning (dno: INTEGER msg: STRING)
			-- Add warning for design number `dno' with message
			-- `msg' to `waraings'
		local
			s: STRING
		do
			s.append ("     ")
			s.append_integer (dno)
			s.append ("     ")
			s.append (msg)
			warnings.extend (s)
		end
	
	append_vendor_geometry (g: GEOMETRIES_ROW list: like dies)
			-- Append new die for `g' to `list'
		do
			list.extend (new_vendor_geometry (g))
		end
		
	append_test_vehicle (mtv: M_TEST_VEHICLES_ROW list: like dies)
			-- Append new die for `mtv' to `list'
		do
			list.extend (new_test_vehicle_die (mtv.design_no))
		end
		
	add_vendor_geometry_locations (locs: LIST [GEOMETRYXFIXED_LOCATIONS_ROW] d: like anchored_die)
			-- Create a list of die locations based on `locs'
			-- and put it in `d'
		local
			loc: GEOMETRYXFIXED_LOCATIONS_ROW
			die_locs: ARRAYED_LIST [PACKING_DIE_LOCATIONS_MROW]
			die_loc: PACKING_DIE_LOCATIONS_MROW
			n: INTEGER
		do
			create die_locs.make (locs.count)
			from
				locs.start
			until
				locs.after
			loop
				loc := locs.item
				n := n + 1
				create die_loc.make_detached (any_connection)
				die_loc.set_packing_id (packing.packing_id)
				die_loc.set_version_no (0)
				die_loc.set_die_no (d.die_no)
				die_loc.set_site_no (n)
				die_loc.set_field_name (loc.field_name)
				-- FIXME: technically, we need to adjust 
				-- location X and Y by offset of payload from 
				-- center of field (doesn't matter for now)
				die_loc.set_location_x (loc.location_x)
				die_loc.set_location_y (loc.location_y)
				die_loc.set_fixed_vendor_geometry_packer_attributes
				die_locs.extend (die_loc)
				locs.forth
			end
			d.set_die_locations (die_locs)
		end
		
	new_initialized_die (dtype: STRING): like anchored_die
			-- New die of type `dtype' initialized with
			-- key (except version_no, which is set when
			-- save new version) and not rotated
		do
			max_die_no := max_die_no + 1
			create Result.make_detached (any_connection)
			Result.set_packing_id (packing.packing_id)
			Result.set_version_no (0)
			Result.set_die_no (max_die_no)
			Result.set_die_type (dtype)
			Result.set_die_wafer_rotation (0)
			Result.set_die_locations (create {ARRAYED_LIST [like anchored_die_location]}.make (0))
			Result.set_packing (packing)
		end
		
	new_initialized_partition (a_name: STRING): like anchored_partition
			-- New partition initialized with
			-- key (except version_no, which is set when
			-- save new version) for etest name `a_name'
		require
			a_name_not_void: a_name /= Void
		do
			max_partition_no := max_partition_no + 1
			create Result.make_detached (any_connection)
			Result.set_packing_id (packing.packing_id)
			Result.set_version_no (0)
			Result.set_partition_no (max_partition_no)
			Result.set_wafer_etest_name (a_name)
			Result.set_default_packer_attributes
		end
		
	detach_partitions_deep
			-- Detach `partitions' from database connection, along
			-- with all dependents
		do
			original_partitions := partitions.twin
			original_partitions.wipe_out
			original_partitions.append (partitions)
			partitions.wipe_out
			original_partitions.do_all (agent append_detached_partition (?, partitions))
		end

	detach_dies_deep
			-- Detach `dies' from database connection, along
			-- with all dependents
		do
			original_dies := dies.twin
			original_dies.wipe_out
			original_dies.append (dies)
			dies.wipe_out
			original_dies.do_all (agent append_detached_die (?, dies))
		end

	detach_cut_patterns_deep
			-- Detach `cut_patterns' from database connection,
			-- along with all dependents
		do
			original_cut_patterns := cut_patterns.twin
			original_cut_patterns.wipe_out
			original_cut_patterns.append (cut_patterns)
			cut_patterns.wipe_out
			original_cut_patterns.do_all (agent append_detached_cut_pattern (?, cut_patterns))
		end

	detach_wafer_processes_deep
			-- Detach `wafer_processes' from database connection,
			-- along with all dependents
		do
			original_wafer_processes := wafer_processes.twin
			original_wafer_processes.wipe_out
			original_wafer_processes.append (wafer_processes)
			wafer_processes.wipe_out
			original_wafer_processes.do_all (agent append_detached_wafer_process (?, wafer_processes))
		end

	append_detached_partition (p: like anchored_partition list: like partitions)
			-- Append detached copy of `p' to `list'
		do
			list.extend (p.deep_detached_twin)
		end
		
	append_detached_die (d: like anchored_die list: like dies)
			-- Append detached copy of `d' to `list'
		do
			list.extend (d.deep_detached_twin)
		end
		
	append_detached_wafer_process (wp: like anchored_wafer_process list: like wafer_processes)
			-- Append detached copy of `wp' to `list'
		do
			list.extend (wp.deep_detached_twin)
		end
		
	append_detached_cut_pattern (cp: like anchored_cut_pattern list: like cut_patterns)
			-- Append detached copy of `cp' to `list'
		do
			list.extend (cp.deep_detached_twin)
		end
		
	register_dependents_deep
			-- Register all dependent rows on implicit
			-- connection
		do
			-- Move "detached" versions back to "cached" versions
			-- since they will no longer be detached at end
			-- of routine
			
			cached_partitions := detached_partitions
			detached_partitions := Void
			partitions.do_all (agent {like anchored_partition}.register_deep)
			
			cached_dies := detached_dies
			detached_dies := Void
			dies.do_all (agent {like anchored_die}.register_deep)
			
			cached_wafer_processes := detached_wafer_processes
			detached_wafer_processes := Void
			wafer_processes.do_all (agent {like anchored_wafer_process}.register_deep)
			
			cached_cut_patterns := detached_cut_patterns
			detached_cut_patterns := Void
			cut_patterns.do_if (agent {like anchored_cut_pattern}.register_deep, agent {like anchored_cut_pattern}.has_no_parent)
			cut_patterns.do_if (agent {like anchored_cut_pattern}.register_deep, agent {like anchored_cut_pattern}.has_parent)
		end

	update_dependents_for_version (n: INTEGER)
			-- Update all dependent rows prior to save of
			-- new version `n'
		do
			max_partition_no := 0
			sorted_partitions.do_all (agent renumber_partition)
			partitions.do_all (agent {like anchored_partition}.update_for_new_version (n))
			
			max_die_no := 0
			dies.do_all (agent renumber_die)
			dies.do_all (agent {like anchored_die}.update_for_new_version (n))
			
			wafer_processes.do_all (agent {like anchored_wafer_process}.update_for_new_version (n))
			cut_patterns.do_all (agent {like anchored_cut_pattern}.update_for_new_version (n))
		end

	sorted_partitions: like partitions
			-- List of partitions sorted by lower left
			-- corner Y coordinate and by X coordinate
			-- within same Y coordinate
		do
		end

	renumber_partition (p: PACKING_PARTITIONS_MROW)
		do
			max_partition_no := max_partition_no + 1
			p.set_partition_no (max_partition_no)
		end
	
	renumber_die (d: PACKING_DIES_MROW)
		do
			max_die_no := max_die_no + 1
			d.set_die_no (max_die_no)
		end
	
	update_max_partition_no (p: PACKING_PARTITIONS_MROW)
		do
			max_partition_no := max_partition_no.max (p.partition_no)
		end
	
	update_max_die_no (d: PACKING_DIES_MROW)
		do
			max_die_no := max_die_no.max (d.die_no)
		end
	
	update_max_cut_pattern_no (cp: like anchored_cut_pattern)
		do
			max_cut_pattern_no := max_cut_pattern_no.max (cp.cut_pattern_no)
		end
	
	anchored_partition: PACKING_PARTITIONS_MROW
			-- For anchoring only
		do
		end
	
	anchored_die: PACKING_DIES_MROW
			-- For anchoring only
		do
		end
	
	anchored_wafer_process: PACKING_WAFER_PROCESSES_MROW
			-- For anchoring only
		do
		end
	
	anchored_die_location: PACKING_DIE_LOCATIONS_MROW
			-- For anchoring only
		do
		end

	anchored_cut_pattern: PACKING_CUT_PATTERNS_MROW
			-- For anchoring only
		do
		end

	anchored_cut_die_site: PACKING_CUT_DIE_SITES_MROW
			-- For anchoring only
		do
		end

	anchored_project: PACKING_PROJECTS_MROW
			-- For anchoring only
		do
		end

	anchored_packaging: PACKING_PROJECT_PACKAGINGS_MROW
			-- For anchoring only
		do
		end

	anchored_option: PACKING_PROJECT_OPTIONS_MROW
			-- For anchoring only
		do
		end

	detached_partitions: like partitions
			-- List of detached partitions (no explict DB connection)

	detached_dies: like dies
			-- List of detached dies (no explict DB connection)

	detached_cut_patterns: like cut_patterns
			-- List of detached cut patterns (no explict 
			-- DB connection)
		
	detached_wafer_processes: like wafer_processes
			-- List of detached wafer processes (no explict 
			-- DB connection)
		
	original_partitions: like partitions
			-- Original list of partitions before detach

	original_dies: like dies
			-- Original list of dies before detach

	original_wafer_processes: like wafer_processes
			-- Original list of wafer processes before detach

	original_cut_patterns: like cut_patterns
			-- Original list of cut patterns before detach

feature {PACKING_VERSIONS_MROW} -- Implementation

	max_partition_no: INTEGER
			-- Maximum partition number used so far (0 if
			-- none used) in this version

	max_die_no: INTEGER
			-- Maximum die number used so far (0 if none used)
			-- in this version

	max_cut_pattern_no: INTEGER
			-- Maximum cut pattern number assigned so far
			-- (0 if none used) in this version

	warnings: LIST [STRING]
			-- Warnings from last call to 
			-- `assign_process_to_projects'

end
