
note

	author: "Generated"
	date: "Mon Mar 29 14:53:11 PST 2004"

class RUN_PACKAGING_LOTS_ROW
inherit
	RUN_PACKAGING_LOTS_RAW
	ESSENCE_OF_DATE_RECEIVED
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	NULL_VALUE_CHECK_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
create
	make_template, make_null, make_by_key

feature

	lots_for_run_and_production (run, prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All packaging lots for run `run' and
			-- production `prod', ordered by packaging lot
			-- number
		require
			run_exists: run /= Void
			prod_exists: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (run, prod)
			Result := all_rows_filtered_ordered (conn, "run_id = :1 and production_id = :2", b, "packaging_lot_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	lots_for_run (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All packaging lots for run `run' (on all 
			-- productions), ordered by packaging lot
			-- number
		require
			run_exists: run /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, "run_id = :1", b, "packaging_lot_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	lots_for_order_type (prod, a_process, ord_type: STRING is_tv: BOOLEAN conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All packaging lots for production `prod',
			-- wafer process `a_process' and packaging order
			-- tpye `ord_type'.  `is_tv' indicates whether
			-- we're looking for lots for a test vehicle
		require
			prod_not_void: prod /= Void
			a_process_not_void: a_process /= Void
			ord_type_not_void: ord_type /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SSS
			btv: DB_BIND_SS
		do
			if is_tv then
				create btv.make (prod, ord_type)
				Result := all_rows_filtered (conn, "production_id = :1 and packaging_order_type = :2", btv)
			else
				create b.make (prod, a_process, ord_type)
				Result := all_rows_filtered (conn, "production_id = :1 and wafer_process = :2 and packaging_order_type = :3", b)
			end
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	non_null_wafer_process: like wafer_process
			-- Value of `wafer_process',
			-- with error check for nullness
		do
			check_not_null ("wafer_process", is_null_wafer_process, Current)
			Result := wafer_process
		end

	process: WAFER_PROCESSES_ROW
			-- Process of wafers in lot
		require
			has_wafer_process: not is_null_wafer_process
		do
                        if cached_process = Void then
                                create cached_process.make_by_key (connection, option.Old_row, non_null_wafer_process)
                        end
			Result := cached_process
                ensure
                        result_not_void: Result /= Void
		end
	

	is_cached_process: BOOLEAN
			-- Is `process' currently cached?
		do  
			Result := cached_process /= Void
		end

	organization: ORGANIZATIONS_ROW
			-- Organization that will package this lot
		do
                        if cached_organization = Void then
                                create cached_organization.make_by_key (connection, option.Old_row, packaging_org_no)
                        end
			Result := cached_organization
                ensure
                        result_not_void: Result /= Void
		end
	
	is_cached_organization: BOOLEAN
			-- Is `organization' currently cached?
		do  
			Result := cached_organization /= Void
		end

	is_itar_approved_vendor: BOOLEAN
			-- Is packaging vendor for `Current' approved
			-- to handle ITAR material?
		do  
			Result := organization.is_itar_approved
		end

	cut_patterns: LIST [RUN_CUT_PATTERNS_ROW]
			-- Cut patterns associated with this lot
		local
			row: RUN_CUT_PATTERNS_ROW
		do
                        if cached_cut_patterns = Void then
                                create row.make_template
                                cached_cut_patterns := row.rows_for_packaging_lot (run_id, packaging_lot_no, connection, expected_count.At_least_zero)
                        end
			Result := cached_cut_patterns
                ensure
                        result_not_void: Result /= Void
		end
	
	is_cached_cut_patterns: BOOLEAN
			-- Is `cut_patterns' currently cached?
		do  
			Result := cached_cut_patterns /= Void
		end

	wafer_lots: LIST [RUN_WAFER_LOTS_ROW]
			-- List of wafer lots that may be used for
			-- this packaging lot (ones with same process)
	local b: DB_BIND_SIS; r: RUN_WAFER_LOTS_ROW
	do  if cached_wafer_lots = Void
	     then create r.make_template;
		  create b.make(run_id, packaging_lot_no, wafer_process)
		  cached_wafer_lots := r.all_rows_filtered_ordered
				(connection, wfr_lot_filter, b,
				 "wafer_lot_no")
	    end;
	    Result := cached_wafer_lots
	end;

	is_cached_wafer_lots: BOOLEAN
	do  Result := cached_wafer_lots /= Void
	end;

	ic_packagings_needing_bonding: LIST [IC_PACKAGINGS_ROW]
			-- List of IC packagings for this packaging lot
			-- that need a bonding diagram
		local 
			r: IC_PACKAGINGS_ROW
		do  
			if cached_ic_packagings_needing_bonding = Void then 
				create r.make_template
		  		cached_ic_packagings_needing_bonding := r.rows_needing_diagram_for_pkg_lot (production_id, packaging_lot_no, False, connection, expected_count.At_least_zero)
	    		end
	    		Result := cached_ic_packagings_needing_bonding
		ensure
			result_not_void: Result /= Void
			packagings_cached: is_cached_ic_packagings_needing_bonding
		end;

	is_cached_ic_packagings_needing_bonding: BOOLEAN
		do  
			Result := cached_ic_packagings_needing_bonding /= Void
		end

	wafer_etest_id: STRING
			-- Wafer etest identifier
		require
			has_wafer_process: not is_null_wafer_process
		do
			Result := process.wafer_etest_id
		ensure
			result_exists: Result /= Void
		end

	map_wafer_lot_ids: STRING
			-- Description of wafer lot ids of lots that
			-- may be used for packaging lots, suitable
			-- for inclusion in maps
		do
			if wafer_lots.for_all (agent {RUN_WAFER_LOTS_ROW}.has_vendor_lot_id) then
				create temp.make (40)
				wafer_lots.do_all (agent append_lot_id)
				Result := temp
			else
				Result := "UNKNOWN"
			end
		ensure
			result_exists: Result /= Void
		end

	packager_short_id: STRING
			-- Short id of organizaton that will package this lot
		do
			Result := organization.short_id
		ensure
			result_exists: Result /= Void
		end

	identification: STRING
			-- Descriptive string identifiying lot
		do
			Result := su.left_padded (packaging_lot_no.out, 2) + "    " + su.right_padded (packager_short_id, 10) + " " + wafer_etest_id
		ensure
			result_exists: Result /= Void
		end

	symbolic_cut_patterns_map_name: STRING
			-- Symbolic name for map for cut patterns for
			-- this lot
		do
			Result := "Cut Patterns for Packaging Lot " + identification
		ensure
			result_exists: Result /= Void
		end

	symbolic_bond_diagrams_map_name: STRING
			-- Symbolic name for map for bonding diagrams for
			-- this lot
		do
			Result := "Bonding Diagrams for Packaging Lot " + identification
		ensure
			result_exists: Result /= Void
		end

	processing_stage: STRING
		-- descriptive text re this lot's progress
	local wlot: RUN_WAFER_LOTS_ROW; wdat: DB_DATE;
	do  if date_received.is_null
	     then 
	    elseif date_received < connection.transaction_date
	     then Result := "house"
	    elseif wafer_lots.is_empty
	     then Result := "progress"
	     else wlot := wafer_lots.first;	-- (no way to choose)
		  wdat := wlot.date_received
		  if wdat.is_null or else wdat < connection.transaction_date
		   then Result := "packaging"
		  elseif wlot.is_null_processing_stage
		   then Result := "fabrication"
		   else Result := "fabrication at " + wlot.processing_stage
		  end
	    end
	end;

	is_non_virtual: BOOLEAN
			-- Is packaging order type not virtual?
		local
			type: PACKAGING_ORDER_TYPE_VALUES
		do
			Result := not packaging_order_type.is_equal (type.Virtual)
		end

	is_date_received_actual: BOOLEAN
			-- Is `date_received' an actual date
			-- (not an estimate)?
		local
			status: DATE_STATUS_VALUES
		do
			Result := date_received_status.is_equal (status.Actual)
		end

feature -- Modification

	set_process (p: like process)
		require
			p_not_void: p /= Void
		do
			cached_process := p
		ensure
			value_set: process = p
		end

	set_organization (p: like organization)
		require
			p_not_void: p /= Void
		do
			cached_organization := p
		ensure
			value_set: organization = p
		end

	set_wafer_lots (list: like wafer_lots)
		require
			list_not_void: list /= Void
		do
			cached_wafer_lots := list
		ensure
			value_set: wafer_lots = list
		end

	select_wafer_lots (master: like wafer_lots)
	require	master_not_void: master /= Void
	local wpid: STRING; b: DB_BIND_SI; it: RUN_WAFER_LOTS_ROW
	do  create b.make(run_id, packaging_lot_no);
	    wpid := connection.selected_string
			("m_lookup.wafer_source_for_packaging(:1, :2)",
			 "dual", Void, b);
	    create {LINKED_LIST[RUN_WAFER_LOTS_ROW]}
			cached_wafer_lots.make;
	    from  master.start
	    until master.after
	    loop  it := master.item
		  if equal(wpid, it.production_id) and
		     equal(wafer_process, it.wafer_process)
		   then cached_wafer_lots.extend(it)
		  end;
		  master.forth
	    end;
	ensure is_cached_wafer_lots
	end;

	uncache_wafer_lots
	do  cached_wafer_lots := Void
	end;

	uncache_ic_packagings_needing_bonding
		do  
			cached_ic_packagings_needing_bonding := Void
		end

	uncache_organization
			-- Remove any cached organization
		do  
			cached_organization := Void
		end

	uncache_cut_patterns
			-- Remove any cached cut patterns
		do  
			cached_cut_patterns := Void
		end

	uncache_process
			-- Remove any cached process
		do  
			cached_process := Void
		end

feature {NONE} -- Implementation

	temp: STRING
			-- Temporary use

	cached_wafer_lots: LIST[RUN_WAFER_LOTS_ROW]

	cached_ic_packagings_needing_bonding: LIST [IC_PACKAGINGS_ROW]

	cached_organization: ORGANIZATIONS_ROW
			-- Organization that will package this lot

	cached_cut_patterns: LIST [RUN_CUT_PATTERNS_ROW]
			-- Cut patterns associated with this lot

	cached_process: WAFER_PROCESSES_ROW
			-- Wafer process for lot

	wfr_lot_filter: STRING =
	"production_id = m_lookup.wafer_source_for_packaging(:1, :2) %
	% and wafer_process = :3";

	append_lot_id (row: RUN_WAFER_LOTS_ROW)
		do
			if not temp.is_empty then
				temp.append (", ")
			end
			temp.append (row.wafer_vendor_lot_id)
		end

end
