
note

	author: "Generated"
	date: "Mon Mar 29 14:49:51 PST 2004"

class RUN_WAFER_LOTS_ROW
inherit
	RUN_WAFER_LOTS_RAW
	ESSENCE_OF_DATE_RECEIVED
	ROW_COUNT_CHECK_ACCESS
	STRING_UTILITIES_ACCESS
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end


create
	make_template, make_null, make_by_key

feature

	lots_for_run_and_prod (run, prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All wafer lots for run `run' and production `prod',
			-- ordered by wafer lot number
		require
			run_exists: run /= Void
			prod_exists: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SS
		do
			create b.make (run, prod)
			Result := all_rows_filtered_ordered (conn, "run_id = :1 and production_id = :2", b, "wafer_lot_no")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	lots_for_cust_wo (order_no: INTEGER;
			  conn: DB_CONNECTION; 
			  range: ITEM_COUNT_RANGE): LIST [like Current]
		-- all run_wafer_lots for a cust_wo
		-- (matching production_id and wafer_process)
	require
		connection_exists: conn /= Void
		connection_open: conn.is_open
	local b: DB_BIND_I;
	do
		create b.make (order_no);
		Result := all_rows_filtered_ordered (conn,
				"(production_id, wafer_process) in %
				%(select cwo.production_id, ifip.wafer_process%
				% from ic_fab_instance_processes ifip, cust_wos cwo%
				% where ifip.m_fab_no = cwo.m_fab_no%
				% and cwo.m_order_no = :1)",
				b, "wafer_lot_no");
		if range /= Void then
			check_row_count (Result, range)
		end
	ensure
		result_exists: Result /= Void
	end;

	field_step_pattern_offset: CONST_POINT
			-- Offset of field step pattern for lot
		do
			create Result.make_xy (field_step_pattern_offset_x, field_step_pattern_offset_y)
		ensure
			result_exists: Result /= Void
		end

	field_step_patterns_row: FIELD_STEP_PATTERNS_ROW
			-- Field step patterns row for `Current'
		require
			has_step_pattern: not field_step_pattern.is_empty
		do
			if cached_field_step_patterns_row = Void then
				create cached_field_step_patterns_row.make_by_key (connection, option.Old_row, field_step_pattern)
			end
			Result := cached_field_step_patterns_row
		ensure
			result_exists: Result /= Void
		end

	uncache_field_step_pattern
			-- Remove field step pattern row from cache, if present
		do
			cached_field_step_patterns_row := Void
		ensure
			field_step_pattern_not_cached: not is_cached_field_step_pattern
		end

	is_cached_field_step_pattern: BOOLEAN
			-- Is `field_step_patterns_row' currently cached?
		do
			Result := cached_field_step_patterns_row /= Void
		end

	is_preliminary_step_pattern: BOOLEAN
			-- Is field step pattern for this
			-- lot marked as preliminary?
		do
			Result := field_step_patterns_row.is_preliminary
		end

	has_same_stepping(other: RUN_WAFER_LOTS_ROW): BOOLEAN
		-- do we have same field_step_pattern AND offsets?
	require have_other: other /= Void
	do  Result := equal(field_step_pattern, other.field_step_pattern) and
			(field_step_pattern_offset_x = other.field_step_pattern_offset_x) and
			(field_step_pattern_offset_y = other.field_step_pattern_offset_y)
	end;

	has_vendor_lot_id: BOOLEAN
			-- Does lot have a vendor lot id assigned?
		do
			Result := not is_null_wafer_vendor_lot_id
		end

	maximum_quantity_wafers_ordered: INTEGER
			-- Largest value for `quantity_wafers_ordered'
			-- that will fit in the database
		do
			Result := (10 ^ precision_quantity_wafers_ordered - 1).truncated_to_integer
		end

	has_quantity_wafers_received: BOOLEAN
			-- Does lot have a quantity of
			-- wafers received?
		do
			Result := not is_null_quantity_wafers_received
		end

	has_date_accepted: BOOLEAN
			-- Does lot have a date accepted?
		do
			Result := not is_null_date_accepted
		end

	has_vendor_target_date: BOOLEAN
			-- Does lot have a vendor target date?
		do
			Result := not is_null_vendor_target_date
		end

feature	-- wafer_etest_id

	wafer_etest_id: STRING
	local b: DB_BIND_S
	do  create b.make (wafer_process)
	    Result := connection.selected_string
				("wafer_etest_id", "wafer_processes",
				 "wafer_process = :1", b)
	end;

	wafer_etest_names_row: WAFER_ETEST_NAMES_ROW
		local 
			wtf: STRING
			r: RUNS_ROW
		do  
			create r.make_by_key (connection, option.Old_row, run_id)
	    		wtf := r.wafer_techn_row.wafer_techn_family
			create Result.make_by_etest_id (connection, option.Old_row, wtf, wafer_etest_id)
		end

	wafer_etest_name: STRING
		do  
			Result := wafer_etest_names_row.wafer_etest_name
		end

	vendor_PCM_data_handle: STRING
		-- a unique handle for the PCM data
	do  if has_vendor_lot_id then
		Result := su.string_of_5(production_id, su.underscore, 
					 wafer_etest_id, su.underscore,
					 wafer_vendor_lot_id)
	    else
		Result := su.string_of_3(production_id, su.underscore, 
					 wafer_etest_id)
	    end
	end;

feature {NONE} -- Implementation

	cached_field_step_patterns_row: like field_step_patterns_row
			-- Field step patterns row, cached in memory

end
