
note

	author: "David Hollenberg"
	date: "Thu Jun 12 13:50:27 PDT 2003"

class RUNS_ROW
inherit
	RUNS_RAW
	ROW_COUNT_CHECK_ACCESS
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	STRING_UTILITIES_ACCESS
		export	{NONE} all
		end

create
	make_template, make_null, make_by_key

feature -- Properties

	is_shared_run: BOOLEAN
			-- Is this a shared run?
		require
			run_type_exists: run_type /= Void
			run_type_nonempty: not run_type.is_empty
		do
			Result := run_type.is_equal (run_type_values.Shr)
		end

	is_dedicated_run: BOOLEAN
			-- Is this a dedicated run?
		require
			run_type_exists: run_type /= Void
			run_type_nonempty: not run_type.is_empty
		do
			Result := run_type.is_equal (run_type_values.Ded)
		end

	is_trusted_foundry_run: BOOLEAN
			-- Is this a trusted foundry run?
		require
			run_type_exists: run_type /= Void
			run_type_nonempty: not run_type.is_empty
		do
			Result := run_type.is_equal (run_type_values.Trf)
		end

	is_development_run: BOOLEAN
			-- Is this a development run?
		require
			run_type_exists: run_type /= Void
			run_type_nonempty: not run_type.is_empty
		do
			Result := run_type.is_equal (run_type_values.Dev)
		end

	is_released_to_manufacturing: BOOLEAN
			-- Does a "released to manufacturing" event
			-- for run exist?
		local
			event: EVENT_NAME_VALUES
			row: RUN_EVENTS_ROW
		do
			create row.make_by_key (connection, option.Old_no_exception_row, run_id, event.Released_to_manufacturing)
			Result := not row.is_empty
		end

	runs_assigned_to_sched_run (run_no: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Runs that are assigned to `run_no'
		require
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_I
		do
			create b.make (run_no)
			Result := all_rows_filtered_ordered (conn, "sched_run_no = :1", b, "run_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	runs_using_part_id (a_part_id: STRING wt: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Runs that use `a_part_id' and
			-- whose wafer techn is `wt'
		require
			a_part_id_not_void: a_part_id /= Void
			wt_not_void: wt /= Void
			connection_not_void: conn /= Void
			range_not_void: range /= Void
		local
			b: DB_BIND_SSS
			ptype: PRODUCTION_TYPE_VALUES
		do
			create b.make (wt, a_part_id, ptype.Initial_run)
			Result := all_rows_filtered_ordered (conn, "run_id in (select r.run_id from runs r, productions p where r.run_id = p.run_id and r.wafer_techn = :1 and (p.wafer_part_id_source = :2 or p.wafer_part_id = :2) and p.production_type = :3)", b, "run_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	wafer_techn_row: WAFER_TECHNS_ROW
			-- wafer_techns row for this run
	local wt: WAFER_TECHNS_ROW
	do  if is_null_wafer_techn
	     then cached_wafer_techn := Void
	    elseif cached_wafer_techn = Void or else
		   (not equal(wafer_techn,
				cached_wafer_techn.wafer_techn))
	     then create wt.make_template
		  cached_wafer_techn := wt.wafer_techns_with_name
				(wafer_techn, connection, 
				 expected_count.exactly_one).first
	    end;
	    Result := cached_wafer_techn
	end;

	external_design_die_offset: CONST_POINT
			-- Offset of center of design from center of die
			-- for dies whose effective overhead style is EXTERNAL
		require
			wafer_techn_not_null: not is_null_wafer_techn
			other_border_not_null: not is_null_other_border
		do
			Result := wafer_techn_row.external_design_die_offset (other_border)
		ensure
			result_not_void: Result /= Void
		end

	internal_design_die_offset: CONST_POINT
			-- Offset of center of design from center of die
			-- for dies whose effective overhead style is 
			-- not EXTERNAL
		do
			create Result.make_xy (0.0, 0.0)
		ensure
			result_not_void: Result /= Void
		end

	overhead_id: STRING 
			-- nvl(run_id_for_overhead, run_id)
	do  Result := su.empty_value(run_id_for_overhead, run_id)
	end;

	closer: STRING 
			-- Production manager for initial run
		do  
			Result := initial_run_production.production_manager
		end

	merge_type: STRING 
			-- Type of merge to be done
		local
			type: MERGE_TYPE_VALUES
		do  
			if is_field_merged then
				Result := type.Field_merge
			elseif send_cust_ic_design_files then
				Result := type.No_merge
			else
				Result := type.Die_merge
			end
		end

feature	-- wafer_lots interface

	wafer_lots: LIST[like anchored_wafer_lot]
	local r: like anchored_wafer_lot; b: DB_BIND_S;
	do  if cached_wafer_lots = Void then
		create r.make_template;
		create b.make(run_id);
		cached_wafer_lots := r.all_rows_filtered_ordered
			(connection, "run_id = :1", b, "wafer_lot_no")
	    end;
	    Result := cached_wafer_lots
	ensure Result /= Void
	end;

	select_wafer_lots(master: like wafer_lots)
		-- pick off my elements from those in master
		-- whoever built master is responsible for its contents
	do  create {LINKED_LIST[like anchored_wafer_lot]}
				cached_wafer_lots.make;
	    from  master.start
	    until master.after
	    loop  if equal(master.item.run_id, run_id)
		   then cached_wafer_lots.extend(master.item)
		  end;
		  master.forth
	    end
	ensure have_wafer_lots
	end;

	uncache_wafer_lots
		-- clear local cache
	do  cached_wafer_lots := Void
	ensure not have_wafer_lots
	end;

	have_wafer_lots: BOOLEAN
		-- do we currently have a cached list of wafer_lots?
	do  Result := cached_wafer_lots /= Void
	end;

	all_lots_have_same_stepping: BOOLEAN
		-- do all my wafer_lots have identical field_step_pattern
		--  AND offsets?
	local  rwl: like wafer_lots; rwl1: RUN_WAFER_LOTS_ROW
	do  Result := true;		-- until proven otherwise
	    rwl := wafer_lots;
	    if rwl.count > 1
	     then from  rwl.start;
			rwl1 := rwl.item; rwl.forth
		  until rwl.after or not Result
		  loop  Result := rwl1.has_same_stepping(rwl.item)
		        rwl.forth
		  end
	    end
	end;

feature	-- Wafer techn interface

	uncache_wafer_techn
		do
			cached_wafer_techn := Void
		end

feature	-- run options inquiry
	-- note that run_options uses run_wafer_lots, so wafer lots must
	--  be established, or nothing will come back

	all_options_matching(an_oid: STRING): DYNAMIC_LIST[STRING]
		-- an_oid is a db matchstring against all option_ids 
		--  "%" gets all the options
	local b: DB_BIND_SS
	do  create b.make(run_id, an_oid);
	    Result := connection.selected_strings("option_id", "run_options",
					"run_id = :1 and option_id like :2",
					"option_id", true, b)

	end;

feature	-- packaging_lots interface

	packaging_lots: LIST[like anchored_packaging_lot]
	local r: like anchored_packaging_lot; b: DB_BIND_S;
	do  if cached_packaging_lots = Void then
		create r.make_template;
		create b.make(run_id);
		cached_packaging_lots := r.all_rows_filtered_ordered
			(connection, "run_id = :1", b, "packaging_lot_no")
	    end;
	    Result := cached_packaging_lots
	ensure Result /= Void
	end;

	select_packaging_lots(master: like packaging_lots)
		-- pick off my elements from those in master
		-- whoever built master is responsible for its contents
	do  create {LINKED_LIST[like anchored_packaging_lot]}
			cached_packaging_lots.make;
	    from  master.start
	    until master.after
	    loop  if equal(master.item.run_id, run_id)
		   then cached_packaging_lots.extend(master.item)
		  end;
		  master.forth
	    end
	ensure have_packaging_lots
	end;

	uncache_packaging_lots
		-- clear local cache
	do  cached_packaging_lots := Void
	ensure not have_packaging_lots
	end;

	have_packaging_lots: BOOLEAN
		-- do we currently have a cached list of packaging_lots?
	do  Result := cached_packaging_lots /= Void
	end;

feature	-- Productions interface

	initial_run_production: like anchored_production
			-- Initial run production for `Current'
		do
			if cached_initial_run_production = Void then
				fetch_initial_run_production
			end
			Result := cached_initial_run_production
		ensure
			result_not_void: Result /= Void
		end

	uncache_initial_run_production
			-- Remove initial run production from cache, if present
		do
			cached_initial_run_production := Void
		ensure
			initial_run_production_not_cached: not is_cached_initial_run_production
		end

	is_cached_initial_run_production: BOOLEAN
			-- Is `initial_run_production' currently cached?
		do
			Result := cached_initial_run_production /= Void
		end

	productions: DYNAMIC_LIST [like anchored_production]
			-- All production sfor `Current'
		do
			if cached_productions = Void then
				fetch_productions
			end
			Result := cached_productions
		ensure
			result_not_void: Result /= Void
		end

	uncache_productions
			-- Remove productions from cache, if present
		do
			cached_productions := Void
		ensure
			productions_not_cached: not is_cached_productions
		end

	is_cached_productions: BOOLEAN
			-- Is `productions' currently cached?
		do
			Result := cached_productions /= Void
		end

feature	-- Scheduled run interface

	scheduled_run: like anchored_scheduled_run
			-- Scheduled run for `Current'
		require
			has_sched_run_no: not is_null_sched_run_no
		do
			if not is_cached_scheduled_run then
				create cached_scheduled_run.make_by_key (connection, option.Old_row, sched_run_no)
			end
			Result := cached_scheduled_run
		ensure
			result_not_void: Result /= Void
		end

	uncache_scheduled_run
			-- Remove scheduled run from cache, if present
		do
			cached_scheduled_run := Void
		ensure
			scheduled_run_not_cached: not is_cached_scheduled_run
		end

	is_cached_scheduled_run: BOOLEAN
			-- Is `scheduled_run' currently cached?
		do
			Result := cached_scheduled_run /= Void
		end

feature {NONE}

	cached_wafer_techn: WAFER_TECHNS_ROW

	anchored_wafer_lot: RUN_WAFER_LOTS_ROW
		-- dummy to (re-) declare the type
	do
	ensure Result = Void
	end;

	cached_wafer_lots: like wafer_lots
		-- cached list for wafer_lots

	anchored_packaging_lot: RUN_PACKAGING_LOTS_ROW
		-- dummy to (re-) declare the type
	do
	ensure Result = Void
	end;

	cached_packaging_lots: like packaging_lots
		-- cached list for packaging_lots

feature	-- lot locators, by wafer_process and packaging_order_type
	-- returns first of type if more than one; Void if none

	wafer_lot(wproc: STRING): like anchored_wafer_lot
	do  from  wafer_lots.start
	    until wafer_lots.after or Result /= Void
	    loop  if equal(wproc, wafer_lots.item.wafer_process)
		   then Result := wafer_lots.item end;
		  wafer_lots.forth
	    end
	end;

	packaging_lot(wproc, pot: STRING): like anchored_packaging_lot
		-- args are wafer_process and packaging_order_type
	local rpl: like anchored_packaging_lot
	do  from  packaging_lots.start
	    until packaging_lots.after or Result /= Void
	    loop  rpl := packaging_lots.item;
		  if equal(wproc, rpl.wafer_process) and
		     equal(pot, rpl.packaging_order_type)
		   then Result := rpl end;
		  packaging_lots.forth
	    end
	end;

feature	-- lot-based features hiding as run-based features

	field_step_pattern: STRING
		-- moved to run_wafer_lots; grab one
	require all_same: all_lots_have_same_stepping
	do  if wafer_lots.is_empty
	     then Result := ""
	     else Result := wafer_lots.first.field_step_pattern
	    end
	end;

feature {NONE} -- Implementation

	fetch_initial_run_production
		local
			row: like anchored_production
			type: PRODUCTION_TYPE_VALUES
		do
			create row.make_template
			cached_initial_run_production := row.productions_for_run_and_type (run_id, type.Initial_run, connection, expected_count.Exactly_one).first
		end

	fetch_productions
			-- Fetch all productions for `Current'
			-- and put in `cached_productions'
		local
			row: like anchored_production
		do
			create row.make_template
			cached_productions := row.productions_for_run (run_id, connection, expected_count.At_least_zero)
		end

	cached_initial_run_production: like anchored_production
			-- Initial run production row, cached in memory

	cached_productions: DYNAMIC_LIST [like anchored_production]
			-- Productions for `Current', cached in memory

	cached_scheduled_run: like anchored_scheduled_run
			-- Scheduled run for `Current', cached in
			-- memory

	anchored_production: PRODUCTIONS_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

	anchored_scheduled_run: SCHEDULED_RUNS_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

	run_type_values: RUN_TYPE_VALUES
		once
			create Result
		end
end
