
note

	author: "Generated"
	date: "Thu Jun 12 13:50:27 PDT 2003"

class SCHEDULED_RUNS_ROW
inherit
	SCHEDULED_RUNS_RAW

	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end

	CONDITION_UTILITIES

	DB_QUERY_OPTIONS

create
	make_template, make_null, make_by_key, make_by_org_and_run_name

feature -- Collections

	possible_scheduled_runs_ordered (rtype, wt: STRING is_edu: BOOLEAN days_past, days_future: INTEGER conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All rows for scheduled runs compatible with
			-- run_type `rtype', wafer techn `wt' and `is_edu',
			-- whose scheduled date is within the range
			-- indicated by `days_past' and `days_future'
		require
			rtype_not_void: rtype /= Void
			wafer_techn_exists: wt /= Void
			days_past_nonnegative: days_past >= 0
			days_future_nonnegative: days_future >= 0
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_SSSII
			predicate: CUST_ACCT_PREDICATE_VALUES
		do
			create b.make (rtype, wt, predicate.Is_educational, -days_past, days_future)
			Result := all_rows_filtered_ordered (conn, sched_runs_filter (is_edu, days_past, days_future), b, "date_scheduled")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end


feature	-- status

	is_cancelled: BOOLEAN
		-- has this run been cancelled?
	do  Result := not is_null_date_cancelled
	end;

	is_shr: BOOLEAN
	local rtv: RUN_TYPE_VALUES
	do  Result := equal(run_type, rtv.shr)
	end;

	is_ded: BOOLEAN
	local rtv: RUN_TYPE_VALUES
	do  Result := equal(run_type, rtv.ded)
	end;

	is_trf: BOOLEAN
	local rtv: RUN_TYPE_VALUES
	do  Result := equal(run_type, rtv.trf)
	end;

	is_dev: BOOLEAN
	local rtv: RUN_TYPE_VALUES
	do  Result := equal(run_type, rtv.dev)
	end;

	OK_to_update: BOOLEAN
		-- is a DED run update allowed at this time (and state)?
	require is_ded: is_ded
	local rnams: LIST[STRING]; run: RUNS_ROW
	do  rnams := run_ids
	    if rnams.is_empty then
		Result := true
	    else
		-- a DED scheduled_run CANNOT have more than one real run!!
		if rnams.count > 1 then
			handle_hard_error("More than 1 run for dedicated sched_run_no " + sched_run_no.out)
		end;
		create run.make_by_key(connection, old_row, rnams.first)
		Result := not run.project_set_validated
	    end
	end;

feature	-- printable handles on the scheduled run

	foundry: STRING
	local b: DB_BIND_S;
	do  create b.make(wafer_techn);
	    Result := connection.selected_string(
				"short_id", "organizations",
				org_no_of_wt, b);
	end;

	feature_size: STRING
	local b: DB_BIND_S;
	do  create b.make(wafer_techn);
	    Result := connection.selected_string(
				"feature_size_um_ss", "run_techns",
				run_techn_of_wt, b);
	    if (Result @ 1) = '.'
	     then Result.precede('0')
	    end
	end;

	terse_handle: STRING
	do  Result := su.string_of_5(
				date_scheduled.to_date_string,
				su.colon_space,
				foundry, su.space, feature_size)
	end;

	verbose_handle: STRING
	do  Result := su.string_of_5(
				foundry, su.space, feature_size,
				" run scheduled on ",
				date_scheduled.to_date_string)
	end;

	run_type_word: STRING
	local b: DB_BIND_S
	do  create b.make(run_type)
	    Result := connection.selected_string("run_type_word",
					"run_types", "run_type = :1", b)
	end;

	description: STRING
			-- Description of run for messages
		local
			ded_name: STRING
		do 
			if not is_null_dedicated_run_name then
				ded_name := dedicated_run_name + " "
			else
				ded_name := ""
			end
			Result := ded_name + date_scheduled.to_date_string + " " + run_type + " " + wafer_techn + " (" + sched_run_no.out + ")"
		end

feature	-- info on the associated run(s)

	run_count: INTEGER
		-- number of runs closed on me
	local b: DB_BIND_I;
	do  create b.make(sched_run_no);
	    Result := connection.selected_count("runs", by_sched_run, b)
	end;

	run_ids: LIST[STRING]
		-- the names (run_id) of those runs
	local b: DB_BIND_I;
	do  create b.make(sched_run_no);
	    Result := connection.selected_strings("run_id", "runs",
					by_sched_run, Void, false, b)
	end;

	run_event_count(evnt: STRING): INTEGER
		-- number of my runs that show (at least) one run_event
		--  with event_name = evnt
	local b: DB_BIND_SI;
	do  create b.make(evnt, sched_run_no);
	    Result := connection.selected_integer(
					"count(distinct run_id)",
					"run_events",
					event_filter, b)
	end;

	assigned_runs: LIST [STRING]
			-- List of runs assigned to `Current',
			-- ordered by run ID
		do
			if not is_cached_assigned_runs then
				fetch_assigned_runs
			end
			Result := cached_assigned_runs
		ensure
			result_exists: Result /= Void
		end

	assigned_runs_description: STRING
			-- Comma-separated list of runs assigned to
			-- `Current'
		do
			Result := su.joined_by (assigned_runs, ", ")
		ensure
			result_exists: Result /= Void
		end

	is_cached_assigned_runs: BOOLEAN
			-- Is list of runs assigned to `Current'
			-- cached?
		do
			Result := cached_assigned_runs /= Void
		end

	uncache_assigned_runs
			-- Remove list of assigned runs from cache
		do
			cached_assigned_runs := Void
		end

	dedicated_account_string: STRING
			-- Description of dedicated cust acct
		do
			if not is_cached_dedicated_account_string then
				fetch_dedicated_account_string
			end
			Result := cached_dedicated_account_string
		ensure
			result_exists: Result /= Void
		end

	is_cached_dedicated_account_string: BOOLEAN
			-- Is description of dedicated cust
			-- account cached?
		do
			Result := cached_dedicated_account_string /= Void
		end

	uncache_dedicated_account_string
			-- Remove description of dedicated
			-- cust account from cache
		do
			cached_dedicated_account_string := Void
		end

	cust_accounts: LIST [SCHEDULED_RUN_CUST_ACCTS_ROW]
			-- Additional customer accounts authorized
			-- to fabricate projects on a dedicated run
		do
			if not is_cached_cust_accounts then
				fetch_cust_accounts
			end
			Result := cached_cust_accounts
		ensure
			result_exists: Result /= Void
		end

	is_cached_cust_accounts: BOOLEAN
			-- Is list of additional authorized customer
			-- accounts cached?
		do
			Result := cached_cust_accounts /= Void
		end

	uncache_custs_accounts
			-- Remove list of additional authorized
			-- customer accounts from cache
		do
			cached_cust_accounts := Void
		end

	dedicated_organization: ORGANIZATIONS_ROW
			-- Organization for dedicated run
		do
			if not is_cached_dedicated_organization then
				fetch_dedicated_organization
			end
			Result := cached_dedicated_organization
		ensure
			result_not_void: Result /= Void
		end

	is_cached_dedicated_organization: BOOLEAN
			-- Is organization for dedicated run cached?
		do
			Result := cached_dedicated_organization /= Void
		end

	uncache_dedicated_organization
			-- Remove dedicated organization from cache
		do
			cached_dedicated_organization := Void
		end

feature	-- Properties

	is_dedicated: BOOLEAN
			-- Is `Current' for a dedicated run?
		local
			type: RUN_TYPE_VALUES
		do
			Result := run_type.is_equal (type.Ded)
		end

	is_authorized_account (n: INTEGER): BOOLEAN
			-- Is `n' an additional account authorized
			-- to fabricate projects on `Current"?
		local
			list: LIST [SCHEDULED_RUN_CUST_ACCTS_ROW]
		do
			list := cust_accounts
			from
				list.start
			until
				list.after or Result
			loop
				Result := Result or list.item.cust_acct_no = n
				list.forth
			end
		end

	is_still_open: BOOLEAN
		-- this scheduled run is still open for update 
		-- and/or project changes
	local rcnt: INTEGER; b: DB_BIND_I;
	do  rcnt := run_count
	    if rcnt = 0 then
		-- no runs at all
		Result := true
	    else
		-- have runs; at least one must still be open
		create b.make(sched_run_no);
		Result := connection.selected_count
				("runs", by_sched_run_and_unvalidated, b) > 0
	    end
	end;

feature	-- info on the fab instances

	project_count: INTEGER
		-- number of fab instances targeted to me
	local b: DB_BIND_I;
	do  create b.make(sched_run_no);
	    Result := connection.selected_count
				("ic_fab_instances",
				 "target_sched_run_no = :1", b)
	end;

	m_fab_nos: LIST[INTEGER]
		-- the m_fab_nos that are targeted to me
	local b: DB_BIND_I;
	do  create b.make(sched_run_no);
	    Result := connection.selected_integers("m_fab_no", "ic_fab_instances",
					   "target_sched_run_no = :1", "design_no", false, b)
	end;
	
feature	-- cust_quote_access

	cust_quote: CUST_QUOTES_ROW
	require have_quote: not is_null_m_cust_quote_no
	do  if cached_cust_quote = Void or else 
		cached_cust_quote.m_cust_quote_no /= m_cust_quote_no then
		create cached_cust_quote.make_by_key(connection, old_row, m_cust_quote_no)
	    end;
	    Result := cached_cust_quote
	end;

feature {NONE} -- cached_cust_quote

	cached_cust_quote: like cust_quote

feature	-- find scheduled_runs for secondary acct and run_name

	sched_runs_for_secondary_acct(conn: DB_CONNECTION; 
				      acctno: INTEGER;
				      a_run_name: STRING): DYNAMIC_LIST[like Current]
		-- find all scheduled_runs matching a_run_name for which
		--  acctno is a secondary user (i.e., in scheduled_run_cust_accts)
		--  and the run is NOT cancelled
		-- NOTE: finding more than one is a problem, but we let the caller deal with it!
	local b: DB_BIND_IS
	do  create b.make(acctno, a_run_name);
	    Result := all_rows_filtered(conn, secondary_acct_filter, b)
	end;

feature {NONE} -- alternate creation

	make_by_org_and_run_name(conn: DB_CONNECTION; opt: DB_QUERY_OPTION; 
				 an_org_no: INTEGER; a_run_name: STRING)
		-- NOTE: will find an existing cancelled run
	local b: DB_BIND_IS
	do  create b.make(an_org_no, a_run_name)
	    make_by_alternate_key(conn, opt, by_org_and_name, b)
	end;

feature {NONE} -- details

	by_sched_run: STRING = "sched_run_no = :1"

	by_sched_run_and_unvalidated: STRING = 
	"sched_run_no = :1 and project_set_validated = 'N'"

	org_no_of_wt: STRING =
	"org_no = (select org_no from wafer_techns %
			%where wafer_techn = :1)"

	run_techn_of_wt: STRING =
	"run_techn = (select run_techn from wafer_techns_ %
			%where wafer_techn = :1)"

	event_filter: STRING =
	"event_name = :1 and %
	%run_id in (select run_id from runs where sched_run_no = :2)"

feature {NONE} -- Implementation

	sched_runs_filter (is_edu: BOOLEAN days_past, days_future: INTEGER): STRING
			-- Filter for possible scheduled runs for this run,
			-- where `is_edu' indicates whether it is an
			-- educational run
		do
			Result := "run_type = :1 and (wafer_techn = :2 or wafer_techn in %N(select superset_wafer_techn from subset_wafer_techns where wafer_techn = :2)) and date_cancelled is null%N"
			Result.append ("and date_scheduled - sysdate between :4 and :5%N")
			if is_edu then
				-- Restrict to only IS_EDUCATIONAL
				Result.append ("and cust_acct_access_predicate = :3")
			else
				-- No restriction
				Result.append ("and :3 = :3")
			end
		ensure
			result_exists: Result /= Void
		end

	fetch_assigned_runs
		local
			template_row: RUNS_ROW
			list: LIST [RUNS_ROW]
		do
			create template_row.make_template
			list := template_row.runs_assigned_to_sched_run (sched_run_no, connection, expected_count.at_least_zero)
			create {ARRAYED_LIST [STRING]} cached_assigned_runs.make (list.count)
			from
				list.start
			until
				list.after
			loop
				cached_assigned_runs.extend (list.item.run_id)
				list.forth
			end
		ensure
			is_cached: is_cached_assigned_runs
		end

	fetch_dedicated_account_string
		local
			b: DB_BIND_I
		do
			create b.make (dedicated_cust_acct_no)
			cached_dedicated_account_string := connection.selected_string ("m_lookup.cust_acct_display_name(:1)", "dual", Void, b)
		ensure
			is_cached: is_cached_dedicated_account_string
		end

	fetch_dedicated_organization
		local
			b: DB_BIND_I
		do
			create b.make (dedicated_org_no)
			create cached_dedicated_organization.make_by_key (connection, option.Old_row, dedicated_org_no)
		ensure
			is_cached: is_cached_dedicated_organization
		end

	fetch_cust_accounts
		local
			row: SCHEDULED_RUN_CUST_ACCTS_ROW
		do
			create row.make_template
			cached_cust_accounts := row.rows_for_sched_run_no (sched_run_no, connection, expected_count.At_least_zero)
		ensure
			is_cached: is_cached_cust_accounts
		end

	cached_assigned_runs: like assigned_runs
			-- List of assigned runs, cached in memory

	cached_dedicated_account_string: like dedicated_account_string
			-- Description of dedicated cust account,
			-- cached in memory

	cached_dedicated_organization: like dedicated_organization
			-- Organization for dedicated run,
			-- cached in memory

	cached_cust_accounts: like cust_accounts
			-- List of additional accounts authorized to
			-- fabricate on dedicated run, cached in
			-- memory

	by_org_and_name: STRING =
	"dedicated_org_no = :1 and %
	%m_lookup.comparable_run_name(dedicated_run_name) = m_lookup.comparable_run_name(:2)"

	secondary_acct_filter: STRING =
	"sched_run_no in %
	%(select sched_run_no from scheduled_run_cust_accts where cust_acct_no = :1) and %
	%m_lookup.comparable_run_name(dedicated_run_name) = m_lookup.comparable_run_name(:2) and %
	%date_cancelled is null"

end
