
note

	author: "Generated"
	date: "Thu Feb 26 10:39:46 PST 2004"

class WAFER_PROCESSES_ROW
inherit
	WAFER_PROCESSES_RAW
	ROW_COUNT_CHECK_ACCESS
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end

create
	make_template, make_null, make_by_key

feature

	processes_for_techn_family (wtf: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All wafer processes for wafer techn family `wtf'
		require
			wtf_exists: wtf /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (wtf)
			Result := all_rows_filtered (conn, "wafer_techn_family = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	processes_for_wafer_techn (wt: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All wafer processes offered in wafer techn `wt',
			-- ordered by wafer process name
		require
			wt_exists: wt /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (wt)
			Result := all_rows_filtered_ordered (conn, wt_process_filter, b, "wafer_process")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	processes_for_production (prod: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All wafer processes used in wafer lots for
			-- production `prod', ordered by wafer etest ID
		require
			prod_not_void: prod /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (prod)
			Result := all_rows_filtered_ordered (conn, production_process_filter, b, "wafer_etest_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	processes_for_run_fab_instances (run: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All wafer processes to be ordered for at least
			-- one fab instance on run `run', ordered by
			-- wafer etest ID.  Excludes processes for
			-- a tier project on a 3DIC run
		require
			run_not_void: run /= Void
			connection_not_void: conn /= Void
			connection_open: conn.is_open
			range_not_void: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (run)
			Result := all_rows_filtered_ordered (conn, fab_instance_process_filter, b, "wafer_etest_id")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	is_found_in(lots: LIST[RUN_WAFER_LOTS_ROW]): BOOLEAN
			-- am I the wafer_process for (at least) one lot in lots?
		require lots /= Void
		do
			from	lots.start
			until	lots.after or Result
			loop	Result := equal(wafer_process, lots.item.wafer_process)
				lots.forth
			end
		end

	etest_name: WAFER_ETEST_NAMES_ROW
			-- Row for `wafer_techn_family' and `wafer_etest_id'
		do  
			if not is_cached_etest_name then 
	     			create cached_etest_name.make_by_etest_id (connection, option.Old_row, wafer_techn_family, wafer_etest_id)
	    		end
	    		Result := cached_etest_name
		ensure
			result_not_void: Result /= Void
		end;

	uncache_etest_name
			-- Uncache `etest_name' for `Current'
		do  
			cached_etest_name := Void
		ensure
			etest_name_not_cached: not is_cached_etest_name
		end;

	is_cached_etest_name: BOOLEAN
			-- Is etest name cached?
		do  
			Result := cached_etest_name /= Void
		end;

	name_for_wafer_lot_creation: STRING
			-- Name of lot used in wafer lot creation interface
		do  
			Result := wafer_process
		end;

feature {NONE} -- Implementation
	
	wt_process_filter: STRING =
	"(wafer_techn_family, wafer_process_id) in %
	%(select wtop.wafer_techn_family, wtop.wafer_process_id %
	%from wafer_techn_offered_processes wtop, wafer_techns_ wt %
	%where wtop.wafer_techn_family = wt.wafer_techn_family %
	%  and wtop.wafer_techn_id = wt.wafer_techn_id %
	%  and wt.wafer_techn = :1)"

	production_process_filter: STRING =
	"wafer_process in (select wafer_process from run_wafer_lots where production_id = :1)"

	fab_instance_process_filter: STRING = "[
		wafer_process in 
		  (select wafer_process 
		     from ic_fab_instance_processes ifip, ic_fab_instances ifi, cust_ic_designs cid 
		    where ifip.m_fab_no = ifi.m_fab_no
		      and ifi.design_no = cid.design_no
		      and cid.design_no_3dic is null
		      and ifi.run_id = :1)
			]"
			-- Filter to select all processes to order for
			-- fab instances on a run, excluding tier projects
			-- on a 3DIC run

	cached_etest_name: like etest_name
			-- Row for `wafer_etest_name', cached in memory

end
