class WAFER_TECHN_3D_TIERS_ROW

inherit

	WAFER_TECHN_3D_TIERS_RAW

	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	ROW_COUNT_CHECK_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export 
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end

create

	make_template, make_null, make_detached, make_by_key

feature	-- Collections

	tiers_for_wafer_techn (wt: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- All tiers for wafer techn `wt', 
			-- ordered by tier number
		require
			wt_exists: wt /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (wt)
			Result := all_rows_filtered_ordered (conn, "wafer_techn = :1", b, "tier_no")
			check_row_count (Result, range)
		ensure
			result_not_void: Result /= Void
		end

feature -- Associated rows lookup

	tier_wafer_techns_row: WAFER_TECHNS_ROW
			-- Row from wafer_techns for `tier_wafer_techn'
		local
			r: like cached_tier_wafer_techns_row
		do
			if cached_tier_wafer_techns_row = Void then
				create r.make_template
				cached_tier_wafer_techns_row := r.wafer_techns_with_name (tier_wafer_techn, connection, expected_count.exactly_one).first
			end
			Result := cached_tier_wafer_techns_row
		ensure
			result_not_void: Result /= Void
		end

	uncache_tier_wafer_techns_row
			-- Remove tier wafer techns row from cache, if present
		do
			cached_tier_wafer_techns_row := Void
		ensure
			tier_wafer_techns_row_not_cached: not is_cached_tier_wafer_techns_row 
		end

	is_cached_tier_wafer_techns_row: BOOLEAN
			-- Is `tier_wafer_techns_row' currently cached?
		do
			Result := cached_tier_wafer_techns_row /= Void
		end

feature	-- Properties

	adjusted_location_x (a_x: DOUBLE): DOUBLE
			-- `a_x' adjusted according to
			-- `wafer_alignment'
		local
			align: WAFER_ALIGNMENT_VALUES
		do
			if wafer_alignment.is_equal (align.Rot0) then
				Result := a_x
			elseif wafer_alignment.is_equal (align.Mirror90) then
				Result := -a_x
			else
				handle_hard_error ("unsupported wafer alignment value " + wafer_alignment)
			end
		end

	adjusted_location_y (a_y: DOUBLE): DOUBLE
			-- `a_y' adjusted according to
			-- `wafer_alignment'
		local
			align: WAFER_ALIGNMENT_VALUES
		do
			if wafer_alignment.is_equal (align.Rot0) or wafer_alignment.is_equal (align.Mirror90) then
				Result := a_y
			else
				handle_hard_error ("unsupported wafer alignment value " + wafer_alignment)
			end
		end

feature	-- tier handle

	tier_handle: STRING
		-- descriptive handle using either tier_name or tier_no
	do  if is_null_tier_name then
		Result := su.string_of_2("tier #", tier_no.out)
	    else
		Result := tier_name
	    end
	end;

	description: STRING
			-- Tier description used during run closing
		do  
			if is_null_tier_name then
				Result := "Tier " + tier_no.out
			elseif su.is_prefix_caseless ("tier", tier_name) then
				Result := "Tier " + tier_name.substring (5, tier_name.count)
	    		else
				Result := tier_name
	    		end
		end

feature {NONE} -- Implementation

	cached_tier_wafer_techns_row: WAFER_TECHNS_ROW
			-- Wafer techns row for tier wafer techn,
			-- cached in memory

end -- WAFER_TECHN_3D_TIERS_ROW
