
note

	author: "Generated"
	date: "Thu Jun 12 13:50:27 PDT 2003"

class WAFER_TECHNS_ROW
inherit
	WAFER_TECHNS_RAW
		rename
			max_scribe_width as wt_max_scribe_width,
			is_null_max_scribe_width as wt_is_null_max_scribe_width
		end
	ROW_COUNT_CHECK_ACCESS
		export 
			{NONE} all
		end
	NULL_VALUE_CHECK_ACCESS
		export 
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export 
			{NONE} all
		end
	NUMBER_UTILITIES_ACCESS
		export
			{NONE} all
		end
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end

create
	make_template, make_null

feature -- Properties

	wafer_techns_with_name (wt_name: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Wafer techns with name `wt_name'
		require
			wt_name_exists: wt_name /= Void
			connection_exists: conn /= Void
			connection_open: conn.is_open
			range_exists: range /= Void
		local
			b: DB_BIND_S
		do
			create b.make (wt_name)
			Result := all_rows_filtered (conn, "wafer_techn = :1", b)
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	possible_wafer_techns_ordered (dt, run_type: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Non-retired wafer techns compatible with
			-- design type `dt' and `run_type'
		require
			design_type_exists: dt /= Void
			run_type_exists: run_type /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SSSB
			wt_status: WAFER_TECHN_STATUS_VALUES
		do
			create b.make (wt_status.Ret, dt, run_type, True)
			Result := all_rows_filtered_ordered (conn, "wafer_techn_status <> :1 and design_type = :2 and %N  exists %N  (select 1 from wafer_techn#run_types wtrt%N   where wtrt.wafer_techn = wt.wafer_techn and wtrt.run_type = :3 and wtrt.available = :4)", b, "wafer_techn")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	current_wafer_techns_ordered (dt: STRING conn: DB_CONNECTION range: ITEM_COUNT_RANGE): LIST [like Current]
			-- Current wafer techns compatible with
			-- design type `dt'
		require
			design_type_exists: dt /= Void
			connection_exists: conn /= Void
			range_exists: range /= Void
		local
			b: DB_BIND_SS
			wt_status: WAFER_TECHN_STATUS_VALUES
		do
			create b.make (wt_status.Cur, dt)
			Result := all_rows_filtered_ordered (conn, "wafer_techn_status = :1 and design_type = :2", b, "wafer_techn")
			check_row_count (Result, range)
		ensure
			result_exists: Result /= Void
		end

	has_wafer_techn (s: STRING): BOOLEAN
			-- Does `Current' have `s' as its wafer techn?
		require
			s_not_void: s /= Void
		do
			Result := wafer_techn.is_equal (s)
		end

	non_null_design_techn_scribe: like design_techn_scribe
			-- Value of `design_techn_scribe', with error check 
			-- for nullness
		do
			check_not_null ("design_techn_scribe", is_null_design_techn_scribe, Current)
			Result := design_techn_scribe
		end

	max_scribe_width: like wt_max_scribe_width
			-- Default width of scribe
		local
			tiers: like wafer_techn_3d_tiers
			tier: WAFER_TECHN_3D_TIERS_ROW
			w: like max_scribe_width
			widths: LIST [like max_scribe_width]
		do
			if is_3dic then
				tiers := wafer_techn_3d_tiers
				create {ARRAYED_LIST [like max_scribe_width]} widths.make (1)
				from
					tiers.start
				until
					tiers.after
				loop
					tier := tiers.item
					w := tier.tier_wafer_techns_row.non_null_max_scribe_width
					if not widths.has (w) then
						widths.extend (w)
					end
					tiers.forth
				end
				if widths.count > 1 then
					handle_unsupported ("not all tier wafer techns for " + wafer_techn + " have the same max_scribe_width")
				end
				Result := widths.first
			else
				Result := wt_max_scribe_width
			end
		end

	is_null_max_scribe_width: like wt_is_null_max_scribe_width
			-- Is default width of scribe null?
		local
			tiers: like wafer_techn_3d_tiers
			tier: WAFER_TECHN_3D_TIERS_ROW
		do
			if is_3dic then
				tiers := wafer_techn_3d_tiers
				Result := tiers.is_empty
				from
					tiers.start
				until
					tiers.after
				loop
					tier := tiers.item
					Result := Result or tier.tier_wafer_techns_row.is_null_max_scribe_width
					tiers.forth
				end
			else
				Result := wt_is_null_max_scribe_width
			end
		end

	non_null_max_scribe_width: like max_scribe_width
			-- Value of `max_scribe_width', with error check 
			-- for nullness
		do
			check_not_null ("max_scribe_width", is_null_max_scribe_width, Current)
			Result := max_scribe_width
		end

	external_design_die_offset (an_other_border: DOUBLE): CONST_POINT
			-- Offset of center of design from center of die
			-- for dies whose effective overhead style is EXTERNAL,
			-- using `an_other_border' instead of `other_border'
		do
			create Result.make_xy (0.0, (an_other_border - (top_border.min (an_other_border) + overhead_strip_height + design_strip_separation)) / 2.0)
		ensure
			result_not_void: Result /= Void
		end

	minimum_scribe_width: DOUBLE
			-- Minimum width of scribes for wafer techn
		require
			has_design_techn_scribe: not is_null_design_techn_scribe
		local
			layer: DESIGN_TECHN_LAYERS_ROW
			found: BOOLEAN
		do
			if not is_cached_scribe_layers then
				fetch_scribe_layers
			end
			from
				Result := 0
				found := False
				cached_scribe_layers.start
			until
				cached_scribe_layers.after
			loop
				layer := cached_scribe_layers.item
				if not layer.is_null_near_edge then
					found := True
					Result := Result.max (layer.near_edge + layer.min_width)
						-- Guarantee that no scribe layer
						-- can disappear or go below 
						-- minimum width
				end
				cached_scribe_layers.forth
			end
			if not found then
				handle_hard_error ("cannot compute minimum scribe width for wafer techn " + wafer_techn + " - no scribe layers with non-null near_edge value")
			elseif Result <= 0 then
				handle_hard_error ("cannot compute minimum scribe width for wafer techn " + wafer_techn + "%NNear_edge + min_width <= 0 for every scribe layer with non-null near_edge value")
			end
		end

	top_overhead_for_other_border (an_other_border: DOUBLE): DOUBLE
			-- Amount of overhead added to top side of design,
			-- using `an_other_border' instead of `other_border'
			-- in `Current'.  Applicable only when die overhead
			-- style is EXTERNAL
		do
			Result := design_strip_separation + overhead_strip_height + an_other_border.min (top_border)
		end

	top_overhead_description (an_overhead_style: STRING an_other_border: DOUBLE): STRING
			-- Description of top overhead
		local
			style: OVERHEAD_STYLE_VALUES
			ext_val, int_val: STRING
		do
			if an_overhead_style.is_equal (style.External_) then
				Result := number_as_string (top_overhead_for_other_border (an_other_border))
			elseif an_overhead_style.is_equal (style.None) or an_overhead_style.is_equal (style.Internal) then
				Result := number_as_string (an_other_border)
			elseif an_overhead_style.is_equal ("") then
				ext_val := number_as_string (top_overhead_for_other_border (an_other_border))
				int_val := number_as_string (an_other_border)
				create Result.make (30)
				if not int_val.is_empty then
					Result.append (int_val)
					Result.append (" (internal)")
				end
				if not ext_val.is_empty then
					Result.append ("   ")
					Result.append (ext_val)
					Result.append (" (external)")
				end
			else
				Result := "(unknown style " + an_overhead_style + ")"
			end
		end

	project_overhead (an_overhead_style: STRING an_other_border: DOUBLE): CONST_POINT
			-- Amount of overhead in X and Y dimensions that
			-- is added around a cust project or test vehicle
			-- using `an_overhead_style' and `an_other_border'
		require
			an_overhead_style_not_void: an_overhead_style /= Void
		local
			style: OVERHEAD_STYLE_VALUES
		do
			if an_overhead_style.is_equal (style.External_) then
				create Result.make_xy (2 * an_other_border, top_overhead_for_other_border (an_other_border) + an_other_border)
			else
				create Result.make_xy (2 * an_other_border, 2 * an_other_border)
			end
		end

	uniform_white_space_border (an_overhead_style: STRING an_other_border: DOUBLE): DOUBLE
			-- Amount of white space added around a cust
			-- project or test vehicle using `an_overhead_style'
			-- and `an_other_border'
		local
			style: OVERHEAD_STYLE_VALUES
		do
			if an_overhead_style.is_equal (style.External_) then
				Result := an_other_border.min (top_border)
			else
				Result := an_other_border
			end
		end

	die_size_with_overhead (an_overhead_style: STRING an_other_border: DOUBLE size: CONST_POINT): CONST_POINT
			-- Size of die for project whose design size
			-- is `size' using `an_overhead_style' and
			-- `an_other_border'.  Result is rounded up to
			-- multiple of packing grid resolution
		require
			size_not_void: size /= Void
			an_overhead_style_not_void: an_overhead_style /= Void
		local
			res: DOUBLE
		do
			res := packing_grid_resolution
			Result := size.twin
			Result := Result + project_overhead (an_overhead_style, an_other_border)
			create Result.make_xy (ceiling_multiple_of_res (Result.x, res), ceiling_multiple_of_res (Result.y, res))
		end

	project_size_without_overhead (an_overhead_style: STRING an_other_border: DOUBLE size: CONST_POINT): CONST_POINT
			-- Maximum size of project for die whose size
			-- is `size' using `an_overhead_style' and
			-- `an_other_border'
		require
			size_not_void: size /= Void
			an_overhead_style_not_void: an_overhead_style /= Void
		local
			res: DOUBLE
		do
			res := packing_grid_resolution
			Result := size.twin
			Result := Result - project_overhead (an_overhead_style, an_other_border)
			create Result.make_xy (floor_multiple_of_res (Result.x, res), floor_multiple_of_res (Result.y, res))
		end

	needs_merged_gds_files: BOOLEAN
			-- Does `Current' require that we generate
			-- merged GDS files?
		do
			Result := not is_null_gds_merge_mode
		end

	is_3dic: BOOLEAN
			-- Is this a 3DIC wafer techn?
		do
			Result := design_types_row.is_3dic
		end

	scale_factor: DOUBLE
			-- Scale factor for `Current'
		local
			levels: like wafer_techn_levels
			tiers: like wafer_techn_3d_tiers
			tier : WAFER_TECHN_3D_TIERS_ROW
			sf: DOUBLE
			factors: LIST [DOUBLE]
		do
			if is_3dic then
				tiers := wafer_techn_3d_tiers
				create {ARRAYED_LIST [DOUBLE]} factors.make (1)
				from
					tiers.start
				until
					tiers.after
				loop
					tier := tiers.item
					sf := tier.tier_wafer_techns_row.scale_factor
					if not factors.has (sf) then
						factors.extend (sf)
					end
					tiers.forth
				end
				if factors.count > 1 then
					handle_unsupported ("not all tier wafer techns for " + wafer_techn + " have the same scale factor")
				end
				Result := factors.first
			else
				levels := wafer_techn_levels
				if not levels.is_empty then
					Result := levels.first.scale_factor
				else
					handle_hard_error ("no wafer techn levels for wafer techn " + wafer_techn)
				end
			end
		end

	nth_tier (n: INTEGER): WAFER_TECHN_3D_TIERS_ROW
			-- `n'th tier for `Current'
		require
			is_3dic: is_3dic
		local
			tiers: like wafer_techn_3d_tiers
			tier : WAFER_TECHN_3D_TIERS_ROW
			c: CURSOR
		do
			tiers := wafer_techn_3d_tiers
			c := tiers.cursor
			from
				tiers.start
			until
				tiers.after or Result /= Void
			loop
				tier := tiers.item
				if tier.tier_no = n then
					Result := tier
				end
				tiers.forth
			end
			tiers.go_to (c)
			if Result = Void then
				create tier.make_template
				handle_hard_error ("tier number " + n.out + " not found for wafer techn " + wafer_techn + " in table " + tier.table_name)
			end
		end
		
feature -- Operations

	check_one_scale_factor
			-- Report unsupported condition if there is more
			-- than one scale factor among levels for run
		local
			tiers: like wafer_techn_3d_tiers
			tier : WAFER_TECHN_3D_TIERS_ROW
			sf: DOUBLE
			factors: LIST [DOUBLE]
		do
			if is_3dic then
				tiers := wafer_techn_3d_tiers
				create {ARRAYED_LIST [DOUBLE]} factors.make (1)
				from
					tiers.start
				until
					tiers.after
				loop
					tier := tiers.item
					tier.tier_wafer_techns_row.check_one_scale_factor
					sf := tier.tier_wafer_techns_row.scale_factor
					if not factors.has (sf) then
						factors.extend (sf)
					end
					tiers.forth
				end
				if factors.count > 1 then
					handle_unsupported ("not all tier wafer techns for " + wafer_techn + " have the same scale factor")
				end
			else
				if not same_scale_factor (wafer_techn_levels) then
					handle_unsupported ("not all levels for wafer_techn " + wafer_techn + " have the same scale factor")
				end
			end
		end

feature -- Associated rows lookup

	design_types_row: DESIGN_TYPES_ROW
			-- Design types row for `Current'
		do
			if cached_design_types_row = Void then
				create cached_design_types_row.make_by_key (connection, option.Old_row, design_type)
			end
			Result := cached_design_types_row
		ensure
			result_not_void: Result /= Void
		end

	uncache_design_types_row
			-- Remove design types row from cache, if present
		do
			cached_design_types_row := Void
		ensure
			design_types_row_not_cached: not is_cached_design_types_row
		end

	is_cached_design_types_row: BOOLEAN
			-- Is `design_types_row' currently cached?
		do
			Result := cached_design_types_row /= Void
		end

	wafer_techn_3d_tiers: LIST [WAFER_TECHN_3D_TIERS_ROW]
			-- Tiers for `Current', in tier number order.
		require
			is_3dic: is_3dic
		do
			if cached_wafer_techn_3d_tiers = Void then
				fetch_wafer_techn_3d_tiers
			end
			Result := cached_wafer_techn_3d_tiers
		ensure
			result_not_void: Result /= Void
		end

	uncache_wafer_techn_3d_tiers
			-- Remove wafer techn 3D tiers from cache, if
			-- present
		do
			cached_wafer_techn_3d_tiers := Void
		ensure
			tiers_not_cached: not is_cached_wafer_techn_3d_tiers
		end

	is_cached_wafer_techn_3d_tiers: BOOLEAN
			-- Is `wafer_techn_3d_tiers' currently cached?
		do
			Result := cached_wafer_techn_3d_tiers /= Void
		end

	wafer_techn_levels: LIST [WAFER_TECHN_LEVELS_ROW]
			-- List of levels for `Current', ordered by
			-- level number.  List will be empty
			-- for a 3DIC wafer techn
		do
			if cached_wafer_techn_levels = Void then
				fetch_wafer_techn_levels
			end
			Result := cached_wafer_techn_levels
		ensure
			result_not_void: Result /= Void
		end

	uncache_wafer_techn_levels
			-- Remove wafer techn levels from cache, if
			-- present
		do
			cached_wafer_techn_levels := Void
		ensure
			levels_not_cached: not is_cached_wafer_techn_levels
		end

	is_cached_wafer_techn_levels: BOOLEAN
			-- Is `wafer_techn_levels' currently cached?
		do
			Result := cached_wafer_techn_levels /= Void
		end

feature {NONE} -- Implementation

	ceiling_multiple_of_res (size: DOUBLE n: DOUBLE): DOUBLE
			-- Size increased (if necessary) to multiple of n
		require
			size_positive: size >= 0.0
			n_positive: n > 0.0
		do
			if nu.is_multiple_of_resolution (size, n, scale_top_border) then
				Result := size
			else
				Result := nu.ceiling_real_64 (size / n) * n
			end
		end
	
	floor_multiple_of_res (size: DOUBLE n: DOUBLE): DOUBLE
			-- Size decreased (if necessary) to multiple of n
		require
			n_positive: n > 0.0
		do
			if nu.is_multiple_of_resolution (size, n, scale_top_border) then
				Result := size
			else
				Result := nu.floor_real_64 (size / n) * n
			end
		end
	
	fetch_scribe_layers
			-- Fetch layers for scribe design techn
			-- and put in `cached_scribe_layers'
		local
			row: DESIGN_TECHN_LAYERS_ROW
		do
			create row.make_template
			cached_scribe_layers := row.layers_for_dt (design_techn_scribe, connection, expected_count.At_least_one)
		end

	fetch_wafer_techn_3d_tiers
			-- Fetch 3D tiers for wafer techn
			-- and put in `cached_wafer_techn_3d_tiers'
		local
			r: WAFER_TECHN_3D_TIERS_ROW
		do
			create r.make_template
			cached_wafer_techn_3d_tiers := r.tiers_for_wafer_techn (wafer_techn, connection, expected_count.At_least_one)
		end

	fetch_wafer_techn_levels
			-- Fetch levels for wafer techn
			-- and put in `cached_wafer_techn_levels'
		local
			r: WAFER_TECHN_LEVELS_ROW
		do
			create r.make_template
			cached_wafer_techn_levels := r.levels_for_wafer_techn (wafer_techn, connection, expected_count.At_least_zero)
		end

	number_as_string (n: DOUBLE): STRING
			-- Value of `n' as a string suitable for display,
			-- using `scale_top_border' as the scale
		do
			Result := nu.formatted_number_or_integer (n, Scale_top_border)
		end

	same_scale_factor (levels: LIST [WAFER_TECHN_LEVELS_ROW]): BOOLEAN
			-- Do all elements of `levels' have the same scale
			-- factor?
		require
			levels_exists: levels /= Void
		local
			factor: DOUBLE
		do
			Result := True
			if not levels.is_empty then
				factor := levels.first.scale_factor
			end
			from
				levels.start
			until
				levels.after or not Result
			loop
				if levels.item.scale_factor /= factor then
					Result := False
				end
				levels.forth
			end
		end

	is_cached_scribe_layers: BOOLEAN
			-- Are layers for scribe design techn
			-- currently cached?

	cached_scribe_layers: LIST [DESIGN_TECHN_LAYERS_ROW]
			-- Layers for scribe design techn
			-- cached in memory
	
	cached_design_types_row: DESIGN_TYPES_ROW
			-- Design types row for Current,
			-- cached in memory
	
	cached_wafer_techn_3d_tiers: LIST [WAFER_TECHN_3D_TIERS_ROW]
			-- 3D tiers for Current, cached in memory
	
	cached_wafer_techn_levels: LIST [WAFER_TECHN_LEVELS_ROW]
			-- Levels for Current, cached in memory
	
end
