
note

	author: "David Hollenberg"

deferred class ABSTRACT_CUT_PATTERN
inherit
	ANY
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end
	CONST_ABSTRACT_EXT_CUT_DIST_SPEC

feature -- Properties

	current_xform_type: CUT_PATTERN_XFORM_TYPES_ROW
			-- Xform type row for current cut 
			-- pattern xform type
		do
			all_xform_types.search (cut_pattern_xform_type)
			if all_xform_types.found then
				Result := all_xform_types.found_item
			else
				handle_hard_error ("cannot find xform type " + cut_pattern_xform_type + " in internal table")
			end
		end

	all_xform_types: HASH_TABLE [CUT_PATTERN_XFORM_TYPES_ROW, STRING]
			-- All cut pattern xform types, indexed by
			-- xform type name
		do
			if cached_all_xform_types = Void then
				fetch_all_xform_types
			end
			Result := cached_all_xform_types
		end

	source_type: STRING
			-- Type of entity to which cut pattern
			-- is applied
		do
			Result := current_xform_type.source_type
		end

	type_for_maps: STRING
			-- Description of type of transform cut
			-- pattern performs, for cut pattern maps
		local
			src_type, res_type: STRING
			stype: CUT_PATTERN_SOURCE_TYPE_VALUES
			rtype: CUT_PATTERN_RESULT_TYPE_VALUES
		do
			src_type := current_xform_type.source_type
			res_type := current_xform_type.result_type
			if src_type.is_equal (stype.Wafers) then
				if res_type.is_equal (rtype.Wafer_strips) then
					Result := "Wafer to Strips"
				elseif res_type.is_equal (rtype.Isolated_sites) then
					Result := "Wafer"
			 	else
					handle_hard_error ("unknown cut pattern result type " + res_type)
				end
			elseif src_type.is_equal (stype.Wafer_strips) then
				Result := "Strip"
			else
				handle_hard_error ("unknown cut pattern source type " + src_type)
			end
		end

	transform_type_description: STRING
			-- Description of type of transform cut
			-- pattern performs that is suitable for
			-- interactive display
		local
			src_type, res_type: STRING
			stype: CUT_PATTERN_SOURCE_TYPE_VALUES
			rtype: CUT_PATTERN_RESULT_TYPE_VALUES
		do
			src_type := current_xform_type.source_type
			res_type := current_xform_type.result_type
			if src_type.is_equal (stype.Wafers) then
				if res_type.is_equal (rtype.Wafer_strips) then
					Result := "Wafers to Strips"
				elseif res_type.is_equal (rtype.Isolated_sites) then
					Result := "Wafers to Sites"
			 	else
					handle_hard_error ("unknown cut pattern result type " + res_type)
				end
			elseif src_type.is_equal (stype.Wafer_strips) then
				Result := "Strips to Sites"
			else
				handle_hard_error ("unknown cut pattern source type " + src_type)
			end
		end

	result_type: STRING
			-- Type of entity the application of cut pattern
			-- produces
		do
			Result := current_xform_type.result_type
		end

	produces_strips: BOOLEAN
			-- Does this type of cut pattern
			-- produce wafer strips?
		do
			Result := current_xform_type.produces_strips
		end

	produces_non_strips: BOOLEAN
			-- Does this type of cut pattern
			-- not produce wafer strips?
		do
			Result := not produces_strips
		end

feature -- Deferred properties

	production_id: STRING
			-- Production ID
		deferred
		end

	cut_pattern_no: INTEGER
			-- Cut pattern number
		deferred
		end

	cut_pattern_no_parent: INTEGER
			-- Parent cut pattern number
		deferred
		end

	cut_pattern_xform_type: STRING
			-- Type of transform application of cut
			-- pattern performs
		deferred
		end

	field_name: STRING
			-- Name of field to which cut pattern applies
		deferred
		end

	is_completed: BOOLEAN
			-- Is cut pattern completed (not still
			-- under construction)?
		deferred
		end

	wafer_etest_name: STRING
			-- Etest name of projects isolated by cut pattern
		deferred
		end

	external_cut_distance_left: DOUBLE
			-- External cut distance on left side
			-- of payload
		deferred
		end

	external_cut_distance_right: DOUBLE
			-- External cut distance on right side
			-- of payload
		deferred
		end

	external_cut_distance_bottom: DOUBLE
			-- External cut distance on bottom side
			-- of payload
		deferred
		end

	external_cut_distance_top: DOUBLE
			-- External cut distance on top side
			-- of payload
		deferred
		end

	saw_blade_width: DOUBLE
			-- Width of saw blade to be used for pattern
		deferred
		end

	quantity_wafers_needed: INTEGER
			-- Number of wafers to cut with cut pattern
		deferred
		end

	is_null_production_id: BOOLEAN
			-- Is production ID null?
		deferred
		end

	is_null_cut_pattern_no: BOOLEAN
			-- Is cut pattern number null?
		deferred
		end

	is_null_cut_pattern_no_parent: BOOLEAN
			-- Is parent cut pattern number null?
		deferred
		end

	is_null_field_name: BOOLEAN
			-- Is field name null?
		deferred
		end

	is_null_is_completed: BOOLEAN
			-- Is completed indicator null?
		deferred
		end

	is_null_wafer_etest_name: BOOLEAN
			-- Is wafer test name null?
		deferred
		end

	is_null_cut_pattern_xform_type: BOOLEAN
			-- Is cut pattern transform type null?
		deferred
		end

	is_null_external_cut_distance_left: BOOLEAN
			-- Is external cut distance on left
			-- side floating?
		deferred
		end

	is_null_external_cut_distance_right: BOOLEAN
			-- Is external cut distance on right
			-- side floating?
		deferred
		end

	is_null_external_cut_distance_bottom: BOOLEAN
			-- Is external cut distance on bottom
			-- side floating?
		deferred
		end

	is_null_external_cut_distance_top: BOOLEAN
			-- Is external cut distance on top
			-- side floating?
		deferred
		end

	is_null_quantity_wafers_needed: BOOLEAN
			-- Is number of wafers to cut null?
		deferred
		end

	any_connection: DB_CONNECTION
			-- Explicit or implicit connection 
		deferred
		end

feature {NONE} -- Implementation

	fetch_all_xform_types
			-- Fetch all cut pattern xform types
		local
			row: CUT_PATTERN_XFORM_TYPES_ROW
			list: LIST [CUT_PATTERN_XFORM_TYPES_ROW]
		do
			create row.make_template
			list := row.all_xform_types (any_connection, expected_count.At_least_one)
			create cached_all_xform_types.make (list.count)
			list.do_all (agent add_xform_type)
		end
		
	add_xform_type (row: CUT_PATTERN_XFORM_TYPES_ROW)
		do
			cached_all_xform_types.put (row, row.cut_pattern_xform_type)
		end

	cached_all_xform_types: like all_xform_types
			-- All cut pattern xform types, indexed by 
			-- cut pattern xform type name and cached in memory

end
