
note
	
	description: "Abstraction for read-only object that may have external cut distances"
	author: "David Hollenberg"

deferred class CONST_ABSTRACT_EXT_CUT_DIST_SPEC
inherit
	ANY
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end

feature -- Properties
	
	has_external_cut_distance_left: BOOLEAN
			-- Does object have an external cut
			-- distance on left side?
		do
			Result := not is_null_external_cut_distance_left
		end

	has_external_cut_distance_right: BOOLEAN
			-- Does object have an external cut
			-- distance on right side?
		do
			Result := not is_null_external_cut_distance_right
		end

	has_external_cut_distance_bottom: BOOLEAN
			-- Does object have an external cut
			-- distance on bottom side?
		do
			Result := not is_null_external_cut_distance_bottom
		end

	has_external_cut_distance_top: BOOLEAN
			-- Does object have an external cut
			-- distance on top side?
		do
			Result := not is_null_external_cut_distance_top
		end

	external_cut_distance_left: DOUBLE
			-- External cut distance on left side
		deferred
		end
	
	external_cut_distance_right: DOUBLE
			-- External cut distance on right side
		deferred
		end
	
	external_cut_distance_top: DOUBLE
			-- External cut distance on top side
		deferred
		end
	
	external_cut_distance_bottom: DOUBLE
			-- External cut distance on bottom side
		deferred
		end
	
	is_null_external_cut_distance_left: BOOLEAN
			-- Is external cut distance on left side 
			-- null (floating)?
		deferred
		end

	is_null_external_cut_distance_right: BOOLEAN
			-- Is external cut distance on right side 
			-- null (floating)?
		deferred
		end

	is_null_external_cut_distance_bottom: BOOLEAN
			-- Is external cut distance on bottom side 
			-- null (floating)?
		deferred
		end

	is_null_external_cut_distance_top: BOOLEAN
			-- Is external cut distance on top side 
			-- null (floating)?
		deferred
		end

	is_same_as (other: CONST_ABSTRACT_EXT_CUT_DIST_SPEC): BOOLEAN
			-- Are `Current' and `other' the same
			-- external cut specifications?
		do
			Result := is_null_external_cut_distance_left = other.is_null_external_cut_distance_left and
			   is_null_external_cut_distance_right = other.is_null_external_cut_distance_right and 
			   is_null_external_cut_distance_bottom = other.is_null_external_cut_distance_bottom and 
			   is_null_external_cut_distance_top = other.is_null_external_cut_distance_top and
			   external_cut_distance_left = other.external_cut_distance_left and
			   external_cut_distance_right = other.external_cut_distance_right and
			   external_cut_distance_bottom = other.external_cut_distance_bottom and
			   external_cut_distance_top = other.external_cut_distance_top
		end

	copy_of: EXT_CUT_DIST_SPEC
			-- Copy of external cut specifications of `Current'
		do
			create Result.set_external_cut_specs (Current)
		end

	description_for_grid (a_grid: GRID): STRING
			-- Description suitable for messages,
			-- using `a_grid' to format distances
		local
			sp: STRING
		do
			create Result.make (50)
			sp := "  "
			append_description (Result, "Left", is_null_external_cut_distance_left, external_cut_distance_left, a_grid)
			Result.append (sp)
			append_description (Result, "Right", is_null_external_cut_distance_right, external_cut_distance_right, a_grid)
			Result.append (sp)
			append_description (Result, "Top", is_null_external_cut_distance_top, external_cut_distance_top, a_grid)
			Result.append (sp)
			append_description (Result, "Bottom", is_null_external_cut_distance_bottom, external_cut_distance_bottom, a_grid)
		end

feature {NONE} -- Implementation

	append_description (dest, side: STRING is_null_dist: BOOLEAN dist: DOUBLE a_grid: GRID)
		do
			dest.append (side)
			dest.append (su.Colon_space)
			if is_null_dist then
				dest.append (Floating)
			else
				dest.append (a_grid.formatted_number (dist))
			end
		end

	Floating: STRING = "floating"

end
