
note
	
	description: "Read only database row on whose behalf a batch %
		%job needs to be run"
	author: "David Hollenberg";
	date: "January 29, 2004"

deferred class CONST_BATCH_JOB_REQUESTOR
inherit
	DB_LOCKABLE_ROW

feature -- Properties
	
	job_status: STRING
			-- Status of job for requestor
		deferred
		end

	has_job_status (s: STRING): BOOLEAN
			-- Is `job_status' equal to `s'?
		require
			string_exists: s /= Void
		do
			Result := job_status.is_equal (s)
		end

	is_status_monitorable: BOOLEAN
			-- Does `job_status' indicate that there is a
			-- job which can be monitored?
		do
			Result := is_status_stopable
		end

	is_status_stopable: BOOLEAN
			-- Does `job_status' indicate that there is a
			-- job which can be stopped?
		local
			status_values: JOB_STATUS_VALUES
		do
			Result := job_status.is_equal (status_values.Pending) or job_status.is_equal (status_values.Executing)
		end

	is_status_unstopable: BOOLEAN
			-- Does `job_status' indicate that there is no
			-- job which can be stopped?
		do
			Result := not is_status_stopable
		end

	is_status_resettable: BOOLEAN
			-- Does `job_status' indicate that there is a
			-- job which can be reset (a reset undoes
			-- whatever the job did)
		local
			status_values: JOB_STATUS_VALUES
		do
			Result := job_status.is_equal (status_values.Passed) or job_status.is_equal (status_values.Failed) or job_status.is_equal (status_values.Aborted) or job_status.is_equal (status_values.Stopped)
		end

	is_status_unresettable: BOOLEAN
			-- Does `job_status' indicate that there is no
			-- job which can be reset?
		do
			Result := not is_status_resettable
		end

	is_status_submittable: BOOLEAN
			-- Does `job_status' indicate that there is a
			-- job which can be submitted?
		local
			status_values: JOB_STATUS_VALUES
		do
			Result := job_status.is_equal (status_values.Created)
		end

	is_status_unsubmittable: BOOLEAN
			-- Does `job_status' indicate that there is no
			-- job which can be submitted?
		do
			Result := not is_status_submittable
		end

	is_status_passed: BOOLEAN
			-- Does `job_status' indicate that job
			-- has completed successfully?
		local
			status_values: JOB_STATUS_VALUES
		do
			Result := job_status.is_equal (status_values.Passed)
		end

	is_status_failed: BOOLEAN
			-- Does `job_status' indicate that job
			-- completed unsuccessfully?
		local
			status_values: JOB_STATUS_VALUES
		do
			Result := job_status.is_equal (status_values.Failed)
		end

	is_status_aborted: BOOLEAN
			-- Does `job_status' indicate that job
			-- aborted before completion?
		local
			status_values: JOB_STATUS_VALUES
		do
			Result := job_status.is_equal (status_values.Aborted)
		end

	is_status_unsuccessful: BOOLEAN
			-- Does `job_status' indicate that job
			-- did not succeed?
		do
			Result := is_status_failed or is_status_aborted
		end

	date_job_started: DB_DATE
			-- Date batch job started
		deferred
		end

	date_job_ended: DB_DATE
			-- Date batch job ended
		deferred
		end

	batch_job_no: INTEGER
			-- Batch job number
		deferred
		end

	is_null_batch_job_no: BOOLEAN
			-- Is batch job number null?
		deferred
		end

	is_null_date_job_started: BOOLEAN
			-- Is date job started null?
		deferred
		end

	is_null_date_job_ended: BOOLEAN
			-- Is date job ended null?
		deferred
		end

	is_null_elapsed_job_time: BOOLEAN
			-- Is elapsed time of job null?
		do
			Result := is_null_date_job_started or is_null_date_job_ended
		end

	elapsed_job_time: INTEGER
			-- Elapsed time of job, in seconds
		require
			time_not_null: not is_null_elapsed_job_time
		do
			if not is_known_job_interval then
				computed_job_interval := (connection.date_difference (date_job_ended, date_job_started) * 86400).rounded
				is_known_job_interval := True
			end
			Result := computed_job_interval
		end

	error_explanation: STRING
			-- Description of error encountered during job
		deferred
		end

	maxlen_error_explanation: INTEGER
			-- Maximum length of `error_explanation'
		deferred
		end

feature {NONE} -- Implementation

	computed_job_interval: INTEGER
			-- Last computed number of seconds job took
			-- (elapsed time, not CPU time)

	is_known_job_interval: BOOLEAN
			-- Is `computed_job_interval' meaningful?

end
