deferred class MULTI_LINE_TEXT_ROW 
	[G -> NUMBERED_TEXT_LINE create make_template end]

	-- implements a NUMBERED_TEXT_LINE detail table as
	-- though a direct attribute of the master table.
	-- NOTE: this is an _MROW version; there is no read_only version

feature -- our detail table of text; fetched on demand

	the_numbered_rows: LIST[G]
	local r: G
	do  if cached_numbered_rows = Void
	     then create r.make_template;
		  cached_numbered_rows := 
			r.all_rows_filtered_ordered
				(connection, 
				 numbered_rows_filter,
				 numbered_rows_binder,
				 "line_no")
	    end;
	    Result := cached_numbered_rows
	end;

feature	-- attribute value and setting (as though in this row)

	the_text: LIST[STRING]
		-- current value, copied from the_numbered_rows
		-- NOTE: do NOT modify the_text; use set_text to replace it
	local tnr: LIST[G];
	do  tnr := the_numbered_rows;
	    create {ARRAYED_LIST[STRING]} Result.make(tnr.count);
	    from  tnr.start
	    until tnr.after
	    loop  Result.extend(tnr.item.text);
		  tnr.forth
	    end
	ensure the_text /= Void
	end;

	set_the_text(txt: LIST[STRING])
		-- replace with new value
		--  (creating or deleting rows as needed)
	require the_text /= Void
	local row: G; tnr: like the_numbered_rows;
	do  tnr := the_numbered_rows;
	    from  txt.start;  tnr.start
	    until txt.after
	    loop  if tnr.after		-- put into tnr
		   then -- add a row to put the text into
			row := create_numbered_row;
			tnr.extend(row);	-- (becomes current)
			row.set_line_no(tnr.count);
		   else row := tnr.item
		  end;
		  row.set_text(txt.item);	-- normal copy of a line
		  txt.forth;
		  tnr.forth
	    end;
	    from  -- here to the end
	    until tnr.after
	    loop  tnr.item.delete	-- extra line: delete from db
		  tnr.remove		-- and remove from list

	    end;
	end;

	delete
		-- delete me and the detail table too
		-- use this to replace original delete
	local tnr: LIST[G]
	do  tnr := the_numbered_rows;
	    from  tnr.start
	    until tnr.after
	    loop  tnr.item.delete
		  tnr.forth
	    end;
	    tnr.wipe_out
	    raw_delete		-- get rid of me too
	end;

feature {NONE} -- the cached numbered_rows of text

	cached_numbered_rows: LIST[G]

feature {NONE} -- more deferred stuff to flesh out

	numbered_rows_filter: STRING deferred end;
		-- filter for fetching rows from the db

	numbered_rows_binder: DB_BINDER deferred end;
		-- binder to go with the filter

	create_numbered_row: G deferred end;
		-- create and fill in all of a new row EXCEPT line_no and text

feature {NONE} -- deferred stuff that usually comes from DB_CHANGEABLE_ROW

	connection: DB_CONNECTION deferred end;

	raw_delete deferred end;
		-- really delete me

end -- MULTI_LINE_TEXT_ROW
