class MOSIS_PROD_VALUES
	-- known attribute values with semantic meaning
	-- found primarily in PROD section tables
	-- They are collected here for ease of use.

feature -- SPECIAL_HANDLING_TYPES

	shtype_is_attn:		STRING = "ATTENTION"
	shtype_is_bsa:		STRING = "BOND-SAME-AS"
	shtype_is_option:	STRING = "OPTION"
	shtype_is_late_po:	STRING = "LATE-PO"
	shtype_is_package:	STRING = "PACKAGE"
	shtype_is_qty:		STRING = "QUANTITY"
	shtype_is_sh:		STRING = "SPECIAL-HANDLING"
	shtype_is_ship:		STRING = "SHIPPING-PRIORITY"
	shtype_is_ip_included:	STRING = "IP-INCLUDED"
	shtype_is_bond_verification:
				STRING = "BONDING-VERIFICATION"

feature -- DESIGN_CHECKS.DESIGN_CHECK_STATE values
	-- FILE_TRANSFERS.TRANSFER_STATE values

	frs_is_new,
	dcs_is_new:		STRING = "NEW"
	frs_is_pending,
	dcs_is_pending:		STRING = "PENDING"
	dcs_is_retry:		STRING = "RETRY"
	dcs_is_obsolete:	STRING = "OBSOLETE"
	frs_is_passed,
	dcs_is_passed:		STRING = "PASSED"
	frs_is_failed,
	dcs_is_failed:		STRING = "FAILED"
	frs_is_internal_error,
	dcs_is_internal_error:	STRING = "INTERNAL_ERROR"
	dcs_is_holding:		STRING = "HOLDING"
	dcs_is_aborted:		STRING = "ABORTED"
	dcs_is_waiting:		STRING = "WAITING"
	dcs_is_checksum_error:	STRING = "CHECKSUM_ERROR"
	dcs_is_pads:		STRING = "PADDING"	-- generate cid_pads
	dcs_is_bond:		STRING = "BONDING"	-- bonding check(s)

	dcs_is_initial(dcs: STRING): BOOLEAN
		-- is this an "initial" state?
		-- (one from which we can begin a check)
	do  Result := equal(dcs, dcs_is_new) or
			equal(dcs, dcs_is_pads) or
			equal(dcs, dcs_is_bond)
	end

	dcs_is_terminal(dcs: STRING): BOOLEAN
		-- is this a terminal dc state?
		-- (one where the dc processor has finished and the
		--  daemon takes over)
	do  Result := equal(dcs, dcs_is_passed) or
			equal(dcs, dcs_is_failed) or
			equal(dcs, dcs_is_obsolete)
	end

	dcs_needs_help(dcs: STRING): BOOLEAN
		-- is this an intervention-required dc state?
	do  Result := equal(dcs, dcs_is_internal_error) or
			equal(dcs, dcs_is_holding) or
			equal(dcs, dcs_is_checksum_error) or
			equal(dcs, dcs_is_aborted)
	end

feature -- FILE_TRANSFER_ATTEMPTS.TERMINATION_STATEs and TERMINATION_REASONs

	fts_is_not_conn: STRING = "NOT_CONNECTED"
	fts_is_conn:	 STRING = "CONNECTED"
	fts_is_logged:	 STRING = "LOGGED_IN"
	fts_is_setup:	 STRING = "SETUP_DONE"
	fts_is_retr:	 STRING = "RETRIEVE_SENT"
	fts_is_store:	 STRING = "STORE_SENT"
	fts_is_dco:	 STRING = "DATA_CONN_OPENED"
	fts_is_done:	 STRING = "TRANSFER_DONE"

	ftr_is_success:	 STRING = "SUCCESS"
	ftr_is_host_err: STRING = "HOST_LOOKUP_ERROR"
	ftr_is_conn_err: STRING = "CONNECTION_ERROR"
	ftr_is_ptcl_err: STRING = "PROTOCOL_ERROR"
	ftr_is_parm_err: STRING = "MISSING_PARAMETER"
	ftr_is_huge_err: STRING = "FILE_TOO_BIG"
	ftr_is_time_err: STRING = "TIMEOUT_ERROR"
	ftr_is_tran_err: STRING = "TRANSIENT_NEG_REPLY"
	ftr_is_perm_err: STRING = "PERMANENT_NEG_REPLY"
	ftr_is_cons_err: STRING = "CONSUMPTION_ERROR"
	ftr_is_cons_wrn: STRING = "CONSUMPTION_WARNING"

feature -- IC_FAB_INSTANCE_TYPES

	type_is_customer_fab: STRING = "CUSTOMER_FAB"
	type_is_test_vehicle: STRING = "TEST_VEHICLE"

feature	-- INTENDED_DISPOSITIONS (IBM-requested item in IFI)

	idisp_is_research:	STRING = "RESEARCH"
	idisp_is_development:	STRING = "DEVELOPMENT"
	idisp_is_production:	STRING = "PRODUCTION"
	
feature -- Project states (validated by DESIGN_STATES)

	pstate_is_new: 			STRING = "WAITING FOR LAYOUT"
	pstate_is_incomplete:		STRING = "INCOMPLETE"
	pstate_is_awaiting_itar:	STRING = "WAITING FOR ITAR APPROVAL"
	pstate_is_in_transit: 		STRING = "IN TRANSIT"
	pstate_is_being_checked:	STRING = "BEING REVIEWED"
	pstate_is_failed:		STRING = "FAILED"
	pstate_is_passed:		STRING = "PASSED"
	pstate_is_preliminary:		STRING = "AWAITING FINAL LAYOUT"
	pstate_is_awaiting_funding:	STRING = "FUNDING AUTHORIZATION PENDING"
	pstate_is_queued_for_fab:	STRING = "QUEUED FOR FAB"
	pstate_is_foundry_checks:	STRING = "FOUNDRY CHECKS IN PROGRESS"
	pstate_is_locked:		STRING = "LOCKED (FABRICATION CANDIDATE)"
	pstate_is_in_fab:		STRING = "IN FAB"
	pstate_is_shipped:		STRING = "SHIPPED"
	pstate_is_cancelled:		STRING = "CANCELLED"

feature -- DESIGN_ACCEPT_STATE values

	das_is_empty : STRING = ""		-- these are the only
	das_is_xmit  : STRING = "XMIT"		-- allowed values for
	das_is_check : STRING = "CHECK"		-- DESIGN_ACCEPT_STATE
	das_is_passed: STRING = "PASSED"
	das_is_failed: STRING = "FAILED"
	das_is_itar  : STRING = "ITAR"

feature -- known formats for GEOMETRY_FORMAT

	fmt_is_cif  : STRING = "CIF"
	fmt_is_gds  : STRING = "GDS"
	fmt_is_uugds: STRING = "UUGDS"
	fmt_is_mebes: STRING = "MEBES"

	is_multifile_format(fmt: STRING): BOOLEAN
		-- is fmt a format that uses a set of files?
	do  Result := equal(fmt, fmt_is_mebes)
	end

feature	-- known values of ic_fab_instance_message.message_class

	msg_class_is_cats:	STRING = "CATS"
	msg_class_is_anomaly:	STRING = "ANOMALY"
	msg_class_is_layer:	STRING = "LAYER"
	msg_class_is_pad:	STRING = "PAD"
	msg_class_is_bond:	STRING = "BOND"
	msg_class_is_postcheck:	STRING = "POSTCHECK"
	msg_class_is_request:	STRING = "REQUEST"
		-- this class only appears in design_check_messages;
		-- it instructs the fe to make this item a request
		-- response rather than add it to ifi_messages

feature -- values of runs.run_status and runs.run_type

	rstatus_is_in_fab:	STRING = "IN_FAB"
	rstatus_is_complete:	STRING = "COMPLETE"
	rtype_is_dedicated:	STRING = "DED"
	rtype_is_development:	STRING = "DEV"
	rtype_is_shared:	STRING = "SHR"
	rtype_is_trusted_foundry: STRING = "TRF"

feature -- Values of runs.run_archive_state

	runarch_is_in_progress:	STRING = "IN PROGRESS"
	runarch_is_complete:	STRING = "COMPLETE"

feature -- transform classes and their output modes; transform_names

	xform_is_compress:	STRING = "COMPRESSED"
	xform_is_encode:	STRING = "ENCODED"
	xform_is_checksum:	STRING = "CHECKSUMMED"
	xform_is_fragment:	STRING = "FRAGMENTED"
	xform_is_encrypt:	STRING = "ENCRYPTED"

	output_is_binary:	STRING = "BINARY"
	output_is_text:		STRING = "TEXT"
	output_is_input:	STRING = "INPUT"	-- same as input
	output_is_medium:	STRING = "MEDIUM"	-- guess based on medium

	encode_is_uuencode:	STRING = "UUENCODED"
	encode_is_base64:	STRING = "BASE64"
	encode_is_quoted_printable:
				STRING = "QUOTED_PRINTABLE"

	checksum_is_cif:	STRING = "CIF"
	checksum_is_crc:	STRING = "CRC"

feature	-- pattern_files.pattern_type

	pattern_type_is_die:	STRING = "DIE"
	pattern_type_is_merge:	STRING = "MERGE"
	pattern_type_is_fill:	STRING = "FILL"

feature	-- run_events.event_name

	event_is_closed:	STRING = "RUN CLOSED"
	event_is_posted:	STRING = "FAB POSTED"
	event_is_rtm:		STRING = "RELEASED TO MANUFACTURING"
	event_is_eccn_reported: STRING = "ECCN REPORTED"
	event_is_design_files_copied: STRING = "DESIGN FILES COPIED"
	event_is_primary_archive: STRING = "PRIMARY ARCHIVE"
	event_is_secondary_archive: STRING = "SECONDARY ARCHIVE"
	event_is_directory_cleanup: STRING = "DIRECTORY CLEANUP"

feature	-- special package_requested values

	package_is_none:	STRING = "NONE"
	package_is_null:	STRING = "NULL"
	package_is_default:	STRING = "DEFAULT"

feature	-- packaging_order_type values

	packaging_order_is_ceramic:	STRING = "CERAMIC"
	packaging_order_is_plastic:	STRING = "PLASTIC"

feature	-- die_types

	die_type_is_cust_die:		STRING = "CUST_DIE"
	die_type_is_placard:		STRING = "PLACARD"
	die_type_is_test_vehicle:	STRING = "TEST_VEHICLE"
	die_type_is_vendor_geometry:	STRING = "VENDOR_GEOMETRY"
	die_type_is_filler:		STRING = "FILLER"
	die_type_is_scribe:		STRING = "SCRIBE"
	die_type_is_vendor_ghost:	STRING = "VENDOR_GHOST"
	die_type_is_macrocell:		STRING = "MACROCELL"

feature	-- orientations

	orientation_is_horizontal:	STRING = "HORIZONTAL"
	orientation_is_vertical:	STRING = "VERTICAL"

feature	-- FILE_ROLES

	fir_is_submitted_layout:	STRING = "SUBMITTED_LAYOUT"
	fir_is_merged_layout:		STRING = "MERGED_LAYOUT"
	fir_is_auto_drc:		STRING = "AUTO_DRC_RESULT"
	fir_is_f_bnd_diagram:		STRING = "F_BND_DIAGRAM"
	fir_is_p_bnd_diagram:		STRING = "P_BND_DIAGRAM"
	fir_is_price_quote:		STRING = "PRICE_QUOTATION"
	fir_is_wafer_lot_data:		STRING = "WAFER_LOT_DATA"
	fir_is_ibm_kerf_data:		STRING = "IBM_KERF_DATA"
	fir_is_price_list:		STRING = "PRICE_LIST"

feature	-- QUEUE_FAILURE_REASON_IDs

	qfr_is_misprice:	STRING = "MISPRICE"
	qfr_is_misalloc:	STRING = "MISALLOC"
	qfr_is_nopropsl:	STRING = "NOPROPSL"
	qfr_is_noorder:		STRING = "NOORDER"
	qfr_is_nsf:		STRING = "NSF"
	qfr_is_nsedu:		STRING = "NSEDU"
	qfr_is_trfquot:		STRING = "TRFQUOT"
	qfr_is_unknown:		STRING = "UNKNOWN"
	qfr_is_xxpropsl:	STRING = "XXPROPSL"

end
