
note
	
	description: "Object that is an sub-element of a model element that %
		%is comprised of more than one part (each one rectangular)"
	author: "David Hollenberg"
	date: "July 25, 2006"

deferred class MODEL_SUB_ELEMENT

feature -- Properties

	identification: STRING
			-- Identifying name of element
		deferred
		ensure
			result_not_void: Result /= Void
		end

	is_selectable: BOOLEAN
			-- May object be selected?
		deferred
		end

	rotation: INTEGER
			-- Rotation of object in degrees clockwise
		deferred
		ensure
			valid: is_valid_rotation (Result)
		end

	is_valid_rotation (n: INTEGER): BOOLEAN
			-- Is `n' a valid rotation for object in degrees
			-- clockwise?
		do
			Result := n = 0 or n = 90 or n = 180 or n = 270
		end

	has_required_rotation: BOOLEAN
			-- Must object have specific rotation?
		deferred
		end

	required_rotation: INTEGER
			-- Rotation object is required to have in degrees clockwise
		require
			has_required_rotation: has_required_rotation
		deferred
		ensure
			valid: is_valid_rotation (Result)
		end

	is_rotated_sideways: BOOLEAN
			-- Is object rotated by 90 degrees clockwise or
			-- 90 degrees counterclockwise?
		deferred
		end

	size_x: DOUBLE
			-- Size of minimum bounding box in X dimension
		deferred
		ensure
			result_positive: Result > 0
		end

	size_y: DOUBLE
			-- Size of minimum bounding box in Y dimension
		deferred
		ensure
			result_positive: Result > 0
		end

	unrotated_size_x: DOUBLE
			-- Size of minimum bounding box in X dimension
			-- when element is not rotated
		do
			if is_rotated_sideways then
				Result := size_y
			else
				Result := size_x
			end
		end

	unrotated_size_y: DOUBLE
			-- Size of minimum bounding box in Y dimension
			-- when element is not rotated
		do
			if is_rotated_sideways then
				Result := size_x
			else
				Result := size_y
			end
		end

	largest_dimension: DOUBLE
			-- Largest of `size_x' and `size_y'
		do
			Result := size_x.max (size_y)
		ensure
			result_positive: Result > 0
		end

	center_x: DOUBLE
			-- Center of minimum bounding box in X dimension
		deferred
		end

	center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension
		deferred
		end

	left_x: DOUBLE
			-- Left edge of minimum bounding box in X dimension
		do
			Result := center_x - (size_x / 2.0)
		end

	right_x: DOUBLE
			-- Right edge of minimum bounding box in X dimension
		do
			Result := center_x + (size_x / 2.0)
		end

	top_y: DOUBLE
			-- Top edge of minimum bounding box in Y dimension
		do
			Result := center_y + (size_y / 2.0)
		end

	bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box in X dimension
		do
			Result := center_y - (size_y / 2.0)
		end

	bounding_box: CONST_RECTANGLE
			-- Minimum bounding box
		do
			create Result.make_by_size_and_center (size_x, size_y, center_x, center_y)
		end

	grid: GRID
			-- Grid for object sizes and locations
		do
			Result := model_element.grid
		end

	model_element: like anchored_model_element
			-- Model element of which this is one part

	model: like anchored_model
			-- Model of which element is one part
		do
			Result := model_element.model
		end

	anchored_model: MODEL
			-- Type of model of which sub-element is part
		do
		ensure
			result_void: Result = Void
		end

	anchored_model_element: MODEL_ELEMENT
			-- Type of model element of which `Current' is
			-- one part
		do
		ensure
			result_void: Result = Void
		end

end
