
note
	
	description: "A named packing.  Used to select and pack projects. "
	author: "David Hollenberg"
	date: "November 14, 2005"

class RC_PACKING
inherit
	ANY
	DB_OPTIONS_ACCESS
		export
			{NONE} all
		end
	ITEM_COUNT_RANGES_ACCESS
		export
			{NONE} all
		end
	CONDITION_UTILITIES
		export
			{NONE} all
		end

feature -- Properties

	packing_id: STRING
			-- Packing unique identifer
		do
			Result := packings_row.packing_id
		end

	production_id: STRING
			-- Production ID if editing cut patterns for
			-- specific production.  Must be Void otherwise

	connection: DB_CONNECTION
			-- Database connection
		do
			Result := packings_row.any_connection
		end

	wafer_techn: STRING
			-- Wafer techn for packing
		do
			Result := packings_row.wafer_techn
		end

	wafer_techns_row: WAFER_TECHNS_ROW
			-- Wafer techns row for packing's wafer techn
		do
			Result := packings_row.wafer_techns_row
		end

	is_3dic: BOOLEAN
			-- Is wafer techn for packing a 3DIC techn?
		do
			Result := wafer_techns_row.is_3dic
		end

	nth_tier (n: INTEGER): WAFER_TECHN_3D_TIERS_ROW
			-- `n'th tier for packing's wafer techn
		require
			is_3dic: is_3dic
		do
			Result := wafer_techns_row.nth_tier (n)
		end
		
	field_set: FIELD_SETS_ROW
			-- Field set for packing
		do
			Result := packings_row.field_set
		end

	field_set_name: STRING
			-- Name of field set for packing
		do
			Result := field_set.field_set_name
		end

	litho_sizes: LITHO_SIZES_MROW
			-- Lithography sizes for packing
		do
			Result := packings_row.litho_sizes
		end

	packing_target: PACKING_TARGETS_ROW
			-- Packing target for packing
		do
			Result := packings_row.packing_target
		end

	current_payload_size: CONST_POINT
			-- Current size of payload
		do
			if packing_target.is_custom and then not current_version.is_null_payload_size_x then
				-- Custom packing target and not old
				-- packing version
				Result := current_version.custom_payload_size
			else
				-- Non-custom packing target or old packing 
				-- version.  Get payload
				-- size from litho_sizes row
				Result := litho_payload_size
			end
		end

	litho_payload_size: CONST_POINT
			-- Size of payload in `litho_sizes'
		do
			Result := litho_sizes.payload_size
		end

	current_field_size: CONST_POINT
			-- Current size of field
		require
			reticle_packing_target: packing_target.is_reticle
		local
			pt: like packing_target
		do
			pt := packing_target
			if pt.is_custom and then not current_version.is_null_field_size_x then
				-- Custom reticle packing target and not old
				-- packing version
				Result := current_version.custom_field_size
			else
				-- Reticle packing target that is not custom 
				-- or old packing version
				Result := litho_field_size
			end
		end

	litho_field_size: CONST_POINT
			-- Size of field in `litho_sizes'
		require
			reticle_packing_target: packing_target.is_reticle
		do
			Result := litho_sizes.field_size
		end

	current_external_kerf: CONST_POINT
			-- Current size of external kerf
		local
			pt: like packing_target
		do
			pt := packing_target
			if pt.is_custom and then not current_version.is_null_field_size_x then
				Result := current_version.custom_external_kerf
			else
				Result := litho_external_kerf
			end
		end

	litho_external_kerf: CONST_POINT
			-- Size of external kerf in `litho_sizes'
		do
			Result := litho_sizes.external_kerf
		end

	wafer_techn_family_processes: LIST [WAFER_PROCESSES_ROW]
			-- All wafer processes for packing's wafer techn family
		do
			Result := packings_row.wafer_techn_family_processes
		end

	run_type: STRING
			-- Run type for packing
		do
			Result := packings_row.run_type
		end

	scheduled_run: SCHEDULED_RUNS_ROW
			-- Scheduled run for `Current'
		do
			Result := packings_row.scheduled_run
		end

	packings_row: PACKINGS_MROW
			-- Row describing packing

	versions: LIST [PACKING_VERSIONS_MROW]
			-- Versions of packing

	current_version: PACKING_VERSIONS_MROW
			-- Currently selected version of packing

	version_with_number (n: INTEGER): PACKING_VERSIONS_MROW
			-- Version of `Current' with whose version number
			-- is `n'
		local
			v: PACKING_VERSIONS_MROW
			found: BOOLEAN
		do
			from
				versions.start
			until
				versions.after or found
			loop
				v := versions.item
				if v.version_no = n then
					Result := v
					found := True
				end
				versions.forth
			end
			if not found then
				handle_internal_error ("cannot find version " + n.out + " in list of versions for packing " + packing_id)
			end
		end

	is_current_version_modified: BOOLEAN
			-- Has currently selected version of packing
			-- been modified?
		do
			Result := not current_version.is_deep_unchanged
		end

	is_cut_pattern_set_modified: BOOLEAN
			-- Has set of cut patterns in currently selected 
			-- version of packing been modified?
		do
			Result := not current_version.is_cut_pattern_set_deep_unchanged
		end

	is_version_id_used (s: STRING): BOOLEAN
			-- Has `s' already been used as the version ID
			-- in an existing version?
		do
			Result := versions.there_exists (agent {PACKING_VERSIONS_MROW}.has_version_id (s))
		end

	highest_used_version: INTEGER
			-- Highest version previously used
		do
			Result := current_version.highest_used_version
		end

	selected_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected
		do
			Result := current_version.selected_projects
			Result.do_all (agent {PACKING_PROJECTS_MROW}.set_selected (True))
		end

	selected_unlocked_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are not locked
		do
			uncache_fab_instances
			Result := current_version.selected_unlocked_projects
			set_selected (Result)
		end

	selected_locked_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are locked
		do
			uncache_fab_instances
			Result := current_version.selected_locked_projects
			set_selected (Result)
		end

	selected_unassigned_projects (a_run_id: STRING): LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are not assigned to `a_run_id'
		require
			a_run_id_not_void: a_run_id /= Void
		do
			uncache_fab_instances
			Result := current_version.selected_unassigned_projects (a_run_id)
			set_selected (Result)
		end

	selected_assigned_projects (a_run_id: STRING): LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are assigned to `a_run_id'
		require
			a_run_id_not_void: a_run_id /= Void
		do
			uncache_fab_instances
			Result := current_version.selected_assigned_projects (a_run_id)
			set_selected (Result)
		end

	selected_reworkable_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are in rework
		do
			uncache_fab_instances
			Result := current_version.selected_reworkable_projects
			set_selected (Result)
		end

	selected_customer_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects currently selected that
			-- are customer projects
		do
			uncache_fab_instances
			Result := current_version.selected_customer_projects
			set_selected (Result)
		end

	selected_projects_matching_filter (f: FUNCTION [ANY, TUPLE [PACKING_PROJECTS_MROW], BOOLEAN]): LIST [PACKING_PROJECTS_MROW]
			-- List of all projects that are currently selected
			-- and match filter `f'
		do
			Result := current_version.selected_projects_matching_filter (f)
			Result.do_all (agent {PACKING_PROJECTS_MROW}.set_selected (True))
		end

	unselected_assigned_fab_instances (a_run_id: STRING): LIST [IC_FAB_INSTANCES_ROW]
			-- List of all fab instances that are not included
			-- in the current packing version, but are assigned
			-- to `a_run_id'
		require
			a_run_id_not_void: a_run_id /= Void
		do
			Result := current_version.unselected_assigned_fab_instances (a_run_id)
		end

	unselected_targeted_fab_instances (a_sched_run_no: INTEGER): LIST [IC_FAB_INSTANCES_ROW]
			-- List of all fab instances that are not included
			-- in the current packing version, but are targeted
			-- to `a_sched_run_no'
		do
			Result := current_version.unselected_targeted_fab_instances (a_sched_run_no)
		end

	uncache_fab_instances
			-- Uncache fab instances for all selected projects
		do
			selected_projects.do_all (agent {PACKING_PROJECTS_MROW}.uncache_fab_instance)
		end

	uncache_all
			-- Uncache cached database rows
		do
			selected_projects.do_all (agent {PACKING_PROJECTS_MROW}.uncache_foreign_keys)
		end

	set_selected (list: LIST [PACKING_PROJECTS_MROW])
			-- Mark each project in `list' as "selected"
		do
			list.do_all (agent {PACKING_PROJECTS_MROW}.set_selected (True))
		end

	deselected_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects which were selected previously
			-- but are now deselected
		do
		end

	deselected_projects_matching_filter (f: FUNCTION [ANY, TUPLE [PACKING_PROJECTS_MROW], BOOLEAN]): LIST [PACKING_PROJECTS_MROW]
			-- List of all projects which were selected previously
			-- but are now deselected and which match filter `f'
		local
			list: like deselected_projects
		do
			list := deselected_projects
			Result := list.twin
			Result.wipe_out
			list.do_if (agent Result.extend, f)
			Result.do_all (agent {PACKING_PROJECTS_MROW}.set_selected (False))
		end

	all_projects: LIST [PACKING_PROJECTS_MROW]
			-- List of all projects for packing (highest version
			-- only, for each one)
		local
			row: PACKING_PROJECTS_MROW
		do
			create row.make_template
			Result := row.highest_version_rows_for_packing (packing_id, connection, expected_count.At_least_zero)
		end

	cust_project_highest_version (mfn: INTEGER): like anchored_project
			-- Highest version of cust project whose
			-- fab number is `mfn', among all versions of
			-- packing.  Void if not in any versions.
			-- Must have previously called
			-- `setup_for_fast_project_access'
		do
			Result := cust_project_table.item (mfn)
		end

	test_vehicle_highest_version (dno: INTEGER): like anchored_project
			-- Highest version of test vehicle project whose
			-- design number is `dno', among all versions of
			-- packing.  Void if not in any versions.
			-- Must have previously called
			-- `setup_for_fast_project_access'
		do
			Result := test_vehicle_project_table.item (dno)
		end

feature -- Modification

	set_packings_row (p: like packings_row)
			-- Set `packings_row' to `p'
		do
			packings_row := p
			packings_row.set_parent (Current)
		end

	set_production_id (an_id: like production_id)
			-- Set `production_id' to `an_id'
		do
			production_id := an_id
		end

	set_current_version (pv: like current_version)
			-- Set `current_version' to `pv' and detach it
		require
			pv_not_void: pv /= Void
		do
			set_current_version_without_processes (pv)
			current_version.compute_compatible_processes
			current_version.assign_process_to_projects
			current_version.report_process_warnings
		end

	set_current_version_from_official_data (pv: like current_version)
			-- Set `current_version' to `pv' and detach it.
			-- Alternate version that uses data in official
			-- database tables to set processes instead
			-- of computing compatible processes based on
			-- options
		require
			pv_not_void: pv /= Void
		do
			set_current_version_without_processes (pv)
			current_version.set_processes_from_official_data
		end

	set_current_version_without_processes (pv: like current_version)
			-- Set `current_version' to `pv' and detach it,
			-- without setting process data
		require
			pv_not_void: pv /= Void
		do
			current_version := pv
			current_version.set_packing (packings_row)
			current_version.set_highest_used_numbers
			detach_current_version
		end

feature -- Operations

	refresh_versions
			-- Refresh `versions'
		local
			row: PACKING_VERSIONS_MROW
		do
			create row.make_template
			versions := row.rows_for_packing (packing_id, "date_created desc, version_no desc", connection, expected_count.At_least_zero)
		end

	create_initial_version
			-- Create the initial version for a packing
			-- which has no versions
		do
			create current_version.make_initial_version (packings_row, litho_sizes, connection)
		end

	detach_current_version
			-- Replace `current_version' with detached copy
			-- of itself
		do
			current_version := current_version.deep_detached_twin
		end

	save_new_version (a_name: STRING r: RUNS_MROW)
			-- Save contents of `current_version' as a new
			-- packing version under `a_name' (under no
			-- name if Void).  Update `r' if not Void
		do
		end

	save_new_dummy_vms_version (r: RUNS_MROW)
			-- Save contents of `current_version' as a new
			-- dummy VMS packing version (projects were selected
			-- on VMS, not Unix).  Update `r' if not Void
		do
			save_new_version (current_version.vms_selection_id, r)
		end

	setup_for_fast_project_access
			-- Do setup which allows quickly finding out
			-- whether a particular cust die or test
			-- vehicle project has ever been part of a
			-- packing version, so that we can assign it
			-- the same project_no
		local
			list: LIST [like anchored_project]
		do
			list := all_projects
			create cust_project_table.make (list.count)
			create test_vehicle_project_table.make (list.count)
			list.do_all (agent add_project_to_table)
		end

	save_new_scribe_widths (old_horiz_width, old_vert_width, new_horiz_width, new_vert_width, def_width: DOUBLE)
			-- Save new scribe widths in the database,
			-- if they changed, so that they will be
			-- preserved over sessions
		local
			horiz_changed, vert_changed: BOOLEAN
		do
			horiz_changed := old_horiz_width /= new_horiz_width
			vert_changed := old_vert_width /= new_vert_width
			if horiz_changed or vert_changed then
				if horiz_changed then
					if new_horiz_width /= def_width then
						packings_row.set_horizontal_scribe_width (new_horiz_width)
					else
						packings_row.set_null_horizontal_scribe_width
					end
				end
				if vert_changed then
					if new_vert_width /= def_width then
						packings_row.set_vertical_scribe_width (new_vert_width)
					else
						packings_row.set_null_vertical_scribe_width
					end
				end
				connection.commit
			end
		end

	is_lock_required_for_sizes (a_payload_size_x, a_payload_size_y, a_field_size_x, a_field_size_y, a_scale_factor: DOUBLE): BOOLEAN
			-- Would changing the payload and field sizes
			-- as indicated by the arguments cause
			-- the litho_sizes row to be modified
			-- (and require locking it in advance
			-- to avoid crashes if lock cannot be acquired)?
		require
			is_custom_packing_target: packing_target.is_custom
		local
			pt: like packing_target
			ls: like litho_sizes
			ps, fs: CONST_POINT
			merged_x, merged_y: DOUBLE
		do
			pt := packing_target
			ls := litho_sizes
			create ps.make_xy (a_payload_size_x, a_payload_size_y)
			Result := Result or ls.payload_size_x /= ps.x or ls.payload_size_y /= ps.y
			if pt.is_reticle then
				create fs.make_xy (a_field_size_x, a_field_size_y)
				Result := Result or ls.is_null_field_size_x or ls.field_size_x /= fs.x
				Result := Result or ls.is_null_field_size_y or ls.field_size_y /= fs.y
			end

			merged_x := ps.x * a_scale_factor
			merged_y := ps.y * a_scale_factor
			Result := Result or ls.is_null_merged_pattern_file_size_x or ls.merged_pattern_file_size_x /= merged_x
			Result := Result or ls.is_null_merged_pattern_file_size_y or ls.merged_pattern_file_size_y /= merged_y
		end
	
	lock_litho_size
			-- Try to lock `litho_sizes' row
		do
		end

	update_payload_and_field_sizes (a_payload_size_x, a_payload_size_y, a_field_size_x, a_field_size_y, a_scale_factor: DOUBLE)
			-- Update current packing version with payload
			-- and field sizes that have changed
		require
			is_custom_packing_target: packing_target.is_custom
		local
			cv: like current_version
		do
			cv := current_version
			if cv.is_null_payload_size_x or cv.payload_size_x /= a_payload_size_x then
				cv.set_payload_size_x (a_payload_size_x)
			end
			if cv.is_null_payload_size_y or cv.payload_size_y /= a_payload_size_y then
				cv.set_payload_size_y (a_payload_size_y)
			end
			if packing_target.is_reticle then
				-- Update field sizes too
				if cv.is_null_field_size_x or cv.field_size_x /= a_field_size_x then
					cv.set_field_size_x (a_field_size_x)
				end
				if cv.is_null_field_size_y or cv.field_size_y /= a_field_size_y then
					cv.set_field_size_y (a_field_size_y)
				end
			else
				-- Set field sizes to null if not already null
				if not cv.is_null_field_size_x then
					cv.set_null_field_size_x
				end
				if not cv.is_null_field_size_y then
					cv.set_null_field_size_y
				end
			end
		end
	
	propagate_custom_litho_sizes (a_scale_factor: DOUBLE)
			-- Propagate custom litho sizes to
			-- litho sizes table
		require
			is_custom_packing_target: packing_target.is_custom
		local
			pt: like packing_target
			ls: like litho_sizes
			ps, fs: CONST_POINT
			merged_x, merged_y: DOUBLE
		do
			pt := packing_target
			if pt.is_custom then
				ls := litho_sizes
				ps := current_payload_size
				if ls.payload_size_x /= ps.x then
					ls.set_payload_size_x (ps.x)
				end
				if ls.payload_size_y /= ps.y then
					ls.set_payload_size_y (ps.y)
				end
				if pt.is_reticle then
					fs := current_field_size
					if ls.is_null_field_size_x or ls.field_size_x /= fs.x then
						ls.set_field_size_x (fs.x)
					end
					if ls.is_null_field_size_y or ls.field_size_y /= fs.y then
						ls.set_field_size_y (fs.y)
					end
				end

				merged_x := ps.x * a_scale_factor
				merged_y := ps.y * a_scale_factor
				if ls.is_null_merged_pattern_file_size_x or ls.merged_pattern_file_size_x /= merged_x then
					ls.set_merged_pattern_file_size_x (merged_x)
				end
				if ls.is_null_merged_pattern_file_size_y or ls.merged_pattern_file_size_y /= merged_y then
					ls.set_merged_pattern_file_size_y (merged_y)
				end

				if ls.is_changed then
					connection.commit
				end
				-- Uncache litho_sizes row to avoid
				-- later row locking problem if in-memory
				-- size differs slightly from in-database size
				packings_row.uncache_field_set
			end
		end

feature {NONE} -- Implementation

	add_project_to_table (proj: like anchored_project)
			-- Add `proj' to appropriate table, based on
			-- fab instance type
		do
			if proj.is_customer_project then
				cust_project_table.put (proj, proj.m_fab_no)
				if not cust_project_table.inserted then
					handle_internal_error ("duplicate m_fab_no among distinct cust die projects for all packing versions")
				end
			elseif proj.is_test_vehicle then
				test_vehicle_project_table.put (proj, proj.design_no)
				if not test_vehicle_project_table.inserted then
					handle_internal_error ("duplicate design_no among distinct test vehicles for all packing versions")
				end
			else
				handle_internal_error ("project is not customer project or test vehicle - design_no is " + proj.design_no.out)
			end
		end

	cust_project_table: HASH_TABLE [like anchored_project, INTEGER]
			-- Packer boxes of type cust_die indexed by m_fab_no

	test_vehicle_project_table: HASH_TABLE [like anchored_project, INTEGER]
			-- Packer boxes of type test_vehicle indexed by
			-- design_no

	anchored_project: PACKING_PROJECTS_ROW
			-- For anchoring only
		do
		ensure
			result_void: Result = Void
		end

end

