
note
	
	description: "Virtual element of reticle model that may be fixed or movable"
	author: "David Hollenberg"

deferred class RETICLE_ABSTRACT_VIRTUAL_ELEMENT
inherit
	RETICLE_MODEL_ELEMENT
		redefine
			is_virtual, has_field_edge_restriction
		end

feature {NONE} -- Initialization

	initialize (a_size, a_center: CONST_POINT a_model: like anchored_model)
			-- Initialize newly created object
		require
			a_size_not_void: a_size /= Void
			a_center_not_void: a_center /= Void
			a_size_x_positive: a_size.x > 0.0
			a_size_y_positive: a_size.y > 0.0
			a_model_not_void: a_model /= Void
		do
			size_x := a_size.x
			size_y := a_size.y
			center_x := a_center.x
			center_y := a_center.y
			model := a_model
			create {ARRAYED_LIST [like Current]} all_copies.make (1)
			all_copies.extend (Current)
		end

feature -- Properties

	is_virtual: BOOLEAN = True
			-- Is element virtual (label, field of view edge)
			-- rather than something that can potentially be
			-- manipulated (selected, moved, etc)?

	size_x: DOUBLE
			-- Size of minimum bounding box in X dimension

	size_y: DOUBLE
			-- Size of minimum bounding box in Y dimension

	center_x: DOUBLE
			-- Location of center of minimum bounding box in
			-- X dimension

	center_y: DOUBLE
			-- Location of center of minimum bounding box in
			-- Y dimension

	has_field_edge_restriction: BOOLEAN = False
			-- Does element have a restriction on how close it
			-- can be to edge of field?

	is_selectable: BOOLEAN = False
			-- May object be selected?

	is_never_deletable: BOOLEAN = True
			-- Can object never be deleted?

feature {NONE} -- Modification

	set_is_deletable (b: BOOLEAN)
			-- Set `is_deletable' to `b'
		do
			-- Inapplicable
		end

feature {NONE} -- Implementation

	is_deletable_internal: BOOLEAN = False
			-- May object currently be deleted?

end
