
note
	
	description: "Change applied to reticle during packing"
	author: "David Hollenberg"
	date: "January 25, 2007"

deferred class RETICLE_CHANGE
inherit
	ANY
	STRING_UTILITIES_ACCESS
		export
			{NONE} all
		end
	
feature {NONE} -- Creation

	make (a_name: like name)
		require
			a_name_not_void: a_name /= Void
		do
			name := a_name
		end

feature -- Properties

	name: STRING
			-- Name of operation involved in change

	description: STRING
			-- Description of operation involved in change
		do
			create Result.make (80)
			Result.append (name)
			Result.append (" operation (")
			Result.append_integer (simple_change_count)
			Result.extend (' ')
			Result.append (su.number (simple_change_count, "change", "changes"))
			Result.extend (')')
		end

	view: RETICLE_VIEW
			-- View affected by changes

	changes: LIST [RETICLE_SIMPLE_CHANGE]
			-- List of simple changes that comprise this change

	simple_change_count: INTEGER
			-- Number of simple changes that comprise this change
		do
			Result := changes.count
		end

	failed: BOOLEAN
			-- Did last undo or redo of this
			-- change fail?  If so, the change
			-- was not applied

	displayed_area: CONST_RECTANGLE
			-- Area of view displayed both before and after change

	initial_locked_mode: BOOLEAN
			-- Were we in locked mode before change?

	final_locked_mode: BOOLEAN
			-- Were we in locked mode after change?

	initial_selected_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- Elements that were selected before change

	final_selected_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- Elements that were selected after change (Void
			-- if same as `initial_selected_elements')

	initial_partitions_hidden: BOOLEAN
			-- Were partitions hidden before change?

	final_partitions_hidden: BOOLEAN
			-- Were partitions hidden after change?

	is_valid: BOOLEAN
			-- Is `Current' valid?
		do
		end

feature -- Modification

	set_name (a_name: like name)
			-- Set `name' to `a_name'
		do
			name := a_name
		ensure
			name_set: name = a_name
		end

	set_view (a_view: like view)
			-- Set `view' to `a_view'
		do
		end

	set_changes (a_change_list: like changes)
			-- Set `changes' to `a_change_list'
		do
			changes := a_change_list
		ensure
			changes_set: changes = a_change_list
		end

	set_displayed_area (an_area: like displayed_area)
			-- Set `displayed_area' to `an_area'
		do
			displayed_area := an_area
		ensure
			displayed_area_set: displayed_area = an_area
		end

	set_initial_locked_mode (a_mode: like initial_locked_mode)
			-- Set `initial_locked_mode' to `a_mode'
		do
			initial_locked_mode := a_mode
			is_set_initial_locked_mode := True
		ensure
			initial_locked_mode_set: initial_locked_mode = a_mode
		end

	set_final_locked_mode (a_mode: like final_locked_mode)
			-- Set `final_locked_mode' to `a_mode'
		do
			final_locked_mode := a_mode
			is_set_final_locked_mode := True
		ensure
			final_locked_mode_set: final_locked_mode = a_mode
		end

	set_initial_selected_elements (a_list: like initial_selected_elements)
			-- Set `initial_selected_elements' to `a_list'
		do
			initial_selected_elements := a_list
		ensure
			initial_selected_elements_set: initial_selected_elements = a_list
		end

	set_final_selected_elements (a_list: like final_selected_elements)
			-- Set `final_selected_elements' to `a_list'
		do
			final_selected_elements := a_list
		ensure
			final_selected_elements_set: final_selected_elements = a_list
		end

	set_initial_partitions_hidden (a_list: like initial_partitions_hidden)
			-- Set `initial_partitions_hidden' to `a_list'
		do
			initial_partitions_hidden := a_list
			is_set_initial_partitions_hidden := True
		ensure
			initial_partitions_hidden_set: initial_partitions_hidden = a_list
		end

	set_final_partitions_hidden (a_list: like final_partitions_hidden)
			-- Set `final_partitions_hidden' to `a_list'
		do
			final_partitions_hidden := a_list
			is_set_final_partitions_hidden := True
		ensure
			final_partitions_hidden_set: final_partitions_hidden = a_list
		end

feature -- Operations

	undo
			-- Undo change
		require
			is_valid: is_valid
		do
			failed := False
			check_undo_valid
			if not failed then
				start_undo
				from
					changes.finish
				until
					changes.before
				loop
					undo_simple_change (changes.item)
					changes.back
				end
				finish_undo
			end
		end

	redo
			-- Redo change
		require
			is_valid: is_valid
		do
			failed := False
			check_redo_valid
			if not failed then
				start_redo
				from
					changes.start
				until
					changes.after
				loop
					redo_simple_change (changes.item)
					changes.forth
				end
				finish_redo
			end
		end

feature {NONE} -- Implementation

	undo_simple_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Undo `a_change'
		do
			a_change.undo
		end

	redo_simple_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Redo `a_change'
		do
			a_change.redo
		end

	check_undo_valid
			-- Check that undo can be done now.
			-- Set `failed' to True if it cannot be done
		do
		end

	check_redo_valid
			-- Check that redo can be done now.
			-- Set `failed' to True if it cannot be done
		do
		end

	start_undo
			-- Take actions needed before undoing all simple
			-- changes
		deferred
		end

	finish_undo
			-- Take actions needed after undoing all simple
			-- changes
		deferred
		end

	start_redo
			-- Take actions needed before redoing all simple
			-- changes
		deferred
		end

	finish_redo
			-- Take actions needed after redoing all simple
			-- changes
		deferred
		end

	model: RETICLE_MODEL
			-- Model for view
		do
		end

	
	is_set_initial_locked_mode: BOOLEAN
			-- Has `initial_locked_mode' been set?

	is_set_final_locked_mode: BOOLEAN
			-- Has `final_locked_mode' been set?

	is_set_initial_partitions_hidden: BOOLEAN
			-- Has `initial_partitions_hidden' been set?

	is_set_final_partitions_hidden: BOOLEAN
			-- Has `final_partitions_hidden' been set?


end
