
note
	
	description: "A cut pattern in reticle model"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_CUT_PATTERN
inherit
	ANY
	CONDITION_UTILITIES
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Creation

	make (a_pattern: like pattern a_table: like cut_pattern_xform_types_table a_model: RETICLE_MODEL)
		require
			a_pattern_not_void: a_pattern /= Void
			a_table_not_void: a_table /= Void
			a_model_not_void: a_model /= Void
		do
			pattern := a_pattern
			cut_pattern_xform_types_table := a_table
			model := a_model
			set_current_xform_type
			pattern.detach_actions.extend (agent update_pattern)
		end

feature -- Properties

	pattern: PACKING_CUT_PATTERNS_MROW
			-- Cut pattern row

	is_read_only: BOOLEAN
			-- Is pattern read-only (cannot be modified
			-- or deleted)?
		do
			Result := pattern.is_read_only
		end

	cut_pattern_no: INTEGER
			-- Number assigned to cut pattern
		do
			Result := pattern.cut_pattern_no
		end

	production_id: STRING
			-- Production ID cut pattern is for
		do
			Result := pattern.production_id
		end

	cut_pattern_xform_type: STRING
			-- Type of transform cut pattern performs
		do
			Result := pattern.cut_pattern_xform_type
		end

	cut_pattern_no_parent: INTEGER
			-- Number of parent cut pattern, if not null
		do
			Result := pattern.cut_pattern_no_parent
		end

	is_null_cut_pattern_no_parent: BOOLEAN
			-- Is parent cut pattern number null?
		do
			Result := pattern.is_null_cut_pattern_no_parent
		end

	source_type: STRING
			-- Type of entity to which cut pattern is applied
		do
			Result := current_xform_type.source_type
		end

	result_type: STRING
			-- Type of entity application of cut
			-- pattern produces
		do
			Result := current_xform_type.result_type
		end

	isolates_sites: BOOLEAN
			-- Does this type of cut pattern
			-- isolate site?
		do
			Result := current_xform_type.isolates_sites
		end

	produces_strips: BOOLEAN
			-- Does this type of cut pattern
			-- produce wafer strips?
		do
			Result := current_xform_type.produces_strips
		end

	applies_to_strips: BOOLEAN
			-- Is this type of cut pattern
			-- applied to wafer strips?
		do
			Result := current_xform_type.applies_to_strips
		end

	presentation_text: STRING
			-- Text that describes current cut pattern
			-- transformation type
		do
			Result := current_xform_type.presentation_text
		end

	field_name: STRING
			-- Name of field to which cut pattern applies
		do
			Result := pattern.field_name
		end

	has_wafer_etest_name: BOOLEAN
			-- Is there a wafer etest name for cut pattern?
		do
			Result := not pattern.is_null_wafer_etest_name
		end

	wafer_etest_name: STRING
			-- Wafer etest name for cut pattern
		do
			Result := pattern.wafer_etest_name
		end

	wafer_etest_id: STRING
			-- Wafer etest ID for cut pattern
		require
			has_wafer_etest_name: has_wafer_etest_name
		do
			Result := pattern.wafer_etest_id
		end

	has_quantity_wafers_needed: BOOLEAN
			-- Is there a number wafers to cut for pattern?
		do
			Result := not pattern.is_null_quantity_wafers_needed
		end

	saw_blade_width: DOUBLE
			-- Width of saw blade to be used for pattern
		do
			Result := pattern.saw_blade_width
		end

	quantity_wafers_needed: INTEGER
			-- Number of wafers to cut for pattern or -1 if
			-- not yet specified
		do
			if has_quantity_wafers_needed then
				Result := pattern.quantity_wafers_needed
			else
				Result := -1
			end
		end

	is_completed: BOOLEAN
			-- Is cut pattern completed (not under construction)?
		do
			Result := pattern.is_completed
		end

	has_cuts: BOOLEAN
			-- Are there any cuts for pattern?
		do
			Result := pattern.has_cuts
		end

	has_horizontal_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a horizontal cut at
			-- `a_location'?
		do
			Result := pattern.has_horizontal_cut_at (a_location)
		end

	has_vertical_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a vertical cut at
			-- `a_location'?
		do
			Result := pattern.has_vertical_cut_at (a_location)
		end

	has_external_cut_left (fld_left_edge: DOUBLE): BOOLEAN
			-- Does pattern include an external cut
			-- to left of `fld_left_edge'?
		local
			vert_cuts: like vertical_cuts
		do
			vert_cuts := vertical_cuts
			from
				vert_cuts.start
			until
				vert_cuts.after or Result
			loop
				Result := vert_cuts.item.location < fld_left_edge
				vert_cuts.forth
			end
		end

	has_external_cut_right (fld_right_edge: DOUBLE): BOOLEAN
			-- Does pattern include an external cut
			-- to right of `fld_right_edge'?
		local
			vert_cuts: like vertical_cuts
		do
			vert_cuts := vertical_cuts
			from
				vert_cuts.start
			until
				vert_cuts.after or Result
			loop
				Result := vert_cuts.item.location > fld_right_edge
				vert_cuts.forth
			end
		end

	has_external_cut_bottom (fld_bottom_edge: DOUBLE): BOOLEAN
			-- Does pattern include an external cut
			-- below `fld_bottom_edge'?
		local
			horiz_cuts: like horizontal_cuts
		do
			horiz_cuts := horizontal_cuts
			from
				horiz_cuts.start
			until
				horiz_cuts.after or Result
			loop
				Result := horiz_cuts.item.location < fld_bottom_edge
				horiz_cuts.forth
			end
		end

	has_external_cut_top (fld_top_edge: DOUBLE): BOOLEAN
			-- Does pattern include an external cut
			-- above `fld_top_edge'?
		local
			horiz_cuts: like horizontal_cuts
		do
			horiz_cuts := horizontal_cuts
			from
				horiz_cuts.start
			until
				horiz_cuts.after or Result
			loop
				Result := horiz_cuts.item.location > fld_top_edge
				horiz_cuts.forth
			end
		end

	horizontal_cuts: LIST [PACKING_CUT_LOCATIONS_ROW]
			-- horizontal cuts for cut pattern
		do
			Result := pattern.horizontal_cuts
		end

	vertical_cuts: LIST [PACKING_CUT_LOCATIONS_ROW]
			-- Vertical cuts for cut pattern
		do
			Result := pattern.vertical_cuts
		end

	has_isolated_sites: BOOLEAN
			-- Are there any isolated sites for pattern?
		do
			Result := pattern.has_isolated_sites
		end

	isolates (a_location: PACKING_DIE_LOCATIONS_MROW): BOOLEAN
			-- Does pattern isolate `a_location'?
		do
			Result := pattern.isolates_location (a_location)
		end

	isolated_cut_die (an_element: RETICLE_MODEL_ELEMENT): RECTANGLE
			-- Cut die for isolated element `an_element' in 
			-- cut pattern, determined from its cut die site
		require
			an_element_not_void: an_element /= Void
			an_element_is_die: an_element.is_die
		local
			rect: RECTANGLE
		do
			rect := pattern.isolated_cut_die (an_element.die_location)
			Result := an_element.unoriented_model_relative_rectangle (rect)
		end

	isolated_location_count: INTEGER
			-- Number of die locations isolated by cut pattern
		do
			Result := pattern.isolated_location_count
		end

	description: STRING
			-- Description of cut pattern for messages to user
		do
			Result := "Pattern " + cut_pattern_no.out + ", Field " + field_name
		end

	description_for_selection: STRING
			-- Description of cut pattern when it
			-- is included in a list from which user
			-- selects one item
		do
			Result := pattern.description_for_selection
		end

	status: STRING
			-- Status of cut pattern
		do
			Result := pattern.status
		end

	model: RETICLE_MODEL
			-- Model containing cut pattern

feature -- Modification

	set_is_completed (a_value: BOOLEAN)
			-- Set `is_completed' to `a_value'
		do
			pattern.set_is_completed (a_value)
		ensure
			completed_set: is_completed = a_value
		end

	set_cut_pattern_xform_type (a_value: STRING)
			-- Set `cut_pattern_xform_type' to `a_value'
		do
			pattern.set_cut_pattern_xform_type (a_value)
			set_current_xform_type
		ensure
			value_set: cut_pattern_xform_type.is_equal (a_value)
		end

	set_cut_pattern_no_parent (a_value: INTEGER)
			-- Set `cut_pattern_no_parent' to `a_value'
		do
			pattern.set_cut_pattern_no_parent (a_value)
		ensure
			value_set: cut_pattern_no_parent = a_value
		end

	set_null_cut_pattern_no_parent
			-- Set `cut_pattern_no_parent' to null
		do
			pattern.set_null_cut_pattern_no_parent
		ensure
			value_null: is_null_cut_pattern_no_parent
		end

	set_production_id (a_value: STRING)
			-- Set `production_id' to `a_value'
		require
			a_value_not_void: a_value /= Void
			a_value_not_empty: not a_value.is_empty
		do
			pattern.set_production_id (a_value)
		ensure
			value_set: production_id.is_equal (a_value)
		end

	set_wafer_etest_name (a_name: STRING)
			-- Set `wafer_etest_name' to `a_name'
		do
			pattern.set_wafer_etest_name (a_name)
		ensure
			name_set: wafer_etest_name.is_equal (a_name)
		end

	set_quantity_wafers_needed (a_value: INTEGER has_value: BOOLEAN)
			-- Set `quantity_wafers_needed' to `a_value' where
			-- `has_value' indicates whether there really is a
			-- value
		do
			if has_value then
				pattern.set_quantity_wafers_needed (a_value)
			else
				pattern.set_null_quantity_wafers_needed
			end
		end

	set_external_cut_distance_spec (a_value: CONST_ABSTRACT_EXT_CUT_DIST_SPEC)
			-- Set external cut distance specifications
			-- according to `a_value'
		require
			a_value_not_void: a_value /= Void
		do
			pattern.set_external_cut_specs (a_value)
		end

	set_saw_blade_width (a_width: DOUBLE)
			-- Set saw blade width to `a_width'
		require
			a_width_positive: a_width > 0.0
		do
			pattern.set_saw_blade_width (a_width)
		end

feature -- Operations

	remove_isolated_sites
			-- Remove all isolated sites for cut pattern
		do
			pattern.remove_isolated_sites
		end

	isolate (an_element: RETICLE_MODEL_ELEMENT)
			-- Isolate `an_element' in cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			pattern.isolate (an_element.die_location, an_element.virtual_oriented_field_relative_cut_cut_die)
		end

	unisolate (an_element: RETICLE_MODEL_ELEMENT)
			-- Unisolate `an_element' in cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			pattern.unisolate (an_element.die_location)
		end

	set_isolated_site_cut_die (an_element: RETICLE_MODEL_ELEMENT)
			-- Set cut die size and center from `an_element'
		require
			an_element_not_void: an_element /= Void
		do
			pattern.set_isolated_site_cut_die (an_element.die_location, an_element.virtual_oriented_field_relative_cut_cut_die)
		end

feature {NONE} -- Implementation

	set_current_xform_type
			-- Set `current_xform_type' row based on 
			-- `cut_pattern_xform_type' value
		local
			xf_type: like cut_pattern_xform_type
		do
			xf_type := cut_pattern_xform_type
			cut_pattern_xform_types_table.search (xf_type)
			if cut_pattern_xform_types_table.found then
				current_xform_type := cut_pattern_xform_types_table.found_item
			else
				handle_hard_error ("cannot find cut_pattern_xform_type " + xf_type + " in internal table")
			end
		end

	update_pattern (an_original, a_replacement: like pattern)
			-- If `pattern' equals `an_original', replace it
			-- with `a_replacement'
		do
			if pattern = an_original then
				pattern := a_replacement
			end
		end

	cut_pattern_xform_types_table: HASH_TABLE [CUT_PATTERN_XFORM_TYPES_ROW, STRING]
			-- Cut pattern transform types, index by
			-- transform type name

	current_xform_type: CUT_PATTERN_XFORM_TYPES_ROW
			-- Row for current cut pattern transform type

end
