
note
	
	description: "Label for a cut pattern"
	author: "David Hollenberg"
	date: "July 3, 2007"

class RETICLE_CUT_PATTERN_LABEL
inherit
	RETICLE_VIRTUAL_ELEMENT
		redefine
			field, is_cut_pattern_label
		end

create
	make

feature {NONE} -- Creation

	make (a_cut_pattern: like cut_pattern a_field: like field a_size, a_center: CONST_POINT a_model: like anchored_model)
		require
			a_cut_pattern_not_void: a_cut_pattern /= Void
			a_field_not_void: a_field /= Void
			a_size_not_void: a_size /= Void
			a_center_not_void: a_center /= Void
			a_size_x_positive: a_size.x > 0.0
			a_size_y_positive: a_size.y > 0.0
			a_model_not_void: a_model /= Void
		do
			initialize (a_size, a_center, a_model)
			cut_pattern := a_cut_pattern
			field := a_field
			cut_pattern.detach_actions.extend (agent update_pattern)
		end

feature -- Properties

	identification: STRING
			-- Identifying name of element
		do
			Result := cut_pattern.label_description
		end

	cut_pattern: PACKING_CUT_PATTERNS_MROW
			-- Cut pattern for which `Current' is a label

	field: RETICLE_FIELD
			-- Field of view with which element is associated.
			-- It isn't really associated with this field, but
			-- the model assumes that every element has an
			-- associated field of view, so this may avoid
			-- breaking some code

	name: STRING
			-- Name of field
		do
			Result := field.name
		end

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := center_x - field.center_x
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := center_y - field.center_y
		end

	is_cut_pattern_label: BOOLEAN = True
			-- Is element a label for a cut pattern?

feature {NONE} -- Implementation

	associated_field_name: STRING
		do
			Result := name
		end

	update_pattern (an_original, a_replacement: like cut_pattern)
			-- If `cut_pattern' equals `an_original', replace it
			-- with `a_replacement'
		do
			if cut_pattern = an_original then
				cut_pattern := a_replacement
			end
		end

end
