
note
	
	description: "One copy of a die element (of any type) of reticle model"
	author: "David Hollenberg"
	date: "July 28, 2006"

deferred class RETICLE_DIE
inherit
	RETICLE_ABSTRACT_DIE
		redefine
			die, die_location,
			reticle_die,
			packaging_max_die_size_fixed_rotation,
			packaging_max_die_size_free_rotation,
			user_max_die_size,
			die_id, non_scribe_die_id, is_die, is_non_scribe_die,
			mask_site_number, has_mask_site_number,
			is_oversize_die_always_allowed, 
			is_oversize_die_possibly_allowed,
			oversize_die_inclusion_conditions,
			participates_in_scan
		end

feature {NONE} -- Creation

	make (a_die: like die a_die_location: like die_location a_model: ANY)
		require
			a_die_not_void: a_die /= Void
			a_die_location_not_void: a_die_location /= Void
			is_valid_for_creation: is_valid_for_creation (a_die)
		do
			die := a_die
			die_location := a_die_location
			die.detach_actions.extend (agent update_die)
			die_location.detach_actions.extend (agent update_die_location)
			create temp_point.make_xy (0, 0)
			
			-- Make sure preferences for movability, etc
			-- are set appropriately if object is
			-- stationary, etc.  Not strictly needed but
			-- might avoid confusion for people looking at
			-- contents of packing database tables
			
			if is_stationary_in_x then
				set_is_fixed_in_x_internal (True)
				set_is_movable_in_x_internal (False)
			end
			if is_stationary_in_y then
				set_is_fixed_in_y_internal (True)
				set_is_movable_in_y_internal (False)
			end
			if is_stationary_orientation then
				set_is_rotatable_internal (False)
			end
			if is_never_duplicatable then
				set_is_duplicatable_internal (False)
			end
			if is_never_deletable then
				set_is_deletable_internal (False)
			end
		end

feature -- Properties

	die: PACKING_DIES_MROW
			-- Die of which this is one location
	
	die_location: PACKING_DIE_LOCATIONS_MROW
			-- Location of this copy of die
	
	reticle_die: RETICLE_DIE
			-- Reticle die for `Current'
		do
			Result := Current
		end

	unique_identifier: STRING
			-- Unique identifier for `Current'
		do
			Result := die_location.concatenated_key_columns
		end

	packaging_max_die_size_fixed_rotation: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes for packagings which specify a
			-- fixed rotation of die in package.  Void
			-- indicates no maximum
		do
			Result := Void
		end

	packaging_max_die_size_free_rotation: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes for packagings which do not
			-- specify a rotation of die in package.  Void
			-- indicates no maximum
		do
			Result := Void
		end

	user_max_die_size: CONST_POINT
			-- Maximum die size, as constrained by
			-- user-specified limits.  Void indicates no maximum
		do
			Result := Void
		end

	is_oversize_die_always_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed unconditionally?
		do
			Result := False
		end

	is_oversize_die_possibly_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed but only under specific conditions?
		do
			Result := False
		end

	oversize_die_inclusion_conditions: STRING
			-- Description of conditions under which 
			-- die that is bigger than `max_die_size'
			-- allowed
		do
			Result := Void
		end

	is_die: BOOLEAN = True
			-- Is element a die?

	is_non_scribe_die: BOOLEAN
			-- Is element a die that is not a scribe
			-- (a "real" die)?
		do
			Result := True
		end

	die_id: STRING
			-- Die ID of element or empty string if
			-- no associated die ID
		do
			Result := die.die_id
		end

	non_scribe_die_id: STRING
			-- Die ID of element or empty string if
			-- element is a scribe or has no associated
			-- die ID
		do
			Result := die.die_id
		end

	mask_site_number: INTEGER
			-- Mask site number of element
		do
			Result := die_location.mask_site_no
		end

	has_mask_site_number: BOOLEAN
			-- Does element have an associated mask
			-- site number?
		do
			Result := not die_location.is_null_mask_site_no
		end

	is_selectable: BOOLEAN = True
			-- May object be selected?

	rotation: INTEGER
			-- Rotation of object in degrees clockwise
		do
			Result := die.die_wafer_rotation
		end

	is_rotated_sideways: BOOLEAN
			-- Is object rotated by 90 degrees clockwise or
			-- 90 degrees counterclockwise?
		do
			Result := die.is_rotated_sideways
		end

	is_fixed_in_x_internal: BOOLEAN
			-- Is object temporarily fixed in the X dimension
			-- so that it cannot move in that dimension?
		do
			Result := die_location.is_fixed_in_x
		end

	is_fixed_in_y_internal: BOOLEAN
			-- Is object temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension?
		do
			Result := die_location.is_fixed_in_y
		end

	is_movable_in_x_internal: BOOLEAN
			-- May object currently be moved in the X dimension?
		do
			Result := die_location.is_movable_in_x
		end

	is_movable_in_y_internal: BOOLEAN
			-- May object currently be moved in the X dimension?
		do
			Result := die_location.is_movable_in_y
		end

	is_rotatable_internal: BOOLEAN
			-- May object currently be moved in the X dimension?
		do
			Result := die_location.is_rotatable
		end

	is_duplicatable_internal: BOOLEAN
			-- May object currently be duplicated?
		do
			Result := die_location.is_duplicatable
		end

	is_deletable_internal: BOOLEAN
			-- May object currently be deleted?
		do
			Result := die_location.is_deletable
		end

	can_be_fixed_in_x: BOOLEAN = True
			-- Can object be temporarily fixed in the X dimension
			-- so that it cannot move in that dimension?

	can_be_fixed_in_y: BOOLEAN = True
			-- Can object be temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension?

	is_stationary_in_x: BOOLEAN
			-- Can object never be moved in the X dimension?
		do
			Result := False
		end

	is_stationary_in_y: BOOLEAN
			-- Can object never be moved in the Y dimension?
		do
			Result := False
		end

	is_stationary_orientation: BOOLEAN
			-- Can orientation of object never be changed?
		do
			Result := False
		end

	is_never_duplicatable: BOOLEAN
			-- Can object never be duplicated?
		do
			Result := False
		end

	is_never_deletable: BOOLEAN
			-- Can object never be deleted?
		do
			Result := False
		end

	size_x: DOUBLE
			-- Size of minimum bounding box in X dimension
			-- of reticle field of view
		do
			Result := die.non_null_die_size_printed_x
		end

	size_y: DOUBLE
			-- Size of minimum bounding box in Y dimension
			-- of reticle field of view
		do
			Result := die.non_null_die_size_printed_y
		end

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := die_location.location_x
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := die_location.location_y
		end

	participates_in_scan: BOOLEAN
			-- Does `Current' participate in horizontal
			-- and vertical scan operations?
		do
			Result := True
		end

feature -- Modification

	set_is_fixed_in_x (b: BOOLEAN)
			-- Set `is_fixed_in_x' to `b'
		do
			set_is_fixed_in_x_internal (b)
		end

	set_is_fixed_in_y (b: BOOLEAN)
			-- Set `is_fixed_in_y' to `b'
		do
			set_is_fixed_in_y_internal (b)
		end

	set_is_movable_in_x (b: BOOLEAN)
			-- Set `is_movable_in_x' to `b'
		do
			set_is_movable_in_x_internal (b)
		end

	set_is_movable_in_y (b: BOOLEAN)
			-- Set `is_movable_in_y' to `b'
		do
			set_is_movable_in_y_internal (b)
		end

	set_is_rotatable (b: BOOLEAN)
			-- Set `is_rotatable' to `b'
		do
			set_is_rotatable_internal (b)
		end

	set_is_duplicatable (b: BOOLEAN)
			-- Set `is_duplicatable' to `b'
		do
			set_is_duplicatable_internal (b)
		end

	set_is_deletable (b: BOOLEAN)
			-- Set `is_deletable' to `b'
		do
			set_is_deletable_internal (b)
		end

	set_rotation (n: INTEGER)
			-- Set rotation of `Current' to `n' degrees clockwise
		do
			die.set_rotation (n)
		end

	toggle_rotation
			-- Toggle `is_rotated'
		do
			die.toggle_rotation
		end

	rotate_90_clockwise
			-- Rotate `Current' 90 degrees clockwise
		do
			die.rotate_90_clockwise
		end

	rotate_90_counterclockwise
			-- Rotate `Current' 90 degrees counterclockwise
		do
			die.rotate_90_counterclockwise
		end

	move_location_by (a_distance: CONST_POINT)
			-- Move center of `Current' by `a_distance'
		do
			die_location.move_by (a_distance)
		end

	duplicate
		local
			dl: like die_location
		do
			die.add_die_location (die_location.field_name, die_location.location_x, die_location.location_y, die_location.is_rotatable, True)
			dl := die.last_die_location
		end

	set_size_x (val: DOUBLE)
			-- Set `size_x' to `val'
		do
			die.set_die_size_printed_x (val)
		end

	set_size_y (val: DOUBLE)
			-- Set `size_y' to `val'
		do
			die.set_die_size_printed_y (val)
		end

	set_field_relative_center_x (val: DOUBLE)
			-- Set `field_relative_center_x' to `val'
		do
			die_location.set_location_x (val)
		end

	set_field_relative_center_y (val: DOUBLE)
			-- Set `field_relative_center_y' to `val'
		do
			die_location.set_location_y (val)
		end

	set_associated_field_name (a_name: STRING)
			-- Set name of associated field to `a_name'
		do
			die_location.set_field_name (a_name)
		end

feature -- Contract support

	is_valid_for_creation (a_die: like die): BOOLEAN
			-- Does `a_die' have a die type that is valid
			-- for creation of `Current'?
		require
			a_die_not_void: a_die /= Void
		deferred
		end

feature {NONE} -- Implementation

	set_is_fixed_in_x_internal (b: BOOLEAN)
			-- Set `is_fixed_in_x_internal' to `b'
		do
			die_location.set_is_fixed_in_x (b)
		end

	set_is_fixed_in_y_internal (b: BOOLEAN)
			-- Set `is_fixed_in_y_internal' to `b'
		do
			die_location.set_is_fixed_in_y (b)
		end

	set_is_movable_in_x_internal (b: BOOLEAN)
			-- Set `is_movable_in_x_internal' to `b'
		do
			die_location.set_is_movable_in_x (b)
		end

	set_is_movable_in_y_internal (b: BOOLEAN)
			-- Set `is_movable_in_y_internal' to `b'
		do
			die_location.set_is_movable_in_y (b)
		end

	set_is_rotatable_internal (b: BOOLEAN)
			-- Set `is_rotatable_internal' to `b'
		do
			die_location.set_is_rotatable (b)
		end

	set_is_duplicatable_internal (b: BOOLEAN)
			-- Set `is_duplicatable_internal' to `b'
		do
			die_location.set_is_duplicatable (b)
		end

	set_is_deletable_internal (b: BOOLEAN)
			-- Set `is_deletable_internal' to `b'
		do
			die_location.set_is_deletable (b)
		end

	y_coordinate_translated_to_field (a_field: RETICLE_FIELD): DOUBLE
			-- Y coordinate of field-relative center
			-- translated from its value for current associated
			-- field to the value it would have if associated
			-- field was changed to `a_field'
		require
			a_field_not_void: a_field /= Void
		local
			old_field: like field
		do
			old_field := field
			Result := die_location.location_y + field.center_y - a_field.center_y
		end

	update_die (an_original, a_replacement: like die)
			-- If `die' equals `an_original', replace it
			-- with `a_replacement'
		do
			if die = an_original then
				die := a_replacement
			end
		end

	update_die_location (an_original, a_replacement: like die_location)
			-- If `die_location' equals `an_original', replace it
			-- with `a_replacement'
		do
			if die_location = an_original then
				die_location := a_replacement
			end
		end

	associated_field_name: STRING
		do
			Result := die_location.field_name
		end
	
invariant
	die_not_void: die /= Void
	die_location_not_void: die_location /= Void

end
