
note
	
	description: "Field of view element of reticle model"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_FIELD
inherit
	RETICLE_FIELD_PART
		redefine
			is_field
		end

create
	make

feature {NONE} -- Creation

	make (a_field_set_name, a_field_name: STRING a_size, a_center: CONST_POINT a_spacing_fraction: DOUBLE a_model: like anchored_model)
		require
			a_field_set_name_not_void: a_field_set_name /= Void
			a_field_name_not_void: a_field_name /= Void
			a_size_not_void: a_size /= Void
			a_center_not_void: a_center /= Void
			a_size_x_positive: a_size.x > 0.0
			a_size_y_positive: a_size.y > 0.0
			a_spacing_fraction_nonnegative: a_spacing_fraction >= 0.0
			a_model_not_void: a_model /= Void
		do
			initialize (a_size, a_center, a_model)
			field_set_name := a_field_set_name
			name := a_field_name
			spacing_fraction := a_spacing_fraction
		end

feature -- Properties

	identification: STRING = ""
			-- Identifying name of element
	
	name: STRING
			-- Name of field

	field_set_name: STRING
			-- Name of field set field is part of

	tier: WAFER_TECHN_3D_TIERS_ROW
			-- Tier for field (3DIC runs only).  Void for
			-- non-3DIC runs or for "virtual 3DIC" tier

	label: RETICLE_FIELD_LABEL
			-- Associated label

	bounding_box_including_label: CONST_RECTANGLE
			-- Minimum bounding box of field and its associated
			-- label
		local
			rect: RECTANGLE
		do
			create rect.make_copy (bounding_box)
			rect.extend_to_include (label.bounding_box)
			Result := rect
		end

	field_relative_center_x: DOUBLE = 0.0
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated

	field_relative_center_y: DOUBLE = 0.0
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated

	field_spacing: DOUBLE
			-- Distance between edge of one field and
			-- edge of adjacent
		do
			Result := size_y * spacing_fraction
		end

	is_field: BOOLEAN = True
			-- Is element a field of view?
	
feature -- Modification

	set_label (a_label: like label)
		require
			a_label_not_void: a_label /= Void
		do
			label := a_label
		ensure
			label_set: label = a_label
		end

	set_tier (a_tier: like tier)
		do
			tier := a_tier
		ensure
			tier_set: tier = a_tier
		end

feature {NONE} -- Implementation

	associated_field_name: STRING
		do
			Result := name
		end

	spacing_fraction: DOUBLE
			-- Spacing between one field of view and the next
			-- is this fraction of the height of one field

end
