
note
	
	description: "Field of view label element of reticle model"
	author: "David Hollenberg"
	date: "October 4, 2006"

class RETICLE_FIELD_LABEL
inherit
	RETICLE_FIELD_PART
		redefine
			field,
			is_field_label
		end

create
	make

feature {NONE} -- Creation

	make (a_field: RETICLE_FIELD a_size, a_center: CONST_POINT a_model: like anchored_model)
		require
			a_field_not_void: a_field /= Void
			a_size_not_void: a_size /= Void
			a_center_not_void: a_center /= Void
			a_size_x_positive: a_size.x > 0.0
			a_size_y_positive: a_size.y > 0.0
			a_model_not_void: a_model /= Void
		do
			initialize (a_size, a_center, a_model)
			field := a_field
		end

feature -- Properties

	identification: STRING
			-- Identifying name of element
		local
			l_tier: WAFER_TECHN_3D_TIERS_ROW
		do
			Result := "Field " + name
			l_tier := field.tier
			if l_tier /= Void then
				Result.append (" (")
				Result.append (l_tier.description)
				Result.append (")")
			elseif is_3dic then
				Result.append (" (3DIC)")
			end
		end

	field: RETICLE_FIELD
			-- Field of view with which element is associated

	name: STRING
			-- Name of field
		do
			Result := field.name
		end

	is_3dic: BOOLEAN
			-- Is label for a 3DIC?

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := center_x - field.center_x
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := center_y - field.center_y
		end

	is_field_label: BOOLEAN = True
			-- Is element a field of view label?
	
feature -- Modification

	set_is_3dic (b: BOOLEAN)
			-- Set `is_3dic' to `b'
		do
			is_3dic := b
		ensure
			is_3dic_set: is_3dic = b
		end

feature {NONE} -- Implementation

	associated_field_name: STRING
		do
			Result := name
		end

end
