
note
	
	description: "Field of view or field of view label of reticle model"
	author: "David Hollenberg"
	date: "October 4, 2006"

deferred class RETICLE_FIELD_PART
inherit
	RETICLE_VIRTUAL_ELEMENT
		redefine
			is_field_part
		end

feature -- Properties

	is_field_part: BOOLEAN = True
			-- Is element one of the parts of a field of view?
			
feature -- Operations

	save_initial_size_and_center
			-- Save initial size and center in history
			-- list to support undo/redo of payload size
			-- changes
		do
			if current_position = 0 then
				create {ARRAYED_LIST [CONST_POINT]} historical_sizes.make (Initial_history_size)
				create {ARRAYED_LIST [CONST_POINT]} historical_centers.make (Initial_history_size)
				save_current_size_and_center
			end
		end

	save_current_size_and_center
			-- Save current size and center in history
			-- list to support undo/redo of payload size
			-- changes
		local
			pos: INTEGER
		do
			-- Remove all history entries after
			-- the current position
			
			from
				pos := historical_sizes.count
			until
				pos = current_position
			loop
				historical_sizes.go_i_th (pos)
				historical_sizes.remove
				historical_centers.go_i_th (pos)
				historical_centers.remove
				pos := pos - 1
			end

			historical_sizes.extend (create {CONST_POINT}.make_xy (size_x, size_y))
			historical_centers.extend (create {CONST_POINT}.make_xy (center_x, center_y))
			current_position := current_position + 1
		end

	set_new_size_and_center (f: like Current)
			-- Set new size and center from `f'
		do
			size_x := f.size_x
			size_y := f.size_y
			center_x := f.center_x
			center_y := f.center_y
		end

	restore_previous_size_and_center
			-- Restore previous size and center from history
			-- list to support undo/redo of payload size
			-- changes
		do
			current_position := current_position - 1
			restore_current_size_and_center
		end

	restore_next_size_and_center
			-- Restore next size and center from history
			-- list to support undo/redo of payload size
			-- changes
		do
			current_position := current_position + 1
			restore_current_size_and_center
		end

	clear_historical_sizes_and_centers
			-- Clear history lists with sizes and centers
		do
			historical_sizes := Void
			historical_centers := Void
			current_position := 0
		end

feature {NONE} -- Implementation

	restore_current_size_and_center
			-- Restore size and center from entry
			-- at `current_position' in history lists
		local
			p: CONST_POINT
		do
			p := historical_sizes.i_th (current_position)
			size_x := p.x
			size_y := p.y
			p := historical_centers.i_th (current_position)
			center_x := p.x
			center_y := p.y
		end

	current_position: INTEGER
			-- Position of current size and center
			-- in history lists

	historical_sizes: DYNAMIC_LIST [CONST_POINT]
			-- Historical sizes of field part (one entry
			-- per payload size change that can still be
			-- undone/redone)

	historical_centers: DYNAMIC_LIST [CONST_POINT]
			-- Historical centers of field part (one entry
			-- per payload size change that can still be
			-- undone/redone)

	Initial_history_size: INTEGER = 10
			-- Initial size of history lists for sizes
			-- and centers

end
