
note
	
	description: "Element of reticle model that is fixed %
		%(cannot never be moved in X or Y, rotated, resized or duplicated)"
	author: "David Hollenberg"

deferred class RETICLE_FIXED_ELEMENT
inherit
	RETICLE_MODEL_ELEMENT

feature -- Properties

	can_be_fixed_in_x: BOOLEAN = False
			-- Can object be temporarily fixed in the X dimension
			-- so that it cannot move in that dimension?

	can_be_fixed_in_y: BOOLEAN = False
			-- Can object be temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension?

	is_fixed_in_x_internal: BOOLEAN = False
			-- Is object currently fixed in the Y dimension
			-- for vertical scanning purposes?

	is_fixed_in_y_internal: BOOLEAN = False
			-- Is object currently fixed in the Y dimension
			-- for vertical scanning purposes?

	db_maximum_move_up: DOUBLE = 0.0
			-- Maximum distance element can move up and still
			-- fit in database (inapplicable)

	db_maximum_move_down: DOUBLE = 0.0
			-- Maximum distance element can move down and still
			-- fit in database (inapplicable)

	db_maximum_move_left: DOUBLE = 0.0
			-- Maximum distance element can move left and still
			-- fit in database (inapplicable)

	db_maximum_move_right: DOUBLE = 0.0
			-- Maximum distance element can move right and still
			-- fit in database (inapplicable)

	is_stationary_in_x: BOOLEAN = True
			-- Can object never be moved in the X dimension?

	is_stationary_in_y: BOOLEAN = True
			-- Can object never be moved in the Y dimension?

	is_stationary_orientation: BOOLEAN = True
			-- Can orientation of object never be changed?

	rotation: INTEGER = 0
			-- Rotation of object in degrees clockwise

	is_rotated_sideways: BOOLEAN = False
			-- Is object rotated by 90 degrees clockwise or
			-- 90 degrees counterclockwise?

	is_never_duplicatable: BOOLEAN = True
			-- Can object never be duplicated?

feature {NONE} -- Inapplicable

	move_by (a_distance: CONST_POINT)
			-- Move `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_by_without_adjustment (a_distance: CONST_POINT)
			-- Move `Current' by `a_distance'
			-- but do not adjust the center to
			-- keep the edges on grid
		do
			-- Inapplicable
		end

	move_left_edge (a_distance: DOUBLE)
			-- Move left edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_right_edge (a_distance: DOUBLE)
			-- Move right edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_top_edge (a_distance: DOUBLE)
			-- Move top edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	move_bottom_edge (a_distance: DOUBLE)
			-- Move bottom edge of `Current' by `a_distance'
		do
			-- Inapplicable
		end

	duplicate
			-- Duplicate element and add it to model
		do
			-- Inapplicable
		end

	set_rotation (n: INTEGER)
			-- Set rotation of `Current' to `n' degrees clockwise
		do
			-- Inapplicable
		end

	toggle_rotation
			-- Toggle `is_rotated'
		do
			-- Inapplicable
		end

	rotate_90_clockwise
			-- Rotate object 90 degrees clockwise
		do
			-- Inapplicable
		end

	rotate_90_counterclockwise
			-- Rotate object 90 degrees counterclockwise
		do
			-- Inapplicable
		end

	round_edge_to_grid
			-- Move lower left corner of `Current' to nearest
			-- grid point.  Used to prevent accumulation of
			-- errors from multiple moves
		do
			-- Inapplicable
		end

	set_is_fixed_in_x (b: BOOLEAN)
			-- Set `is_fixed_in_x' to `b'
		do
			-- Inapplicable
		end

	set_is_fixed_in_y (b: BOOLEAN)
			-- Set `is_fixed_in_y' to `b'
		do
			-- Inapplicable
		end

	set_is_movable_in_x (b: BOOLEAN)
			-- Set `is_movable_in_x' to `b'
		do
			-- Inapplicable
		end

	set_is_movable_in_y (b: BOOLEAN)
			-- Set `is_movable_in_y' to `b'
		do
			-- Inapplicable
		end

	set_is_rotatable (b: BOOLEAN)
			-- Set `is_rotatable' to `b'
		do
			-- Inapplicable
		end

	set_is_duplicatable (b: BOOLEAN)
			-- Set `is_duplicatable' to `b'
		do
			-- Inapplicable
		end

feature {NONE} -- Implementation
	
	is_movable_in_x_internal: BOOLEAN = False
			-- May object currently be moved in the X dimension?

	is_movable_in_y_internal: BOOLEAN = False
			-- May object currently be moved in the Y dimension?

	is_rotatable_internal: BOOLEAN = False
			-- May rotation of object currently be changed?

	is_duplicatable_internal: BOOLEAN = False
			-- May object currently be duplicated?

end
