
note
	
	description: "Object that is an element of a model of a reticle"
	author: "David Hollenberg"
	date: "July 28, 2006"

deferred class RETICLE_MODEL_ELEMENT
inherit
	MODEL_ELEMENT
		redefine
			anchored_model_element,
			anchored_model_sub_element
		end
	PACKER_UTILITIES
		export
			{NONE} all
		end
	HAS_DESIGN_NAME
	HAS_DESIGN_NUMBER

feature -- Properties

	identification: STRING
			-- Identifying name of element
		deferred
		end

	identification_for_message: STRING
			-- Identifying name of element for displaying
			-- error and warning messages
		do
			Result := identification
		end

	location_description: STRING
			-- Description of location of element, for use in
			-- error and warning messages
		do
			Result := "at (" + grid.formatted_number (field_relative_center_x) + ", " + grid.formatted_number (field_relative_center_y) + ") from field " + associated_field_name + " center"
		end

	has_zero_size_in_x: BOOLEAN
			-- Does `Current' actually have zero size in X
			-- dimension?
		do
			Result := size_x = 0.0
		end
		
	has_zero_size_in_y: BOOLEAN
			-- Does `Current' actually have zero size in Y
			-- dimension?
		do
			Result := size_y = 0.0
		end
		
	size_description: STRING
			-- Description of size of element, for use in
			-- error and warning messages
		do
			Result := grid.formatted_number (size_x) + " x " + grid.formatted_number (size_y)
		end

	virtual_scribe_cut_size_description: STRING
			-- Description of virtual cut die size of
			-- element determined by surrounding scribes,
			-- for use in error and warning messages
		do
			Result := grid.formatted_number (virtual_scribe_cut_die_size_x) + " x " + grid.formatted_number (virtual_scribe_cut_die_size_y)
		end

	actual_scribe_cut_size_description: STRING
			-- Description of actual cut die size of
			-- element determined by surrounding scribes,
			-- for use in error and warning messages
		do
			Result := grid.formatted_number (actual_scribe_cut_die_size_x) + " x " + grid.formatted_number (actual_scribe_cut_die_size_y)
		end

	virtual_cut_cut_size_description: STRING
			-- Description of virtual cut die size of
			-- element determined by surrounding cuts, for
			-- use in error and warning messages
		do
			Result := grid.formatted_number (virtual_cut_cut_die_size_x) + " x " + grid.formatted_number (virtual_cut_cut_die_size_y)
		end

	actual_cut_cut_size_description: STRING
			-- Description of actual cut die size of
			-- element determined by surrounding cuts, for
			-- use in error and warning messages
		do
			Result := grid.formatted_number (actual_cut_cut_die_size_x) + " x " + grid.formatted_number (actual_cut_cut_die_size_y)
		end

	offset_cut_center_description: STRING
			-- Description of offset of die center from cut
			-- die center, for use in error and warning message
		do
			Result := grid.formatted_number (offset_cut_center_x) + " x " + grid.formatted_number (offset_cut_center_y)
		end

	packing_name: STRING
			-- Identifying name of element for packing
			-- sort purposes
		do
			Result := identification_for_message
		end

	has_design_number: BOOLEAN
			-- Does `Current' have an associated design number?
		do
			Result := is_project
		end

	design_number: INTEGER
			-- Design number of element
		do
			-- Inapplicable
		end

	has_design_name: BOOLEAN = True
			-- Does `Current' have a associated design name?

	design_name: STRING
			-- Design name of element or empty string if
			-- no associated design name
		do
			Result := ""
		end

	die_id: STRING
			-- Die ID of element or empty string if
			-- no associated die ID
		do
			Result := ""
		end

	non_scribe_die_id: STRING
			-- Die ID of element or empty string if
			-- element is a scribe or has no associated
			-- die ID
		do
			Result := ""
		end

	mask_site_number: INTEGER
			-- Mask site number of element
		require
			has_mask_site_number: has_mask_site_number
		do
		end

	has_mask_site_number: BOOLEAN
			-- Does element have an associated mask
			-- site number?
		do
			Result := False
		end

	info_name: STRING
			-- Name of element for packer information window
		do
			Result := identification_for_message
		end

	account_number: STRING
			-- Account number of element or empty string if
			-- no associated account number
		do
			Result := ""
		end

	account_type: STRING
			-- Account type of element or empty string if
			-- no associated account type
		do
			Result := ""
		end

	effective_overhead_style: STRING
			-- Effective overhead style for element
		do
			Result := ""
		end

	effective_overhead_style_description: STRING
			-- Description of effective overhead 
			-- style for element
		do
			Result := ""
		end

	is_layout_preliminary: BOOLEAN
			-- Does element have preliminary layout
			-- rather than final layout?
		do
			Result := False
		end

	layout_status_description: STRING
			-- Description of layout status 
			-- for element
		do
			Result := ""
		end

	test_vehicle_priority: STRING
			-- Description of test vehicle priority
			-- for element
		do
			Result := ""
		end

	organization_short_id: STRING
			-- Short name of organization type for element or
			-- empty string if no associated organization
		do
			Result := ""
		end

	wafer_etest_name: STRING
			-- Wafer etest name for element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	alternate_wafer_etest_names: LIST [STRING]
			-- Wafer etest names for element other than
			-- the primary one
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	alternate_assigned_wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW]
			-- Wafer etest name rows for element other
			-- than the primary one
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	assigned_wafer_etest_id: STRING
			-- Wafer etest ID assigned to element or empty string
			-- if none assigned yet or if not applicable
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := ""
		end

	assigned_wafer_etest_name: WAFER_ETEST_NAMES_ROW
			-- Wafer etest name assigned to element or Void
			-- if none assigned yet
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	unassigned_wafer_etest_ids: LIST [STRING]
			-- Wafer etest IDs not assigned to element or
			-- empty list if no unassigned etest IDs or
			-- if not applicable
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			create {ARRAYED_LIST [STRING]} Result.make (0)
		end

	has_wafer_etest_id: BOOLEAN
			-- Does object have associated wafer etest ID(s)?
		do
			Result := False
		end

	is_wafer_etest_id_assigned: BOOLEAN
			-- Does object with associated wafer etest ID(s)
			-- have a specific one assigned to it?
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			Result := not assigned_wafer_etest_id.is_empty
		end

	is_wafer_etest_id_missing: BOOLEAN
			-- Does object require that a wafer etest ID be
			-- assigned but not have a specific one assigned?
		do
			Result := has_wafer_etest_id and then not is_wafer_etest_id_assigned
		end

	compatible_wafer_etest_ids: LIST [STRING]
			-- Compatible wafer etest IDs for element or
			-- empty list if not applicable to this element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			create {ARRAYED_LIST [STRING]} Result.make (0)
		end

	compatible_wafer_etest_names: LIST [WAFER_ETEST_NAMES_ROW]
			-- Compatible wafer etest names rows for element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
			create {ARRAYED_LIST [WAFER_ETEST_NAMES_ROW]} Result.make (0)
		end

	has_constraining_wafer_etest_id: BOOLEAN
			-- Does object constrain other elements
			-- contained within it to have a particular
			-- wafer etest ID(s)?
		do
			Result := False
		end

	constraining_wafer_etest_id: STRING
			-- Wafer etest ID that elements contained
			-- in `Current' are required to have if 
			-- they have a wafer etest ID
		require
			has_constraining_wafer_etest_id: has_constraining_wafer_etest_id
		do
			Result := ""
		end

	constraining_wafer_etest_name: STRING
			-- Wafer etest ID that elements contained
			-- in `Current' are required to have if 
			-- they have a wafer etest ID
		require
			has_constraining_wafer_etest_id: has_constraining_wafer_etest_id
		do
			Result := ""
		end

	constraining_wafer_etest_names_row: WAFER_ETEST_NAMES_ROW
			-- Wafer etest names row for `Current'
		require
			has_constraining_wafer_etest_id: has_constraining_wafer_etest_id
		do
		ensure
			result_not_void: Result /= Void
		end

	has_quantity_requested: BOOLEAN
			-- Does object have a quantity of parts
			-- requested?
		do
			Result := False
		end

	quantity_requested: INTEGER
			-- Quantity of parts requested
		require
			has_quantity_requested: has_quantity_requested
		do
		end

	has_quantity_ordered: BOOLEAN
			-- Does object have a quantity of parts
			-- ordered?
		do
			Result := False
		end

	quantity_ordered: INTEGER
			-- Quantity of parts ordered
		require
			has_quantity_ordered: has_quantity_ordered
		do
		end

	packaging_description: STRING
			-- Description of packaging for element or 
			-- empty string if none
		do
			Result := ""
		end

	has_ceramic_packaging: BOOLEAN
			-- Will element be packaged in at least one ceramic
			-- package?
		do
			Result := False
		end

	has_plastic_packaging: BOOLEAN
			-- Will element be packaged in at least one plastic
			-- package?
		do
			Result := False
		end

	is_commercial_project: BOOLEAN
			-- Is element a commercial project?
		do
			Result := False
		end

	is_mep_project: BOOLEAN
			-- Is element an MEP project?
		do
			Result := False
		end

	is_mep_instructional_project: BOOLEAN
			-- Is element an MEP instructional project?
		do
			Result := False
		end

	is_mep_research_project: BOOLEAN
			-- Is element an MEP research project?
		do
			Result := False
		end

	is_mep_other_project: BOOLEAN
			-- Is element an MEP project that is
			-- not instructional or research?
		do
			Result := False
		end

	is_other_project: BOOLEAN
			-- Is element a project whose account type is not
			-- commercial or MEP?
		do
			Result := False
		end

	is_project: BOOLEAN
			-- Is element a project?
		do
			Result := False
		end

	is_virtual: BOOLEAN
			-- Is element virtual (label, field of view edge)
			-- rather than something that can potentially be
			-- manipulated (selected, moved, etc)?
		do
			Result := False
		end

	is_group: BOOLEAN
			-- Is element really a group of elements
			-- treated as a unit?
		do
			Result := False
		end

	is_field_part: BOOLEAN
			-- Is element one of the parts of a field of view?
		do
			Result := False
		end

	is_field: BOOLEAN
			-- Is element a field of view?
		do
			Result := False
		end

	is_field_label: BOOLEAN
			-- Is element a field of view label?
		do
			Result := False
		end

	is_non_field_label: BOOLEAN

			-- field of view?
		do
			Result := False
		end

	is_cut_pattern_label: BOOLEAN
			-- Is element a label for a cut pattern?
		do
			Result := False
		end

	is_partition: BOOLEAN
			-- Is element a partition?
		do
			Result := False
		end

	is_die: BOOLEAN
			-- Is element a die?
		do
			Result := False
		end

	is_non_filler_die: BOOLEAN
			-- Is element a die that is not a filler?
		do
			Result := is_die and not is_filler
		end

	reticle_die: RETICLE_DIE
			-- Reticle die for `Current'
		require
			is_die: is_die
		do
		end

	is_non_scribe_die: BOOLEAN
			-- Is element a die that is not a scribe
			-- (a "real" die)?
		do
			Result := False
		end

	is_abstract_die: BOOLEAN
			-- Is element an abstract die (die or
			-- partition)?
		do
			Result := False
		end

	is_cut: BOOLEAN
			-- Is element a cut line?
		do
			Result := False
		end

	is_horizontal_cut: BOOLEAN
			-- Is element a horizontal cut line?
		do
			Result := False
		end

	is_vertical_cut: BOOLEAN
			-- Is element a vertical cut line?
		do
			Result := False
		end

	is_external_cut: BOOLEAN
			-- Is element a cut line that is outside
			-- its associated field (external to payload)?
		do
			Result := is_cut and not is_center_in_field
		end

	is_isolated_in_cut_pattern (a_pattern: RETICLE_CUT_PATTERN): BOOLEAN
			-- Is element isolated in `a_pattern'?
		require
			is_die: is_die
		do
			Result := a_pattern.isolates (die_location)
		end

	cut_line: RETICLE_CUT_SUB_ELEMENT
			-- Line that is the actual cut,
			-- without handles
		require
			is_cut: is_cut
		do
		end
	
	external_cut_position_for (dist: EXTERNAL_CUT_DISTANCE_SET): DOUBLE
			-- Position of external cut dictated by
			-- `dist'
		require
			is_external_cut: is_external_cut
		local
			f: RETICLE_FIELD
			loc: DOUBLE
		do
			f := field
			if is_horizontal_cut then
				loc := field_relative_center_y
				if loc > 0.0 then
					-- Cut at top
					Result := grid.rounded_center (f.field_relative_top_y + dist.top)
				else
					-- Cut at bottom
					Result := grid.rounded_center (f.field_relative_bottom_y - dist.bottom)
				end
			else
				loc := field_relative_center_x
				if loc > 0.0 then
					-- Cut at right
					Result := grid.rounded_center (f.field_relative_right_x + dist.right)
				else
					-- Cut at left
					Result := grid.rounded_center (f.field_relative_left_x - dist.left)
				end
			end
		end

	is_external_cut_at_wrong_position: BOOLEAN
			-- Is element an external cut that is not at
			-- the position dictated by external cut
			-- specifications
		do
		end

	cut_position: DOUBLE
			-- Position of cut, in model coordinates (not
			-- relative to center of field)
		require
			is_cut: is_cut
		do
			if is_horizontal_cut then
				Result := center_y
			else
				Result := center_x
			end
		end
	
	cut_field_relative_position: DOUBLE
			-- Position of cut relative to center of field
		require
			is_cut: is_cut
		do
			if is_horizontal_cut then
				Result := field_relative_center_y
			else
				Result := field_relative_center_x
			end
		end

	is_cust_project: BOOLEAN
			-- Is element a customer project?
		do
			Result := False
		end

	is_unlocked_project: BOOLEAN
			-- Is element a customer project that is not
			-- locked against customer update?
		do
			Result := is_cust_project and then not die.project.is_currently_locked
		end

	is_unassigned_project: BOOLEAN
			-- Is element a customer project that is not
			-- not assigned to this run?
		do
		end

	is_test_vehicle: BOOLEAN
			-- Is element a test vehicle?
		do
			Result := False
		end

	is_high_priority_test_vehicle: BOOLEAN
			-- Is element a test vehicle that is
			-- high priority for inclusion on the run?
		do
			Result := False
		end

	is_vendor_geometry: BOOLEAN
			-- Is element a vendor geometry/ghost?
		do
			Result := False
		end

	is_movable_vendor_geometry: BOOLEAN
			-- Is element a vendor geometry/ghost
			-- that can be moved?
		do
			Result := False
		end

	is_scribe: BOOLEAN
			-- Is element a scribe line?
		do
			Result := False
		end

	is_horizontal_scribe: BOOLEAN
			-- Is element a horizontal scribe line?
		do
			Result := False
		end

	is_vertical_scribe: BOOLEAN
			-- Is element a vertical scribe line?
		do
			Result := False
		end

	is_filler: BOOLEAN
			-- Is element a filler?
		do
			Result := False
		end

	is_normal_filler: BOOLEAN
			-- Is element a normal filler?
		do
			Result := False
		end

	is_temporary_filler: BOOLEAN
			-- Is element a temporary filler used for
			-- positioning other elements or reserving area?
		do
			Result := False
		end

	is_placeholder_filler: BOOLEAN
			-- Is element a placeholder filler used to
			-- represent a design that has not yet been created
		do
			Result := False
		end

	is_below_minimum_size: BOOLEAN
			-- Is element size below the minimum allowed die size
			-- in either dimension?
		local
			l_min_size: DOUBLE
		do
			if is_abstract_die then
				Result := size_x < l_min_size or size_y < l_min_size
			else
				Result := False
			end
		end

	is_small_scribe_or_filler: BOOLEAN
			-- Is element a scribe or filler whose size is
			-- below the small object threshold in either
			-- dimension (just length for scribes)?
		local
			l_x, l_y, limit: DOUBLE
		do
			l_x := size_x
			l_y := size_y
			Result := (is_horizontal_scribe and l_x < limit) or (is_vertical_scribe and l_y < limit) or (is_filler and (l_x < limit or l_y < limit))
		end

	uniform_white_space_border: DOUBLE
			-- Uniform amount of white space around element
		do
			Result := 0.0
		end

	minimum_separation: DOUBLE
			-- Minimum separation between object that is a
			-- project (contains circuitry) and other objects
			-- or field of view edge
		do
		end

	has_field_edge_restriction: BOOLEAN
			-- Does element have a restriction on how close it
			-- can be to edge of field?
		do
			Result := True
		end

	needs_min_sep_white_space: BOOLEAN
			-- Does element need "minimum separation" amount of
			-- white space around it?
		do
			Result := False
		end

	minimum_gap: DOUBLE
			-- Gridded minimum gap needed between object and field
			-- of view edge or another object (zero gap is also
			-- allowed if not `needs_min_sep_white_space')
		do
			if needs_min_sep_white_space then
				Result := needed_white_space
			else
				Result := minimum_separation
			end
		ensure
			result_nonnegative: Result >= 0.0
		end

	minimum_separation_from (an_element: RETICLE_MODEL_ELEMENT): DOUBLE
			-- Minimum separation needed between `Current'
			-- and `an_element' (zero is also allowed if neither
			-- element `needs_min_sep_white_space')
		do
			Result := minimum_separation - (uniform_white_space_border + an_element.uniform_white_space_border)
		end

	participates_in_measure: BOOLEAN
			-- Does `Current' participate in measure in X/Y
			-- operations?
		do
			Result := True
		end

	participates_in_scan: BOOLEAN
			-- Does `Current' participate in horizontal
			-- and vertical scan operations?
		do
			Result := False
		end

	is_resizable: BOOLEAN
			-- Can `Current' be resized?
		do
			Result := False
		end

	can_be_isolated: BOOLEAN
			-- Is `Current' allowed to be isolated, based on its
			-- type?
		do
			Result := False
		end

	is_isolated_by_default: BOOLEAN
			-- Should element be isolated when default
			-- isolation is computed?
		do
			Result := False
		end

	offset_cut_center_x: DOUBLE
			-- Offset of center of die from center of cut die
			-- in X dimension, using closest edges of surrounding
			-- scribes instead of centers
		require
			virtual_edge_cut_die_not_void: virtual_edge_cut_die /= Void
		local
			l_cut_die: like virtual_edge_cut_die
		do
			l_cut_die := virtual_edge_cut_die
			Result := (grid.rounded_edge (left_x - l_cut_die.low_x) - grid.rounded_edge (l_cut_die.high_x - right_x)) / 2.0
			if grid.rounded_center_units (Result) \\ 2 = 0 then
				Result := grid.rounded_edge (Result)
			else
				Result := grid.truncated_edge (Result)
			end
		end

	offset_cut_center_y: DOUBLE
			-- Offset of center of die from center of cut die
			-- in X dimension, using closest edges of surrounding
			-- scribes instead of centers
		require
			virtual_edge_cut_die_not_void: virtual_edge_cut_die /= Void
		local
			l_cut_die: like virtual_edge_cut_die
		do
			l_cut_die := virtual_edge_cut_die
			Result := (grid.rounded_edge (bottom_y - l_cut_die.low_y) - grid.rounded_edge (l_cut_die.high_y - top_y)) / 2.0
			if grid.rounded_center_units (Result) \\ 2 = 0 then
				Result := grid.rounded_edge (Result)
			else
				Result := grid.truncated_edge (Result)
			end
		end

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		deferred
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		deferred
		end

	field_relative_left_x: DOUBLE
			-- Left edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := grid.rounded_center (field_relative_center_x - size_x / 2.0)
		end

	field_relative_right_x: DOUBLE
			-- Right edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := grid.rounded_center (field_relative_center_x + size_x / 2.0)
		end

	field_relative_top_y: DOUBLE
			-- Top edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := grid.rounded_center (field_relative_center_y + size_y / 2.0)
		end

	field_relative_bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := grid.rounded_center (field_relative_center_y - size_y / 2.0)
		end

	field: RETICLE_FIELD
			-- Field of view with which element is associated
		do
			if cached_field = Void then
				fetch_field
			end
			Result := cached_field
		end

	associated_field_name: STRING
			-- Name of field with which element is associated
		deferred
		ensure
			result_not_void: Result /= Void
		end
	
	is_center_in_field: BOOLEAN
			-- Is center of element inside field of view associated
			-- with element (closest field)?
		do
			Result := field.bounding_box.contains (create {CONST_POINT}.make_xy (center_x, center_y))
		end

	is_center_x_in_field: BOOLEAN
			-- Is center of element in X inside field of view
			-- associated with element (closest field)?
		do
			Result := field.bounding_box.contains (create {CONST_POINT}.make_xy (center_x, field.bounding_box.center_y))
		end

	is_center_y_in_field: BOOLEAN
			-- Is center of element in Y inside field of view
			-- associated with element (closest field)?
		do
			Result := field.bounding_box.contains (create {CONST_POINT}.make_xy (field.bounding_box.center_x, center_y))
		end

	intersects_field (f: RETICLE_FIELD): BOOLEAN
			-- Does element bounding box intersect bounding
			-- box of `f'?
		do
			Result := bounding_box.intersects (f.bounding_box)
		end

	intersects_any_field: BOOLEAN
			-- Does element bounding box intersect any
			-- field of view?
		local
			list: like all_fields
			mbb: like bounding_box
		do
			list := all_fields
			mbb := bounding_box
			from
				list.start
			until
				list.after or Result
			loop
				Result := list.item.bounding_box.intersects (mbb)
				list.forth
			end
		end

	intersects_more_than_one_field: BOOLEAN
			-- Does element bounding box intersect more than one
			-- field of view?
		local
			list: like all_fields
			mbb: like bounding_box
			l_count: INTEGER
		do
			list := all_fields
			mbb := bounding_box
			from
				list.start
			until
				list.after or l_count > 1
			loop
				if list.item.bounding_box.intersects (mbb) then
					l_count := l_count + 1
				end
				list.forth
			end
			Result := l_count > 1
		end

	intersection_with_fields: LIST [RECTANGLE]
			-- Rectangles that represent intersection of element
			-- bounding box with fields of view
		local
			list: like all_fields
			mbb: like bounding_box
			fbb, r: RECTANGLE
			f: RETICLE_FIELD
			lx, ly, hx, hy: DOUBLE
		do
			list := all_fields
			create {ARRAYED_LIST [RECTANGLE]} Result.make (list.count)
			mbb := bounding_box
			from
				list.start
			until
				list.after
			loop
				f := list.item
				create fbb.make_copy (f.bounding_box)
				if fbb.intersects (mbb) then
					r := fbb.intersection_with (mbb)
					lx := grid.rounded_edge (r.low_x)
					ly := grid.rounded_edge (r.low_y)
					hx := grid.rounded_edge (r.high_x)
					hy := grid.rounded_edge (r.high_y)
					r.make_by_corners (lx, ly, hx, hy)
					Result.extend (r)
					
				end
				list.forth
			end
		end

	intersecting_fields: LIST [RETICLE_FIELD]
			-- Fields that element bounding box intersects
		local
			list: like all_fields
			mbb: like bounding_box
			f: RETICLE_FIELD
		do
			list := all_fields
			create {ARRAYED_LIST [RETICLE_FIELD]} Result.make (list.count)
			mbb := bounding_box
			from
				list.start
			until
				list.after
			loop
				f := list.item
				if f.bounding_box.intersects (mbb) then
					Result.extend (f)
				end
				list.forth
			end
		end

	enclosed_by_one_field: BOOLEAN
			-- Does element bounding box completely enclosed by
			-- exactly one field of view (edges may touch field
			-- edges)?
		local
			list: like all_fields
			mbb: like bounding_box
		do
			list := all_fields
			mbb := bounding_box
			from
				list.start
			until
				list.after or Result
			loop
				if list.item.bounding_box.encloses (mbb) then
					Result := True
				end
				list.forth
			end
		end

	partially_outside_fields: BOOLEAN
			-- Is element bounding box partially outside fields
			-- of view (intersects at least one field but is not
			-- completely enclosed by any field)?
		do
			Result := intersects_any_field and not enclosed_by_one_field
		end

	is_too_close_to_field_edge: BOOLEAN
			-- Is at least one edge of element too close to
			-- corresponding edge of its associated field?
		local
			dist, l, r, t, b: DOUBLE
		do
			if has_field_edge_restriction and then intersects_any_field then
				if needs_min_sep_white_space then
					dist := needed_white_space
					if dist > 0.0 then
						l := left_to_field_edge
						r := right_to_field_edge
						t := top_to_field_edge
						b := bottom_to_field_edge
						Result := (l >= 0.0 and l < dist) or (r >= 0.0 and r < dist) or (t >= 0.0 and t < dist) or (b >= 0.0 and b < dist)
					end
				else
					dist := minimum_separation
					l := left_to_field_edge
					r := right_to_field_edge
					t := top_to_field_edge
					b := bottom_to_field_edge
					Result := (l > 0.0 and l < dist) or (r > 0.0 and r < dist) or (t > 0.0 and t < dist) or (b > 0.0 and b < dist)
				end
			end
		end

	is_edge_too_close_to_field_edge (dist: DOUBLE): BOOLEAN
			-- Is edge of element that is `dist' away from
			-- corresponding edge of its associated field
			-- too close?
		local
			required_dist: DOUBLE
		do
			if has_field_edge_restriction and then intersects_any_field then
				if needs_min_sep_white_space then
					required_dist := needed_white_space
					if required_dist > 0.0 then
						Result := dist >= 0.0 and dist < required_dist
					end
				else
					required_dist := minimum_separation
					Result := dist > 0.0 and dist < required_dist
				end
			end
		end

	distance_in_x_from (an_element: like Current): DOUBLE
			-- Gridded distance of element from
			-- `an_element' in X dimension.  Result is
			-- zero if elements overlap in X
		local
			l, r: DOUBLE
			l2, r2: DOUBLE
		do
			l := grid.rounded_edge (left_x)
			r := grid.rounded_edge (right_x)
			l2 := grid.rounded_edge (an_element.left_x)
			r2 := grid.rounded_edge (an_element.right_x)
			if l >= r2 then
				Result := l - r2
			elseif r <= l2 then
				Result := l2 - r
			else
				Result := 0.0
			end
			Result := grid.rounded_edge (Result)
		end

	distance_in_y_from (an_element: like Current): DOUBLE
			-- Gridded distance of element from
			-- `an_element' in Y dimension.  Result is
			-- zero if elements overlap in Y
		local
			t, b: DOUBLE
			t2, b2: DOUBLE
		do
			t := grid.rounded_edge (top_y)
			b := grid.rounded_edge (bottom_y)
			t2 := grid.rounded_edge (an_element.top_y)
			b2 := grid.rounded_edge (an_element.bottom_y)
			if b >= t2 then
				Result := b - t2
			elseif t <= b2 then
				Result := b2 - t
			else
				Result := 0.0
			end
			Result := grid.rounded_edge (Result)
		end

	abuts_left_field_edge: BOOLEAN
			-- Does left side of `Current' abut the left edge 
			-- of its associated field?
		do
			Result := left_to_field_edge = 0.0
		end

	abuts_right_field_edge: BOOLEAN
			-- Does right side of `Current' abut the right edge 
			-- of its associated field?
		do
			Result := right_to_field_edge = 0.0
		end

	abuts_bottom_field_edge: BOOLEAN
			-- Does bottom side of `Current' abut the bottom edge 
			-- of its associated field?
		do
			Result := bottom_to_field_edge = 0.0
		end

	abuts_top_field_edge: BOOLEAN
			-- Does top side of `Current' abut the top edge 
			-- of its associated field?
		do
			Result := top_to_field_edge = 0.0
		end

	left_to_field_edge: DOUBLE
			-- Gridded distance from left edge of element to left
			-- edge of its associated field (positive if
			-- element edge is to right of field edge)
		do
			Result := grid.rounded_edge (field_relative_left_x - field.field_relative_left_x)
		end

	right_to_field_edge: DOUBLE
			-- Gridded distance from right edge of element to right
			-- edge of its associated field (positive if
			-- element edge is to left of field edge)
		do
			Result := grid.rounded_edge (field.field_relative_right_x - field_relative_right_x)
		end

	top_to_field_edge: DOUBLE
			-- Gridded distance from top edge of element to top
			-- edge of its associated field (positive if
			-- element edge is below field edge)
		do
			Result := grid.rounded_edge (field.field_relative_top_y - field_relative_top_y)
		end

	bottom_to_field_edge: DOUBLE
			-- Gridded distance from bottom edge of
			-- element to bottom edge of its associated
			-- field (positive if element edge is above
			-- field edge)
		do
			Result := grid.rounded_edge (field_relative_bottom_y - field.field_relative_bottom_y)
		end

	field_containing_top_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- top edge of element bounding box or Void if
			-- none
		local
			mbb: like bounding_box
		do
			mbb := bounding_box
			Result := field_containing_edge (mbb.ul, mbb.ur)
		end

	field_containing_bottom_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- bottom edge of element bounding box or Void if
			-- none
		local
			mbb: like bounding_box
		do
			mbb := bounding_box
			Result := field_containing_edge (mbb.ll, mbb.lr)
		end

	field_containing_left_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- left edge of element bounding box or Void if
			-- none
		local
			mbb: like bounding_box
		do
			mbb := bounding_box
			Result := field_containing_edge (mbb.ll, mbb.ul)
		end

	field_containing_right_edge: RETICLE_FIELD
			-- Field that truly contains at least part of
			-- right edge of element bounding box or Void if
			-- none
		local
			mbb: like bounding_box
		do
			mbb := bounding_box
			Result := field_containing_edge (mbb.lr, mbb.ur)
		end

	x_outside_all_fields (a_value: DOUBLE): BOOLEAN
			-- Is `a_value' outside all fields of view in X
			-- dimension or on field edge?
		local
			list: like all_fields
			p: POINT
			mbb: CONST_RECTANGLE
		do
			Result := True
			list := all_fields
			from
				list.start
			until
				list.after or not Result
			loop
				mbb := list.item.bounding_box
				create p.make_xy (a_value, mbb.center_y)
				Result := not mbb.truly_contains (p)
				list.forth
			end
		end

	y_outside_all_fields (a_value: DOUBLE): BOOLEAN
			-- Is `a_value' outside all fields of view in Y
			-- dimension or on field edge?
		local
			list: like all_fields
			p: POINT
			mbb: CONST_RECTANGLE
		do
			Result := True
			list := all_fields
			from
				list.start
			until
				list.after or not Result
			loop
				mbb := list.item.bounding_box
				create p.make_xy (mbb.center_x, a_value)
				Result := not mbb.truly_contains (p)
				list.forth
			end
		end

	closest_field_edge_above (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field lower edge which is
			-- closest to and above or equal to `a_value'
		local
			list: like all_fields
			f: RETICLE_FIELD
			edge: DOUBLE
		do
			Result := Infinity
			list := all_fields
			from
				list.start
			until
				list.after
			loop
				f := list.item
				edge := grid.rounded_edge (f.bottom_y)
				if edge >= a_value and edge < Result then
					Result := edge
				end
				list.forth
			end
		end

	closest_field_edge_below (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field upper edge which is closest to
			-- and below or equal to `a_value'
		local
			list: like all_fields
			f: RETICLE_FIELD
			edge: DOUBLE
		do
			Result := -Infinity
			list := all_fields
			from
				list.start
			until
				list.after
			loop
				f := list.item
				edge := grid.rounded_edge (f.top_y)
				if edge <= a_value and edge > Result then
					Result := edge
				end
				list.forth
			end
		end

	closest_field_edge_to_left (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field right edge which is
			-- closest to and left of or equal to
			-- `a_value'
		local
			list: like all_fields
			f: RETICLE_FIELD
			edge: DOUBLE
		do
			Result := -Infinity
			list := all_fields
			from
				list.start
			until
				list.after
			loop
				f := list.item
				edge := grid.rounded_edge (f.right_x)
				if edge <= a_value and edge > Result then
					Result := edge
				end
				list.forth
			end
		end

	closest_field_edge_to_right (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field left edge which is
			-- closest to and right of or equal to
			-- `a_value'
		local
			list: like all_fields
			f: RETICLE_FIELD
			edge: DOUBLE
		do
			Result := Infinity
			list := all_fields
			from
				list.start
			until
				list.after
			loop
				f := list.item
				edge := grid.rounded_edge (f.left_x)
				if edge >= a_value and edge < Result then
					Result := edge
				end
				list.forth
			end
		end

	all_fields: LIST [RETICLE_FIELD]
			-- All fields of view with which element could be
			-- associated
		do
		end

	distance_from_associated_field_to_field (a_field_name: STRING): CONST_POINT
			-- Distance from center of field with which
			-- `Current' is associated to center of field
			-- named `a_field_name'
		require
			a_field_name_not_void: a_field_name /= Void
		local
			src_field, dest_field: like field
		do
			src_field := field
			dest_field := field_with_name (a_field_name)
			create Result.make_xy (dest_field.center_x - src_field.center_x, dest_field.center_y - src_field.center_y)
		end

	field_with_name (a_field_name: STRING): RETICLE_FIELD
			-- Field with `a_field_name'
		require
			a_field_name_not_void: a_field_name /= Void
		do
		end

	die: PACKING_DIES_MROW
			-- Die for element that is a die
		require
			is_die: is_die
		do
			-- Inapplicable
		ensure
			result_not_void: Result /= Void
		end

	die_location: PACKING_DIE_LOCATIONS_MROW
			-- Die location for element that is a die
		require
			is_die: is_die
		do
			-- Inapplicable
		ensure
			result_not_void: Result /= Void
		end

	cut_location: PACKING_CUT_LOCATIONS_MROW
			-- Cut location for element that is a cut
		require
			is_cut: is_cut
		do
		ensure
			result_not_void: Result /= Void
		end

	project: PACKING_PROJECTS_MROW
			-- Project for element that is a project
		require
			is_project: is_project
		do
			Result := die.project
		ensure
			result_not_void: Result /= Void
		end

	partition: PACKING_PARTITIONS_MROW
			-- Partition for element that is a partition
		require
			is_partition: is_partition
		do
		ensure
			result_not_void: Result /= Void
		end

	packaging_max_die_size_fixed_rotation: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes for packagings which specify a
			-- fixed rotation of die in package.  Void
			-- indicates no maximum
		require
			is_die: is_die
		do
			-- In applicable
		end

	packaging_max_die_size_free_rotation: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes for packagings which do not
			-- specify a rotation of die in package.  Void
			-- indicates no maximum
		require
			is_die: is_die
		do
			-- In applicable
		end

	user_max_die_size: CONST_POINT
			-- Maximum die size, as constrained by
			-- user-specified limits.  Void indicates no maximum
		require
			is_die: is_die
		do
			-- Inapplicable
		end

	is_oversize_die_always_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed unconditionally?
		require
			is_die: is_die
		do
			-- Inapplicable
		end

	is_oversize_die_possibly_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed but only under specific conditions?
		require
			is_die: is_die
		do
			-- Inapplicable
		end

	oversize_die_inclusion_conditions: STRING
			-- Description of conditions under which 
			-- die that is bigger than `max_die_size'
			-- allowed
		require
			is_die: is_die
			oversize_die_possibly_allowed: is_oversize_die_possibly_allowed
		do
			-- Inapplicable
		end

	has_sub_elements: BOOLEAN
			-- Is element composed of more than one sub-element,
			-- each of which has its own size and center?
		do
			Result := False
		end

	sub_elements: LIST [like anchored_model_sub_element]
			-- Sub-elements of `Current'
		do
		end

	last_new_element: RETICLE_MODEL_ELEMENT
			-- Last new element created by call to `duplicate'
		do
		end

	all_copies: LIST [RETICLE_MODEL_ELEMENT]
			-- All copies of this element

	virtual_scribe_cut_die: CONST_RECTANGLE
			-- Last computed virtual cut die for a
			-- project, using centers of surrounding
			-- scribes for edges of cut die

	actual_scribe_cut_die: CONST_RECTANGLE
			-- Last computed actual cut die for a
			-- project, using centers of surrounding
			-- scribes for edges of cut die
		do
			Result := actual_cut_die_for (virtual_scribe_cut_die)
		end

	virtual_oriented_scribe_cut_die: CONST_RECTANGLE
			-- Last computed virtual cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using centers of
			-- surrounding scribes for edges of cut die
		do
			if virtual_scribe_cut_die /= Void then
				create Result.make_by_size_and_center (virtual_scribe_cut_die_size_x, virtual_scribe_cut_die_size_y, virtual_scribe_cut_die.center_x, virtual_scribe_cut_die.center_y)
			else
				Result := Void
			end
		end

	actual_oriented_scribe_cut_die: CONST_RECTANGLE
			-- Last computed actual cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using centers of
			-- surrounding scribes for edges of cut die
		do
			Result := actual_cut_die_for (virtual_oriented_scribe_cut_die)
		end

	virtual_scribe_cut_die_size_x: DOUBLE
			-- Size of virtual cut die in X dimension,
			-- using centers of surrounding scribes for
			-- edges of cut die
		do
			if is_rotated_sideways then
				Result := virtual_scribe_cut_die.size_y
			else
				Result := virtual_scribe_cut_die.size_x
			end
		end

	virtual_scribe_cut_die_size_y: DOUBLE
			-- Size of virtual cut die in Y dimension,
			-- using centers of surrounding scribes for
			-- edges of cut die
		do
			if is_rotated_sideways then
				Result := virtual_scribe_cut_die.size_x
			else
				Result := virtual_scribe_cut_die.size_y
			end
		end

	actual_scribe_cut_die_size_x: DOUBLE
			-- Size of actual cut die in X dimension,
			-- using centers of surrounding scribes for
			-- edges of cut die
		do
			if is_rotated_sideways then
				Result := actual_scribe_cut_die.size_y
			else
				Result := actual_scribe_cut_die.size_x
			end
		end

	actual_scribe_cut_die_size_y: DOUBLE
			-- Size of actual cut die in Y dimension,
			-- using centers of surrounding scribes for
			-- edges of cut die
		do
			if is_rotated_sideways then
				Result := actual_scribe_cut_die.size_x
			else
				Result := actual_scribe_cut_die.size_y
			end
		end

	virtual_edge_cut_die: CONST_RECTANGLE
			-- Last computed virtual cut die for a
			-- project, using closest edges of surrounding
			-- scribes for edges of cut die

	virtual_oriented_cut_cut_die: CONST_RECTANGLE
			-- Last computed virtual cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using cuts for edges
			-- of cut die
		do
			if virtual_cut_cut_die /= Void then
				create Result.make_by_size_and_center (virtual_cut_cut_die_size_x, virtual_cut_cut_die_size_y, virtual_cut_cut_die.center_x, virtual_cut_cut_die.center_y)
			else
				Result := Void
			end
		end

	actual_oriented_cut_cut_die: CONST_RECTANGLE
			-- Last computed actual cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using cuts for edges
			-- of cut die
		do
			Result := actual_cut_die_for (virtual_oriented_cut_cut_die)
		end

	virtual_cut_cut_die: CONST_RECTANGLE
			-- Last computed virtual cut die for a
			-- project, using cuts for edges of cut die

	actual_cut_cut_die: CONST_RECTANGLE
			-- Last computed actual cut die for a
			-- project, using cuts for edges of cut die
		do
			Result := actual_cut_die_for (virtual_cut_cut_die)
		end

	virtual_cut_cut_die_size_x: DOUBLE
			-- Size of virtual cut die in X dimension,
			-- using centers of surrounding cuts for edges
			-- of cut die
		do
			if is_rotated_sideways then
				Result := virtual_cut_cut_die.size_y
			else
				Result := virtual_cut_cut_die.size_x
			end
		end

	virtual_cut_cut_die_size_y: DOUBLE
			-- Size of virtual cut die in Y dimension,
			-- using centers of surrounding cut for edges
			-- of cut die
		do
			if is_rotated_sideways then
				Result := virtual_cut_cut_die.size_x
			else
				Result := virtual_cut_cut_die.size_y
			end
		end

	actual_cut_cut_die_size_x: DOUBLE
			-- Size of actual cut die in X dimension,
			-- using centers of surrounding cuts for edges
			-- of cut die
		do
			if is_rotated_sideways then
				Result := actual_cut_cut_die.size_y
			else
				Result := actual_cut_cut_die.size_x
			end
		end

	actual_cut_cut_die_size_y: DOUBLE
			-- Size of actual cut die in Y dimension,
			-- using centers of surrounding cut for edges
			-- of cut die
		do
			if is_rotated_sideways then
				Result := actual_cut_cut_die.size_x
			else
				Result := actual_cut_cut_die.size_y
			end
		end

	virtual_oriented_field_relative_cut_cut_die: CONST_RECTANGLE
			-- Last computed virtual cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using surrounding cuts for
			-- for edges of cut die and with location relative
			-- to center of field
		do
			if virtual_cut_cut_die /= Void then
				create Result.make_by_size_and_center (virtual_cut_cut_die_size_x, virtual_cut_cut_die_size_y, virtual_cut_cut_die.center_x - field.center_x, virtual_cut_cut_die.center_y - field.center_y)
			else
				Result := Void
			end
		end

	virtual_field_relative_cut_cut_die: CONST_RECTANGLE
			-- Last computed virtual cut die for a project, using
			-- surrounding cuts for for edges of cut die
			-- and with location relative to center of
			-- field
		do
			if virtual_cut_cut_die /= Void then
				create Result.make_by_size_and_center (virtual_cut_cut_die.size_x, virtual_cut_cut_die.size_y, virtual_cut_cut_die.center_x - field.center_x, virtual_cut_cut_die.center_y - field.center_y)
			else
				Result := Void
			end
		end

	unoriented_model_relative_rectangle (a_rectangle: CONST_RECTANGLE): RECTANGLE
			-- Transformation of oriented (rotated if die
			-- is rotated) field-relative rectangle `a_rectangle' 
			-- to an unoriented model-relative rectangle
		
		local
			l_size_x, l_size_y: DOUBLE
		do
			if a_rectangle /= Void then
				if is_rotated_sideways then
					l_size_x := a_rectangle.size_y
					l_size_y := a_rectangle.size_x
				else
					l_size_x := a_rectangle.size_x
					l_size_y := a_rectangle.size_y
				end
				create Result.make_by_size_and_center (l_size_x, l_size_y, a_rectangle.center_x + field.center_x, a_rectangle.center_y + field.center_y)
			else
				Result := Void
			end
		end

	anchored_model_element: RETICLE_MODEL_ELEMENT
			-- Type of model element of which `Current' is
			-- one part (inapplicable)
		do
		end

	anchored_model_sub_element: RETICLE_MODEL_SUB_ELEMENT
			-- Type of sub-elements if model element has
			-- sub-elements
		do
		end

	is_reworkable: BOOLEAN
			-- Is element currently reworkable?
		do
			Result := False
		end

	moved_center_fits_in_database (a_distance: CONST_POINT): BOOLEAN
			-- Does model location moved by `a_distance' fit 
			-- in database?
		require
			a_distance_not_void: a_distance /= Void
		local
			p: POINT
		do
			create p.make_xy (center_x, center_y)
			p := p + a_distance
		end

	has_required_rotation: BOOLEAN
			-- Must object have specific rotation?
		do
			Result := False
		end

	required_rotation: INTEGER
			-- Rotation object is required to have in degrees clockwise
		do
			Result := 0
		end

	area: DOUBLE
			-- Area of element, including any overhead
		do
			Result := size_x * size_y
		end

	project_area: DOUBLE
			-- Area of element, not including any added
			-- overhead for a project
		do
			Result := size_x * size_y
		end

feature -- Modification

	set_all_copies (a_list: like all_copies)
		require
			a_list_not_void: a_list /= Void
		do
			all_copies := a_list
		ensure
			all_copies_set: all_copies = a_list
		end

	set_virtual_scribe_cut_die (a_cut_die: like virtual_scribe_cut_die)
			-- Set `virtual_scribe_cut_die' to `a_cut_die'
		do
			virtual_scribe_cut_die := a_cut_die
		end

	set_virtual_edge_cut_die (a_cut_die: like virtual_edge_cut_die)
			-- Set `virtual_edge_cut_die' to `a_cut_die'
		do
			virtual_edge_cut_die := a_cut_die
		end

	set_virtual_cut_cut_die (a_cut_die: like virtual_cut_cut_die)
			-- Set `virtual_cut_cut_die' to `a_cut_die'
		do
			virtual_cut_cut_die := a_cut_die
		end

feature -- Operations

	uncache_field
			-- Remove `field' from cache, if present
		do
			cached_field := Void
		end

	duplicate
			-- Leave new element in `last_new_element'
		require
			is_abstract_die: is_abstract_die
		deferred
		end

	move_left_edge (a_distance: DOUBLE)
			-- Move left edge of `Current' by `a_distance'
		require
			is_abstract_die: is_abstract_die
		deferred
		end

	move_right_edge (a_distance: DOUBLE)
			-- Move right edge of `Current' by `a_distance'
		require
			is_abstract_die: is_abstract_die
		deferred
		end

	move_top_edge (a_distance: DOUBLE)
			-- Move top edge of `Current' by `a_distance'
		require
			is_abstract_die: is_abstract_die
		deferred
		end

	move_bottom_edge (a_distance: DOUBLE)
			-- Move bottom edge of `Current' by `a_distance'
		require
			is_abstract_die: is_abstract_die
		deferred
		end

	save_initial_center
			-- Save initial center in history
			-- list to support undo/redo of payload size
			-- changes
		require
			is_abstract_die: is_abstract_die
		do
		end

	save_current_center
			-- Save current center in history
			-- list to support undo/redo of payload size
			-- changes
		require
			is_abstract_die: is_abstract_die
		do
		end

	adjust_center
			-- Adjust center of element, if necessary,
			-- so that edges remain on grid
		require
			is_abstract_die: is_abstract_die
		do
		end

	restore_previous_center
			-- Restore previous center of element
			-- as part of undo of payload size
			-- change operation
		require
			is_abstract_die: is_abstract_die
		do
		end

	restore_next_center
			-- Restore next center of element
			-- as part of redo of payload size
			-- change operation
		require
			is_abstract_die: is_abstract_die
		do
		end

	clear_historical_centers
			-- Clear history list with centers when
			-- payload size changes can no longer be
			-- undone/redone
		require
			is_abstract_die: is_abstract_die
		do
		end

feature {NONE} -- Implementation

	needed_white_space: DOUBLE
			-- Gridded amount of extra white space needed between
			-- object that is a project (contains circuitry) and
			-- other objects or field of view edge
		local
			val: DOUBLE
		do
			val := minimum_separation - uniform_white_space_border
			if not grid.is_edge_on_grid (val) and then grid.is_center_on_grid (val) then
				-- Uniform white space border may be on half
				-- grid, so leave `va' on half grid if so
				Result := grid.rounded_center (val)
			else
				Result := grid.rounded_edge (val)
			end
		end

	field_containing_edge (p1, p2: CONST_POINT): RETICLE_FIELD
			-- Field that truly contains some portion of
			-- the line segment from `p1' to `p2'
			-- (assumed to be horizontal or vertical segment)
			-- or Void if none
		local
			list: like all_fields
			f: RETICLE_FIELD
			mbb: CONST_RECTANGLE
			x1, x2, y1, y2: DOUBLE
			low_x, low_y, high_x, high_y: DOUBLE
			is_horiz: BOOLEAN
		do
			x1 := grid.rounded_edge (p1.x)
			y1 := grid.rounded_edge (p1.y)
			x2 := grid.rounded_edge (p2.x)
			y2 := grid.rounded_edge (p2.y)
			is_horiz := y1 = y2
			list := all_fields
			from
				list.start
			until
				list.after or Result /= Void
			loop
				f := list.item
				mbb := f.bounding_box
				low_x := grid.rounded_edge (mbb.low_x)
				low_y := grid.rounded_edge (mbb.low_y)
				high_x := grid.rounded_edge (mbb.high_x)
				high_y := grid.rounded_edge (mbb.high_y)
				if (is_horiz and y1 > low_y and y1 < high_y and x1 < high_x and x2 > low_x) or
				   (not is_horiz  and x1 > low_x and x1 < high_x and y1 < high_y and y2 > low_y) then
					Result := f
				end
				list.forth
			end
		end

	actual_cut_die_for (d: CONST_RECTANGLE): CONST_RECTANGLE
			-- Corresponding actual cut die for virtual
			-- cut die `d'
		local
			kerf, sx, sy: DOUBLE
		do
			if d /= Void then
				-- If size comes out negative, use 0.0
				-- to avoid contract violation
				sx := grid.rounded_size (d.size_x - kerf).max (0.0)
				sy := grid.rounded_size (d.size_y - kerf).max (0.0)
				create Result.make_by_size_and_center (sx, sy, d.center_x, d.center_y)
			end
		end

	fetch_field
			-- Fetch field with which die is associated, based
			-- on its name, and put in `cached_field'
		do
		end

	cached_field: RETICLE_FIELD
			-- Field with which die is associated, cached
			-- from lookup
	
end
