
note
	
	description: "Label not for a specific field of view in reticle model"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_NON_FIELD_LABEL
inherit
	RETICLE_VIRTUAL_ELEMENT
		redefine
			field, is_non_field_label
		end

create
	make

feature {NONE} -- Creation

	make (a_text: STRING a_field: RETICLE_FIELD a_size, a_center: CONST_POINT a_model: like anchored_model)
		require
			a_text_not_void: a_text /= Void
			a_field_not_void: a_field /= Void
			a_size_not_void: a_size /= Void
			a_center_not_void: a_center /= Void
			a_size_x_positive: a_size.x > 0.0
			a_size_y_positive: a_size.y > 0.0
			a_model_not_void: a_model /= Void
		do
			initialize (a_size, a_center, a_model)
			identification := a_text
			field := a_field
		end

feature -- Properties

	identification: STRING
			-- Identifying name of element

	field: RETICLE_FIELD
			-- Field of view with which element is associated.
			-- It isn't really associated with this field, but
			-- the model assumes that every element has an
			-- associated field of view, so this may avoid
			-- breaking some code

	name: STRING
			-- Name of field
		do
			Result := field.name
		end

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := center_x - field.center_x
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := center_y - field.center_y
		end

	is_non_field_label: BOOLEAN = True
			-- Is element a label that is not for a specific
			-- field of view?
	
feature {NONE} -- Implementation

	associated_field_name: STRING
		do
			Result := name
		end

end
