
note
	
	description: "A rectangle that covers part of the area of the reticle and %
		%which contains only customer projects of the same lot split. %
		%All dies in a partition are merged into one big die."
	author: "David Hollenberg"

class RETICLE_PARTITION
inherit
	RETICLE_ABSTRACT_DIE
		redefine
			is_partition,
			partition,
			identification_for_message,
			has_constraining_wafer_etest_id,
			constraining_wafer_etest_id,
			constraining_wafer_etest_name,
			constraining_wafer_etest_names_row,
			has_field_edge_restriction,
			duplicate,
			is_resizable
		end

create
	make

feature {NONE} -- Creation

	make (a_partition: like partition a_model: ANY)
		require
			a_partition_not_void: a_partition /= Void
		do
			partition := a_partition
			create {ARRAYED_LIST [like Current]} all_copies.make (1)
			all_copies.extend (Current)
			partition.detach_actions.extend (agent update_partition)
			create temp_point.make_xy (0, 0)
		end

feature -- Properties

	is_partition: BOOLEAN = True
			-- Is element a partition?

	partition: PACKING_PARTITIONS_MROW
			-- Partition represented by `Current'
	
	identification: STRING = ""
			-- Identifying name of element

	identification_for_message: STRING = "PARTITION"
			-- Identifying name of element for displaying
			-- error and warning messages

	is_selectable: BOOLEAN = True
			-- May object be selected?

	has_constraining_wafer_etest_id: BOOLEAN = True
			-- Does object have constrain other elements
			-- contained within it to have a particular
			-- wafer etest ID(s)?

	constraining_wafer_etest_id: STRING
			-- Wafer etest ID that elements contained
			-- in `Current' are required to have if 
			-- they have a wafer etest ID
		do
			Result := partition.wafer_etest_id
		end

	constraining_wafer_etest_name: STRING
			-- Wafer etest ID that elements contained
			-- in `Current' are required to have if 
			-- they have a wafer etest ID
		do
			Result := partition.wafer_etest_name
		end

	constraining_wafer_etest_names_row: WAFER_ETEST_NAMES_ROW
			-- Wafer etest names row for `Current'
		do
			Result := partition.etest_name
		end

	rotation: INTEGER = 0
			-- Rotation of object in degrees clockwise

	is_rotated_sideways: BOOLEAN = False
			-- Is object rotated by 90 degrees clockwise or
			-- 90 degrees counterclockwise?

	is_fixed_in_x_internal: BOOLEAN = False
			-- Is object temporarily fixed in the X dimension
			-- so that it cannot move in that dimension?

	is_fixed_in_y_internal: BOOLEAN = False
			-- Is object temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension?

	is_movable_in_x_internal: BOOLEAN
			-- May object currently be moved in the X dimension?
		do
			Result := partition.is_movable_in_x
		end

	is_movable_in_y_internal: BOOLEAN
			-- May object currently be moved in the Y dimension?
		do
			Result := partition.is_movable_in_y
		end

	is_rotatable_internal: BOOLEAN = False
			-- May rotation of object currently be changed?

	is_duplicatable_internal: BOOLEAN = True
			-- May object currently be duplicated?

	is_deletable_internal: BOOLEAN
			-- May object currently be deleted?
		do
			Result := partition.is_deletable
		end

	can_be_fixed_in_x: BOOLEAN = False
			-- Can object be temporarily fixed in the X dimension
			-- so that it cannot move in that dimension?

	can_be_fixed_in_y: BOOLEAN = False
			-- Can object be temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension?

	is_stationary_in_x: BOOLEAN = False
			-- Can object never be moved in the X dimension?

	is_stationary_in_y: BOOLEAN = False
			-- Can object never be moved in the Y dimension?

	is_stationary_orientation: BOOLEAN = True
			-- Can orientation of object never be changed?

	is_never_duplicatable: BOOLEAN = False
			-- Can object never be duplicated?

	is_never_deletable: BOOLEAN = False
			-- Can object never be deleted?

	size_x: DOUBLE
			-- Size of minimum bounding box in X dimension
			-- of reticle field of view
		do
			Result := partition.partition_size_x
		end

	size_y: DOUBLE
			-- Size of minimum bounding box in Y dimension
			-- of reticle field of view
		do
			Result := partition.partition_size_y
		end

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := partition.location_x
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		do
			Result := partition.location_y
		end

	has_field_edge_restriction: BOOLEAN = False
			-- Does element have a restriction on how close it
			-- can be to edge of field?  (No, because
			-- handled by other placement restrictions)

	is_resizable: BOOLEAN = True
			-- Can `Current' be resized?

feature -- Modification

	set_is_fixed_in_x (b: BOOLEAN)
			-- Set `is_fixed_in_x' to `b'
		do
			-- Inapplicable
		end

	set_is_fixed_in_y (b: BOOLEAN)
			-- Set `is_fixed_in_y' to `b'
		do
			-- Inapplicable
		end

	set_is_movable_in_x (b: BOOLEAN)
			-- Set `is_movable_in_x' to `b'
		do
			set_is_movable_in_x_internal (b)
		end

	set_is_movable_in_y (b: BOOLEAN)
			-- Set `is_movable_in_y' to `b'
		do
			set_is_movable_in_y_internal (b)
		end

	set_is_rotatable (b: BOOLEAN)
			-- Set `is_rotatable' to `b'
		do
			-- Inapplicable
		end

	set_is_duplicatable (b: BOOLEAN)
			-- Set `is_duplicatable' to `b'
		do
			-- Inapplicable
		end

	set_is_deletable (b: BOOLEAN)
			-- Set `is_deletable' to `b'
		do
			set_is_deletable_internal (b)
		end

	set_rotation (n: INTEGER)
			-- Set rotation of `Current' to `n' degrees clockwise
		do
			-- Inapplicable
		end

	toggle_rotation
			-- Toggle `is_rotated'
		do
			-- Inapplicable
		end

	rotate_90_clockwise
			-- Rotate `Current' 90 degrees clockwise
		do
			-- Inapplicable
		end

	rotate_90_counterclockwise
			-- Rotate `Current' 90 degrees counterclockwise
		do
			-- Inapplicable
		end

	move_location_by (a_distance: CONST_POINT)
			-- Move center of `Current' by `a_distance'
		do
			partition.move_by (a_distance)
		end

	duplicate
			-- we create a new partition with a single location to
			-- ensure that each partition has only one location
		do
		end

	set_size_x (val: DOUBLE)
			-- Set `size_x' to `val'
		do
			partition.set_partition_size_x (val)
		end

	set_size_y (val: DOUBLE)
			-- Set `size_y' to `val'
		do
			partition.set_partition_size_y (val)
		end

	set_field_relative_center_x (val: DOUBLE)
			-- Set `field_relative_center_x' to `val'
		do
			partition.set_location_x (val)
		end

	set_field_relative_center_y (val: DOUBLE)
			-- Set `field_relative_center_y' to `val'
		do
			partition.set_location_y (val)
		end

	set_associated_field_name (a_name: STRING)
			-- Set name of associated field to `a_name'
		do
			partition.set_field_name (a_name)
		end

feature {NONE} -- Implementation

	set_is_movable_in_x_internal (b: BOOLEAN)
			-- Set `is_movable_in_x_internal' to `b'
		do
			partition.set_is_movable_in_x (b)
		end

	set_is_movable_in_y_internal (b: BOOLEAN)
			-- Set `is_movable_in_y_internal' to `b'
		do
			partition.set_is_movable_in_y (b)
		end

	set_is_deletable_internal (b: BOOLEAN)
			-- Set `is_deletable_internal' to `b'
		do
			partition.set_is_deletable (b)
		end

	y_coordinate_translated_to_field (a_field: RETICLE_FIELD): DOUBLE
			-- Y coordinate of field-relative center
			-- translated from its value for current associated
			-- field to the value it would have if associated
			-- field was changed to `a_field'
		require
			a_field_not_void: a_field /= Void
		local
			old_field: like field
		do
			old_field := field
			Result := partition.location_y + field.center_y - a_field.center_y
		end

	update_partition (an_original, a_replacement: like partition)
			-- If `partition' equals `an_original', replace it
			-- with `a_replacement'
		do
			if partition = an_original then
				partition := a_replacement
			end
		end

	associated_field_name: STRING
		do
			Result := partition.field_name
		end
	
invariant
	partition_not_void: partition /= Void

end
