
note
	
	description: "Context of reticle view, which can be saved and restored"
	author: "David Hollenberg"
	date: "July 15, 2006"

class RETICLE_VIEW_CONTEXT

feature -- Properties

	elements: LIST [RETICLE_VIEW_ELEMENT]
			-- Elements of view, each corresponding to an
			-- element of the model, in stacking order
			-- (first element is at bottom, last element is on
			-- top of all other elements)

	copies_table: HASH_TABLE [RETICLE_VIEW_ELEMENT_LIST, PACKING_DIES_MROW]
			-- Table that maps a packing die to the list
			-- of view elements for that die (each
			-- element is one location of die)

	selected_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of elements that are selected

	displayed_model_area: RECTANGLE
			-- Area of model that is displayed on view

	displayed_area_history_list: TWO_WAY_LIST [RECTANGLE]
			-- History list of model areas that have been displayed

	reticle_change_history_list: TWO_WAY_LIST [RETICLE_CHANGE]
			-- History list of changes that can be undone/redone

	partitions_hidden: BOOLEAN
			-- Are partitions currently hidden in most
			-- views (removed from list of view elements)?

	model_context: RETICLE_MODEL_CONTEXT
			-- Model context corresponding to `Current'

feature -- Modification

	set_elements (a_list: like elements)
			-- Set `elements' to `a_list'
		do
			elements := a_list
		ensure
			elements_set: elements = a_list
		end

	set_copies_table (a_table: like copies_table)
			-- Set `copies_table' to `a_table'
		do
			copies_table := a_table
		ensure
			copies_table_set: copies_table = a_table
		end

	set_selected_elements (a_list: like selected_elements)
			-- Set `selected_elements' to `a_list'
		do
			selected_elements := a_list
		ensure
			selected_elements_set: selected_elements = a_list
		end

	set_displayed_model_area (an_area: like displayed_model_area)
			-- Set `displayed_model_area' to `an_area'
		do
			displayed_model_area := an_area
		ensure
			displayed_model_area_set: displayed_model_area = an_area
		end

	set_displayed_area_history_list (a_list: like displayed_area_history_list)
			-- Set `displayed_area_history_list' to `a_list'
		do
			displayed_area_history_list := a_list
		ensure
			displayed_area_history_list_set: displayed_area_history_list = a_list
		end

	set_reticle_change_history_list (a_list: like reticle_change_history_list)
			-- Set `reticle_change_history_list' to `a_list'
		do
			reticle_change_history_list := a_list
		ensure
			reticle_change_history_list_set: reticle_change_history_list = a_list
		end

	set_partitions_hidden (a_list: like partitions_hidden)
			-- Set `partitions_hidden' to `a_list'
		do
			partitions_hidden := a_list
		ensure
			partitions_hidden_set: partitions_hidden = a_list
		end

	set_model_context (a_context: like model_context)
			-- Set `model_context' to `a_context'
		do
			model_context := a_context
		ensure
			model_context_set: model_context = a_context
		end

end
