
note
	
	description: "A cut pattern in reticle view"
	author: "David Hollenberg"
	date: "July 28, 2006"

class RETICLE_VIEW_CUT_PATTERN

create
	make

feature {NONE} -- Creation

	make (a_pattern: like pattern a_view: RETICLE_VIEW)
		require
			a_pattern_not_void: a_pattern /= Void
			a_view_not_void: a_view /= Void
		do
			pattern := a_pattern
			view := a_view
		end

feature -- Properties

	pattern: RETICLE_CUT_PATTERN
			-- Cut pattern in model

	is_read_only: BOOLEAN
			-- Is pattern read-only (cannot be modified
			-- or deleted)?
		do
			Result := pattern.is_read_only
		end

	cut_pattern_no: INTEGER
			-- Number assigned to cut pattern
		do
			Result := pattern.cut_pattern_no
		end

	cut_pattern_xform_type: STRING
			-- Type of transform cut pattern performs
		do
			Result := pattern.cut_pattern_xform_type
		end

	cut_pattern_no_parent: INTEGER
			-- Number of parent cut pattern, if not null
		do
			Result := pattern.cut_pattern_no_parent
		end

	cut_pattern_no_parent_or_minus_one: INTEGER
			-- Number of parent cut pattern, or -1
			-- if null
		do
			if not is_null_cut_pattern_no_parent then
				Result := cut_pattern_no_parent
			else
				Result := -1
			end
		end

	is_null_cut_pattern_no_parent: BOOLEAN
			-- Is parent cut pattern number null?
		do
			Result := pattern.is_null_cut_pattern_no_parent
		end

	production_id: STRING
			-- Production ID cut pattern is for
		do
			Result := pattern.production_id
		end

	cut_patterns_row: PACKING_CUT_PATTERNS_MROW
			-- Row in PACKING_CUT_PATTERNS to which this cut 
			-- pattern corresponds
		do
			Result := pattern.pattern
		end

	source_type: STRING
			-- Type of entity to which cut pattern is applied
		do
			Result := pattern.source_type
		end

	result_type: STRING
			-- Type of entity application of cut
			-- pattern produces
		do
			Result := pattern.result_type
		end

	isolates_sites: BOOLEAN
			-- Does this type of cut pattern
			-- isolate site?
		do
			Result := pattern.isolates_sites
		end

	produces_strips: BOOLEAN
			-- Does this type of cut pattern
			-- produce wafer strips?
		do
			Result := pattern.produces_strips
		end

	applies_to_strips: BOOLEAN
			-- Is this type of cut pattern
			-- applied to wafer strips?
		do
			Result := pattern.applies_to_strips
		end

	presentation_text: STRING
			-- Text that describes current cut pattern
			-- transformation type
		do
			Result := pattern.presentation_text
		end

	field_name: STRING
			-- Name of field to which cut pattern applies
		do
			Result := pattern.field_name
		end

	has_wafer_etest_name: BOOLEAN
			-- Is there a wafer etest name for cut pattern?
		do
			Result := pattern.has_wafer_etest_name
		end

	wafer_etest_name: STRING
			-- Wafer etest name for cut pattern
		do
			Result := pattern.wafer_etest_name
		end

	wafer_etest_id: STRING
			-- Wafer etest ID for cut pattern
		require
			has_wafer_etest_name: has_wafer_etest_name
		do
			Result := pattern.wafer_etest_id
		end

	wafer_etest_id_description: STRING
			-- Description of wafer etest ID for cut pattern
		do
			if has_wafer_etest_name  then
				Result := wafer_etest_id
			else
				Result := "No wafer etest ID yet"
			end
		end

	has_quantity_wafers_needed: BOOLEAN
			-- Is there a number wafers to cut for pattern?
		do
			Result := pattern.has_quantity_wafers_needed
		end

	quantity_wafers_needed: INTEGER
			-- Number of wafers to cut for pattern or -1 if
			-- not yet specified
		do
			Result := pattern.quantity_wafers_needed
		end

	external_cut_distance_spec: CONST_ABSTRACT_EXT_CUT_DIST_SPEC
			-- Specifications for external cuts
		do
			Result := cut_patterns_row
		end

	saw_blade_width: DOUBLE
			-- Width of saw blade to be used for pattern
		do
			Result := pattern.saw_blade_width
		end

	is_completed: BOOLEAN
			-- Is cut pattern completed (not under construction)?
		do
			Result := pattern.is_completed
		end

	has_cuts: BOOLEAN
			-- Are there any cuts for pattern?
		do
			Result := pattern.has_cuts
		end

	has_horizontal_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a horizontal cut at
			-- `a_location'?
		do
			Result := pattern.has_horizontal_cut_at (a_location)
		end

	has_vertical_cut_at (a_location: DOUBLE): BOOLEAN
			-- Does pattern include a vertical cut at
			-- `a_location'?
		do
			Result := pattern.has_vertical_cut_at (a_location)
		end

	has_external_cut_left (fld_left_edge: DOUBLE): BOOLEAN
			-- Does pattern include an external cut
			-- to left of `fld_left_edge'?
		do
			Result := pattern.has_external_cut_left (fld_left_edge)
		end

	has_external_cut_right (fld_right_edge: DOUBLE): BOOLEAN
			-- Does pattern include an external cut
			-- to right of `fld_right_edge'?
		do
			Result := pattern.has_external_cut_right (fld_right_edge)
		end

	has_external_cut_bottom (fld_bottom_edge: DOUBLE): BOOLEAN
			-- Does pattern include an external cut
			-- below `fld_bottom_edge'?
		do
			Result := pattern.has_external_cut_bottom (fld_bottom_edge)
		end

	has_external_cut_top (fld_top_edge: DOUBLE): BOOLEAN
			-- Does pattern include an external cut
			-- above `fld_top_edge'?
		do
			Result := pattern.has_external_cut_top (fld_top_edge)
		end

	horizontal_cuts: LIST [PACKING_CUT_LOCATIONS_ROW]
			-- Horizontal cuts for cut pattern
		do
			Result := pattern.horizontal_cuts
		end

	vertical_cuts: LIST [PACKING_CUT_LOCATIONS_ROW]
			-- Vertical cuts for cut pattern
		do
			Result := pattern.vertical_cuts
		end

	has_isolated_sites: BOOLEAN
			-- Are there any isolated sites for pattern?
		do
			Result := pattern.has_isolated_sites
		end

	isolated_cut_die (an_element: RETICLE_VIEW_ELEMENT): CONST_RECTANGLE
			-- Cut die for isolated element `an_element' in 
			-- cut pattern, determined from its cut die site
		require
			an_element_not_void: an_element /= Void
		do
			Result := pattern.isolated_cut_die (an_element.model_element)
		end

	isolated_location_count: INTEGER
			-- Number of die locations isolated by cut pattern
		do
			Result := pattern.isolated_location_count
		end

	description: STRING
			-- Description of cut pattern for messages to user
		do
			Result := pattern.description
		end

	description_for_selection: STRING
			-- Description of cut pattern when it
			-- is included in a list from which user
			-- selects one item
		do
			Result := pattern.description_for_selection
		end

	status: STRING
			-- Status of cut pattern
		do
			Result := pattern.status
		end

	view: RETICLE_VIEW
			-- View containing cut pattern

	context: RETICLE_VIEW_CONTEXT
			-- Context to be restored when view displays
			-- this cut pattern

feature -- Modification

	set_context (a_context: like context)
			-- Set `context' to `a_context'
		do
			context := a_context
		ensure
			context_set: context = a_context
		end

	set_is_completed (a_value: BOOLEAN)
			-- Set `is_completed' to `a_value'
		do
			pattern.set_is_completed (a_value)
		ensure
			completed_set: is_completed = a_value
		end

	set_cut_pattern_xform_type (a_value: STRING)
			-- Set `cut_pattern_xform_type' to `a_value'
		do
			pattern.set_cut_pattern_xform_type (a_value)
		ensure
			value_set: cut_pattern_xform_type.is_equal (a_value)
		end

	set_cut_pattern_no_parent (a_value: INTEGER)
			-- Set `cut_pattern_no_parent' to `a_value'
		do
			if a_value >= 0 then
				pattern.set_cut_pattern_no_parent (a_value)
			else
				set_null_cut_pattern_no_parent
			end
		end

	set_null_cut_pattern_no_parent
			-- Set `cut_pattern_no_parent' to null
		do
			pattern.set_null_cut_pattern_no_parent
		ensure
			value_null: is_null_cut_pattern_no_parent
		end

	set_production_id (a_value: STRING)
			-- Set `production_id' to `a_value'
		require
			a_value_not_void: a_value /= Void
			a_value_not_empty: not a_value.is_empty
		do
			pattern.set_production_id (a_value)
		ensure
			value_set: production_id.is_equal (a_value)
		end

	set_wafer_etest_name (a_name: STRING)
			-- Set `wafer_etest_name' to `a_name'
		do
			pattern.set_wafer_etest_name (a_name)
		ensure
			name_set: wafer_etest_name.is_equal (a_name)
		end

	set_quantity_wafers_needed (a_value: INTEGER has_value: BOOLEAN)
			-- Set `quantity_wafers_needed' to `a_value' where
			-- `has_value' indicates whether there really is a
			-- value
		do
			pattern.set_quantity_wafers_needed (a_value, has_value)
		end

	set_external_cut_distance_spec (a_value: like external_cut_distance_spec)
			-- Set `external_cut_distance_spec' to `a_value'
		require
			a_value_not_void: a_value /= Void
		do
			pattern.set_external_cut_distance_spec (a_value)
		end

	set_saw_blade_width (a_value: like saw_blade_width)
			-- Set `saw_blade_width' to `a_value'
		require
			a_value_positive: a_value > 0.0
		do
			pattern.set_saw_blade_width (a_value)
		end

feature -- Operations

	clear_change_history_list
			-- Clear the change history list for `Current'
		do
			context.reticle_change_history_list.wipe_out
		end

	remove_isolated_sites
			-- Remove all isolated sites for cut pattern
		do
			pattern.remove_isolated_sites
		end

	isolate (an_element: RETICLE_VIEW_ELEMENT)
			-- Isolate `an_element' in cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			pattern.isolate (an_element.model_element)
		end

	unisolate (an_element: RETICLE_VIEW_ELEMENT)
			-- Unisolate `an_element' in cut pattern
		require
			an_element_not_void: an_element /= Void
		do
			pattern.unisolate (an_element.model_element)
		end

	set_isolated_site_cut_die (an_element: RETICLE_VIEW_ELEMENT)
			-- Set cut die size and center from `an_element'
		require
			an_element_not_void: an_element /= Void
		do
			pattern.set_isolated_site_cut_die (an_element.model_element)
		end

end
