
note
	
	description: "List of elements of a reticle view, with added properties"
	author: "David Hollenberg"
	date: "October 4, 2006"

class RETICLE_VIEW_ELEMENT_LIST

create
	make

feature {NONE} -- Creation

	make (a_capacity: INTEGER)
			-- Create with initial capacity `a_capacity'
		require
			a_capacity_nonnegative: a_capacity >= 0
		do
			create list.make (a_capacity)
		end

feature -- Properties

	all_elements_selected: BOOLEAN
			-- Are all elements currently selected?
		local
			b: BOOLEAN
		do
			if not is_all_cached then
				b := True
				from
					list.start
				until
					list.after or not b
				loop
					b := b and list.item.is_selected
					list.forth
				end
				cached_all_elements_selected := b
				is_all_cached := True
			end
			Result := cached_all_elements_selected
		end

	any_elements_selected: BOOLEAN
			-- Are any elements currently selected?
		local
			b: BOOLEAN
		do
			if not is_any_cached then
				b := False
				from
					list.start
				until
					list.after or b
				loop
					b := b or list.item.is_selected
					list.forth
				end
				cached_any_elements_selected := b
				is_any_cached := True
			end
			Result := cached_any_elements_selected
		end

	any_elements_stationary_orientation: BOOLEAN
			-- Do any elements have a fixed orientation that
			-- cannot be changed?
		local
			b: BOOLEAN
		do
			if not is_any_stationary_orientation_cached then
				b := False
				from
					list.start
				until
					list.after or b
				loop
					b := b or list.item.is_stationary_orientation
					list.forth
				end
				cached_any_elements_stationary_orientation := b
				is_any_stationary_orientation_cached := True
			end
			Result := cached_any_elements_stationary_orientation
		end

	any_elements_rotation_locked: BOOLEAN
			-- Are any elements rotation-locked, so that their
			-- orientation cannot be changed?
		local
			b: BOOLEAN
		do
			if not is_any_rotation_locked_cached then
				b := False
				from
					list.start
				until
					list.after or b
				loop
					b := b or not list.item.is_rotatable and not list.item.is_stationary_orientation
					list.forth
				end
				cached_any_elements_rotation_locked := b
				is_any_rotation_locked_cached := True
			end
			Result := cached_any_elements_rotation_locked
		end

	is_processed: BOOLEAN
			-- Have any elements in `Current' already been
			-- processed for current operation?

	rotation_move_distance: DOUBLE
			-- Distance `Current' had to be moved in both
			-- X and Y after rotation to keep the edges on
			-- grid

	elements: LIST [RETICLE_VIEW_ELEMENT]
			-- Elements in list
		do
			Result := list
		end

feature -- Modification

	extend (an_element: RETICLE_VIEW_ELEMENT)
			-- Add `an_element' to end of list
		require
			an_element_not_void: an_element /= Void
		do
			list.extend (an_element)
		end

	remove (an_element: RETICLE_VIEW_ELEMENT)
			-- Remove `an_element' from list
		require
			an_element_not_void: an_element /= Void
		do
			list.prune_all (an_element)
			uncache_any_elements_selected
			uncache_any_elements_stationary_orientation
			uncache_any_elements_rotation_locked
			uncache_any_elements_contain
		end

	uncache_all
			-- Remove all cached values
		do
			uncache_all_elements_selected
			uncache_any_elements_selected
			uncache_any_elements_stationary_orientation
			uncache_any_elements_rotation_locked
			uncache_any_elements_contain
		end

	uncache_all_elements_selected
			-- Remove value computed for `all_elements_selected'
			-- from cache
		do
			is_all_cached := False
		end

	uncache_any_elements_selected
			-- Remove value computed for `any_elements_selected'
			-- from cache
		do
			is_any_cached := False
		end

	uncache_any_elements_stationary_orientation
			-- Remove value computed for
			-- `any_elements_stationary_orientation' from cache
		do
			is_any_stationary_orientation_cached := False
		end

	uncache_any_elements_rotation_locked
			-- Remove value computed for
			-- `any_elements_rotation_locked' from cache
		do
			is_any_rotation_locked_cached := False
		end

	uncache_any_elements_contain
			-- Remove value computed for
			-- `any_elements_contain' from cache
		do
			is_any_contain_cached := False
		end

	set_processed
			-- Set `is_processed' to true
		do
			is_processed := True
		end

	clear_processed
			-- Set `is_processed' to false
		do
			is_processed := False
		end

	set_rotation_move_distance (dist: like rotation_move_distance)
			-- Set `rotation_move_distance' to `dist'
		do
			rotation_move_distance := dist
		ensure
			rotation_move_distance_set: rotation_move_distance = dist
		end

feature {NONE} -- Implementation

	list: ARRAYED_LIST [RETICLE_VIEW_ELEMENT]

	cached_all_elements_selected: BOOLEAN
			-- Cached value for `all_elements_selected'

	is_all_cached: BOOLEAN
			-- Is value for `all_elements_selected' cached?

	cached_any_elements_selected: BOOLEAN
			-- Cached value for `any_elements_selected'

	is_any_cached: BOOLEAN
			-- Is value for `any_elements_selected' cached?

	cached_any_elements_stationary_orientation: BOOLEAN
			-- Cached value for
			-- `any_elements_stationary_orientation'

	is_any_stationary_orientation_cached: BOOLEAN
			-- Is value for `any_elements_stationary_orientation'
			-- cached?

	cached_any_elements_rotation_locked: BOOLEAN
			-- Cached value for
			-- `any_elements_rotation_locked'

	is_any_rotation_locked_cached: BOOLEAN
			-- Is value for `any_elements_rotation_locked'
			-- cached?

	cached_any_elements_contain: BOOLEAN
			-- Cached value for
			-- `any_elements_contain'

	is_any_contain_cached: BOOLEAN
			-- Is value for `any_elements_contain'
			-- cached?

end
