
note
	
	description: "Object that is a graphical element of a view"
	author: "David Hollenberg"
	date: "July 25, 2006"

deferred class VIEW_GRAPHICAL_ELEMENT

feature -- Properties

	view: RETICLE_VIEW
			-- View of which this is one element

	x: INTEGER
			-- X coordinate of upper left corner of bounding box.
			-- May not be the "true" value if very
			-- large coordinates are involved, but will work
			-- correctly for drawing the element

	y: INTEGER
			-- Y coordinate of upper left corner of bounding box.
			-- May not be the "true" value if very
			-- large coordinates are involved, but will work
			-- correctly for drawing the element

	height: INTEGER
			-- Size of minimum bounding box in Y dimension.
			-- May not be the "true" value if very
			-- large coordinates are involved, but will work
			-- correctly for drawing the element

	width: INTEGER
			-- Size of minimum bounding box in X dimension.
			-- May not be the "true" value if very
			-- large coordinates are involved, but will work
			-- correctly for drawing the element

	line_width: INTEGER
			-- Width of line to draw for border, in pixels
		deferred
		end

	label: STRING
			-- Current label to display
		deferred
		ensure
			result_not_void: Result /= Void
		end

	label_rotation: INTEGER
			-- Rotation of label in degrees clockwise
		deferred
		end

	is_border_drawn: BOOLEAN
			-- Should border be drawn for `Current'?
		do
			Result := True
		end

	is_interior_filled: BOOLEAN
			-- Should interior of element be filled when displayed?
		deferred
		end

	is_opaque: BOOLEAN
			-- Should element be displayed as opaque, so that
			-- it completely obscures the part of any element
			-- that is below it in stacking order?
		deferred
		end

	is_transparent: BOOLEAN
			-- Should element be displayed as transparent, so that
			-- we attempt to draw parts of elements that fall
			-- below it in stacking order so that they are visible?
		deferred
		end

feature -- Operations

	set_view_coordinates
			-- Set `width', `height', `x', `y' based on
			-- both size and location of model element
			-- on `view'
		deferred
		end

	unload_view_coordinates
		do
		end

	set_clip_area
			-- Set clip area for view to bounding box of
			-- element, if no clip region is already
			-- defined for view
		do
		end

	remove_clip_area
			-- Remove clip area for view, if no clip
			-- region is already defined for view
		do
		end

feature {NONE} -- Implementation


end

