class DIRECTION_OF_TRAVERSAL
	-- a small class that will help determine whether a path around a
	--  point is going CW or CCW (and even how many times it wraps)

inherit
	GEOMETRY_RESOLUTION_ACCESS

	MORE_DOUBLE_MATH

create
	make

feature	-- public data

	reference_point,
		-- the fixed point around which we are traversing

	first_point, 
	last_point: CONST_POINT
		-- the first and last points in the traversal to date

	traversed_angle: DOUBLE
		-- the traversed angle between first_point and last_point
		-- angle is in degrees; positive is CCW; negative is CW

feature	-- extension, either fore or aft

	traverse_to(nxt_pt: CONST_POINT)
		-- include traversal from last_point to nxt_pt,
		-- which then becomes the new last_point
	require have_pt: nxt_pt /= Void
		not_at_ref: not reference_point.is_same_point(nxt_pt)
		not_opposite: last_point = Void or else
				gres.ne(last_point.direction_degrees(reference_point),
					reference_point.direction_degrees(nxt_pt))
	local alst, anxt, adif: DOUBLE
	do  if first_point = Void then
		first_point := nxt_pt.twin
	    else
		-- note angular change from reference
		alst := reference_point.direction_degrees(last_point);
		anxt := reference_point.direction_degrees(nxt_pt);
		adif := normalized_degrees(anxt - alst);
		-- now update
		traversed_angle := traversed_angle + adif
	    end;
	    last_point := nxt_pt.twin
	end

	traverse_from(prev_pt: CONST_POINT)
		-- include traversal from prev_pt to first_point,
		-- which then becomes the new first_point
	require have_pt: prev_pt /= Void
		not_at_ref: not reference_point.is_same_point(prev_pt)
		not_opposite: first_point = Void or else
				gres.ne(first_point.direction_degrees(reference_point),
					reference_point.direction_degrees(prev_pt))
	local afrst, aprev, adif: DOUBLE
	do  if first_point = Void then
		last_point := prev_pt.twin
	    else
		-- note angular change from reference
		afrst := reference_point.direction_degrees(first_point);
		aprev := reference_point.direction_degrees(prev_pt);
		adif := normalized_degrees(afrst - aprev);
		-- now update
		traversed_angle := traversed_angle + adif
	    end;
	    first_point := prev_pt.twin
	end

	direction_of_traversal: INTEGER
		-- cumulative direction of traversal from first to last
		-- +1 = CCW; 0 = none; -1 = CW
		-- (same values as going_CCW and going CW)
	do  Result := traversed_angle.sign
	end;

feature {NONE} -- creation

	make(ref_pt: CONST_POINT)
	require	have_point: ref_pt /= Void
	do  reference_point := ref_pt.twin
	end;

end -- DIRECTION_OF_TRAVERSAL
