deferred class LIST_OF_POINTS
		-- adds line_segment handling to SIMPLE_LIST_OF_POINTS
		-- supports both TRACE (open) and POLYGON (closed)

inherit
	SIMPLE_LIST_OF_POINTS
	redefine out
	end;

	GEOMETRY_SUPPORT
	export {NONE} all
	redefine out
	end;

-- creation	make, make_copy, make_transformed
			-- (for children who are real)

feature -- segment from i-th point to the next one
	-- circular (if allowed); count-th segment goes back to first point

	valid_segment(i: INTEGER): BOOLEAN
		-- either 1 to count or 1 to count-1
	deferred
	end;

	i_th_segment(i: INTEGER): LINE_SEGMENT
		-- segment from point i.abs to point i.abs+1 (circularly if allowed)
		-- if i < 0, then same segment, but reversed ends
	require valid_segment(i.abs)
	do  create Result.make_xy(0,0,0,0);
	    put_i_th_segment(i, Result)
	end;

	put_i_th_segment(i: INTEGER; here: LINE_SEGMENT)
		-- build (i.abs)-th segment in here
	require valid_segment(i.abs); here /= Void
	local i1,i2: INTEGER;
	do  i1 := i.abs;
	    if i1 = count then i2 := 1 else i2 := i1+1 end;
	    here.make_xy(i_th_X(i1), i_th_Y(i1), i_th_X(i2), i_th_Y(i2))
	    if i < 0 then here.reverse end;
	end;

	segment_length(i: INTEGER): DOUBLE
	require valid_segment(i)
	local i2: INTEGER;
	do  if i = count then i2 := 1 else i2 := i+1 end;
	    Result := radius(i_th_X(i) - i_th_X(i2), i_th_Y(i) - i_th_Y(i2))
	end;

feature -- first and last points

	first: POINT
	require not is_empty
	do  Result := i_th(1)
	end;

	last: POINT
	require not is_empty
	do  Result := i_th(count)
	end;

feature -- current point (the item)

	item: POINT
	require on: not off
	do  Result := i_th(index)
	end;

	put_item_segment(here: LINE_SEGMENT)
	require on: valid_segment(index)
	do  put_i_th_segment(index, here)
	end;

feature -- other things one can do

	is_Manhattan: BOOLEAN
		-- is every segment horizontal or vertical?
	local ls: LINE_SEGMENT; nope: BOOLEAN
	do  ls := scratch_line_segment
	    from  start
	    until off or nope
	    loop  if valid_segment(index)	-- dont look at last seg
		   then put_item_segment(ls);	--  for a trace
			nope := not ls.is_Manhattan
		  end
		  forth
	    end
	end;

	bounding_box: CONST_RECTANGLE
		-- Void if the list is empty
	local lx, ly, hx, hy, ix, iy: DOUBLE; i: INTEGER
	do  if not is_empty
	     then lx := first_X; hx := lx;  
		  ly := first_Y; hy := ly
		  from  i := 2
		  until i > count
		  loop  ix := i_th_X(i);   iy := i_th_Y(i);
			lx := lx.min(ix);  hx := hx.max(ix);
			ly := ly.min(iy);  hy := hy.max(iy);
			i := i + 1
		  end;
		  create Result.make_by_corners(lx, ly, hx, hy)
	    end
	end;

	out: STRING
	local i: INTEGER
	do  create Result.make(100);
	    Result.extend('[');
	    if is_empty
	     then Result.append("<empty>")
	    elseif count < 5
	     then from  i := 1
		  until i > count
		  loop  append_one_point(Result, i);
			if i < count then Result.extend(';') end
			i := i + 1
		  end
	     else append_one_point(Result, 1);
		  Result.append(";<");
		  Result.append_integer(count-2);
		  Result.append(" intervening points>");
		  append_one_point(Result, count)
	    end
	    Result.extend(']');
	end;

feature {LIST_OF_POINTS} -- Implementation

	append_one_point(res: STRING; i: INTEGER)
	require valid_index(i);
	do  res.extend('(');  res.append_double(i_th_X(i));
	    res.extend(',');  res.append_double(i_th_Y(i));
	    res.extend(')')
	end;

feature {NONE} -- creation (for all my children)

	make_transformed(src: LIST_OF_POINTS; at: MATRIX)
	require src /= Void; at /= Void
	local cnt, i: INTEGER; x, y: DOUBLE
	do  cnt := src.count;
	    make(cnt);
	    from  i := 1
	    until i > cnt
	    loop  x := src.i_th_X(i); y := src.i_th_Y(i);
		  extend(at.transform_xy(x, y))
		  i := i + 1
	    end;
	ensure count = src.count
	end;

end -- LIST_OF_POINTS
