class RECTANGLE
	-- a Manhattan rectangle that can be altered and/or reused

inherit
	ANY
		undefine
			out
		end

	CONST_RECTANGLE
	export {ANY} make_copy, make_by_corners, make_by_size_and_center
	end;

create 
	make_copy,
	make_corners, make_by_corners,
	make_size_and_center, make_by_size_and_center

feature -- alteration of self

	move_by(dx, dy: DOUBLE)
		-- move by dx in x and dy in y
	do  center_x := center_x + dx;
	    center_y := center_y + dy;
	end;

	move_to(cx, cy: DOUBLE)
		-- move so that cx, cy is the new center
	do  center_x := cx; center_y := cy
	ensure center_x = cx and center_y = cy
	end;

	scale_by(sf: DOUBLE)
		-- scale size by sf; retain center
	require sf > 0;
	do  size_x := size_x*sf;
	    size_y := size_y*sf
	end;

	scale_coordinates_by(sf: DOUBLE)
		-- scale size AND center by sf
	require sf > 0;
	do  size_x := size_x * sf;
	    size_y := size_y * sf;
	    center_x := center_x * sf;
	    center_y := center_y * sf;
	end;

	size_by(dx, dy: DOUBLE)
		-- grow by dx, dy (mebes onesided sizing)
	require size_x + dx >= 0; size_y + dy >= 0;
	do  size_x := size_x + dx;
	    size_y := size_y + dy
	ensure	size_x = dx + old size_x;
		size_y = dy + old size_y
	end;

	extend_to_enclose(pt: VIRTUAL_POINT)
		-- extend me to enclose pt
		-- (pt will be on the edge unless it was already inside)
	do  if pt /= Void
	     then make_by_corners(
			low_x.min(pt.x), low_y.min(pt.y),
			high_x.max(pt.x), high_y.max(pt.y))
	    end
	end;

	extend_to_include(other: CONST_RECTANGLE)
		-- extend me to include other 
		-- (same rectangle as union_with, but done in place)
	do  if other /= Void
	     then make_by_corners(
			low_x.min(other.low_x), low_y.min(other.low_y),
			high_x.max(other.high_x), high_y.max(other.high_y))
	    end
	end;

	transform_by(m: MATRIX)
		-- become box after being transformed by m
	require m.is_Manhattan
	local  lx, ly, hx, hy: DOUBLE
	do  -- since m is Manhattan, we can just do two opposite corners
	    --  to determine the new rectangle
	    lx := m.transformed_X(low_x, low_y);
	    ly := m.transformed_Y(low_x, low_y);
	    hx := m.transformed_X(high_x, high_y);
	    hy := m.transformed_Y(high_x, high_y);
	    make_by_corners(lx, ly, hx, hy)
		-- the edges will be resorted into low and high
	end;

end -- RECTANGLE
