note
	description:
		"A basic list of points."
	author: "Barden E. Smith subsetted from J.Goldberg"
	date:	"September 8, 2004"

class
	SIMPLE_LIST_OF_POINTS

inherit
	GEOMETRY_RESOLUTION_ACCESS

create
	make,
	make_copy

feature	-- status

	capacity: INTEGER
			-- current capacity of the list
		do
			Result := xy_list.capacity // 2
		end

	count: INTEGER
			-- list size (in points)
			-- NOTE: may be smaller than xy_list.count // 2

	is_empty: BOOLEAN
		do
			Result := count = 0
		end
	
	valid_index (i: INTEGER): BOOLEAN
		-- use standard list indexing of 1...count
		do
			Result := 1 <= i and i <= count
		end

feature -- Access, as separate coordinates or as a POINT

	i_th_x (i: INTEGER): DOUBLE
		require
			i_valid: valid_index (i)
		do
			Result := xy_list.item (2*i - 1)
		end
	
	i_th_y (i: INTEGER): DOUBLE
		require
			i_valid: valid_index (i)
		do
			Result := xy_list.item (2*i)
		end

	i_th(I: INTEGER): POINT
	require i_valid: valid_index(i)
	do  create Result.make_xy(i_th_X(i), I_th_Y(i))
	ensure Result.x = i_th_X(i) and Result.y = i_th_Y(i)
	end;

feature {SIMPLE_LIST_OF_POINTS} -- alteration

	put_i_th_x (x: DOUBLE; i: INTEGER)
		require
			i_valid: valid_index (i)
		do
			xy_list.put (x, 2*i - 1)
		end

	put_i_th_y (y: DOUBLE; i: INTEGER)
		require
			i_valid: valid_index (i)
		do
			xy_list.put (y, 2*i)
		end


feature -- first and last values

	first_x: DOUBLE
		require
			not_empty: not is_empty
		do
			Result := i_th_x (1)
		end

	first_y: DOUBLE
		require
			not_empty: not is_empty
		do
			Result := i_th_y (1)
		end

	last_x: DOUBLE
		require
			not_empty: not is_empty
		do
			Result := i_th_x (count)
		end

	last_y: DOUBLE
		require
			not_empty: not is_empty
		do
			Result := i_th_y (count)
		end

feature -- traversal

	index: INTEGER
		-- where we currently are in list

	before: BOOLEAN
		do
			Result := index < 1
		end

	after: BOOLEAN
		do
			Result := index > count
		end

	off: BOOLEAN
		do
			Result := not valid_index (index)
		end

	start
		do
			index := 1
		end

	finish
		do
			index := count
		end

	back
		require
			not_before: not before
		do
			index := index - 1
		end

	forth
		require
			not_after: not after
		do
			index := index + 1
		end

feature -- current point (the item)

	item_X: DOUBLE
		require
			on: not off
		do
			Result := i_th_x (index)
		end

	item_Y: DOUBLE
		require
			on: not off
		do
			Result := i_th_y (index)
		end

feature -- list extension

	extend_xy (x, y: DOUBLE)
		do
			count := count + 1
			xy_list.force(x, 2*count - 1)
			xy_list.force(y, 2*count)
		ensure
			count = old count + 1
		end

	extend_if_different_XY(x, y: DOUBLE)
		do
			if is_empty or else 
			   not (gres.eq(x, last_X) and gres.eq(y, last_Y)) then
				extend_xy (x, y)
			    end
		ensure
			gres.eq(x, last_X) and gres.eq(y, last_Y)
	end

feature -- list extension using points

	extend(p: VIRTUAL_POINT)
	require have_point: p /= Void
	do  extend_XY(p.x, p.y);
	ensure count = old count + 1
	end;

	extend_if_different(p: VIRTUAL_POINT)
		-- extend our list UNLESS p = last
	require have_point: p /= Void
	do  extend_if_different_XY(p.x, p.y)
	end;

	append (src: SIMPLE_LIST_OF_POINTS)
		require src /= Void
		local ndx: INTEGER
		do  from  ndx := 1
		    until not src.valid_index (ndx)
		    loop  extend_xy (src.i_th_x(ndx), src.i_th_y(ndx))
			  ndx := ndx + 1
		    end
		ensure count = old count + src.count
		end;

	append_if_different (src: SIMPLE_LIST_OF_POINTS)
		local ndx: INTEGER
		do  from  ndx := 1
		    until not src.valid_index (ndx)
		    loop  extend_if_different_xy
				 (src.i_th_x(ndx), src.i_th_y(ndx))
			  ndx := ndx + 1
		    end
		end;

feature -- list erasure

	wipe_out
		do
			count := 0
			-- we used to wipe out the xy_list also
		ensure
			is_empty
		end

feature {SIMPLE_LIST_OF_POINTS} -- actual list of coordinates, X followed by Y

	xy_list: ARRAY[DOUBLE]

feature {NONE} -- Creation 

	make (n: INTEGER)
			-- `n' is initial capacity
		require
			positive_capacity: n > 0
		do
			create xy_list.make_filled (0.0, 1, 2*n)
			count := 0
			start
		end

	make_copy(src: SIMPLE_LIST_OF_POINTS)
		require
			have_source: src /= Void
		do
			create xy_list.make_filled (0.0, 1, 2*src.count)
			xy_list.copy (src.xy_list)
			count := src.count
			 start
		ensure
			count = src.count
		end

end -- class SIMPLE_LIST_OF_POINTS
