
note
	
	description: "An operation that checks that count of database %
		%rows in a list matches expected number"
	author: "David Hollenberg"
	date: "May 14, 2003"

class ROW_COUNT_CHECK_OP
inherit
	OPERATION
		rename
			try_operation as try_to_check_count
		end

create
	default_create

feature -- Operations

	check_count_within_range (list: LIST [DB_ROW] qt, qb: STRING r: ITEM_COUNT_RANGE)
			-- Check that number of rows in `list', which
			-- was retrieved using query with text `qt' and
			-- bindings described by `qb', falls within
			-- range `r'
		require
			list_exists: list /= Void
			query_text_exists: qt /= Void
			query_bindings_exist: qb /= Void
			range_exists: r /= Void
		do
			check_actual_count_within_range (list.count, qt, qb, r)
		end

	check_actual_count_within_range (n: INTEGER qt, qb: STRING r: ITEM_COUNT_RANGE)
			-- Check that number of rows `n', which
			-- was retrieved using query with text `qt' and
			-- bindings described by `qb', falls within
			-- range `r'
		require
			actual_count_nonnegative: n >= 0
			query_text_exists: qt /= Void
			query_bindings_exist: qb /= Void
			range_exists: r /= Void
		do
			actual_count := n
			query_text := qt
			query_bindings := qb
			range := r
			execute
		end

feature {NONE} -- Implementation

	actual_count: INTEGER
			-- Actual number of rows in list

	range: ITEM_COUNT_RANGE
			-- Range within which count of items in list
			-- must fall

	query_text: STRING
			-- Text of query used to retrieve rows

	query_bindings: STRING
			-- Binding of placeholders in query used to
			-- retrieve rows

	try_to_check_count
			-- Try to check count of rows
		do
		end

end
