
note
	
	author: "David Hollenberg"
	date:	"February 8, 1996"
	description: "A class which can fill in a string from an arbitrary %
		%source of characters (pointer and length)"

class STRING_FILLER

inherit
	STRING
		rename
			set_count as string_set_count,
			area as string_area,
			make as string_make,
			append_substring as string_append_substring
		export
			{ANY} default_pointer
			{STRING_FILLER} all
		end

create
	make
create {STRING_FILLER}
	string_make
feature

	make
			-- Initialize newly created object
		do
			string_make (0)
		end

	fill_string (s: STRING addr: POINTER len: INTEGER)
			-- Fill `s' from character source which is at
			-- address `addr' and is of length `len'
		require
			string_exists: s /= Void
			addr_exists: addr /= default_pointer
			nonnegative_length: len >= 0
		local
			area: ANY
		do
			if s.capacity < len then
				s.resize (len)
			end
			s.set_count (len)
			area := s.area
			memcpy ($area, addr, len)
		ensure
			count_set: s.count = len
		end

	set_count (s: STRING len: INTEGER)
			-- Set `count' of `s' to `len'
		require
			string_exists: s /= Void
			nonnegative_length: len >= 0
			length_small_enough: len <= s.capacity
		do
			s.set_count (len)
		ensure
			count_set: s.count = len
		end

	append_substring (dest, src: STRING n1, n2: INTEGER)
			-- Append to `dest' the substring of `src' from
			-- indexes `n1' to `n2'
		require
			destination_exists: dest /= Void
			source_exists: src /= Void
		local
			old_len, new_len, src_offset, dest_offset: INTEGER
			dest_area, src_area: ANY
		do
			if (1 <= n1) and (n1 <= n2) and (n2 <= src.count) then
				old_len := dest.count
				new_len := old_len + n2 - n1 + 1
				dest.grow (new_len)
				src_area := src.area
				dest_area := dest.area
				src_offset := n1 - 1
				dest_offset := old_len
				copy_offset_area ($src_area, $dest_area,
					src_offset, dest_offset, n2 - n1 + 1)
				dest.set_count (new_len)
			end
		end
	
feature {NONE} -- Externals

	memcpy (dest, src: POINTER len: INTEGER)
		external
			-- Changed for release 4.3 (was "C (char *, char *, int)"
			"C use <memory.h>"
		end

	copy_offset_area (src, dest: POINTER src_offset, dest_offset, len: INTEGER)
		external
			"C"
		end

end

		
